@@ -52,14 +52,18 @@ import net.sf.freecol.client.gui.action.ActionManager;
 import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.client.gui.sound.MusicLibrary;
 import net.sf.freecol.client.gui.sound.SfxLibrary;
+import net.sf.freecol.client.gui.sound.SoundLibrary;
 import net.sf.freecol.client.gui.sound.SoundPlayer;
 import net.sf.freecol.client.networking.Client;
 import net.sf.freecol.common.PseudoRandom;
 import net.sf.freecol.common.model.Game;
 import net.sf.freecol.common.model.Player;
+import net.sf.freecol.common.model.UnitType;
 import net.sf.freecol.common.networking.Message;
+import net.sf.freecol.common.option.AudioMixerOption;
 import net.sf.freecol.common.option.LanguageOption;
 import net.sf.freecol.common.option.Option;
+import net.sf.freecol.common.option.PercentageOption;
 import net.sf.freecol.common.option.LanguageOption.Language;
 import net.sf.freecol.server.FreeColServer;
 
@@ -70,11 +74,9 @@ import org.w3c.dom.Element;
  * keeps references to the GUI and the control objects.
  */
 public final class FreeColClient {
+
     private static final Logger logger = Logger.getLogger(FreeColClient.class.getName());
 
-
-
-
     // Control:
     private ConnectController connectController;
 
@@ -106,7 +108,6 @@ public final class FreeColClient {
 
     private SfxLibrary sfxLibrary;
 
-    @SuppressWarnings("unused")
     private SoundPlayer musicPlayer;
 
     private SoundPlayer sfxPlayer;
@@ -150,6 +151,11 @@ public final class FreeColClient {
     
     private Rectangle windowBounds;
 
+    /**
+     * Describe headless here.
+     */
+    private boolean headless;
+
 
     /**
      * Creates a new <code>FreeColClient</code>. Creates the control objects
@@ -164,8 +170,10 @@ public final class FreeColClient {
      * @param musicLibrary The object holding the music.
      * @param sfxLibrary The object holding the sound effects.
      */
-    public FreeColClient(boolean windowed, final Dimension innerWindowSize, ImageLibrary imageLibrary, MusicLibrary musicLibrary,
-            SfxLibrary sfxLibrary) {
+    public FreeColClient(boolean windowed, final Dimension innerWindowSize, 
+                         ImageLibrary imageLibrary, MusicLibrary musicLibrary,
+                         SfxLibrary sfxLibrary) {
+        headless = "true".equals(System.getProperty("java.awt.headless", "false"));
         this.windowed = windowed;
         this.imageLibrary = imageLibrary;
         this.musicLibrary = musicLibrary;
@@ -174,7 +182,14 @@ public final class FreeColClient {
         mapEditor = false;
         
         clientOptions = new ClientOptions();
+        if (FreeCol.getClientOptionsFile() != null
+                && FreeCol.getClientOptionsFile().exists()) {
+            clientOptions.load(FreeCol.getClientOptionsFile());
+        }
         actionManager = new ActionManager(this);
+        if (!headless) {
+            actionManager.initializeActions();
+        }
         // Control:
         connectController = new ConnectController(this);
         preGameController = new PreGameController(this);
@@ -185,30 +200,32 @@ public final class FreeColClient {
         mapEditorController = new MapEditorController(this);
         
         // Gui:
-        SwingUtilities.invokeLater(new Runnable() {
-            public void run() {
-                startGUI(innerWindowSize);
-            }
-        });
+        if (!headless) {
+            SwingUtilities.invokeLater(new Runnable() {
+                    public void run() {
+                        startGUI(innerWindowSize);
+                    }
+                });
+        }
         worker = new Worker();
         worker.start();
         
         if (FreeCol.getClientOptionsFile() != null
                 && FreeCol.getClientOptionsFile().exists()) {
-            clientOptions.load(FreeCol.getClientOptionsFile());
-            Option o = clientOptions.getObject(ClientOptions.LANGUAGE);
-            o.addPropertyChangeListener(new PropertyChangeListener() {
-                public void propertyChange(PropertyChangeEvent e) {
-                    if (((Language) e.getNewValue()).getKey().equals(LanguageOption.AUTO)) {
-                        canvas.showInformationMessage("autodetectLanguageSelected");
-                    } else {
-                        Locale l = ((Language) e.getNewValue()).getLocale();
-                        Messages.setMessageBundle(l);
-                        canvas.showInformationMessage("newLanguageSelected",
-                                                      new String[][] {{"%language%", l.getDisplayName()}});
-                    }
-                }
-            });
+            if (!headless) {
+                Option o = clientOptions.getObject(ClientOptions.LANGUAGE);
+                o.addPropertyChangeListener(new PropertyChangeListener() {
+                        public void propertyChange(PropertyChangeEvent e) {
+                            if (((Language) e.getNewValue()).getKey().equals(LanguageOption.AUTO)) {
+                                canvas.showInformationMessage("autodetectLanguageSelected");
+                            } else {
+                                Locale l = ((Language) e.getNewValue()).getLocale();
+                                Messages.setMessageBundle(l);
+                                canvas.showInformationMessage("newLanguageSelected", "%language%", l.getDisplayName());
+                            }
+                        }
+                    });
+            }
         }
     }
 
@@ -216,14 +233,21 @@ public final class FreeColClient {
      * Starts the GUI by creating and displaying the GUI-objects.
      */
     private void startGUI(Dimension innerWindowSize) {
+        final AudioMixerOption amo = (AudioMixerOption) getClientOptions().getObject(ClientOptions.AUDIO_MIXER);
         if (musicLibrary != null) {
-            musicPlayer = new SoundPlayer(false, true, true);
+            musicPlayer = new SoundPlayer(amo,
+                    (PercentageOption) getClientOptions().getObject(ClientOptions.MUSIC_VOLUME),
+                    false,
+                    true);
+            playMusic("intro");
         } else {
             musicPlayer = null;
         }
-        // TODO: Start playing some music here.
         if (sfxLibrary != null) {
-            sfxPlayer = new SoundPlayer(true, false, false);
+            sfxPlayer = new SoundPlayer(amo,
+                    (PercentageOption) getClientOptions().getObject(ClientOptions.SFX_VOLUME),
+                    true,
+                    false);
         } else {
             sfxPlayer = null;
         }
@@ -251,13 +275,36 @@ public final class FreeColClient {
         gui = new GUI(this, innerWindowSize, imageLibrary);
         canvas = new Canvas(this, innerWindowSize, gui);
         changeWindowedMode(windowed);
+
+        UnitType galleonType = FreeCol.getSpecification().getUnitType("model.unit.galleon");
+        frame.setIconImage(imageLibrary.getUnitImageIcon(galleonType).getImage());
         
-        frame.setIconImage(imageLibrary.getUnitImage(ImageLibrary.GALLEON));
-        
-        canvas.showMainPanel();
+        SwingUtilities.invokeLater(new Runnable() {
+        	public void run() {
+        		canvas.showMainPanel();
+        	}
+        });
         gui.startCursorBlinking();
     }
-    
+
+    /**
+     * Get the <code>Headless</code> value.
+     *
+     * @return a <code>boolean</code> value
+     */
+    public boolean isHeadless() {
+        return headless;
+    }
+
+    /**
+     * Set the <code>Headless</code> value.
+     *
+     * @param newHeadless The new Headless value.
+     */
+    public void setHeadless(final boolean newHeadless) {
+        this.headless = newHeadless;
+    }
+
     /**
      * Change the windowed mode.
      * @param windowed Use <code>true</code> for windowed mode
@@ -454,7 +501,7 @@ public final class FreeColClient {
     }
 
     /**
-     * Gets the <code>GUI</code> that is beeing used to draw the map on the
+     * Gets the <code>GUI</code> that is being used to draw the map on the
      * {@link Canvas}.
      * 
      * @return The <code>GUI</code>.
@@ -474,6 +521,19 @@ public final class FreeColClient {
         System.exit(0);
     }
 
+    /**
+     * Retires the player from the game. Returns true if the player
+     * achieved a new high score.
+     *
+     * @return a <code>boolean</code> value
+     */
+    public boolean retire() {
+        Element retireElement = Message.createNewRootElement("retire");
+        Element reply = client.ask(retireElement);
+        return ("true".equals(reply.getAttribute("highScore")));
+    }
+
+
     /**
      * Continue playing after win the game
      */
@@ -506,9 +566,9 @@ public final class FreeColClient {
     // this.windowed = windowed;
     // }
     /**
-     * Sets wether or not this game is a singleplayer game.
+     * Sets whether or not this game is a singleplayer game.
      * 
-     * @param singleplayer Indicates wether or not this game is a singleplayer
+     * @param singleplayer Indicates whether or not this game is a singleplayer
      *            game.
      * @see #isSingleplayer
      */
@@ -605,12 +665,51 @@ public final class FreeColClient {
         return client;
     }
 
+    /**
+     * Plays the music.
+     */
+    public void playMusic(String music) {
+        if (musicPlayer != null) {
+            musicPlayer.play(musicLibrary.get(music));
+        }
+    }
+    
+    /**
+     * Plays a random music from the given playlist.
+     */
+    public void playMusicOnce(String music) {
+        if (musicPlayer != null) {
+            musicPlayer.playOnce(musicLibrary.get(music));
+        }
+    }
+    
+    /**
+     * Plays a random music from the given playlist.
+     * @param delay A delay before playing the sound (ms).
+     */
+    public void playMusicOnce(String music, int delay) {
+        if (musicPlayer != null) {
+            musicPlayer.playOnce(musicLibrary.get(music), delay);
+        }
+    }
+    
     /**
      * Plays the given sound effect.
      * 
      * @param sound The key sound effect given by {@link SfxLibrary}.
      */
-    public void playSound(int sound) {
+    public void playSound(String sound) {
+        if (sfxPlayer != null) {
+            sfxPlayer.play(sfxLibrary.get(sound));
+        }
+    }
+
+    /**
+     * Plays the given sound effect.
+     * 
+     * @param sound The key sound effect given by {@link SfxLibrary}.
+     */
+    public void playSound(SoundLibrary.SoundEffect sound) {
         if (sfxPlayer != null) {
             sfxPlayer.play(sfxLibrary.get(sound));
         }
