@@ -25,15 +25,20 @@ import java.util.logging.Logger;
 import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.networking.Client;
 import net.sf.freecol.common.PseudoRandom;
+import net.sf.freecol.common.model.Building;
+import net.sf.freecol.common.model.BuildingType;
 import net.sf.freecol.common.model.FreeColGameObject;
+import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.Game;
 import net.sf.freecol.common.model.Location;
 import net.sf.freecol.common.model.ModelController;
 import net.sf.freecol.common.model.Ownable;
 import net.sf.freecol.common.model.Player;
+import net.sf.freecol.common.model.Player.Stance;
 import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.TradeRoute;
 import net.sf.freecol.common.model.Unit;
+import net.sf.freecol.common.model.UnitType;
 import net.sf.freecol.common.networking.Message;
 
 import org.w3c.dom.Element;
@@ -42,11 +47,10 @@ import org.w3c.dom.Element;
  * A client-side implementation of the <code>ModelController</code> interface.
  */
 public class ClientModelController implements ModelController {
+
     private static final Logger logger = Logger.getLogger(ClientModelController.class.getName());
 
 
-
-
     private final FreeColClient freeColClient;
 
 
@@ -80,9 +84,9 @@ public class ClientModelController implements ModelController {
         getRandomElement.setAttribute("taskID", taskID);
         getRandomElement.setAttribute("n", Integer.toString(n));
 
-        logger.info("TaskID is " + taskID + " Waiting for the server to reply...");
+        //logger.info("TaskID is " + taskID + " Waiting for the server to reply...");
         Element reply = client.ask(getRandomElement);
-        logger.info("Reply received from server.");
+        //logger.info("Reply received from server.");
 
         if (!reply.getTagName().equals("getRandomConfirmed")) {
             logger.warning("Wrong tag name.");
@@ -108,17 +112,16 @@ public class ClientModelController implements ModelController {
      * @param type The type of unit (Unit.FREE_COLONIST...).
      * @return The created <code>Unit</code>.
      */
-    public Unit createUnit(String taskID, Location location, Player owner, int type) {
-        Client client = freeColClient.getClient();
+    public Unit createUnit(String taskID, Location location, Player owner, UnitType type) {
 
         Element createUnitElement = Message.createNewRootElement("createUnit");
         createUnitElement.setAttribute("taskID", taskID);
         createUnitElement.setAttribute("location", location.getId());
         createUnitElement.setAttribute("owner", owner.getId());
-        createUnitElement.setAttribute("type", Integer.toString(type));
+        createUnitElement.setAttribute("type", type.getId());
 
         logger.info("Waiting for the server to reply...");
-        Element reply = client.ask(createUnitElement);
+        Element reply = freeColClient.getClient().ask(createUnitElement);
         logger.info("Reply received from server.");
 
         if (!reply.getTagName().equals("createUnitConfirmed")) {
@@ -126,13 +129,51 @@ public class ClientModelController implements ModelController {
             throw new IllegalStateException();
         }
 
-        Unit unit = new Unit(freeColClient.getGame(), (Element) reply.getElementsByTagName(Unit.getXMLElementTagName())
-                .item(0));
-        unit.setLocation(unit.getLocation());
+        Element unitElement = (Element) reply.getElementsByTagName(Unit.getXMLElementTagName()).item(0);
+        Unit unit = new Unit(freeColClient.getGame(), unitElement);
+        // unit is not really at location yet, set it carefully
+        unit.setLocationNoUpdate(null);
+        unit.setLocation(location);
 
         return unit;
     }
 
+    /**
+     * Creates a new building.
+     * 
+     * @param taskID The <code>taskID</code> should be a unique identifier.
+     *            One method to make a unique <code>taskID</code>: <br>
+     *            <br>
+     *            getId() + "methodName:taskDescription" <br>
+     *            <br>
+     *            As long as the "taskDescription" is unique within the method
+     *            ("methodName"), you get a unique identifier.
+     * @param colony The <code>Colony</code> where the <code>Building</code>
+     *            will be created.
+     * @param type The type of building (Building.FREE_COLONIST...).
+     * @return The created <code>Building</code>.
+     */
+    public Building createBuilding(String taskID, Colony colony, BuildingType type) {
+
+        Element createBuildingElement = Message.createNewRootElement("createBuilding");
+        createBuildingElement.setAttribute("taskID", taskID);
+        createBuildingElement.setAttribute("colony", colony.getId());
+        createBuildingElement.setAttribute("type", type.getId());
+
+        logger.info("Waiting for the server to reply...");
+        Element reply = freeColClient.getClient().ask(createBuildingElement);
+        logger.info("Reply received from server.");
+
+        if (!reply.getTagName().equals("createBuildingConfirmed")) {
+            logger.warning("Wrong tag name.");
+            throw new IllegalStateException();
+        }
+        
+        Element buildingElement = (Element) reply.getElementsByTagName(Building.getXMLElementTagName()).item(0);
+        Building building = new Building(freeColClient.getGame(), buildingElement);
+        return building;
+    }
+
     /**
      * Puts the specified <code>Unit</code> in America.
      * 
@@ -165,7 +206,7 @@ public class ClientModelController implements ModelController {
      * @param second The second <code>Player</code>.
      * @param stance The new stance.
      */
-    public void setStance(Player first, Player second, int stance) {
+    public void setStance(Player first, Player second, Stance stance) {
         // Nothing to do.
     }
 
@@ -215,8 +256,8 @@ public class ClientModelController implements ModelController {
             throw new IllegalStateException();
         }
 
-        TradeRoute tradeRoute = new TradeRoute(game, (Element) reply.getElementsByTagName(
-                TradeRoute.getXMLElementTagName()).item(0));
+        Element routeElement = (Element) reply.getElementsByTagName(TradeRoute.getXMLElementTagName()).item(0);
+        TradeRoute tradeRoute = new TradeRoute(game, routeElement);
 
         return tradeRoute;
     }
