@@ -41,11 +41,15 @@ import net.sf.freecol.client.ClientOptions;
 import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.gui.Canvas;
 import net.sf.freecol.client.gui.i18n.Messages;
+import net.sf.freecol.client.gui.panel.ChoiceItem;
 import net.sf.freecol.client.gui.panel.LoadingSavegameDialog;
 import net.sf.freecol.client.networking.Client;
 import net.sf.freecol.common.FreeColException;
 import net.sf.freecol.common.ServerInfo;
+import net.sf.freecol.common.Specification;
+import net.sf.freecol.common.io.FreeColSavegameFile;
 import net.sf.freecol.common.model.Game;
+import net.sf.freecol.common.model.GameOptions;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.networking.Connection;
 import net.sf.freecol.common.networking.Message;
@@ -68,7 +72,9 @@ public final class ConnectController {
 
     private final FreeColClient freeColClient;
 
-
+    private int numberOfPlayers = 4;
+    private int advantages;
+    private boolean additionalNations = true;
 
     /**
     * Creates a new <code>ConnectController</code>.
@@ -80,14 +86,18 @@ public final class ConnectController {
 
     
     /**
-    * Starts a multiplayer server and connects to it.
-    *
-    * @param publicServer Should this server be listed at the meta server.
-    * @param username The name to use when logging in.
-    * @param port The port in which the server should listen for new clients.
-    */
-    public void startMultiplayerGame(boolean publicServer, String username, int port) {
+     * Starts a multiplayer server and connects to it.
+     *
+     * @param publicServer Should this server be listed at the meta server.
+     * @param username The name to use when logging in.
+     * @param port The port in which the server should listen for new clients.
+     * @param players The number of selectable European players in this game.
+     * @param advantages Whether to use national advantages.
+     */
+    public void startMultiplayerGame(boolean publicServer, String username, int port, int players, int advantages) {
         freeColClient.setMapEditor(false);
+
+        this.advantages = advantages;
         
         if (freeColClient.isLoggedIn()) {
             logout(true);
@@ -102,7 +112,8 @@ public final class ConnectController {
         }
 
         try {
-            FreeColServer freeColServer = new FreeColServer(publicServer, false, port, null);
+            FreeColServer freeColServer = new FreeColServer(publicServer, false, port, null, players,
+                                                            advantages, additionalNations);
             freeColClient.setFreeColServer(freeColServer);
         } catch (NoRouteToServerException e) {
             freeColClient.getCanvas().errorMessage("server.noRouteToServer");
@@ -117,11 +128,21 @@ public final class ConnectController {
 
 
     /**
-    * Starts a new singleplayer game by connecting to the server.
-    *
-    * @param username The name to use when logging in.
-    */
-    public void startSingleplayerGame(String username) {
+     * Starts a new singleplayer game by connecting to the server.
+     *
+     * @param username The name to use when logging in.
+     * @param players The number of selectable European players.
+     * @param additionalNations Whether to use additional nations not in the original game.
+     * @param advantages Whether national advantages can be selected.
+     */
+    public void startSingleplayerGame(String username, int players, boolean additionalNations,
+                                      int advantages) {
+
+        if (players > 4) {
+            additionalNations = true;
+        }
+        this.advantages = advantages;
+
         freeColClient.setMapEditor(false);
         
         if (freeColClient.isLoggedIn()) {
@@ -129,7 +150,7 @@ public final class ConnectController {
         }
 
         // TODO-MUCH-LATER: connect client/server directly (not using network-classes)
-        int port = 3541;
+        int port = FreeCol.getDefaultPort();
 
         if (freeColClient.getFreeColServer() != null && freeColClient.getFreeColServer().getServer().getPort() == port) {
             if (freeColClient.getCanvas().showConfirmDialog("stopServer.text", "stopServer.yes", "stopServer.no")) {
@@ -140,7 +161,8 @@ public final class ConnectController {
         }
 
         try {
-            FreeColServer freeColServer = new FreeColServer(false, true, port, null);
+            FreeColServer freeColServer = new FreeColServer(false, true, port, null, players,
+                                                            advantages, additionalNations);
             freeColClient.setFreeColServer(freeColServer);
         } catch (NoRouteToServerException e) {
             logger.warning("Illegal state: An exception occured that can only appear in public multiplayer games.");
@@ -151,11 +173,12 @@ public final class ConnectController {
         }
 
         freeColClient.setSingleplayer(true);
-        boolean loggedIn = login(username, "127.0.0.1", 3541);
+        boolean loggedIn = login(username, "127.0.0.1", port);
 
         if (loggedIn) {
             freeColClient.getPreGameController().setReady(true);
-            freeColClient.getCanvas().showStartGamePanel(freeColClient.getGame(), freeColClient.getMyPlayer(), true);
+            freeColClient.getCanvas().showStartGamePanel(freeColClient.getGame(), freeColClient.getMyPlayer(), true,
+                                                         players, advantages, additionalNations);
         }
     }
 
@@ -177,11 +200,12 @@ public final class ConnectController {
 
         List<String> vacantPlayers = getVacantPlayers(host, port);
         if (vacantPlayers != null) {
-            Object choice = canvas.showChoiceDialog(Messages.message("connectController.choicePlayer"),
-                                                    Messages.message("cancel"),
-                                                    vacantPlayers.iterator());
+            ChoiceItem choice = (ChoiceItem) canvas
+                .showChoiceDialog(Messages.message("connectController.choicePlayer"),
+                                  Messages.message("cancel"),
+                                  vacantPlayers.iterator());
             if (choice != null) {
-                username = (String) choice;
+                username = (String) choice.getObject();
             } else {
                 return;
             }
@@ -189,7 +213,8 @@ public final class ConnectController {
 
         freeColClient.setSingleplayer(false);
         if (login(username, host, port) && !freeColClient.getGUI().isInGame()) {
-            canvas.showStartGamePanel(freeColClient.getGame(), freeColClient.getMyPlayer(), false);
+            canvas.showStartGamePanel(freeColClient.getGame(), freeColClient.getMyPlayer(), false,
+                                      numberOfPlayers, advantages, additionalNations);
         }
     }
 
@@ -202,7 +227,7 @@ public final class ConnectController {
     * @param port The port to use when connecting to the host.
     * @return <i>true</i> if the login was completed successfully.
     */
-    private boolean login(String username, String host, int port) {
+    public boolean login(String username, String host, int port) {
         Client client = freeColClient.getClient();
         Canvas canvas = freeColClient.getCanvas();
         
@@ -237,10 +262,16 @@ public final class ConnectController {
                 final String startGameStr = in.getAttributeValue(null, "startGame");
                 boolean startGame = (startGameStr != null) && Boolean.valueOf(startGameStr).booleanValue();
                 boolean singleplayer = Boolean.valueOf(in.getAttributeValue(null, "singleplayer")).booleanValue();
-                boolean isCurrentPlayer = Boolean.valueOf(in.getAttributeValue(null, "isCurrentPlayer")).booleanValue();                
-                
+                boolean isCurrentPlayer = Boolean.valueOf(in.getAttributeValue(null, "isCurrentPlayer")).booleanValue();
+                additionalNations = Boolean.valueOf(in.getAttributeValue(null, "additionalNations")).booleanValue();
+                advantages = Integer.parseInt(in.getAttributeValue(null, "advantages"));
                 in.nextTag();
                 Game game = new Game(freeColClient.getModelController(), in, username);
+                
+                // this completes the client's view of the spec with options obtained from the server difficulty
+                // it should not be required in the client, to be removed later, when newTurn() only runs in the server
+                Specification.getSpecification().applyDifficultyLevel(game.getGameOptions().getInteger(GameOptions.DIFFICULTY));
+                
                 Player thisPlayer = game.getPlayerByName(username);
 
                 freeColClient.setGame(game);
@@ -297,7 +328,7 @@ public final class ConnectController {
     * Reconnects to the server.
     */
     public void reconnect() {
-        final String username = freeColClient.getMyPlayer().getUsername();
+        final String username = freeColClient.getMyPlayer().getName();
         final String host = freeColClient.getClient().getHost();
         final int port = freeColClient.getClient().getPort();
         
@@ -315,6 +346,7 @@ public final class ConnectController {
     public void loadGame() {
         File file = freeColClient.getCanvas().showLoadDialog(FreeCol.getSaveDirectory());
         if (file != null) {
+            //FreeCol.setSaveDirectory(file.getParentFile());
             loadGame(file);
         }
     }
@@ -338,16 +370,18 @@ public final class ConnectController {
                 canvas.closeMenus();
                 canvas.errorMessage( message );
             }
-        };
+        }
 
         final boolean publicServer;
         final boolean singleplayer;
         final String name;
         final int port;
+        FreeColSavegameFile fis = null;
         try {
             // Get suggestions for "singleplayer" and "public game" settings from the file:
-            XMLStreamReader in = FreeColServer.createXMLStreamReader(theFile);
-            in.nextTag();                    
+            fis = new FreeColSavegameFile(theFile);
+            XMLStreamReader in = FreeColServer.createXMLStreamReader(fis);
+            in.nextTag();
             final boolean defaultSingleplayer = Boolean.valueOf(in.getAttributeValue(null, "singleplayer")).booleanValue();
             final boolean defaultPublicServer;
             final String publicServerStr =  in.getAttributeValue(null, "publicServer");
@@ -374,7 +408,7 @@ public final class ConnectController {
                 publicServer = defaultPublicServer;
                 singleplayer = defaultSingleplayer;
                 name = null;
-                port = 3541;
+                port = FreeCol.getDefaultPort();
             }
         } catch (FileNotFoundException e) {
             SwingUtilities.invokeLater( new ErrorJob("fileNotFound") );
@@ -382,12 +416,19 @@ public final class ConnectController {
         } catch (IOException e) {
             SwingUtilities.invokeLater( new ErrorJob("server.couldNotStart") );
             return;
+        } catch (NullPointerException e) {
+            SwingUtilities.invokeLater( new ErrorJob("couldNotLoadGame") );
+            return;
         } catch (XMLStreamException e) {
             StringWriter sw = new StringWriter();
             e.printStackTrace(new PrintWriter(sw));
             logger.warning(sw.toString());                    
             SwingUtilities.invokeLater( new ErrorJob("server.couldNotStart") );
             return;
+        } finally {
+            if (fis != null) {
+                fis.close();
+            }
         }
         
         if (freeColClient.getFreeColServer() != null && freeColClient.getFreeColServer().getServer().getPort() == port) {
@@ -410,53 +451,37 @@ public final class ConnectController {
                     freeColClient.setSingleplayer(singleplayer);
                     SwingUtilities.invokeLater( new Runnable() {
                         public void run() {
-                            login(username, "127.0.0.1", 3541);               
+                            login(username, "127.0.0.1", FreeCol.getDefaultPort());
                             canvas.closeStatusPanel();
                         }
                     } );                    
                 } catch (NoRouteToServerException e) {
-                    final FreeColServer theFreeColServer = freeColServer;
                     SwingUtilities.invokeLater(new Runnable() {
                         public void run() {
-                            if (theFreeColServer != null) {
-                                theFreeColServer.getServer().shutdown();
-                            }
                             freeColClient.getCanvas().closeMainPanel();
                             freeColClient.getCanvas().showMainPanel();
                         }
                     });
                     SwingUtilities.invokeLater( new ErrorJob("server.noRouteToServer") );
-                } catch (FileNotFoundException e) {                    
-                    final FreeColServer theFreeColServer = freeColServer;
+                } catch (FileNotFoundException e) {
                     SwingUtilities.invokeLater(new Runnable() {
                         public void run() {
-                            if (theFreeColServer != null) {
-                                theFreeColServer.getServer().shutdown();
-                            }
                             freeColClient.getCanvas().closeMainPanel();
                             freeColClient.getCanvas().showMainPanel();
                         }
                     });
                     SwingUtilities.invokeLater( new ErrorJob("fileNotFound") );
                 } catch (IOException e) {
-                    final FreeColServer theFreeColServer = freeColServer;
                     SwingUtilities.invokeLater(new Runnable() {
                         public void run() {
-                            if (theFreeColServer != null) {
-                                theFreeColServer.getServer().shutdown();
-                            }
                             freeColClient.getCanvas().closeMainPanel();
                             freeColClient.getCanvas().showMainPanel();
                         }
                     });
                     SwingUtilities.invokeLater( new ErrorJob("server.couldNotStart") );
                 } catch (FreeColException e) {
-                    final FreeColServer theFreeColServer = freeColServer;
                     SwingUtilities.invokeLater(new Runnable() {
                         public void run() {
-                            if (theFreeColServer != null) {
-                                theFreeColServer.getServer().shutdown();
-                            }
                             freeColClient.getCanvas().closeMainPanel();
                             freeColClient.getCanvas().showMainPanel();
                         }
@@ -489,7 +514,9 @@ public final class ConnectController {
             logger.warning("Could not close connection!");
         }
 
-        freeColClient.getGUI().setInGame(false);
+        if (!freeColClient.isHeadless()) {
+            freeColClient.getGUI().setInGame(false);
+        }
         freeColClient.setGame(null);
         freeColClient.setMyPlayer(null);
         freeColClient.setClient(null);
@@ -548,12 +575,12 @@ public final class ConnectController {
     *
     * @param host The name of the machine running the <code>FreeColServer</code>.
     * @param port The port to use when connecting to the host.
-    * @return A list of available {@link Player#getUsername() usernames}.
+    * @return A list of available {@link Player#getName() usernames}.
     */
     private List<String> getVacantPlayers(String host, int port) {
         Connection mc;
         try {
-            mc = new Connection(host, port, null);
+            mc = new Connection(host, port, null, FreeCol.CLIENT_THREAD);
         } catch (IOException e) {
             logger.warning("Could not connect to server.");
             return null;
@@ -599,7 +626,7 @@ public final class ConnectController {
 
         Connection mc;
         try {
-            mc = new Connection(FreeCol.META_SERVER_ADDRESS, FreeCol.META_SERVER_PORT, null);
+            mc = new Connection(FreeCol.META_SERVER_ADDRESS, FreeCol.META_SERVER_PORT, null, FreeCol.CLIENT_THREAD);
         } catch (IOException e) {
             logger.warning("Could not connect to meta-server.");
             canvas.errorMessage("metaServer.couldNotConnect");
