@@ -723,7 +723,13 @@ public final class InGameInputHandler extends InputHandler {
         Element unitElement = Message.getChildElement(element, Unit.getXMLElementTagName());
 
         Unit unit = (Unit) getGame().getFreeColGameObject(unitElement.getAttribute("ID"));
-        unit.readFromXMLElement(unitElement);
+        // unit may have come directly from an adjacent Indian settlement, 
+        //so the id may not yet be known to the client
+        if (unit == null) {
+            unit = new Unit(getGame(), unitElement);
+        } else {
+            unit.readFromXMLElement(unitElement);
+        }
 
         Settlement settlement = (Settlement) getGame().getFreeColGameObject(element.getAttribute("settlement"));
         Goods goods = new Goods(getGame(), Message.getChildElement(element, Goods.getXMLElementTagName()));
