@@ -20,6 +20,7 @@
 
 package net.sf.freecol.client.control;
 
+import java.awt.Color;
 import java.util.logging.Logger;
 
 import javax.swing.SwingUtilities;
@@ -27,18 +28,19 @@ import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
+import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.gui.Canvas;
 import net.sf.freecol.common.model.Game;
 import net.sf.freecol.common.model.GameOptions;
-import net.sf.freecol.common.model.Market;
+import net.sf.freecol.common.model.Nation;
+import net.sf.freecol.common.model.NationType;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.networking.Connection;
 import net.sf.freecol.common.networking.StreamedMessageHandler;
 import net.sf.freecol.server.generator.MapGeneratorOptions;
 
 import org.w3c.dom.Element;
-import org.w3c.dom.NodeList;
 
 
 /**
@@ -81,8 +83,6 @@ public final class PreGameInputHandler extends InputHandler implements StreamedM
                 reply = removePlayer(element);
             } else if (type.equals("updateGameOptions")) {
                 reply = updateGameOptions(element);
-            } else if (type.equals("updateMarket")) {
-                reply = updateMarket(element);
             } else if (type.equals("updateMapGeneratorOptions")) {
                 reply = updateMapGeneratorOptions(element);
             } else if (type.equals("chat")) {
@@ -91,6 +91,8 @@ public final class PreGameInputHandler extends InputHandler implements StreamedM
                 reply = playerReady(element);
             } else if (type.equals("updateNation")) {
                 reply = updateNation(element);
+            } else if (type.equals("updateNationType")) {
+                reply = updateNationType(element);
             } else if (type.equals("updateColor")) {
                 reply = updateColor(element);
             } else if (type.equals("startGame")) {
@@ -173,26 +175,6 @@ public final class PreGameInputHandler extends InputHandler implements StreamedM
         return null;
     }
 
-    /**
-     * Handles an "updateMarket"-message.
-     * 
-     * @param updateElement The element (root element in a DOM-parsed XML tree)
-     *            that holds all the information.
-     * @return The reply.
-     */
-    public Element updateMarket(Element updateElement) {
-        Game game = getFreeColClient().getGame();
-        NodeList nodeList = updateElement.getChildNodes();
-        Element element = (Element) nodeList.item(0);
-        Market market = (Market) game.getFreeColGameObjectSafely(element.getAttribute("ID"));
-        if (market != null) {
-            market.readFromXMLElement(element);
-        } else {
-            logger.warning("Could not find 'Market' with ID: " + element.getAttribute("ID"));
-        }
-        return null;
-    }
-
     /**
      * Handles an "updateGameOptions"-message.
      *
@@ -271,7 +253,7 @@ public final class PreGameInputHandler extends InputHandler implements StreamedM
         Game game = getFreeColClient().getGame();
 
         Player player = (Player) game.getFreeColGameObject(element.getAttribute("player"));
-        int nation = Integer.parseInt(element.getAttribute("value"));
+        Nation nation = FreeCol.getSpecification().getNation(element.getAttribute("value"));
 
         player.setNation(nation);
         getFreeColClient().getCanvas().getStartGamePanel().refreshPlayersTable();
@@ -280,6 +262,25 @@ public final class PreGameInputHandler extends InputHandler implements StreamedM
     }
 
 
+    /**
+    * Handles an "updateNationType"-message.
+    *
+    * @param element The element (root element in a DOM-parsed XML tree) that
+    *                holds all the information.
+    */
+    private Element updateNationType(Element element) {
+        Game game = getFreeColClient().getGame();
+
+        Player player = (Player) game.getFreeColGameObject(element.getAttribute("player"));
+        NationType nationType = FreeCol.getSpecification().getNationType(element.getAttribute("value"));
+
+        player.setNationType(nationType);
+        getFreeColClient().getCanvas().getStartGamePanel().refreshPlayersTable();
+
+        return null;
+    }
+
+
     /**
     * Handles an "updateColor"-message.
     *
@@ -292,7 +293,7 @@ public final class PreGameInputHandler extends InputHandler implements StreamedM
         Player player = (Player) game.getFreeColGameObject(element.getAttribute("player"));
         String color = element.getAttribute("value");
 
-        player.setColor(color);
+        player.setColor(new Color(Integer.decode(color)));
 
         getFreeColClient().getCanvas().getStartGamePanel().refreshPlayersTable();
 
@@ -328,7 +329,7 @@ public final class PreGameInputHandler extends InputHandler implements StreamedM
          * when map is still null. Wait in other thread in order not to block and
          * it can receive the map.
          */
-        new Thread() {
+        new Thread(FreeCol.CLIENT_THREAD+"Starting game") {
             public void run() {
                 while (getFreeColClient().getGame().getMap() == null) {
                     try {
