@@ -24,8 +24,10 @@ import java.awt.event.MouseListener;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
+import net.sf.freecol.common.model.Map;
 import net.sf.freecol.common.model.PathNode;
 import net.sf.freecol.common.model.Tile;
+import net.sf.freecol.common.model.TileImprovement;
 import net.sf.freecol.common.model.Unit;
 import net.sf.freecol.common.model.Map.Position;
 
@@ -33,11 +35,9 @@ import net.sf.freecol.common.model.Map.Position;
  * Listens to mouse buttons being pressed at the level of the Canvas.
  */
 public final class CanvasMapEditorMouseListener implements MouseListener {
+
     private static final Logger logger = Logger.getLogger(CanvasMapEditorMouseListener.class.getName());
 
-
-
-
     private final Canvas canvas;
 
     private final GUI gui;
@@ -53,15 +53,30 @@ public final class CanvasMapEditorMouseListener implements MouseListener {
         gui = g;
     }
 
+    /**
+     * This method can be called to make sure the map is loaded
+     * There is no point executing mouse events if the map is not loaded
+     */
+    private Map getMap() {
+        Map map = null;
+        if (canvas.getClient().getGame() != null)
+            map = canvas.getClient().getGame().getMap();
+        return map;
+    }
+    
     /**
      * Invoked when a mouse button was clicked.
      * 
      * @param e The MouseEvent that holds all the information.
      */
     public void mouseClicked(MouseEvent e) {
+        if (getMap() == null) {
+            return;
+        }
         try {
             if (e.getClickCount() > 1) {
-                gui.showColonyPanel(gui.convertToMapCoordinates(e.getX(), e.getY()));
+                Position p = gui.convertToMapCoordinates(e.getX(), e.getY());
+                gui.showColonyPanel(p);
             } else {
                 canvas.requestFocus();
             }
@@ -87,6 +102,7 @@ public final class CanvasMapEditorMouseListener implements MouseListener {
     public void mouseExited(MouseEvent e) {
         // Ignore for now.
     }
+    
 
     /**
      * Invoked when a mouse button was pressed.
@@ -97,17 +113,36 @@ public final class CanvasMapEditorMouseListener implements MouseListener {
         if (!e.getComponent().isEnabled()) {
             return;
         }
+        if (getMap() == null) {
+            return;
+        }
         try {
             if (e.getButton() == MouseEvent.BUTTON3 || e.isPopupTrigger()) {
-                //canvas.showTilePopup(gui.convertToMapCoordinates(e.getX(), e.getY()), e.getX(), e.getY());
                 Position p = gui.convertToMapCoordinates(e.getX(), e.getY());
-                gui.setSelectedTile(p, true);
+                Tile tile = getMap().getTile(p);
+                if (tile != null && tile.hasRiver()) {
+                    TileImprovement river = tile.getRiver();
+                    int style = canvas.showRiverStylePanel();
+                    if (style == -1) {
+                        // user canceled
+                    } else if (style == 0) {
+                        tile.getTileItemContainer().removeTileItem(river);
+                    } else if (0 < style && style < ImageLibrary.RIVER_STYLES) {
+                        river.setStyle(style);
+                    } else {
+                        logger.warning("Unknown river style: " + style);
+                    }
+                } else {
+                    gui.setSelectedTile(p, true);
+                }
             } else if (e.getButton() == MouseEvent.BUTTON1) {
-                if (canvas.getClient().getGame() != null
-                        && canvas.getClient().getGame().getMap() != null
-                        && gui.getFocus() != null) {
+                if (gui.getFocus() != null) {
                     Position p = gui.convertToMapCoordinates(e.getX(), e.getY());
-                    Tile t = canvas.getClient().getGame().getMap().getTile(p);
+                    Tile t = getMap().getTile(p);
+                    if (t == null) {
+                        // can happen on the edges of the map
+                        return;
+                    }
                     canvas.getClient().getMapEditorController().transform(t);
                     canvas.refresh();
                     canvas.requestFocus();
@@ -124,13 +159,16 @@ public final class CanvasMapEditorMouseListener implements MouseListener {
      * @param e The MouseEvent that holds all the information.
      */
     public void mouseReleased(MouseEvent e) {
+        if (getMap() == null) {
+            return;
+        }
         try {
-            if (gui.getDragPath() != null) {
+            if (gui.getGotoPath() != null) {
                 // A mouse drag has ended (see CanvasMouseMotionListener).
 
-                PathNode temp = gui.getDragPath();
+                PathNode temp = gui.getGotoPath();
 
-                gui.stopDrag();
+                gui.stopGoto();
 
                 // Move the unit:
                 Unit unit = gui.getActiveUnit();
@@ -138,8 +176,8 @@ public final class CanvasMapEditorMouseListener implements MouseListener {
                 if (canvas.getClient().getGame().getCurrentPlayer() == canvas.getClient().getMyPlayer()) {
                     canvas.getClient().getInGameController().moveToDestination(unit);
                 }
-            } else if (gui.isDragStarted()) {
-                gui.stopDrag();
+            } else if (gui.isGotoStarted()) {
+                gui.stopGoto();
             }
         } catch (Exception ex) {
             logger.log(Level.WARNING, "Error in mouseReleased!", ex);
