@@ -27,8 +27,10 @@ import java.util.logging.Logger;
 
 import javax.swing.SwingUtilities;
 
+import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.ClientOptions;
 import net.sf.freecol.common.model.Map;
+import net.sf.freecol.common.model.Map.Direction;
 import net.sf.freecol.common.model.PathNode;
 import net.sf.freecol.common.model.Tile;
 
@@ -36,11 +38,9 @@ import net.sf.freecol.common.model.Tile;
  * Listens to the mouse being moved at the level of the Canvas.
  */
 public final class CanvasMouseMotionListener implements MouseMotionListener {
+
     private static final Logger logger = Logger.getLogger(CanvasMouseMotionListener.class.getName());
 
-
-
-
     // Temporary variable for checking if we need to recalculate the path when
     // dragging units.
     private Tile lastTile;
@@ -87,6 +87,31 @@ public final class CanvasMouseMotionListener implements MouseMotionListener {
             scrollThread.stopScrolling();
             scrollThread = null;
         }
+         
+        if (gui.isGotoStarted()) {
+            if (gui.getActiveUnit() == null) {
+                gui.stopGoto();
+            }
+            
+            Map.Position p = gui.convertToMapCoordinates(e.getX(), e.getY());
+
+            if (p == null || !map.isValid(p)) {
+                return;
+            }
+        
+            Tile tile = map.getTile(p);
+            if (tile != null) {
+                if (lastTile != tile) {
+                    lastTile = tile;
+                    if (gui.getActiveUnit().getTile() != tile) {
+                        PathNode dragPath = gui.getActiveUnit().findPath(tile);
+                        gui.setGotoPath(dragPath);
+                    } else {
+                        gui.setGotoPath(null);
+                    }
+                } 
+            }
+        }
     }
 
     private void scroll(int x, int y) {
@@ -99,31 +124,31 @@ public final class CanvasMouseMotionListener implements MouseMotionListener {
          * if (y < canvas.getMenuBarHeight() + SCROLLSPACE) { y -=
          * canvas.getMenuBarHeight(); }
          */
-        int direction;
+        Direction direction;
         if ((x < SCROLLSPACE) && (y < SCROLLSPACE)) {
             // Upper-Left
-            direction = Map.NW;
+            direction = Direction.NW;
         } else if ((x >= gui.getWidth() - SCROLLSPACE) && (y < SCROLLSPACE)) {
             // Upper-Right
-            direction = Map.NE;
+            direction = Direction.NE;
         } else if ((x >= gui.getWidth() - SCROLLSPACE) && (y >= gui.getHeight() - SCROLLSPACE)) {
             // Bottom-Right
-            direction = Map.SE;
+            direction = Direction.SE;
         } else if ((x < SCROLLSPACE) && (y >= gui.getHeight() - SCROLLSPACE)) {
             // Bottom-Left
-            direction = Map.SW;
+            direction = Direction.SW;
         } else if (y < SCROLLSPACE) {
             // Top
-            direction = Map.N;
+            direction = Direction.N;
         } else if (x >= gui.getWidth() - SCROLLSPACE) {
             // Right
-            direction = Map.E;
+            direction = Direction.E;
         } else if (y >= gui.getHeight() - SCROLLSPACE) {
             // Bottom
-            direction = Map.S;
+            direction = Direction.S;
         } else if (x < SCROLLSPACE) {
             // Left
-            direction = Map.W;
+            direction = Direction.W;
         } else {
             // Center
             if (scrollThread != null) {
@@ -134,9 +159,12 @@ public final class CanvasMouseMotionListener implements MouseMotionListener {
         }
 
         if (scrollThread != null) {
+            // continue scrolling in a (perhaps new) direction
             scrollThread.setDirection(direction);
         } else {
+            // start scrolling in a direction
             scrollThread = new ScrollThread(map, gui);
+            scrollThread.setDirection(direction);
             scrollThread.start();
         }
     }
@@ -161,27 +189,29 @@ public final class CanvasMouseMotionListener implements MouseMotionListener {
         }
 
         Tile tile = map.getTile(p);
-        if (tile != null) {
-            if (gui.getActiveUnit() == null) {
-                gui.stopDrag();
-            } else if (gui.getActiveUnit().getTile() != tile) {
-                if (gui.isDragStarted()) {
+        if (tile != null && 
+            (e.getModifiers() & MouseEvent.BUTTON1_MASK) == MouseEvent.BUTTON1_MASK) {
+            // only perform the goto for the left mouse button
+            if (gui.isGotoStarted()) {
+                if (gui.getActiveUnit() == null) {
+                    gui.stopGoto();
+                } else { 
                     if (lastTile != tile) {
                         lastTile = tile;
-                        PathNode dragPath = gui.getActiveUnit().findPath(tile);
-                        // ONLY FOR DEBUGGING: PathNode dragPath =
-                        // map.findPath(gui.getActiveUnit(),
-                        // gui.getActiveUnit().getTile(), tile, (Unit)
-                        // gui.getActiveUnit().getLocation());
-                        gui.setDragPath(dragPath);
+                        if (gui.getActiveUnit().getTile() != tile) {
+                            PathNode dragPath = gui.getActiveUnit().findPath(tile);
+                            // ONLY FOR DEBUGGING: PathNode dragPath =
+                            // map.findPath(gui.getActiveUnit(),
+                            // gui.getActiveUnit().getTile(), tile, (Unit)
+                            // gui.getActiveUnit().getLocation());
+                            gui.setGotoPath(dragPath);
+                        } else {
+                            gui.setGotoPath(null);
+                        }
                     }
                 }
             } else {
-                if (!gui.isDragStarted()) {
-                    gui.startDrag();
-                } else {
-                    gui.setDragPath(null);
-                }
+                gui.startGoto();
             }
         }
     }
@@ -191,11 +221,12 @@ public final class CanvasMouseMotionListener implements MouseMotionListener {
      * Scrolls the view of the Map by moving its focus.
      */
     private class ScrollThread extends Thread {
+
         private final Map map;
 
         private final GUI gui;
 
-        private int direction;
+        private Direction direction;
 
         private boolean cont;
 
@@ -207,6 +238,7 @@ public final class CanvasMouseMotionListener implements MouseMotionListener {
          * @param g The GUI that holds information such as screen resolution.
          */
         public ScrollThread(Map m, GUI g) {
+            super(FreeCol.CLIENT_THREAD+"Mouse scroller");
             map = m;
             gui = g;
             cont = true;
@@ -217,7 +249,7 @@ public final class CanvasMouseMotionListener implements MouseMotionListener {
          * 
          * @param d The direction in which this ScrollThread will scroll.
          */
-        public void setDirection(int d) {
+        public void setDirection(Direction d) {
             direction = d;
         }
 
@@ -243,7 +275,7 @@ public final class CanvasMouseMotionListener implements MouseMotionListener {
                         public void run() {
                             try {
                                 int x, y;
-                                Tile t = map.getTileOrNull(gui.getFocus().getX(), gui.getFocus().getY());
+                                Tile t = map.getTile(gui.getFocus().getX(), gui.getFocus().getY());
                                 if (t == null) {
                                     return;
                                 }
