@@ -20,10 +20,13 @@
 package net.sf.freecol.client.gui;
 
 import java.awt.Graphics;
+import java.awt.Graphics2D;
+import java.awt.RenderingHints;
 import java.awt.event.KeyEvent;
 import java.awt.geom.Rectangle2D;
 import java.util.logging.Logger;
 
+import javax.swing.ButtonGroup;
 import javax.swing.JMenu;
 import javax.swing.JMenuItem;
 
@@ -32,7 +35,6 @@ import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.gui.action.AboutAction;
 import net.sf.freecol.client.gui.action.AssignTradeRouteAction;
 import net.sf.freecol.client.gui.action.BuildColonyAction;
-import net.sf.freecol.client.gui.action.BuildRoadAction;
 import net.sf.freecol.client.gui.action.ChangeAction;
 import net.sf.freecol.client.gui.action.ChangeWindowedModeAction;
 import net.sf.freecol.client.gui.action.ChatAction;
@@ -40,23 +42,31 @@ import net.sf.freecol.client.gui.action.ClearOrdersAction;
 import net.sf.freecol.client.gui.action.ColopediaBuildingAction;
 import net.sf.freecol.client.gui.action.ColopediaFatherAction;
 import net.sf.freecol.client.gui.action.ColopediaGoodsAction;
+import net.sf.freecol.client.gui.action.ColopediaNationAction;
+import net.sf.freecol.client.gui.action.ColopediaNationTypeAction;
+import net.sf.freecol.client.gui.action.ColopediaResourceAction;
 import net.sf.freecol.client.gui.action.ColopediaSkillAction;
 import net.sf.freecol.client.gui.action.ColopediaTerrainAction;
 import net.sf.freecol.client.gui.action.ColopediaUnitAction;
 import net.sf.freecol.client.gui.action.DeclareIndependenceAction;
 import net.sf.freecol.client.gui.action.DisbandUnitAction;
 import net.sf.freecol.client.gui.action.DisplayGridAction;
+import net.sf.freecol.client.gui.action.DisplayTileEmptyAction;
 import net.sf.freecol.client.gui.action.DisplayTileNamesAction;
 import net.sf.freecol.client.gui.action.DisplayTileOwnersAction;
+import net.sf.freecol.client.gui.action.DisplayTileRegionsAction;
 import net.sf.freecol.client.gui.action.EndTurnAction;
 import net.sf.freecol.client.gui.action.EuropeAction;
 import net.sf.freecol.client.gui.action.ExecuteGotoOrdersAction;
+import net.sf.freecol.client.gui.action.FindColonyAction;
 import net.sf.freecol.client.gui.action.FortifyAction;
 import net.sf.freecol.client.gui.action.GotoAction;
+import net.sf.freecol.client.gui.action.GotoTileAction;
+import net.sf.freecol.client.gui.action.ImprovementActionType;
+import net.sf.freecol.client.gui.action.LoadAction;
 import net.sf.freecol.client.gui.action.MapControlsAction;
 import net.sf.freecol.client.gui.action.NewAction;
 import net.sf.freecol.client.gui.action.OpenAction;
-import net.sf.freecol.client.gui.action.PlowAction;
 import net.sf.freecol.client.gui.action.PreferencesAction;
 import net.sf.freecol.client.gui.action.QuitAction;
 import net.sf.freecol.client.gui.action.ReconnectAction;
@@ -64,7 +74,10 @@ import net.sf.freecol.client.gui.action.RenameAction;
 import net.sf.freecol.client.gui.action.ReportCargoAction;
 import net.sf.freecol.client.gui.action.ReportColonyAction;
 import net.sf.freecol.client.gui.action.ReportContinentalCongressAction;
+import net.sf.freecol.client.gui.action.ReportExplorationAction;
 import net.sf.freecol.client.gui.action.ReportForeignAction;
+import net.sf.freecol.client.gui.action.ReportHistoryAction;
+import net.sf.freecol.client.gui.action.ReportHighScoresAction;
 import net.sf.freecol.client.gui.action.ReportIndianAction;
 import net.sf.freecol.client.gui.action.ReportLabourAction;
 import net.sf.freecol.client.gui.action.ReportMilitaryAction;
@@ -73,6 +86,7 @@ import net.sf.freecol.client.gui.action.ReportReligionAction;
 import net.sf.freecol.client.gui.action.ReportRequirementsAction;
 import net.sf.freecol.client.gui.action.ReportTradeAction;
 import net.sf.freecol.client.gui.action.ReportTurnAction;
+import net.sf.freecol.client.gui.action.RetireAction;
 import net.sf.freecol.client.gui.action.SaveAction;
 import net.sf.freecol.client.gui.action.SentryAction;
 import net.sf.freecol.client.gui.action.ShowMainAction;
@@ -88,22 +102,20 @@ import net.sf.freecol.client.gui.menu.DebugMenu;
 
 /**
  * This is the menu bar used in-game.
- * 
+ *
  * <br><br>
- * 
+ *
  * The menu bar that is displayed on the top left corner of the
  * <code>Canvas</code>.
- * 
+ *
  * @see Canvas#setJMenuBar
  * @see MapEditorMenuBar
  */
 public class InGameMenuBar extends FreeColMenuBar {
+
     @SuppressWarnings("unused")
     private static final Logger logger = Logger.getLogger(InGameMenuBar.class.getName());
 
-
-
-
     public static final int UNIT_ORDER_WAIT = 0;
 
     public static final int UNIT_ORDER_FORTIFY = 1;
@@ -114,10 +126,6 @@ public class InGameMenuBar extends FreeColMenuBar {
 
     public static final int UNIT_ORDER_BUILD_COL = 5;
 
-    public static final int UNIT_ORDER_PLOW = 6;
-
-    public static final int UNIT_ORDER_BUILD_ROAD = 7;
-
     public static final int UNIT_ORDER_SKIP = 9;
 
     public static final int UNIT_ORDER_DISBAND = 11;
@@ -128,7 +136,7 @@ public class InGameMenuBar extends FreeColMenuBar {
     /**
      * Creates a new <code>FreeColMenuBar</code>. This menu bar will include
      * all of the submenus and items.
-     * 
+     *
      * @param f The main controller.
      */
     public InGameMenuBar(FreeColClient f) {
@@ -149,23 +157,19 @@ public class InGameMenuBar extends FreeColMenuBar {
         // Okay, I lied.. the update() and paintComponent() methods in this
         // MenuBar use freeColClient, too. But so what.
         // Move those to another class too. :)
-        //
-        // I'm gonna use freeColClient.isSingleplayer() to take out the Chat-function.
-        // So I suggest we keep freeColClient here unless someone can figure out a
-        // better way to know that we're in single-player. -helgihg (helgi@binary.is)
-        
+
         super(f);
 
         reset();
     }
 
-    
+
     /**
      * Resets this menu bar.
      */
     public void reset() {
         removeAll();
-        
+
         buildGameMenu();
         buildViewMenu();
         buildOrdersMenu();
@@ -179,7 +183,7 @@ public class InGameMenuBar extends FreeColMenuBar {
 
         update();
     }
-    
+
     private void buildGameMenu() {
         // --> Game
         JMenu menu = new JMenu(Messages.message("menuBar.game"));
@@ -197,15 +201,15 @@ public class InGameMenuBar extends FreeColMenuBar {
 
         menu.addSeparator();
 
-        if (!freeColClient.isSingleplayer()) {
-            menu.add(getMenuItem(ChatAction.id));
-        }
+        menu.add(getMenuItem(ChatAction.id));
         menu.add(getMenuItem(DeclareIndependenceAction.id));
         menu.add(getMenuItem(EndTurnAction.id));
 
         menu.addSeparator();
 
         menu.add(getMenuItem(ShowMainAction.id));
+        menu.add(getMenuItem(ReportHighScoresAction.id));
+        menu.add(getMenuItem(RetireAction.id));
         menu.add(getMenuItem(QuitAction.id));
 
         add(menu);
@@ -219,12 +223,17 @@ public class InGameMenuBar extends FreeColMenuBar {
         menu.setMnemonic(KeyEvent.VK_V);
 
         menu.add(getCheckBoxMenuItem(MapControlsAction.id));
-        menu.add(getCheckBoxMenuItem(DisplayTileNamesAction.id));
-        menu.add(getCheckBoxMenuItem(DisplayTileOwnersAction.id));
         menu.add(getCheckBoxMenuItem(DisplayGridAction.id));
         menu.add(getMenuItem(ToggleViewModeAction.id));
         menu.add(getCheckBoxMenuItem(ChangeWindowedModeAction.id));
 
+        menu.addSeparator();
+        ButtonGroup group = new ButtonGroup();
+        menu.add(getRadioButtonMenuItem(DisplayTileEmptyAction.id, group));
+        menu.add(getRadioButtonMenuItem(DisplayTileNamesAction.id, group));
+        menu.add(getRadioButtonMenuItem(DisplayTileOwnersAction.id, group));
+        menu.add(getRadioButtonMenuItem(DisplayTileRegionsAction.id, group));
+
         menu.addSeparator();
         menu.add(getMenuItem(ZoomInAction.id));
         menu.add(getMenuItem(ZoomOutAction.id));
@@ -233,6 +242,7 @@ public class InGameMenuBar extends FreeColMenuBar {
 
         menu.add(getMenuItem(EuropeAction.id));
         menu.add(getMenuItem(TradeRouteAction.id));
+        menu.add(getMenuItem(FindColonyAction.id));
 
         add(menu);
     }
@@ -243,28 +253,38 @@ public class InGameMenuBar extends FreeColMenuBar {
         menu.setOpaque(false);
         menu.setMnemonic(KeyEvent.VK_O);
 
-        menu.add(getMenuItem(WaitAction.id));
         menu.add(getMenuItem(SentryAction.id));
         menu.add(getMenuItem(FortifyAction.id));
+
+        menu.addSeparator();
+
         menu.add(getMenuItem(GotoAction.id));
+        menu.add(getMenuItem(GotoTileAction.id));
+        menu.add(getMenuItem(ExecuteGotoOrdersAction.id));
         menu.add(getMenuItem(AssignTradeRouteAction.id));
 
         menu.addSeparator();
 
         menu.add(getMenuItem(BuildColonyAction.id));
-        menu.add(getMenuItem(PlowAction.id));
-        menu.add(getMenuItem(BuildRoadAction.id));
+        // Insert all Improvements here:
+        for (ImprovementActionType iaType : FreeCol.getSpecification().getImprovementActionTypeList()) {
+            menu.add(getMenuItem(iaType.getId()));
+        }
+
+        menu.addSeparator();
+
+        menu.add(getMenuItem(LoadAction.id));
         menu.add(getMenuItem(UnloadAction.id));
 
         menu.addSeparator();
 
-        menu.add(getMenuItem(ExecuteGotoOrdersAction.id));
+        menu.add(getMenuItem(WaitAction.id));
         menu.add(getMenuItem(SkipUnitAction.id));
         menu.add(getMenuItem(ChangeAction.id));
-        menu.add(getMenuItem(ClearOrdersAction.id));
-
+ 
         menu.addSeparator();
 
+        menu.add(getMenuItem(ClearOrdersAction.id));
         menu.add(getMenuItem(RenameAction.id));
         menu.add(getMenuItem(DisbandUnitAction.id));
 
@@ -290,6 +310,8 @@ public class InGameMenuBar extends FreeColMenuBar {
         menu.add(getMenuItem(ReportTradeAction.id));
         menu.add(getMenuItem(ReportTurnAction.id));
         menu.add(getMenuItem(ReportRequirementsAction.id));
+        menu.add(getMenuItem(ReportExplorationAction.id));
+        menu.add(getMenuItem(ReportHistoryAction.id));
 
         add(menu);
 
@@ -307,31 +329,40 @@ public class InGameMenuBar extends FreeColMenuBar {
         menu.setMnemonic(KeyEvent.VK_C);
 
         menu.add(getMenuItem(ColopediaTerrainAction.id));
+        menu.add(getMenuItem(ColopediaResourceAction.id));
         menu.add(getMenuItem(ColopediaUnitAction.id));
         menu.add(getMenuItem(ColopediaGoodsAction.id));
         menu.add(getMenuItem(ColopediaSkillAction.id));
         menu.add(getMenuItem(ColopediaBuildingAction.id));
-        menu.add(getMenuItem(ColopediaFatherAction.id));        
-        menu.addSeparator();        
+        menu.add(getMenuItem(ColopediaFatherAction.id));
+        menu.add(getMenuItem(ColopediaNationAction.id));
+        menu.add(getMenuItem(ColopediaNationTypeAction.id));
+        menu.addSeparator();
         menu.add(getMenuItem(AboutAction.id));
-        
+
         add(menu);
     }
-    
+
     /**
      * Paints information about gold, tax and year.
      */
     public void paintComponent(Graphics g) {
         super.paintComponent(g);
-        
+
+        Graphics2D g2 = (Graphics2D) g;
+        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
+                            RenderingHints.VALUE_ANTIALIAS_ON);
+        g2.setRenderingHint(RenderingHints.KEY_RENDERING,
+                            RenderingHints.VALUE_RENDER_QUALITY);
+
         String displayString = Messages.message("menuBar.statusLine",
-                "%gold%", Integer.toString(freeColClient.getMyPlayer().getGold()),
-                "%tax%", Integer.toString(freeColClient.getMyPlayer().getTax()),
-                "%score%", Integer.toString(freeColClient.getMyPlayer().getScore()),
-                "%year%", freeColClient.getGame().getTurn().toString());
-        Rectangle2D displayStringBounds = g.getFontMetrics().getStringBounds(displayString, g);
+                                                "%gold%", Integer.toString(freeColClient.getMyPlayer().getGold()),
+                                                "%tax%", Integer.toString(freeColClient.getMyPlayer().getTax()),
+                                                "%score%", Integer.toString(freeColClient.getMyPlayer().getScore()),
+                                                "%year%", freeColClient.getGame().getTurn().toString());
+        Rectangle2D displayStringBounds = g2.getFontMetrics().getStringBounds(displayString, g);
         int y = 15 + getInsets().top;
-        g.drawString(displayString, getWidth() - 10 - (int) displayStringBounds.getWidth(), y);
+        g2.drawString(displayString, getWidth() - 10 - (int) displayStringBounds.getWidth(), y);
     }
 
 }
