@@ -70,6 +70,6 @@ public class ColopediaBuildingAction extends MapboardAction {
      * @param e The <code>ActionEvent</code>.
      */
     public void actionPerformed(ActionEvent e) {
-        freeColClient.getCanvas().showColopediaPanel(ColopediaPanel.COLOPEDIA_BUILDING);
+        freeColClient.getCanvas().showColopediaPanel(ColopediaPanel.PanelType.BUILDINGS);
     }
 }
