@@ -70,6 +70,6 @@ public class ColopediaFatherAction extends MapboardAction {
      * @param e The <code>ActionEvent</code>.
      */
     public void actionPerformed(ActionEvent e) {
-        freeColClient.getCanvas().showColopediaPanel(ColopediaPanel.COLOPEDIA_FATHER);
+        freeColClient.getCanvas().showColopediaPanel(ColopediaPanel.PanelType.FATHERS);
     }
 }
