@@ -22,7 +22,6 @@ package net.sf.freecol.client.gui.action;
 import java.awt.FlowLayout;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
-import java.util.Vector;
 import java.util.logging.Logger;
 
 import javax.swing.BoxLayout;
@@ -37,8 +36,6 @@ import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.client.gui.panel.FreeColDialog;
 import net.sf.freecol.common.model.Game;
 import net.sf.freecol.common.model.Map;
-import net.sf.freecol.common.model.Tile;
-import net.sf.freecol.common.model.Map.Position;
 import net.sf.freecol.server.generator.TerrainGenerator;
 
 /**
@@ -199,62 +196,4 @@ public class DetermineHighSeasAction extends FreeColAction {
         }
     }
     
-    /**
-     * Scales the current map into the specified size. The current
-     * map is given by {@link FreeColClient#getGame()#getMap}.   
-     * 
-     * @param width The width of the resulting map.
-     * @param height The height of the resulting map.
-     */
-    private void scaleMapTo(final int width, final int height) {
-        /*
-         * This implementation uses a simple linear scaling, and
-         * the isometric shape is not taken into account.
-         * 
-         * TODO: Find a better method for choosing a group of
-         *       adjacent tiles. This group can then be merged into
-         *       a common tile by using the average value (for
-         *       example: are there a majority of ocean tiles?).
-         */
-        
-        final Game game = freeColClient.getGame();
-        final Map oldMap = game.getMap();
-
-        final int oldWidth = oldMap.getWidth();
-        final int oldHeight = oldMap.getHeight();
-        
-        Vector<Vector<Tile>> columns = new Vector<Vector<Tile>>(width);
-        for (int i = 0; i < width; i++) {
-            Vector<Tile> v = new Vector<Tile>(height);
-            for (int j = 0; j < height; j++) {
-                final int oldX = (i * oldWidth) / width;
-                final int oldY = (j * oldHeight) / height;
-                /*
-                 * TODO: This tile should be based on the average as
-                 *       mentioned at the top of this method.
-                 */
-                Tile importTile = oldMap.getTile(oldX, oldY);
-                Tile t = new Tile(game, importTile.getType(), i, j);
-                t.setForested(importTile.isForested());
-                t.setAddition(importTile.getAddition());
-                t.setBonus(importTile.hasBonus());
-                v.add(t);
-            }
-            columns.add(v);
-        }
-
-        Map map = new Map(game, columns);
-        game.setMap(map);
-        
-        // Update river directions
-        for (Tile t : map.getAllTiles()) {
-            int addition = t.getAddition();
-            t.setAddition(Tile.ADD_NONE);
-            t.addRiver(addition);
-            t.setAddition(addition);
-        }
-        
-        freeColClient.getGUI().setSelectedTile(new Position(0, 0));
-        freeColClient.getCanvas().refresh();
-    }
 }
