@@ -296,6 +296,16 @@ public abstract class FreeColAction extends AbstractAction implements Option {
         } else
             return keyStroke.toString();
     }
+    
+    /**
+     * Should this option be updated directly so that
+     * changes may be previewes?
+     * 
+     * @return <code>false</code>.
+     */
+    public boolean isPreviewEnabled() {
+        return false;
+    }
 
     /**
      * This method writes an XML-representation of this object to the given
@@ -307,12 +317,13 @@ public abstract class FreeColAction extends AbstractAction implements Option {
      */
     protected void toXMLImpl(XMLStreamWriter out) throws XMLStreamException {
         // Start element:
-        out.writeStartElement(getId());
+        out.writeStartElement(getXMLElementTagName());
 
+        out.writeAttribute("id", getId());
         out.writeAttribute("accelerator", getKeyStrokeText(getAccelerator()));
 
         out.writeEndElement();
-    }
+   }
 
     /**
      * Initialize this object from an XML-representation of this object.
@@ -321,7 +332,14 @@ public abstract class FreeColAction extends AbstractAction implements Option {
      * @throws XMLStreamException if a problem was encountered during parsing.
      */
     protected void readFromXMLImpl(XMLStreamReader in) throws XMLStreamException {
+        String id = in.getAttributeValue(null, "id");
         String acc = in.getAttributeValue(null, "accelerator");
+
+        if (id == null){
+            // Old syntax
+            id = in.getLocalName();
+        }
+
         if (!acc.equals("")) {
             putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(acc));
         } else {
@@ -448,8 +466,8 @@ public abstract class FreeColAction extends AbstractAction implements Option {
         public void menuKeyPressed(MenuKeyEvent e) {
 
             if (e.getKeyCode() == mnemonic) {
-                ActionEvent ae = new ActionEvent(e.getSource(), e.getID(), (String) getValue(Action.NAME), e
-                        .getModifiers());
+                ActionEvent ae = new ActionEvent(e.getSource(), e.getID(), (String) getValue(Action.NAME),
+                                                 e.getModifiers());
                 actionPerformed(ae);
 
                 e.consume();
@@ -465,4 +483,13 @@ public abstract class FreeColAction extends AbstractAction implements Option {
         }
 
     }
+
+    /**
+     * Gets the tag name of the root element representing this object.
+     * @return "integerOption".
+     */
+     public static String getXMLElementTagName() {
+         return "action";
+     }
+
 }
