@@ -20,8 +20,11 @@
 package net.sf.freecol.client.gui.action;
 
 import java.awt.event.ActionEvent;
+import java.awt.event.KeyEvent;
 import java.util.logging.Logger;
 
+import javax.swing.KeyStroke;
+
 import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.gui.panel.ReportCargoPanel;
 
@@ -41,7 +44,7 @@ public class ReportCargoAction extends MapboardAction {
      * @param freeColClient The main controller object for the client.
      */
     ReportCargoAction(FreeColClient freeColClient) {
-        super(freeColClient, "menuBar.report.cargo", null, null);
+        super(freeColClient, "menuBar.report.cargo", null, KeyStroke.getKeyStroke(KeyEvent.VK_F10, 0));
     }
     
     /**
