@@ -22,7 +22,6 @@ package net.sf.freecol.client.gui.action;
 import java.awt.FlowLayout;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
-import java.util.Vector;
 import java.util.logging.Logger;
 
 import javax.swing.BoxLayout;
@@ -204,7 +203,7 @@ public class ScaleMapAction extends FreeColAction {
     
     /**
      * Scales the current map into the specified size. The current
-     * map is given by {@link FreeColClient#getGame()#getMap}.   
+     * map is given by freeColClient.getGame().getMap().
      * 
      * @param width The width of the resulting map.
      * @param height The height of the resulting map.
@@ -226,38 +225,30 @@ public class ScaleMapAction extends FreeColAction {
         final int oldWidth = oldMap.getWidth();
         final int oldHeight = oldMap.getHeight();
         
-        Vector<Vector<Tile>> columns = new Vector<Vector<Tile>>(width);
-        for (int i = 0; i < width; i++) {
-            Vector<Tile> v = new Vector<Tile>(height);
-            for (int j = 0; j < height; j++) {
-                final int oldX = (i * oldWidth) / width;
-                final int oldY = (j * oldHeight) / height;
+        Tile[][] tiles = new Tile[width][height];
+        for (int x = 0; x < width; x++) {
+            for (int y = 0; y < height; y++) {
+                final int oldX = (x * oldWidth) / width;
+                final int oldY = (y * oldHeight) / height;
                 /*
                  * TODO: This tile should be based on the average as
                  *       mentioned at the top of this method.
                  */
-                Tile oldTile = oldMap.getTile(oldX, oldY);
-                
-                // Copy values to the new tile:
-                Tile t = new Tile(game, oldTile.getType(), i, j);
-                t.setForested(oldTile.isForested());
-                t.setAddition(oldTile.getAddition());
-                t.setBonus(oldTile.hasBonus());
-                v.add(t);
+                Tile importTile = oldMap.getTile(oldX, oldY);
+                Tile t = new Tile(game, importTile.getType(), x, y);
+                t.getTileItemContainer().copyFrom(importTile.getTileItemContainer());
+                tiles[x][y] = t;
             }
-            columns.add(v);
         }
 
-        Map map = new Map(game, columns);
+        Map map = new Map(game, tiles);
         game.setMap(map);
         
+        /* Commented because it doesn't appear to do anything valuable
         // Update river directions
         for (Tile t : map.getAllTiles()) {
-            int addition = t.getAddition();
-            t.setAddition(Tile.ADD_NONE);
-            t.addRiver(addition);
-            t.setAddition(addition);
-        }
+            t.getTileItemContainer().updateRiver();
+        }*/
         
         freeColClient.getGUI().setSelectedTile(new Position(0, 0));
         freeColClient.getCanvas().refresh();
