@@ -27,14 +27,12 @@ import javax.swing.KeyStroke;
 
 import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.gui.Canvas;
-import net.sf.freecol.client.gui.GUI;
 import net.sf.freecol.client.gui.ImageLibrary;
 
 /**
  * An action for chosing the next unit as the active unit.
  */
 public class ZoomInAction extends FreeColAction {
-    @SuppressWarnings("unused")
     private static final Logger logger = Logger.getLogger(ZoomInAction.class.getName());
 
 
@@ -61,7 +59,7 @@ public class ZoomInAction extends FreeColAction {
     protected boolean shouldBeEnabled() {
         if (!super.shouldBeEnabled()) {
             return false;
-        }
+        } 
         
         Canvas canvas = getFreeColClient().getCanvas();
         
@@ -69,8 +67,8 @@ public class ZoomInAction extends FreeColAction {
         	return false;
         
         float oldScaling = getFreeColClient().getGUI().getImageLibrary().getScalingFactor();
+
         return oldScaling < 1.0;
-  
     }
 
     /**
@@ -89,13 +87,18 @@ public class ZoomInAction extends FreeColAction {
      */
     public void actionPerformed(ActionEvent e) {
         float oldScaling = getFreeColClient().getGUI().getImageLibrary().getScalingFactor();
-        float newScaling = oldScaling + 1/8f;
-        final ImageLibrary im;
+        float newScaling = oldScaling + 1/4f;
+        ImageLibrary im;
         if (newScaling >= 1f) {
             newScaling = 1f;
             im = getFreeColClient().getImageLibrary();
         } else {
-            im = getFreeColClient().getImageLibrary().getScaledImageLibrary(newScaling);
+            try {
+                im = getFreeColClient().getImageLibrary().getScaledImageLibrary(newScaling);
+            } catch(Exception ex) {
+                logger.warning("Failed to retrieve scaled image library.");
+                im = getFreeColClient().getImageLibrary();
+            }
         }
         getFreeColClient().getGUI().setImageLibrary(im);
         getFreeColClient().getGUI().forceReposition();
