@@ -33,7 +33,6 @@ import net.sf.freecol.client.gui.ImageLibrary;
  * An action for chosing the next unit as the active unit.
  */
 public class ZoomOutAction extends FreeColAction {
-    @SuppressWarnings("unused")
     private static final Logger logger = Logger.getLogger(ZoomOutAction.class.getName());
 
 
@@ -70,7 +69,6 @@ public class ZoomOutAction extends FreeColAction {
         float oldScaling = getFreeColClient().getGUI().getImageLibrary().getScalingFactor();
    
         return ((oldScaling - 1/8f) * 8 > 1);
-        
     }
 
     /**
@@ -89,7 +87,13 @@ public class ZoomOutAction extends FreeColAction {
      */
     public void actionPerformed(ActionEvent e) {
         float oldScaling = getFreeColClient().getGUI().getImageLibrary().getScalingFactor();
-        ImageLibrary im = getFreeColClient().getImageLibrary().getScaledImageLibrary(oldScaling - 1/8f);
+        ImageLibrary im;
+        try {
+            im = getFreeColClient().getImageLibrary().getScaledImageLibrary(oldScaling - 1/4f);
+        } catch (Exception ex) {
+            logger.warning("Failed to retrieve scaled image library.");
+            im = getFreeColClient().getImageLibrary();
+        }
         getFreeColClient().getGUI().setImageLibrary(im);
         getFreeColClient().getGUI().forceReposition();
         getFreeColClient().getCanvas().refresh();
