@@ -38,9 +38,6 @@ import net.sf.freecol.FreeCol;
  */
 public class Messages {
 
-
-
-
     private static final Logger logger = Logger.getLogger(Messages.class.getName());
 
     public static final String STRINGS_DIRECTORY = "strings";
@@ -88,8 +85,8 @@ public class Messages {
             variant = "_" + variant;
         }
         String[] fileNames = { FILE_PREFIX + FILE_SUFFIX, FILE_PREFIX + language + FILE_SUFFIX,
-                FILE_PREFIX + language + country + FILE_SUFFIX,
-                FILE_PREFIX + language + country + variant + FILE_SUFFIX };
+                               FILE_PREFIX + language + country + FILE_SUFFIX,
+                               FILE_PREFIX + language + country + variant + FILE_SUFFIX };
 
         for (String fileName : fileNames) {
             File resourceFile = new File(getI18nDirectory(), fileName);
@@ -97,45 +94,15 @@ public class Messages {
         }
     }
 
+    /**
+     * Returns the directory containing language property files.
+     *
+     * @return a <code>File</code> value
+     */
     public static File getI18nDirectory() {
         return new File(FreeCol.getDataDirectory(), STRINGS_DIRECTORY);
     }
 
-    /**
-     * Finds the message with a particular ID in the default locale and performs
-     * string replacements.
-     * 
-     * @param messageId The key of the message to find
-     * @param data Every occurrences of <code>data[x][0]</code> is replaced with
-     *            <code>data[x][1]</code> for every <code>x</code>.
-     * @return The message with the specified id or null if message could not be
-     *         found.
-     * 
-     * @throws NullPointerException if messageId is null
-     * @deprecated
-     */
-    public static String message(String messageId, String[][] data) {
-
-        String message = Messages.message(messageId);
-        if (message == null) {
-            return messageId;
-        }
-
-        if (data != null && message != null) {
-            for (int i = 0; i < data.length; i++) {
-                if (data[i] == null || data[i].length != 2) {
-                    logger.warning("Data has a wrong format for message: " + message);
-                } else if (data[i][0] == null || data[i][1] == null) {
-                    logger.warning("Data in model message is 'null': " + message + ", " + data[i][0] + ", "
-                            + data[i][1]);
-                } else {
-                    message = message.replaceAll(data[i][0], data[i][1]);
-                }
-            }
-        }
-
-        return message;
-    }
 
     /**
      * Finds the message with a particular ID in the default locale and performs
@@ -150,7 +117,7 @@ public class Messages {
         if (messageId == null) {
             throw new NullPointerException();
         }
-        if (data.length % 2 != 0) {
+        if (data!=null && data.length % 2 != 0) {
             throw new IllegalArgumentException("Programming error, the data should consist of only pairs.");
         }
         if (messageBundle == null) {
@@ -162,17 +129,52 @@ public class Messages {
             return messageId;
         }
 
-        if (data.length > 0 && message != null) {
+        if (data!=null && data.length > 0) {
             for (int i = 0; i < data.length; i += 2) {
                 if (data[i] == null || data[i+1] == null) {
-                    throw new IllegalArgumentException("Programming error,no data should be <null>.");
+                    throw new IllegalArgumentException("Programming error, no data should be <null>.");
                 }
-                message = message.replaceAll(data[i], data[i+1]);
+                // if there is a $ in the substituting string, a IllegalArgumentException will be raised
+                //we need to escape it first
+                String escapedStr = data[i+1].replaceAll("\\$","\\\\\\$");
+                message = message.replaceAll(data[i], escapedStr);
             }
         }
-        return message;
+        return message.trim();
     }
 
+    /**
+     * Returns true if the message bundle contains the given key.
+     *
+     * @param key a <code>String</code> value
+     * @return a <code>boolean</code> value
+     */
+    public static boolean containsKey(String key) {
+        if (messageBundle == null) {
+            setMessageBundle(Locale.getDefault());
+        }
+        return (messageBundle.getProperty(key) != null);
+    }
+
+
+    /**
+     * Returns the preferred key if it is contained in the message
+     * bundle and the default key otherwise. This should be used to
+     * select the most specific message key available.
+     *
+     * @param preferredKey a <code>String</code> value
+     * @param defaultKey a <code>String</code> value
+     * @return a <code>String</code> value
+     */
+    public static String getKey(String preferredKey, String defaultKey) {
+        if (containsKey(preferredKey)) {
+            return preferredKey;
+        } else {
+            return defaultKey;
+        }
+    }
+
+
     /**
      * Calling this method can be used to replace the messages used currently
      * with a new bundle. This is used only in the debugging of FreeCol.
