@@ -25,7 +25,9 @@ import java.awt.event.ActionListener;
 import java.awt.event.InputEvent;
 import java.awt.event.KeyEvent;
 import java.io.File;
+import java.util.ArrayList;
 import java.util.Iterator;
+import java.util.List;
 
 import javax.swing.ButtonGroup;
 import javax.swing.JCheckBoxMenuItem;
@@ -40,7 +42,10 @@ import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.gui.Canvas;
 import net.sf.freecol.client.gui.GUI;
 import net.sf.freecol.client.gui.i18n.Messages;
+import net.sf.freecol.client.gui.panel.ChoiceItem;
+import net.sf.freecol.client.gui.panel.StatisticsPanel;
 import net.sf.freecol.common.model.Colony;
+import net.sf.freecol.common.model.FoundingFather;
 import net.sf.freecol.common.model.Map;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Tile;
@@ -181,7 +186,7 @@ public class DebugMenu extends JMenu {
         while (it.hasNext()) {
             final Player p = it.next();
             if (p.isEuropean() && p.canBuildColonies()) {
-                JRadioButtonMenuItem cv2 = new JRadioButtonMenuItem(Player.getNationAsString(p.getNation()),
+                JRadioButtonMenuItem cv2 = new JRadioButtonMenuItem(p.getNationAsString(),
                         gui.displayColonyValue && gui.displayColonyValuePlayer == p);
                 cv2.setOpaque(false);
                 cv2.setMnemonic(KeyEvent.VK_C);
@@ -221,11 +226,8 @@ public class DebugMenu extends JMenu {
             this.add(giveBells);
             giveBells.addActionListener(new ActionListener() {
                 public void actionPerformed(ActionEvent e) {
-                    Iterator<Colony> ci = freeColClient.getMyPlayer().getColonyIterator();
-                    while (ci.hasNext()) {
-                        Colony c = ci.next();
+                    for (Colony c : freeColClient.getMyPlayer().getColonies()) {
                         c.addBells(100);
-
                         Colony sc = (Colony) freeColClient.getFreeColServer().getGame().getFreeColGameObject(c.getId());
                         sc.addBells(100);
                     }
@@ -233,6 +235,31 @@ public class DebugMenu extends JMenu {
             });
         }
 
+        final JMenuItem addFather = new JMenuItem("Add Founding Father");
+        addFather.setOpaque(false);
+        addFather.setMnemonic(KeyEvent.VK_F);
+        this.add(addFather);
+        addFather.addActionListener(new ActionListener() {
+                public void actionPerformed(ActionEvent e) {
+                    Player player = freeColClient.getMyPlayer();
+                    List<ChoiceItem> fathers = new ArrayList<ChoiceItem>();
+                    for (FoundingFather father : FreeCol.getSpecification().getFoundingFathers()) {
+                        if (!player.hasFather(father)) {
+                            fathers.add(new ChoiceItem(father.getName(), father));
+                        }
+                    }
+                    ChoiceItem[] choices = fathers.toArray(new ChoiceItem[fathers.size()]);
+                    ChoiceItem response = (ChoiceItem) freeColClient.getCanvas()
+                        .showChoiceDialog("Select Founding Father", "cancel", choices);
+                    FoundingFather fatherToAdd = (FoundingFather) response.getObject();
+                    player.addFather(fatherToAdd);
+                    Player serverPlayer = (Player) freeColClient.getFreeColServer().getGame().
+                        getFreeColGameObject(player.getId());
+                    serverPlayer.addFather(fatherToAdd);
+                }
+            });
+
+
         this.addSeparator();
 
         final JMenuItem europeStatus = new JMenuItem("Display Europe Status");
@@ -348,8 +375,21 @@ public class DebugMenu extends JMenu {
                 }
             }
         });
+        
+        
+        // statistics
+        final JMenuItem statistics = new JMenuItem("Statistics");
+        statistics.setOpaque(false);
+        statistics.setMnemonic(KeyEvent.VK_I);
+        this.add(statistics);
+        statistics.addActionListener(new ActionListener() {
+            public void actionPerformed(ActionEvent e) {
+                canvas.showPanel(new StatisticsPanel(canvas, freeColClient));
+            }
+        });
 
-        final JMenuItem gc = new JMenuItem("Run the garbage collector");
+        // garbage collector
+        final JMenuItem gc = new JMenuItem(Messages.message("menuBar.debug.memoryManager.gc"));
         gc.setOpaque(false);
         gc.setMnemonic(KeyEvent.VK_G);
         this.add(gc);
