@@ -41,12 +41,14 @@ import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.gui.action.FreeColAction;
 import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.client.gui.panel.FreeColPanel;
+import net.sf.freecol.common.option.AudioMixerOption;
 import net.sf.freecol.common.option.BooleanOption;
 import net.sf.freecol.common.option.FileOption;
 import net.sf.freecol.common.option.IntegerOption;
 import net.sf.freecol.common.option.LanguageOption;
 import net.sf.freecol.common.option.Option;
 import net.sf.freecol.common.option.OptionGroup;
+import net.sf.freecol.common.option.PercentageOption;
 import net.sf.freecol.common.option.RangeOption;
 import net.sf.freecol.common.option.SelectOption;
 
@@ -74,7 +76,7 @@ public final class OptionGroupUI extends JPanel implements OptionUpdater {
      */
     public OptionGroupUI(OptionGroup option, boolean editable, int level, HashMap<String, JComponent> optionUIs) {
         
-        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
+        setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
         
         JPanel horizontalPanel = null;
         boolean buttonAdded = false;
@@ -119,6 +121,14 @@ public final class OptionGroupUI extends JPanel implements OptionUpdater {
                 if (!o.getId().equals(Option.NO_ID)) {
                     optionUIs.put(o.getId(), boi);
                 }
+            } else if (o instanceof PercentageOption) {
+                final PercentageOptionUI soi = new PercentageOptionUI((PercentageOption) o, editable);
+                add(soi);
+                ou.add(soi);
+                buttonAdded = false;
+                if (!o.getId().equals(Option.NO_ID)) {
+                    optionUIs.put(o.getId(), soi);
+                }
             } else if (o instanceof IntegerOption) {
                 final IntegerOptionUI iou = new IntegerOptionUI((IntegerOption) o, editable);
                 add(iou);
@@ -135,14 +145,6 @@ public final class OptionGroupUI extends JPanel implements OptionUpdater {
                 if (!o.getId().equals(Option.NO_ID)) {
                     optionUIs.put(o.getId(), iou);
                 }
-            } else if (o instanceof SelectOption) {
-                final SelectOptionUI soi = new SelectOptionUI((SelectOption) o, editable);
-                add(soi);
-                ou.add(soi);
-                buttonAdded = false;
-                if (!o.getId().equals(Option.NO_ID)) {
-                    optionUIs.put(o.getId(), soi);
-                }
             } else if (o instanceof RangeOption) {
                 final RangeOptionUI soi = new RangeOptionUI((RangeOption) o, editable);
                 add(soi);
@@ -151,6 +153,14 @@ public final class OptionGroupUI extends JPanel implements OptionUpdater {
                 if (!o.getId().equals(Option.NO_ID)) {
                     optionUIs.put(o.getId(), soi);
                 }
+            } else if (o instanceof SelectOption) {
+                final SelectOptionUI soi = new SelectOptionUI((SelectOption) o, editable);
+                add(soi);
+                ou.add(soi);
+                buttonAdded = false;
+                if (!o.getId().equals(Option.NO_ID)) {
+                    optionUIs.put(o.getId(), soi);
+                }
             } else if (o instanceof LanguageOption) {
                 final LanguageOptionUI soi = new LanguageOptionUI((LanguageOption) o, editable);
                 add(soi);
@@ -159,6 +169,14 @@ public final class OptionGroupUI extends JPanel implements OptionUpdater {
                 if (!o.getId().equals(Option.NO_ID)) {
                     optionUIs.put(o.getId(), soi);
                 }
+            } else if (o instanceof AudioMixerOption) {
+                final AudioMixerOptionUI soi = new AudioMixerOptionUI((AudioMixerOption) o, editable);
+                add(soi);
+                ou.add(soi);
+                buttonAdded = false;
+                if (!o.getId().equals(Option.NO_ID)) {
+                    optionUIs.put(o.getId(), soi);
+                }
             } else if (o instanceof FreeColAction) {
                 final FreeColActionUI fau = new FreeColActionUI((FreeColAction) o, this);
                 ou.add(fau);
@@ -189,6 +207,19 @@ public final class OptionGroupUI extends JPanel implements OptionUpdater {
         setOpaque(false);
     }
 
+    /**
+     * Rollback to the original value.
+     * 
+     * This method gets called so that changes made to options with
+     * {@link Option#isPreviewEnabled()} is rolled back
+     * when an option dialoag has been cancelled.
+     */
+    public void rollback() {
+        for (int i = 0; i < optionUpdaters.length; i++) {
+            optionUpdaters[i].rollback();
+        }
+    }
+    
     /**
      * Unregister <code>PropertyChangeListener</code>s.
      */
@@ -263,6 +294,17 @@ public final class OptionGroupUI extends JPanel implements OptionUpdater {
             });
         }
         
+        /**
+         * Rollback to the original value.
+         * 
+         * This method gets called so that changes made to options with
+         * {@link Option#isPreviewEnabled()} is rolled back
+         * when an option dialoag has been cancelled.
+         */
+        public void rollback() {
+            groupUI.rollback();
+        }
+        
         /**
          * Delegates the call to <code>groupUI</code>.
          */
