@@ -32,6 +32,7 @@ import javax.swing.JPanel;
 import javax.swing.JScrollPane;
 import javax.swing.JTabbedPane;
 
+import net.sf.freecol.common.option.AudioMixerOption;
 import net.sf.freecol.common.option.BooleanOption;
 import net.sf.freecol.common.option.FileOption;
 import net.sf.freecol.common.option.IntegerOption;
@@ -39,6 +40,7 @@ import net.sf.freecol.common.option.LanguageOption;
 import net.sf.freecol.common.option.Option;
 import net.sf.freecol.common.option.OptionGroup;
 import net.sf.freecol.common.option.OptionMap;
+import net.sf.freecol.common.option.PercentageOption;
 import net.sf.freecol.common.option.RangeOption;
 import net.sf.freecol.common.option.SelectOption;
 
@@ -47,11 +49,9 @@ import net.sf.freecol.common.option.SelectOption;
  * enable values to be both seen and changed.
  */
 public final class OptionMapUI extends JPanel implements OptionUpdater {
+
     private static final Logger logger = Logger.getLogger(OptionMapUI.class.getName());
 
-
-
-
     private final OptionUpdater[] optionUpdaters;
     
     private final HashMap<String, JComponent> optionUIs;
@@ -100,6 +100,7 @@ public final class OptionMapUI extends JPanel implements OptionUpdater {
                 ou.add(c);
                 JScrollPane scroll = new JScrollPane(c, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                         JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
+                scroll.getVerticalScrollBar().setUnitIncrement(16);
                 scroll.setBorder(BorderFactory.createEmptyBorder());
                 c.setBorder(BorderFactory.createEmptyBorder(OptionGroupUI.H_GAP - 5, OptionGroupUI.H_GAP, 0,
                         OptionGroupUI.H_GAP));
@@ -119,15 +120,15 @@ public final class OptionMapUI extends JPanel implements OptionUpdater {
                 if (!o.getId().equals(Option.NO_ID)) {
                     optionUIs.put(o.getId(), iou);
                 }
-            } else if (o instanceof IntegerOption) {
-                JComponent c = new IntegerOptionUI((IntegerOption) o, editable);
+            } else if (o instanceof PercentageOption) {
+                JComponent c = new PercentageOptionUI((PercentageOption) o, editable);
                 northPanel.add(c);
                 ou.add(c);
                 if (!o.getId().equals(Option.NO_ID)) {
                     optionUIs.put(o.getId(), c);
                 }
-            } else if (o instanceof SelectOption) {
-                JComponent c = new SelectOptionUI((SelectOption) o, editable);
+            } else if (o instanceof IntegerOption) {
+                JComponent c = new IntegerOptionUI((IntegerOption) o, editable);
                 northPanel.add(c);
                 ou.add(c);
                 if (!o.getId().equals(Option.NO_ID)) {
@@ -140,6 +141,13 @@ public final class OptionMapUI extends JPanel implements OptionUpdater {
                 if (!o.getId().equals(Option.NO_ID)) {
                     optionUIs.put(o.getId(), c);
                 }
+            } else if (o instanceof SelectOption) {
+                JComponent c = new SelectOptionUI((SelectOption) o, editable);
+                northPanel.add(c);
+                ou.add(c);
+                if (!o.getId().equals(Option.NO_ID)) {
+                    optionUIs.put(o.getId(), c);
+                }
             } else if (o instanceof LanguageOption) {
                 System.out.println("found languageOption");
                 JComponent c = new LanguageOptionUI((LanguageOption) o, editable);
@@ -148,6 +156,13 @@ public final class OptionMapUI extends JPanel implements OptionUpdater {
                 if (!o.getId().equals(Option.NO_ID)) {
                     optionUIs.put(o.getId(), c);
                 }
+            } else if (o instanceof AudioMixerOption) {
+                JComponent c = new AudioMixerOptionUI((AudioMixerOption) o, editable);
+                northPanel.add(c);
+                ou.add(c);
+                if (!o.getId().equals(Option.NO_ID)) {
+                    optionUIs.put(o.getId(), c);
+                }
             } else {
                 logger.warning("Unknown option.");
             }
@@ -162,6 +177,19 @@ public final class OptionMapUI extends JPanel implements OptionUpdater {
         setOpaque(false);
     }
 
+    /**
+     * Rollback to the original value.
+     * 
+     * This method gets called so that changes made to options with
+     * {@link Option#isPreviewEnabled()} is rolled back
+     * when an option dialoag has been cancelled.
+     */
+    public void rollback() {
+        for (int i = 0; i < optionUpdaters.length; i++) {
+            optionUpdaters[i].rollback();
+        }
+    }
+    
     /**
      * Unregister <code>PropertyChangeListener</code>s.
      */
