@@ -42,9 +42,9 @@ import net.sf.freecol.common.model.Goods;
 import net.sf.freecol.common.model.Unit;
 
 /**
- * Panel for chosing the goods to capture.
+ * Panel for choosing the goods to capture.
  *
- * @see Unit#attack(Unit, int, int)
+ * @see SimpleCombatModel#attack(Unit, Unit, CombatModel#CombatResult, int)
  */
 public final class CaptureGoodsDialog extends FreeColDialog implements ActionListener {
     @SuppressWarnings("unused")
@@ -176,7 +176,7 @@ public final class CaptureGoodsDialog extends FreeColDialog implements ActionLis
      * @param capturingUnit
      */
     public void initialize(Unit capturedUnit, Unit capturingUnit) {
-        maxCargo = capturingUnit.getInitialSpaceLeft();
+        maxCargo = capturingUnit.getType().getSpace();
         GoodsItem[] goods = new GoodsItem[capturedUnit.getGoodsCount()];
         if (goods.length > 0) {
             Iterator<Goods> iter = capturedUnit.getGoodsIterator();
@@ -234,7 +234,7 @@ public final class CaptureGoodsDialog extends FreeColDialog implements ActionLis
         }
 
         public String toString() {
-            return good.getAmount() + " " + Goods.getName(good.getType());
+            return good.getAmount() + " " + good.getType().getName();
         }
     }
 }
