@@ -1051,7 +1051,7 @@ public final class ColonyPanel extends FreeColPanel implements ActionListener,Pr
      */
     private synchronized void setColony(Colony colony) {
     	if(this.colony != null){
-    		this.colony.removePropertyChangeListener(this);
+    		this.colony.removePropertyChangeListener(Colony.ColonyChangeEvent.BONUS_CHANGE.toString(), this);
     	}
         this.colony = colony;
         if(this.colony != null){
