@@ -23,14 +23,15 @@ import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.util.logging.Logger;
 
+import javax.swing.ImageIcon;
 import javax.swing.JButton;
-import javax.swing.JLabel;
+import javax.swing.JTextArea;
 
 import net.sf.freecol.client.gui.Canvas;
 import net.sf.freecol.client.gui.ImageLibrary;
 import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.common.model.Europe;
-import net.sf.freecol.common.model.Unit;
+import net.sf.freecol.common.model.UnitType;
 import cz.autel.dmi.HIGLayout;
 
 /**
@@ -38,16 +39,13 @@ import cz.autel.dmi.HIGLayout;
  */
 public final class EmigrationPanel extends FreeColDialog implements ActionListener {
 
-
-
-
     private static final Logger logger = Logger.getLogger(EmigrationPanel.class.getName());
 
     private static final int NUMBER_OF_PERSONS = 3;
 
     private static final JButton[] person = new JButton[NUMBER_OF_PERSONS];
 
-    private static final JLabel question = new JLabel(Messages.message("chooseImmigrant"));
+    private JTextArea question = getDefaultTextArea(Messages.message("chooseImmigrant"));
 
 
     /**
@@ -73,10 +71,15 @@ public final class EmigrationPanel extends FreeColDialog implements ActionListen
      * 
      * @param europe The Europe Object where we can find the units that are
      *            prepared to emigrate.
+     * @param fountainOfYouth a <code>boolean</code> value
      */
-    public void initialize(Europe europe) {
+    public void initialize(Europe europe, boolean fountainOfYouth) {
 
-        ImageLibrary library = (ImageLibrary) getCanvas().getImageProvider();
+        if (fountainOfYouth) {
+            question.insert(Messages.message("lostCityRumour.FountainOfYouth") + "\n\n", 0);
+        }
+
+        ImageLibrary library = getCanvas().getGUI().getImageLibrary();
 
         int[] widths = { 0 };
         int[] heights = { 0, margin, 0, margin, 0, margin, 0 };
@@ -89,10 +92,10 @@ public final class EmigrationPanel extends FreeColDialog implements ActionListen
         row += 2;
 
         for (int index = 0; index < NUMBER_OF_PERSONS; index++) {
-            int unitType = europe.getRecruitable(index);
-            int graphicsType = ImageLibrary.getUnitGraphicsType(unitType, false, false, 0, false);
-            person[index].setText(Unit.getName(unitType));
-            person[index].setIcon(library.getScaledUnitImageIcon(graphicsType, 0.66f));
+            UnitType unitType = europe.getRecruitable(index);
+            ImageIcon unitIcon = library.getUnitImageIcon(unitType);
+            person[index].setText(unitType.getName());
+            person[index].setIcon(library.getScaledImageIcon(unitIcon, 0.66f));
 
             add(person[index], higConst.rc(row, column));
             row += 2;
