@@ -27,7 +27,6 @@ import javax.swing.ImageIcon;
 import javax.swing.JLabel;
 
 import net.sf.freecol.client.gui.Canvas;
-import net.sf.freecol.common.model.Building;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.GameOptions;
 import net.sf.freecol.common.model.Goods;
@@ -63,7 +62,7 @@ public final class GoodsLabel extends JLabel {// implements ActionListener {
      * @param parent The parent that knows more than we do.
      */
     public GoodsLabel(Goods goods, Canvas parent) {
-        super(parent.getImageProvider().getGoodsImageIcon(goods.getType()));
+        super(parent.getGUI().getImageLibrary().getGoodsImageIcon(goods.getType()));
         this.goods = goods;
         setToolTipText(goods.getName());
         this.parent = parent;
@@ -95,10 +94,10 @@ public final class GoodsLabel extends JLabel {// implements ActionListener {
             player = ((Ownable) location).getOwner();
         }
         if (player == null
-                || goods.getType() >= Goods.NUMBER_OF_TYPES
+                || !goods.getType().isStorable()
                 || player.canTrade(goods)
-                || (location instanceof Colony && player.getGameOptions().getBoolean(GameOptions.CUSTOM_IGNORE_BOYCOTT) && ((Colony) location)
-                .getBuilding(Building.CUSTOM_HOUSE).getLevel() != Building.NOT_BUILT)) {
+                || (location instanceof Colony && player.getGameOptions().getBoolean(GameOptions.CUSTOM_IGNORE_BOYCOTT) &&
+                    ((Colony) location).hasAbility("model.ability.export"))) {
             setToolTipText(goods.getName());
             setEnabled(true);
         } else {
@@ -109,8 +108,8 @@ public final class GoodsLabel extends JLabel {// implements ActionListener {
         if (goods.getType() != Goods.FOOD && location instanceof Colony
                 && ((Colony) location).getWarehouseCapacity() < goods.getAmount()) {
             setForeground(Color.RED);
-        } else if (location instanceof Colony && location != null && goods.getType() < Goods.NUMBER_OF_TYPES
-                && ((Colony) location).getExports(goods)) {
+        } else if (location instanceof Colony && goods.getType().isStorable()
+                   && ((Colony) location).getExportData(goods.getType()).isExported()) {
             setForeground(Color.GREEN);
         } else if (goods.getAmount() == 0) {
             setForeground(Color.GRAY);
@@ -165,11 +164,11 @@ public final class GoodsLabel extends JLabel {// implements ActionListener {
      */
     public void setSmall(boolean isSmall) {
         if (isSmall) {
-            ImageIcon imageIcon = (parent.getImageProvider().getGoodsImageIcon(goods.getType()));
+            ImageIcon imageIcon = parent.getGUI().getImageLibrary().getGoodsImageIcon(goods.getType());
             setIcon(new ImageIcon(imageIcon.getImage().getScaledInstance(imageIcon.getIconWidth() / 2,
                     imageIcon.getIconHeight() / 2, Image.SCALE_DEFAULT)));
         } else {
-            setIcon(parent.getImageProvider().getGoodsImageIcon(goods.getType()));
+            setIcon(parent.getGUI().getImageLibrary().getGoodsImageIcon(goods.getType()));
         }
     }
 }
