@@ -24,7 +24,8 @@ import java.awt.Image;
 
 import javax.swing.ImageIcon;
 
-import net.sf.freecol.common.model.Unit;
+import net.sf.freecol.common.model.GoodsType;
+import net.sf.freecol.common.model.TileType;
 
 /**
  * An object that should be able to provide images upon request.
@@ -44,7 +45,7 @@ public abstract class ImageProvider {
      *            drawn.
      * @return The Image of the terrain with the given type.
      */
-    public abstract Image getTerrainImage(int type, int x, int y);
+    public abstract Image getTerrainImage(TileType type, int x, int y);
 
     /**
      * Should return the Image of the graphic with the given type.
@@ -52,7 +53,7 @@ public abstract class ImageProvider {
      * @param type The type of the graphic of which we need the Image.
      * @return The Image of the graphic with the given type.
      */
-    public abstract Image getGoodsImage(int type);
+    public abstract Image getGoodsImage(GoodsType type);
 
     /**
      * Should return the Image of the graphic with the given type.
@@ -60,7 +61,7 @@ public abstract class ImageProvider {
      * @param type The type of the graphic of which we need the Image.
      * @return The Image of the graphic with the given type.
      */
-    public abstract ImageIcon getGoodsImageIcon(int type);
+    public abstract ImageIcon getGoodsImageIcon(GoodsType type);
 
     /**
      * Should return the Image of the graphic with the given type.
@@ -68,7 +69,7 @@ public abstract class ImageProvider {
      * @param type The type of the graphic of which we need the Image.
      * @return The Image of the graphic with the given type.
      */
-    public abstract Image getMiscImage(int type);
+    public abstract Image getMiscImage(String type);
 
     /**
      * Should return the Image of the color chip with the given color.
@@ -84,8 +85,8 @@ public abstract class ImageProvider {
      * @param index The index of the unit-image to return.
      * @return The unit-image at the given index.
      */
-    public abstract Image getUnitImage(int index);
-
+    //public abstract Image getUnitImage(int index);
+    
     /**
      * Returns the unit-image at the given index.
      * 
@@ -93,7 +94,7 @@ public abstract class ImageProvider {
      * @param grayscale If <code>true</code> return the image in grayscale
      * @return The unit-image at the given index.
      */
-    public abstract Image getUnitImage(int index, boolean grayscale);
+    //public abstract Image getUnitImage(int index, boolean grayscale);
 
     /**
      * Returns the unit-ImageIcon at the given index.
@@ -101,7 +102,7 @@ public abstract class ImageProvider {
      * @param type The type of the Unit of which we need the ImageIcon.
      * @return The unit-ImageIcon of the Unit with the given type.
      */
-    public abstract ImageIcon getUnitImageIcon(int type);
+    //public abstract ImageIcon getUnitImageIcon(int type);
 
     /**
      * Should return the ImageIcon of the Unit with the given type.
@@ -110,7 +111,7 @@ public abstract class ImageProvider {
      * @param grayscale If <code>true</code> return the image in grayscale
      * @return The ImageIcon of the Unit with the given type.
      */
-    public abstract ImageIcon getUnitImageIcon(int type, boolean grayscale);
+    //public abstract ImageIcon getUnitImageIcon(int type, boolean grayscale);
 
     /**
      * Should return the ImageIcon of the Unit Button with the given index.
@@ -124,18 +125,18 @@ public abstract class ImageProvider {
     /**
      * Should return the width of the terrain-image at the given index.
      * 
-     * @param index The index of the terrain-image.
+     * @param type The type of the terrain-image.
      * @return The width of the terrain-image at the given index.
      */
-    public abstract int getTerrainImageWidth(int index);
+    public abstract int getTerrainImageWidth(TileType type);
 
     /**
      * Should return the height of the terrain-image at the given index.
      * 
-     * @param index The index of the terrain-image.
+     * @param type The type of the terrain-image.
      * @return The height of the terrain-image at the given index.
      */
-    public abstract int getTerrainImageHeight(int index);
+    public abstract int getTerrainImageHeight(TileType type);
 
     /**
      * Should return the width of the unit-image at the given index.
@@ -143,7 +144,7 @@ public abstract class ImageProvider {
      * @param index The index of the unit-image.
      * @return The width of the unit-image at the given index.
      */
-    public abstract int getUnitImageWidth(int index);
+    //public abstract int getUnitImageWidth(int index);
 
     /**
      * Should return the height of the unit-image at the given index.
@@ -151,7 +152,7 @@ public abstract class ImageProvider {
      * @param index The index of the unit-image.
      * @return The height of the unit-image at the given index.
      */
-    public abstract int getUnitImageHeight(int index);
+    //public abstract int getUnitImageHeight(int index);
 
     /**
      * Should return the graphic type that can be used to represent the given
@@ -161,5 +162,5 @@ public abstract class ImageProvider {
      * @return The graphic type that can be used to represent the given Unit
      *         visually.
      */
-    public abstract int getUnitGraphicsType(Unit unit);
+    //public abstract int getUnitGraphicsType(Unit unit);
 }
