@@ -27,14 +27,13 @@ import java.awt.Image;
 import java.awt.event.MouseAdapter;
 import java.awt.event.MouseEvent;
 import java.awt.image.BufferedImage;
+import java.util.List;
 import java.util.logging.Logger;
 
 import javax.swing.ImageIcon;
 import javax.swing.JButton;
 import javax.swing.JLabel;
 import javax.swing.JPanel;
-import javax.swing.UIManager;
-
 import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.gui.GUI;
@@ -42,12 +41,15 @@ import net.sf.freecol.client.gui.ImageLibrary;
 import net.sf.freecol.client.gui.ViewMode;
 import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.client.gui.panel.MapEditorTransformPanel.MapTransform;
+import net.sf.freecol.common.model.AbstractGoods;
+import net.sf.freecol.common.model.EquipmentType;
 import net.sf.freecol.common.model.Game;
 import net.sf.freecol.common.model.Goods;
-import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Tile;
-import net.sf.freecol.common.model.TileType;
+import net.sf.freecol.common.model.TileImprovement;
 import net.sf.freecol.common.model.Unit;
+import net.sf.freecol.common.model.Unit.Role;
+import net.sf.freecol.common.resources.ResourceManager;
 import cz.autel.dmi.HIGLayout;
 
 /**
@@ -57,8 +59,6 @@ import cz.autel.dmi.HIGLayout;
  */
 public final class InfoPanel extends FreeColPanel {
     
-    
-    
     @SuppressWarnings("unused")
     private static final Logger logger = Logger.getLogger(InfoPanel.class.getName());
     
@@ -86,22 +86,18 @@ public final class InfoPanel extends FreeColPanel {
      * The constructor that will add the items to this panel.
      *
      * @param freeColClient The main controller object for the client.
-     * @param game The Game object that has all kinds of useful information that
-     *            we want to display here.
-     * @param imageProvider The ImageProvider that can provide us with images to
-     *            display on this panel.
      */
-    public InfoPanel(final FreeColClient freeColClient, Game game, ImageProvider imageProvider) {
+    public InfoPanel(final FreeColClient freeColClient) {
         this.freeColClient = freeColClient;
-        this.game = game;
-        this.library = (ImageLibrary) imageProvider;
+        this.game = freeColClient.getGame();
+        this.library = freeColClient.getImageLibrary();
         
         unitInfoPanel = new UnitInfoPanel();
         setLayout(null);
         
         int internalPanelTop = 0;
         int internalPanelHeight = 128;
-        Image skin = (Image) UIManager.get("InfoPanel.skin");
+        Image skin = ResourceManager.getImage("InfoPanel.skin");
         if (skin == null) {
             setSize(PANEL_WIDTH, PANEL_HEIGHT);
         } else {
@@ -246,7 +242,7 @@ public final class InfoPanel extends FreeColPanel {
             }
         }
         
-        Image skin = (Image) UIManager.get("InfoPanel.skin");
+        Image skin = ResourceManager.getImage("InfoPanel.skin");
         if (skin != null) {
             graphics.drawImage(skin, 0, 0, null);
         }
@@ -322,14 +318,7 @@ public final class InfoPanel extends FreeColPanel {
          * @return the basic movement cost of the tile
          */
         private int getMovementCost(Tile tile) {
-            if (tile.getAddition() == Tile.ADD_MOUNTAINS) {
-                return 9;
-            } else if (tile.getAddition() == Tile.ADD_HILLS) {
-                return 6;
-            }
-
-            TileType t = FreeCol.getSpecification().tileType(tile.getType());
-            return tile.isForested() ? t.whenForested.basicMoveCost : t.basicMoveCost;
+            return tile.getType().getBasicMoveCost();
         }
         
         /**
@@ -339,24 +328,19 @@ public final class InfoPanel extends FreeColPanel {
             int counter = 1;
             int row = 1;
             goodsPanel.removeAll();
-            for (int i = 0; i < Goods.NUMBER_OF_TYPES; i++) {
-                int production = tile.potential(i);
-                if (production > 0) {
-                    int type = i;
-                    if (i == Goods.FOOD && !tile.isLand()) {
-                        type = Goods.FISH;
-                    }
-                    JLabel goodsLabel = new JLabel(String.valueOf(production),
-                            library.getScaledGoodsImageIcon(type, 0.50f), JLabel.RIGHT);
-                    goodsLabel.setToolTipText(Goods.getName(type));
-                    goodsLabel.setFont(goodsLabel.getFont().deriveFont(9f));
-                    goodsPanel.add(goodsLabel, higConst.rc(row, counter));
-                    if (counter == 3) {
-                        row++;
-                        counter = 1;
-                    } else {
-                        counter++;
-                    }
+            List<AbstractGoods> production = tile.getType().getProduction();
+            for (AbstractGoods goods : production) {
+                JLabel goodsLabel = new JLabel(String.valueOf(tile.potential(goods.getType(), null)),
+                                               library.getScaledGoodsImageIcon(goods.getType(), 0.50f),
+                                               JLabel.RIGHT);
+                goodsLabel.setToolTipText(goods.getType().getName());
+                goodsLabel.setFont(goodsLabel.getFont().deriveFont(9f));
+                goodsPanel.add(goodsLabel, higConst.rc(row, counter));
+                if (counter == 3) {
+                    row++;
+                    counter = 1;
+                } else {
+                    counter++;
                 }
             }
             goodsPanel.validate();
@@ -372,64 +356,61 @@ public final class InfoPanel extends FreeColPanel {
             
             labelPanel.removeAll();
             if (tile != null) {
-                tileNameLabel.setText(tile.getName());
-                labelPanel.add(tileNameLabel, higConst.rc(1, 1));
-                String text;
-                int row = 2;
-                
-                text = "";
-                if (tile.getAddition() == Tile.ADD_RIVER_MINOR) {
-                    text = Messages.message("minorRiver");
-                } else if (tile.getAddition() == Tile.ADD_RIVER_MAJOR) {
-                    text = Messages.message("majorRiver");
-                }
-                if (tile.hasRoad()) {
-                    if (!text.equals("")) {
-                        text += ", ";
+                if (tile.isExplored()) {
+                    tileNameLabel.setText(tile.getName());
+                    labelPanel.add(tileNameLabel, higConst.rc(1, 1));
+                    StringBuilder text = new StringBuilder();
+                    int row = 2;
+                    for (TileImprovement tileImprovement : tile.getCompletedTileImprovements()) {
+                        text.append(tileImprovement.getType().getDescription());
+                        text.append(", ");
                     }
-                    text += Messages.message("road");
-                }
-                riverRoadLabel.setText(text);
-                if (!text.equals("")) {
-                    labelPanel.add(riverRoadLabel, higConst.rc(row, 1));
-                    row++;
-                }
-
-                text = "";
-                if (tile.isPlowed()) {
-                    text = Messages.message("plowed");
-                    if (tile.getNationOwner() != Player.NO_NATION) {
-                        text += ", ";
+                    if (tile.getOwner() != null) {
+                        text.append(tile.getOwner().getNationAsString());
+                    } else {
+                        int length = text.length();
+                        if (length > 2) {
+                            text.delete(length - 2, length);
+                        }
                     }
-                }
-                if (tile.getNationOwner() != Player.NO_NATION) {
-                    text += Player.getNationAsString(tile.getNationOwner());
-                }
-                plowOwnerLabel.setText(text);
-                if (!text.equals("")) {
-                    labelPanel.add(plowOwnerLabel, higConst.rc(row, 1));
+
+                    if (text.length() > 0) {
+                        plowOwnerLabel.setText(text.toString());
+                        labelPanel.add(plowOwnerLabel, higConst.rc(row, 1));
+                        row++;
+                    }
+
+                    int defenceBonus = (int) tile.getType().getFeatureContainer()
+                        .applyModifier(100, "model.modifier.defence") - 100;
+                    bonusLabel.setText(Messages.message("colopedia.terrain.defenseBonus") +
+                                       " " + defenceBonus + "%");
+                    labelPanel.add(bonusLabel, higConst.rc(row, 1));
                     row++;
-                }
 
-                bonusLabel.setText(Messages.message("colopedia.terrain.defenseBonus") +
-                        " " + tile.defenseBonus() + "%");
-                labelPanel.add(bonusLabel, higConst.rc(row, 1));
-                row++;
+                    movementLabel.setText(Messages.message("colopedia.terrain.movementCost") +
+                                          " " + String.valueOf(getMovementCost(tile)/3));
+                    labelPanel.add(movementLabel, higConst.rc(row, 1));
+                    row++;
 
-                movementLabel.setText(Messages.message("colopedia.terrain.movementCost") +
-                        " " + String.valueOf(getMovementCost(tile)/3));
-                labelPanel.add(movementLabel, higConst.rc(row, 1));
-                row++;
-
-                labelPanel.add(goodsPanel, higConst.rc(row, 1));
-                addProducedGoods(tile);
+                    labelPanel.add(goodsPanel, higConst.rc(row, 1));
+                    addProducedGoods(tile);
                 
-                int width = library.getTerrainImageWidth(tile.getType());
-                int height = library.getTerrainImageHeight(tile.getType());
-                BufferedImage image = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
-                freeColClient.getGUI().displayTerrain(image.createGraphics(), freeColClient.getGame().getMap(), tile,
-                        0, 0);
-                tileLabel.setIcon(new ImageIcon(image));
+                    int width = library.getTerrainImageWidth(tile.getType());
+                    int height = library.getTerrainImageHeight(tile.getType());
+                    BufferedImage image = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
+                    freeColClient.getGUI().displayTerrain(image.createGraphics(), freeColClient.getGame().getMap(),
+                                                          tile, 0, 0);
+                    tileLabel.setIcon(new ImageIcon(image));
+                } else {
+                    tileNameLabel.setText("Unexplored");
+                    labelPanel.add(tileNameLabel, higConst.rc(1, 1));
+                    int width = library.getTerrainImageWidth(tile.getType());
+                    int height = library.getTerrainImageHeight(tile.getType());
+                    BufferedImage image = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
+                    freeColClient.getGUI().displayTerrain(image.createGraphics(), freeColClient.getGame().getMap(),
+                                                          tile, 0, 0);
+                    tileLabel.setIcon(new ImageIcon(image));
+                }
             } else {
                 tileLabel.setIcon(null);
             }
@@ -466,7 +447,7 @@ public final class InfoPanel extends FreeColPanel {
         public UnitInfoPanel() {
             super(null);
             
-            Image tools = library.getGoodsImageIcon(Goods.TOOLS).getImage();
+            Image tools = library.getGoodsImage(Goods.TOOLS);
             ImageIcon toolsIcon = new ImageIcon(tools.getScaledInstance(tools.getWidth(null) * 2 / 3, tools
                     .getHeight(null) * 2 / 3, Image.SCALE_SMOOTH));
             unitToolsLabel = new JLabel(toolsIcon);
@@ -523,13 +504,16 @@ public final class InfoPanel extends FreeColPanel {
             unitCargoPanel.removeAll();
             if (unit != null) {
                 setUnitNameType(unit);
-                unitLabel.setIcon(library.getUnitImageIcon(library.getUnitGraphicsType(unit)));
+                unitLabel.setIcon(library.getUnitImageIcon(unit));
                 
-                // Handle the special cases.
-                if (unit.isPioneer()) {
-                    unitToolsLabel.setText(String.valueOf(unit.getNumberOfTools()));
+                // Handle the special cases. TODO: make this more generic
+                if (unit.getRole() == Role.PIONEER) {
+                    EquipmentType toolsType = FreeCol.getSpecification()
+                        .getEquipmentType("model.equipment.tools");
+                    int count = unit.getEquipmentCount(toolsType) * 20;
+                    unitToolsLabel.setText(String.valueOf(count));
                     unitCargoPanel.add(unitToolsLabel, higConst.rc(1, 1));
-                } else if (unit.getType() == Unit.TREASURE_TRAIN) {
+                } else if (unit.canCarryTreasure()) {
                     goldLabel.setText(unit.getTreasureAmount() + " " + Messages.message("gold"));
                     unitCargoPanel.add(goldLabel, higConst.rc(1, 1));
                 } else if (unit.isCarrier()) {
@@ -586,8 +570,8 @@ public final class InfoPanel extends FreeColPanel {
                 }
             }
             for (Unit carriedUnit : unit.getUnitList()) {
-                int graphicsType = library.getUnitGraphicsType(carriedUnit);
-                JLabel unitLabel = new JLabel(library.getScaledUnitImageIcon(graphicsType, 0.5f));
+                ImageIcon unitIcon = library.getUnitImageIcon(carriedUnit);
+                JLabel unitLabel = new JLabel(library.getScaledImageIcon(unitIcon, 0.5f));
                 unitLabel.setToolTipText(carriedUnit.getName());
                 unitCargoPanel.add(unitLabel, higConst.rc(row, counter));
                 if (counter == 4) {
