@@ -24,11 +24,9 @@ import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.common.Specification;
 import net.sf.freecol.common.model.Building;
-import net.sf.freecol.common.model.BuildingType;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.ColonyTile;
 import net.sf.freecol.common.model.Europe;
-import net.sf.freecol.common.model.Goods;
 import net.sf.freecol.common.model.GoodsType;
 import net.sf.freecol.common.model.Location;
 import net.sf.freecol.common.model.Player;
@@ -37,11 +35,11 @@ import net.sf.freecol.common.model.Unit;
 import net.sf.freecol.common.model.UnitType;
 import net.sf.freecol.common.model.WorkLocation;
 
+import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.LinkedHashMap;
-import java.util.LinkedHashSet;
+import java.util.List;
 import java.util.Map;
-import java.util.Set;
 
 /**
  * holds labour statistics for the labour report
@@ -94,9 +92,6 @@ public class LabourData {
     }
 
     public static class LocationData {
-        public boolean alwaysShowProfessionals() {
-            return unitData.isSummary() || unitData.getExpertProduction() != null;
-        }
 
         public interface Getter {
             LocationData getLocationData(UnitData unitData);
@@ -211,7 +206,7 @@ public class LabourData {
             boolean isSummary = getUnitData().isSummary();
 
             int rows = 0;
-            if (workingProfessionals.getColonists() > 0 || alwaysShowProfessionals()) rows++;
+            if (workingProfessionals.getColonists() > 0) rows++;
             if (workingAmateurs > 0) rows++;
             if (!isSummary && otherWorkingAmateurs.getColonists() > 0) rows++;
             if (teachers > 0) rows++;
@@ -267,17 +262,6 @@ public class LabourData {
         }
     }
 
-    private static class NonLinkedLocationData extends LocationData {
-        public NonLinkedLocationData(UnitData data) {
-            super(data);
-        }
-
-        @Override
-        public boolean alwaysShowProfessionals() {
-            return false;
-        }
-    }
-
     public static class UnitData {
 
         private UnitType unitType;
@@ -290,9 +274,9 @@ public class LabourData {
         private Map<Colony, LocationData> details = new LinkedHashMap<Colony, LocationData>();
 
         private LocationData total = new LocationData(this, true);
-        private LocationData unitsAtSea = new NonLinkedLocationData(this);
-        private LocationData unitsOnLand = new NonLinkedLocationData(this);
-        private LocationData unitsInEurope = new NonLinkedLocationData(this);
+        private LocationData unitsAtSea = new LocationData(this);
+        private LocationData unitsOnLand = new LocationData(this);
+        private LocationData unitsInEurope = new LocationData(this);
 
         public UnitData(UnitType unitType) {
             this.unitType = unitType;
@@ -366,24 +350,20 @@ public class LabourData {
         }
 
         public boolean showNetProduction() {
-            return showProduction() && GoodsType.getStoredAs(unitType.getExpertProduction().index) >= 0;
+            return showProduction() && unitType.getExpertProduction().isStorable();
         }
 
         public GoodsType getExpertProduction() {
             if (summary) {
                 return null;
             }
-            if (getUnitType().getIndex() == Unit.EXPERT_FISHERMAN) {
-                return FreeCol.getSpecification().goodsType(Goods.FISH);
-            } else {
-                return getUnitType().getExpertProduction();
-            }
+            return getUnitType().getExpertProduction();
         }
     }
 
-    private static final LinkedHashMap<Integer, UnitType> UNIT_TYPE_BY_GOOD_TYPE = new LinkedHashMap<Integer, UnitType>(Goods.NUMBER_OF_ALL_TYPES);
+    private Map<GoodsType, UnitData> experts = new LinkedHashMap<GoodsType, UnitData>();
 
-    private Map<String, UnitData> unitDataMap = new LinkedHashMap<String, UnitData>(Unit.UNIT_COUNT);
+    private Map<String, UnitData> unitDataMap = new LinkedHashMap<String, UnitData>();
 
     private UnitData summary = new UnitData(null);
 
@@ -397,46 +377,42 @@ public class LabourData {
 
     public LabourData(FreeColClient client) {
         Specification spec = FreeCol.getSpecification();
-        if (UNIT_TYPE_BY_GOOD_TYPE.isEmpty()) {
-            for (int unitIndex = 0; unitIndex < Unit.UNIT_COUNT; unitIndex++) {
-                UnitType type = spec.unitType(unitIndex);
-                GoodsType production = type.getExpertProduction();
-                if (production != null) {
-                    UNIT_TYPE_BY_GOOD_TYPE.put(production.index, type);
-                }
+        for (UnitType unitType : spec.getUnitTypeList()) {
+            if (!unitType.isAvailableTo(client.getMyPlayer())) {
+                continue;
+            }
+            GoodsType production = unitType.getExpertProduction();
+            if (production != null) {
+                experts.put(production, getUnitData(unitType));
             }
-            UNIT_TYPE_BY_GOOD_TYPE.put(Goods.FOOD, spec.unitType(Unit.EXPERT_FARMER));
-            UNIT_TYPE_BY_GOOD_TYPE.put(Goods.FISH, spec.unitType(Unit.EXPERT_FISHERMAN));
-        }
 
-        missionary = getUnitData(spec.unitType(Unit.JESUIT_MISSIONARY));
-        pioneer = getUnitData(spec.unitType(Unit.HARDY_PIONEER));
-        soldier = getUnitData(spec.unitType(Unit.VETERAN_SOLDIER));
-        scout = getUnitData(spec.unitType(Unit.SEASONED_SCOUT));
+            if (unitType.hasAbility("model.ability.expertPioneer")) {
+                pioneer = getUnitData(unitType);
+            } else if (unitType.hasAbility("model.ability.expertSoldier")) {
+                soldier = getUnitData(unitType);
+            } else if (unitType.hasAbility("model.ability.expertScout")) {
+                scout = getUnitData(unitType);
+            } else if (unitType.hasAbility("model.ability.expertMissionary")) {
+                missionary = getUnitData(unitType);
+            }
+        }
 
         gatherData(client.getMyPlayer());
     }
 
     private void gatherData(Player player) {
-        Specification spec = FreeCol.getSpecification();
-
-        Set<UnitType> labourTypes = new LinkedHashSet<UnitType>();
-        for (int type : ReportLabourPanel.UNIT_TYPES) {
-            labourTypes.add(spec.unitType(type));
-        }
+        List<UnitType> labourTypes = getLabourTypes(player);
 
         Iterator<Unit> units = player.getUnitIterator();
         while (units.hasNext()) {
             Unit unit = units.next();
-            UnitType type = spec.unitType(unit.getType());
-
-            if (!labourTypes.contains(type)) {
+            if (!labourTypes.contains(unit.getType())) {
                 continue;
             }
 
             Location location = unit.getLocation();
 
-            UnitData data = getUnitData(type);
+            UnitData data = getUnitData(unit.getType());
 
             if (location instanceof WorkLocation) {
                 incrementColonyCount(location.getColony(), unit, data);
@@ -450,22 +426,6 @@ public class LabourData {
                 incrementOutsideWorker(data, unit, UNITS_ON_LAND_GETTER);
             }
         }
-
-        for (UnitType unitType : labourTypes) {
-            UnitData unitData = getUnitData(unitType);
-            for (Colony colony : player.getColonies()) {
-                Building building = getBuilding(colony, unitType);
-                if (building != null) {
-                    BuildingType buildingType = FreeCol.getSpecification().buildingType(building.getType());
-                    if (building.getLevel() != Building.HOUSE || buildingType.level(0).hammersRequired > 0) {
-                        //display this location, because we built this building
-                        unitData.getLocationData(colony);
-                    }
-                }
-            }
-        }
-
-
         summarize();
 
         for (UnitData unitData : unitDataMap.values()) {
@@ -473,11 +433,10 @@ public class LabourData {
 
             GoodsType expertProduction = unitData.getUnitType().getExpertProduction();
             if (expertProduction != null) {
-                int goodsIndex = GoodsType.getStoredAs(expertProduction.index);
                 for (Colony colony : player.getColonies()) {
                     LocationData data = unitData.details.containsKey(colony) ? unitData.getLocationData(colony) : null;
 
-                    int netProduction = colony.getProductionNetOf(goodsIndex);
+                    int netProduction = colony.getProductionNetOf(expertProduction);
                     if (data != null) {
                         data.netProduction = netProduction;
                     }
@@ -487,6 +446,17 @@ public class LabourData {
         }
     }
 
+    public static List<UnitType> getLabourTypes(Player player) {
+        List<UnitType> unitTypes = FreeCol.getSpecification().getUnitTypeList();
+        ArrayList<UnitType> labourTypes = new ArrayList<UnitType>();
+        for (UnitType unitType : unitTypes) {
+            if (unitType.hasSkill() && unitType.isAvailableTo(player)) {
+                labourTypes.add(unitType);
+            }
+        }
+        return labourTypes;
+    }
+
     private void summarize() {
         for (UnitData unitData : unitDataMap.values()) {
             summarize(unitData, UNITS_IN_EUROPE_GETTER);
@@ -513,21 +483,28 @@ public class LabourData {
     }
 
     private void incrementOutsideWorker(UnitData unitData, Unit unit, LocationData.Getter getter) {
-        if (unit.isMissionary()) {
-            incrementOutsideWorker(unitData, unit, missionary, getter);
-        } else if (unit.isPioneer()) {
-            incrementOutsideWorker(unitData, unit, pioneer, getter);
-        } else if (unit.isArmed()) {
-            incrementOutsideWorker(unitData, unit, soldier, getter);
-        } else if (unit.isMounted()) {
-            incrementOutsideWorker(unitData, unit, scout, getter);
-        } else {
-            getter.getLocationData(unitData).notWorking++;
+        switch (unit.getRole()) {
+            case DRAGOON:
+            case SOLDIER:
+                incrementOutsideWorker(unitData, unit, soldier, getter);
+                break;
+            case MISSIONARY:
+                incrementOutsideWorker(unitData, unit, missionary, getter);
+                break;
+            case PIONEER:
+                incrementOutsideWorker(unitData, unit, pioneer, getter);
+                break;
+            case SCOUT:
+                incrementOutsideWorker(unitData, unit, scout, getter);
+                break;
+            default:
+                getter.getLocationData(unitData).notWorking++;
+                break;
         }
     }
 
     private void incrementOutsideWorker(UnitData expert, Unit unit, UnitData workingAs, LocationData.Getter getter) {
-        if (unit.getType() == workingAs.unitType.getIndex()) {
+        if (unit.getType() == workingAs.unitType) {
             getter.getLocationData(expert).workingProfessionals.colonists++;
         } else {
             getter.getLocationData(expert).workingAmateurs++;
@@ -553,38 +530,33 @@ public class LabourData {
         if (teacher != null) {
             colonyData.students++;
 
-            UnitData learning = getUnitData(FreeCol.getSpecification().unitType(
-                Unit.getTrainingResult(unit.getType(), teacher.getType())));
+            UnitData learning = getUnitData(Unit.getUnitTypeTeaching(teacher.getType(), unit.getType()));
             learning.getLocationData(colony).addOtherStudent(unitData.getUnitName());
         }
 
-        int currentlyWorking = unit.getWorkType();
+        GoodsType currentlyWorking = unit.getWorkType();
         int production;
         if (location instanceof Building) {
             Building building = (Building) location;
 
-            if (building.getType() == Building.SCHOOLHOUSE) {
+            if (building.getType().hasAbility("model.ability.teach")) {
                 colonyData.teachers++;
                 return;
             }
 
-            production = building.getProductionFromProductivity(building.getProductivity(unit));
+            production = building.applyModifiers(building.getUnitProductivity(unit));
         } else {
-            Tile workTile = ((ColonyTile) location).getWorkTile();
-            production = unit.getFarmedPotential(currentlyWorking, workTile);
-            if (currentlyWorking == Goods.FOOD && !workTile.isLand()) {
-                currentlyWorking = Goods.FISH;
-            }
+            production = ((ColonyTile) location).getProductionOf(unit, currentlyWorking);
         }
 
-        UnitType workingAs = UNIT_TYPE_BY_GOOD_TYPE.get(currentlyWorking);
+        UnitData workingAs = experts.get(currentlyWorking);
 
-        if (workingAs.getIndex() == unit.getType()) {
+        if (workingAs.getUnitType() == unit.getType()) {
             colonyData.getWorkingProfessionals().addProduction(production);
         } else {
             colonyData.workingAmateurs++;
 
-            getUnitData(workingAs).getLocationData(colony).otherWorkingAmateurs.addProduction(production);
+            workingAs.getLocationData(colony).otherWorkingAmateurs.addProduction(production);
         }
     }
 
@@ -606,16 +578,4 @@ public class LabourData {
     public UnitData getSummary() {
         return summary;
     }
-
-    public static Building getBuilding(Colony colony, UnitType unitType) {
-        if (unitType.getIndex() == Unit.ELDER_STATESMAN) {
-            Building building = colony.getBuilding(Building.PRINTING_PRESS);
-            return building.isBuilt() ? building : null;
-        }
-        GoodsType expert = unitType.getExpertProduction();
-        if (expert != null) {
-            return colony.getBuildingForProducing(expert.index);
-        }
-        return null;
-    }
 }
