@@ -32,7 +32,6 @@ import javax.swing.ImageIcon;
 import javax.swing.JButton;
 import javax.swing.JLabel;
 import javax.swing.JPanel;
-import javax.swing.UIManager;
 import javax.swing.border.BevelBorder;
 import javax.swing.border.CompoundBorder;
 import javax.swing.border.EmptyBorder;
@@ -40,13 +39,14 @@ import javax.swing.border.EmptyBorder;
 import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.gui.Canvas;
 import net.sf.freecol.client.gui.i18n.Messages;
+import net.sf.freecol.common.resources.ResourceManager;
 
 /**
 * A panel filled with 'main' items.
 */
 public final class MainPanel extends FreeColPanel implements ActionListener {
-    private static final Logger logger = Logger.getLogger(MainPanel.class.getName());
 
+    private static final Logger logger = Logger.getLogger(MainPanel.class.getName());
     
     public static final int     NEW = 0,
                                 OPEN = 1,
@@ -70,10 +70,10 @@ public final class MainPanel extends FreeColPanel implements ActionListener {
         this.parent = parent;
         this.freeColClient = freeColClient;
 
-        JButton         openButton = new JButton( Messages.message("menuBar.game.open") ),
-                        mapEditorButton = new JButton( Messages.message("mainPanel.editor") ),
-                        optionsButton = new JButton( Messages.message("mainPanel.options") ),
-                        quitButton = new JButton( Messages.message("menuBar.game.quit") );
+        JButton openButton = new JButton( Messages.message("menuBar.game.open") );
+        JButton mapEditorButton = new JButton( Messages.message("mainPanel.editor") );
+        JButton optionsButton = new JButton( Messages.message("mainPanel.options") );
+        JButton quitButton = new JButton( Messages.message("menuBar.game.quit") );
         
         setCancelComponent(quitButton);
         newButton = new JButton( Messages.message("menuBar.game.new") );
@@ -96,7 +96,7 @@ public final class MainPanel extends FreeColPanel implements ActionListener {
         enterPressesWhenFocused(optionsButton);
         enterPressesWhenFocused(quitButton);
 
-        Image tempImage = (Image) UIManager.get("TitleImage");
+        Image tempImage = ResourceManager.getImage("TitleImage");
 
         if (tempImage != null) {
             JLabel logoLabel = new JLabel(new ImageIcon(tempImage));
@@ -104,7 +104,7 @@ public final class MainPanel extends FreeColPanel implements ActionListener {
             add(logoLabel, BorderLayout.CENTER);
         }
 
-        JPanel buttons = new JPanel(new GridLayout(5, 1, 50, 10));
+        JPanel buttons = new JPanel(new GridLayout(0, 1, 50, 10));
 
         buttons.add(newButton);
         buttons.add(openButton);
@@ -150,7 +150,7 @@ public final class MainPanel extends FreeColPanel implements ActionListener {
             switch (Integer.valueOf(command).intValue()) {
                 case NEW:
                     parent.remove(this);                
-                    parent.showNewGamePanel();
+                    parent.showPanel(new NewPanel(parent));
                     break;
                 case OPEN:
                     freeColClient.getConnectController().loadGame();
