@@ -20,13 +20,17 @@
 
 package net.sf.freecol.client.gui.panel;
 
+import java.awt.Color;
 import java.awt.event.MouseAdapter;
 import java.awt.event.MouseEvent;
+import java.util.ArrayList;
+import java.util.List;
 
 import javax.swing.Action;
 import javax.swing.JLabel;
 import javax.swing.JLayeredPane;
 
+import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.ClientOptions;
 import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.gui.Canvas;
@@ -34,76 +38,70 @@ import net.sf.freecol.client.gui.GUI;
 import net.sf.freecol.client.gui.ViewMode;
 import net.sf.freecol.client.gui.action.ActionManager;
 import net.sf.freecol.client.gui.action.BuildColonyAction;
-import net.sf.freecol.client.gui.action.BuildRoadAction;
 import net.sf.freecol.client.gui.action.DisbandUnitAction;
 import net.sf.freecol.client.gui.action.FortifyAction;
-import net.sf.freecol.client.gui.action.PlowAction;
+import net.sf.freecol.client.gui.action.ImprovementActionType;
 import net.sf.freecol.client.gui.action.SentryAction;
 import net.sf.freecol.client.gui.action.SkipUnitAction;
 import net.sf.freecol.client.gui.action.WaitAction;
 import net.sf.freecol.client.gui.panel.MapEditorTransformPanel.MapTransform;
 import net.sf.freecol.common.model.Tile;
-import net.sf.freecol.common.model.Unit;
+import net.sf.freecol.common.model.Map.Direction;
+import net.sf.freecol.common.resources.ResourceManager;
 
 
 /**
-* A collection of panels and buttons that are used to provide
-* the user with a more detailed view of certain elements on the
-* map and also to provide a means of input in case the user
-* can't use the keyboard.
-*
-* The MapControls are useless by themselves, this object needs to
-* be placed on a JComponent in order to be useable.
-*/
+ * A collection of panels and buttons that are used to provide
+ * the user with a more detailed view of certain elements on the
+ * map and also to provide a means of input in case the user
+ * can't use the keyboard.
+ *
+ * The MapControls are useless by themselves, this object needs to
+ * be placed on a JComponent in order to be usable.
+ */
 public final class MapControls {
-    public static final int EUROPE = 2;
-    public static final int UNITBUTTON = 3;
-    
+
     private final FreeColClient freeColClient;
 
     private final InfoPanel        infoPanel;
     private final MiniMap          miniMap;
     private final UnitButton[]     unitButton;   
     private final JLabel compassRose;
-    @SuppressWarnings("unused")
-    private final GUI                    gui;
-
-
 
+    private static final int CONTROLS_LAYER = JLayeredPane.MODAL_LAYER;
 
     /**
-    * The basic constructor.
-    * @param freeColClient The main controller object for the client
-    * @param gui An object that contains useful GUI-related methods.
-    */
-    public MapControls(final FreeColClient freeColClient, final GUI gui) {
+     * The basic constructor.
+     * @param freeColClient The main controller object for the client
+     */
+    public MapControls(final FreeColClient freeColClient) {
         this.freeColClient = freeColClient;
-        this.gui = gui;
 
         //
         // Create GUI Objects
         //
 
-        infoPanel = new InfoPanel(freeColClient, freeColClient.getGame(), freeColClient.getImageLibrary());
+        infoPanel = new InfoPanel(freeColClient);
         miniMap = new MiniMap(freeColClient);
-        compassRose = new JLabel(freeColClient.getGUI().getImageLibrary().getMiscImageIcon(9));
+        compassRose = new JLabel(ResourceManager.getImageIcon("compass.image"));
         
         final ActionManager am = freeColClient.getActionManager();
-        unitButton = new UnitButton[] {
-            new UnitButton(am.getFreeColAction(WaitAction.id)),
-            new UnitButton(am.getFreeColAction(SkipUnitAction.id)),
-            new UnitButton(am.getFreeColAction(SentryAction.id)),
-            new UnitButton(am.getFreeColAction(FortifyAction.id)),
-            new UnitButton(am.getFreeColAction(PlowAction.id)),
-            new UnitButton(am.getFreeColAction(BuildRoadAction.id)),
-            new UnitButton(am.getFreeColAction(BuildColonyAction.id)),
-            new UnitButton(am.getFreeColAction(DisbandUnitAction.id))
-        };
         
+        List<UnitButton> ubList = new ArrayList<UnitButton>();
+        ubList.add(new UnitButton(am.getFreeColAction(WaitAction.id)));
+        ubList.add(new UnitButton(am.getFreeColAction(SkipUnitAction.id)));
+        ubList.add(new UnitButton(am.getFreeColAction(SentryAction.id)));
+        ubList.add(new UnitButton(am.getFreeColAction(FortifyAction.id)));
+        for (ImprovementActionType iaType : FreeCol.getSpecification().getImprovementActionTypeList()) {
+            ubList.add(new UnitButton(am.getFreeColAction(iaType.getId())));
+        }
+        ubList.add(new UnitButton(am.getFreeColAction(BuildColonyAction.id)));
+        ubList.add(new UnitButton(am.getFreeColAction(DisbandUnitAction.id)));
+        unitButton = (ubList.toArray(new UnitButton[0]));
+
         //
         // Don't allow them to gain focus
         //
-
         infoPanel.setFocusable(false);
         miniMap.setFocusable(false);
         compassRose.setFocusable(false);
@@ -115,14 +113,13 @@ public final class MapControls {
         compassRose.setSize(compassRose.getPreferredSize());
         compassRose.addMouseListener(new MouseAdapter() {
                 public void mouseClicked(MouseEvent e) {
-                    Unit activeUnit = gui.getActiveUnit();
                     int x = e.getX() - compassRose.getWidth()/2;
                     int y = e.getY() - compassRose.getHeight()/2;
                     double theta = Math.atan2(y, x) + Math.PI/2 + Math.PI/8;
                     if (theta < 0) {
                         theta += 2*Math.PI;
                     }
-                    int direction = (int) Math.floor(theta / (Math.PI/4));
+                    Direction direction = Direction.values()[(int) Math.floor(theta / (Math.PI/4))];
                     freeColClient.getInGameController().moveActiveUnit(direction);
                 }
             });
@@ -141,12 +138,12 @@ public final class MapControls {
     }
 
     /**
-    * Adds the map controls to the given component.
-    * @param component The component to add the map controls to.
-    */
+     * Adds the map controls to the given component.
+     * @param component The component to add the map controls to.
+     */
     public void addToComponent(Canvas component) {
         if (freeColClient.getGame() == null
-                || freeColClient.getGame().getMap() == null) {
+            || freeColClient.getGame().getMap() == null) {
             return;
         }
         
@@ -171,15 +168,15 @@ public final class MapControls {
         //
         // Add the GUI Objects to the container
         //
-        component.add(infoPanel, JLayeredPane.DEFAULT_LAYER, false);
-        component.add(miniMap, JLayeredPane.DEFAULT_LAYER, false);
+        component.add(infoPanel, CONTROLS_LAYER, false);
+        component.add(miniMap, CONTROLS_LAYER, false);
         if (freeColClient.getClientOptions().getBoolean(ClientOptions.DISPLAY_COMPASS_ROSE)) {
-            component.add(compassRose, JLayeredPane.DEFAULT_LAYER, false);
+            component.add(compassRose, CONTROLS_LAYER, false);
         }
 
         if (!freeColClient.isMapEditor()) {
             for(int i=0; i<unitButton.length; i++) {
-                component.add(unitButton[i], JLayeredPane.DEFAULT_LAYER, false);
+                component.add(unitButton[i], CONTROLS_LAYER, false);
                 Action a = unitButton[i].getAction();
                 unitButton[i].setAction(null);
                 unitButton[i].setAction(a);
@@ -187,6 +184,24 @@ public final class MapControls {
         }
     }
 
+    /**
+     * Returns the width of the InfoPanel.
+     *
+     * @return an <code>int</code> value
+     */
+    public int getInfoPanelWidth() {
+        return infoPanel.getWidth();
+    }
+
+    /**
+     * Returns the height of the InfoPanel.
+     *
+     * @return an <code>int</code> value
+     */
+    public int getInfoPanelHeight() {
+        return infoPanel.getHeight();
+    }
+
 
     /**
      * Removes the map controls from the parent component.
@@ -208,16 +223,16 @@ public final class MapControls {
 
 
     /**
-    * Zooms in the mini map.
-    */
+     * Zooms in the mini map.
+     */
     public void zoomIn() {
         miniMap.zoomIn();
     }
 
 
     /**
-    * Zooms out the mini map.
-    */
+     * Zooms out the mini map.
+     */
     public void zoomOut() {
         miniMap.zoomOut();
     }
@@ -231,23 +246,31 @@ public final class MapControls {
     }
 
     /**
-    * Updates this <code>MapControls</code>.
-    */
+     * 
+     * @param newColor
+     */
+    public void changeBackgroundColor(Color newColor) {
+    	miniMap.setBackgroundColor(newColor);
+    }
+
+    /**
+     * Updates this <code>MapControls</code>.
+     */
     public void update() {
         GUI gui = freeColClient.getGUI();
         int viewMode = gui.getViewMode().getView();
         switch (viewMode) {
-            case ViewMode.MOVE_UNITS_MODE:
-                infoPanel.update(gui.getActiveUnit());
-                break;
-            case ViewMode.VIEW_TERRAIN_MODE:
-                if (gui.getSelectedTile() != null) {
-                    Tile selectedTile = freeColClient.getGame().getMap().getTile(gui.getSelectedTile());
-                    if (infoPanel.getTile() != selectedTile) {
-                        infoPanel.update(selectedTile);
-                    }
+        case ViewMode.MOVE_UNITS_MODE:
+            infoPanel.update(gui.getActiveUnit());
+            break;
+        case ViewMode.VIEW_TERRAIN_MODE:
+            if (gui.getSelectedTile() != null) {
+                Tile selectedTile = freeColClient.getGame().getMap().getTile(gui.getSelectedTile());
+                if (infoPanel.getTile() != selectedTile) {
+                    infoPanel.update(selectedTile);
                 }
-                break;
+            }
+            break;
         }
     }
 }
