@@ -30,8 +30,9 @@ import java.util.logging.Logger;
 import javax.swing.ImageIcon;
 import javax.swing.JLabel;
 
+import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.gui.Canvas;
-import net.sf.freecol.common.model.Goods;
+import net.sf.freecol.common.model.GoodsType;
 import net.sf.freecol.common.model.Market;
 import net.sf.freecol.common.model.Player;
 
@@ -43,7 +44,7 @@ public final class MarketLabel extends JLabel implements ActionListener {
 
     private static Logger logger = Logger.getLogger(MarketLabel.class.getName());
 
-    private final int type;
+    private final GoodsType type;
     private int amount;
     private final Market market;
     private final Canvas parent;
@@ -56,12 +57,17 @@ public final class MarketLabel extends JLabel implements ActionListener {
     *       and sell <code>Goods</code>.
     * @param parent The parent that knows more than we do.
     */
-    public MarketLabel(int type, Market market, Canvas parent) {
-        super(parent.getImageProvider().getGoodsImageIcon(type));
+    public MarketLabel(GoodsType type, Market market, Canvas parent) {
+        super(parent.getGUI().getImageLibrary().getGoodsImageIcon(type));
         
         this.type = type;
-        setToolTipText(Goods.getName(type));
-
+        /*
+        if (FreeCol.isInDebugMode()) {
+            setToolTipText(type.getName() + " " + market.getMarketData(type).getAmountInMarket());
+        } else {
+            setToolTipText(type.getName());
+        }
+        */
         if (market == null) {
             throw new NullPointerException();
         }
@@ -81,7 +87,7 @@ public final class MarketLabel extends JLabel implements ActionListener {
     * @param parent The parent that knows more than we do.
     * @param isSmall The image will be smaller if set to <code>true</code>.
     */
-    public MarketLabel(int type, Market market, Canvas parent, boolean isSmall) {
+    public MarketLabel(GoodsType type, Market market, Canvas parent, boolean isSmall) {
         this(type, market, parent);
         setSmall(true);
     }
@@ -101,7 +107,7 @@ public final class MarketLabel extends JLabel implements ActionListener {
     * Returns this MarketLabel's goods type.
     * @return This MarketLabel's goods type.
     */
-    public int getType() {
+    public GoodsType getType() {
         return type;
     }
 
@@ -136,10 +142,10 @@ public final class MarketLabel extends JLabel implements ActionListener {
     */
     public void setSmall(boolean isSmall) {
         if (isSmall) {
-            ImageIcon imageIcon = (parent.getImageProvider().getGoodsImageIcon(type));
+            ImageIcon imageIcon = parent.getGUI().getImageLibrary().getGoodsImageIcon(type);
             setIcon(new ImageIcon(imageIcon.getImage().getScaledInstance(imageIcon.getIconWidth() / 2, imageIcon.getIconHeight() / 2, Image.SCALE_DEFAULT)));
         } else {
-            setIcon(parent.getImageProvider().getGoodsImageIcon(type));
+            setIcon(parent.getGUI().getImageLibrary().getGoodsImageIcon(type));
         }
     }
 
@@ -151,13 +157,17 @@ public final class MarketLabel extends JLabel implements ActionListener {
     public void paintComponent(Graphics g) {
 
         Player player = market.getGame().getViewOwner();
+        String toolTipText = type.getName();
         if (player == null || player.canTrade(type)) {
-            setToolTipText(Goods.getName(type));
             setEnabled(true);
         } else {
-            setToolTipText(Goods.getName(type, false));
+            toolTipText = type.getName(false);
             setEnabled(false);
         }
+        if (FreeCol.isInDebugMode()) {
+            toolTipText += " " + market.getMarketData(type).getAmountInMarket();
+        }
+        setToolTipText(toolTipText);
 
         super.setText(Integer.toString(market.paidForSale(type)) + "/" + Integer.toString(market.costToBuy(type)));
         super.paintComponent(g);
@@ -175,7 +185,7 @@ public final class MarketLabel extends JLabel implements ActionListener {
                     default:
                         logger.warning("Invalid action");
                 }
-                setIcon(parent.getImageProvider().getGoodsImageIcon(type));
+                setIcon(parent.getGUI().getImageLibrary().getGoodsImageIcon(type));
                 repaint(0, 0, getWidth(), getHeight());
                 
                 // TODO: Refresh the gold label when goods have prices.
