@@ -30,17 +30,20 @@ import java.util.logging.Logger;
 
 import javax.swing.JButton;
 import javax.swing.JPanel;
-import javax.swing.UIManager;
 import javax.swing.border.BevelBorder;
 import javax.swing.event.MouseInputListener;
 
+import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.ClientOptions;
 import net.sf.freecol.client.FreeColClient;
+import net.sf.freecol.client.gui.action.MiniMapChangeBackgroundAction;
 import net.sf.freecol.client.gui.action.MiniMapZoomInAction;
 import net.sf.freecol.client.gui.action.MiniMapZoomOutAction;
 import net.sf.freecol.common.model.Map;
 import net.sf.freecol.common.model.Settlement;
 import net.sf.freecol.common.model.Tile;
+import net.sf.freecol.common.model.TileType;
+import net.sf.freecol.common.resources.ResourceManager;
 
 
 
@@ -67,6 +70,7 @@ public final class MiniMap extends JPanel implements MouseInputListener {
     private FreeColClient freeColClient;
     private final JButton          miniMapZoomOutButton;
     private final JButton          miniMapZoomInButton;
+    private Color backgroundColor;
 
     private int tileSize; //tileSize is the size (in pixels) that each tile will take up on the mini map
 
@@ -94,6 +98,7 @@ public final class MiniMap extends JPanel implements MouseInputListener {
      */
     public MiniMap(FreeColClient freeColClient) {
         this.freeColClient = freeColClient;
+        backgroundColor = Color.BLACK;
 
         tileSize = 4 * (freeColClient.getClientOptions().getInteger(ClientOptions.DEFAULT_MINIMAP_ZOOM) + 1);
 
@@ -102,7 +107,7 @@ public final class MiniMap extends JPanel implements MouseInputListener {
         setLayout(null);
 
         //boolean usingSkin;
-        Image skin = (Image) UIManager.get("MiniMap.skin");
+        Image skin = ResourceManager.getImage("MiniMap.skin");
         if (skin == null) {
             try {
                 BevelBorder border = new BevelBorder(BevelBorder.RAISED);
@@ -130,7 +135,7 @@ public final class MiniMap extends JPanel implements MouseInputListener {
         miniMapZoomOutButton.setFocusable(false);
         miniMapZoomInButton.setFocusable(false);
 
-        int bh = mapY + MAP_HEIGHT - Math.max(miniMapZoomOutButton.getHeight(), miniMapZoomInButton.getHeight());;
+        int bh = mapY + MAP_HEIGHT - Math.max(miniMapZoomOutButton.getHeight(), miniMapZoomInButton.getHeight());
         int bw = mapX;
         if (getBorder() != null) {
             Insets insets = getBorder().getBorderInsets(this);
@@ -204,7 +209,11 @@ public final class MiniMap extends JPanel implements MouseInputListener {
                 || freeColClient.getGame().getMap() == null) {
             return;
         }        
-        Image skin = (Image) UIManager.get("MiniMap.skin");
+        Image skin = ResourceManager.getImage("MiniMap.skin");
+        
+    	int colorIndex = freeColClient.getClientOptions().getInteger(ClientOptions.MINIMAP_BACKGROUND_COLOR);
+    	Color newBackground = MiniMapChangeBackgroundAction.interpretIndex(colorIndex);
+    	this.setBackgroundColor(newBackground);
         
         scaledFactorX = 1;
         scaledFactorY = 1;
@@ -217,7 +226,7 @@ public final class MiniMap extends JPanel implements MouseInputListener {
             if (!scaleMap) {
                 paintMap(graphics, mapX, mapY, MAP_WIDTH, MAP_HEIGHT);
             } else {
-                graphics.setColor(Color.BLACK);
+                graphics.setColor(backgroundColor);
                 graphics.fillRect(mapX, mapY, MAP_WIDTH, MAP_HEIGHT);
                 
                 final int realMapWidth = freeColClient.getGame().getMap().getWidth();
@@ -333,65 +342,16 @@ public final class MiniMap extends JPanel implements MouseInputListener {
             int yPixels = 0;
             for (int tileY = 0; yPixels <= height + tileSize; tileY++, yPixels += tileSize/4) {
                 /* Check the terrain to find out which color to use */
-                Tile tile = map.getTileOrNull(tileX + xOffset, tileY + yOffset);
+                Tile tile = map.getTile(tileX + xOffset, tileY + yOffset);
                 if (tile == null) {
                     continue;
                 }
-                Settlement settlement = (tile != null ? tile.getSettlement() : null);
-                int units = (tile != null ? tile.getUnitCount() : 0);
+                Settlement settlement = tile.getSettlement();
+                int units = tile.getUnitCount();
                 g.setColor(Color.BLACK); //Default
-
-                if (tile == null) {
-                    g.setColor(Color.BLACK);
-                } else if (tile.getAddition() == Tile.ADD_HILLS) {
-                    g.setColor(new Color(0.44f, 0.50f, 0.32f)); // Grayish orange
-                } else if (tile.getAddition() == Tile.ADD_MOUNTAINS) {
-                    g.setColor(new Color(0.34f, 0.45f, 0.32f)); // Gray
-                } else if (!tile.isForested()) {
-                    int type = tile.getType();
-                    switch (type) {
-                        case Tile.PLAINS:
-                        case Tile.GRASSLANDS:
-                        case Tile.PRAIRIE:
-                        case Tile.SAVANNAH:
-                            g.setColor(new Color(0.14f, 0.50f, 0.12f)); // Green
-                            break;
-
-                        case Tile.MARSH:
-                        case Tile.SWAMP:
-                            g.setColor(new Color(0.14f, 0.50f, 0.24f)); // Bluish green
-                            break;
-
-                        case Tile.DESERT:
-                            g.setColor(new Color(0.39f, 0.45f, 0.17f)); // Orangish
-                            break;
-
-                        case Tile.TUNDRA:
-                            g.setColor(new Color(0.39f, 0.62f, 0.37f)); // Light blue
-                            break;
-
-                        case Tile.ARCTIC:
-                            g.setColor(Color.WHITE);
-                            break;
-
-                        case Tile.OCEAN: //Blue ocean
-                            g.setColor(Color.BLUE);
-                            break;
-
-                        case Tile.HIGH_SEAS: //Darker blue high seas
-                            g.setColor(new Color(0.0f, 0.0f, 0.8f));
-                            break;
-
-                        case Tile.UNEXPLORED:
-                        default:
-                            g.setColor(Color.BLACK);
-                            break;
-                    }
-                } else {
-                  // Tile is forested, so display color of the forest
-                  g.setColor(new Color(0.14f, 0.45f, 0.12f)); // Darker green
+                if (tile.isExplored()) {
+                    g.setColor(tile.getType().getMinimapColor());
                 }
-
                 if (tileSize == 4) {
                     int extra = (((tileY + yOffset) % 2) == 0) ? 0 : 2;
                     g.drawLine(x+extra+ 4*tileX, y+tileY, x+2+extra+4*tileX, y+tileY);
@@ -454,10 +414,11 @@ public final class MiniMap extends JPanel implements MouseInputListener {
          * miniRectX/Y are the center of the rectangle.
          * Use miniRectWidth/Height / 2 to get the upper left corner.
          * x/yTiles are the number of tiles that fit on the large map */
+        TileType tileType = FreeCol.getSpecification().getTileTypeList().get(0);
         int miniRectX = (freeColClient.getGUI().getFocus().getX() - xOffset) * tileSize;
         int miniRectY = (freeColClient.getGUI().getFocus().getY() - yOffset) * tileSize / 4;
-        int miniRectWidth = (getParent().getWidth() / imageProvider.getTerrainImageWidth(0) + 1) * tileSize;
-        int miniRectHeight = (getParent().getHeight() / imageProvider.getTerrainImageHeight(0) + 1) * tileSize / 2;
+        int miniRectWidth = (getParent().getWidth() / imageProvider.getTerrainImageWidth(tileType) + 1) * tileSize;
+        int miniRectHeight = (getParent().getHeight() / imageProvider.getTerrainImageHeight(tileType) + 1) * tileSize / 2;
         if (miniRectX + miniRectWidth / 2 > width) {
             miniRectX = width - miniRectWidth / 2 - 1;
         } else if (miniRectX - miniRectWidth / 2 < 0) {
@@ -473,14 +434,27 @@ public final class MiniMap extends JPanel implements MouseInputListener {
         miniRectY += y;
 
         g.setColor(Color.WHITE);
-        g.drawRect(miniRectX - miniRectWidth / 2, miniRectY - miniRectHeight / 2, miniRectWidth, miniRectHeight);
-        if (scaleMap) {
-            g.drawRect(miniRectX - miniRectWidth / 2 + 1, miniRectY - miniRectHeight / 2 + 1, miniRectWidth - 2, miniRectHeight - 2);
+        /* Use Math max and min to prevent the rect from being larger than the minimap. */
+        int miniRectMaxX = Math.max(miniRectX - miniRectWidth / 2, x);
+        int miniRectMaxY = Math.max(miniRectY - miniRectHeight / 2, y);
+        int miniRectMinWidth = Math.min(miniRectWidth, width - 1);
+        int miniRectMinHeight = Math.min(miniRectHeight, height - 1);
+        /* Prevent the rect from overlapping the bigger adjust rect */
+        if(miniRectMaxX + miniRectMinWidth > x + width - 1) {
+        	miniRectMaxX = x + width - miniRectMinWidth - 1;
         }
-
+        if(miniRectMaxY + miniRectMinHeight > y + height - 1) {
+        	miniRectMaxY = y + height - miniRectMinHeight - 1;
+        }
+        /* Draw the white rect. */
+        g.drawRect(miniRectMaxX, miniRectMaxY, miniRectMinWidth, miniRectMinHeight);
+        if(scaleMap) {
+        	g.drawRect(miniRectX - miniRectWidth / 2 + 1, miniRectY - miniRectHeight / 2 + 1, miniRectWidth - 2, miniRectHeight - 2);
+        }
+		/* Draw an additional white rect, if the whole map is shown on the minimap */
         if (adjustX > 0 && adjustY > 0) {
             g.setColor(Color.WHITE);
-            g.drawRect(x, y, width, height);
+            g.drawRect(x, y, width - 1, height - 1);
         }
     }
 
@@ -557,4 +531,12 @@ public final class MiniMap extends JPanel implements MouseInputListener {
     public void mouseMoved(MouseEvent e) {
 
     }
+
+	public Color getBackgroundColor() {
+		return backgroundColor;
+	}
+
+	public void setBackgroundColor(Color backgroundColor) {
+		this.backgroundColor = backgroundColor;
+	}
 }
