@@ -22,12 +22,14 @@ package net.sf.freecol.client.gui.panel;
 
 import java.awt.Color;
 import java.awt.Component;
-import java.util.Vector;
+import java.util.List;
 
 import javax.swing.JComboBox;
+import javax.swing.JLabel;
 import javax.swing.JTable;
 import javax.swing.table.TableCellRenderer;
 
+import net.sf.freecol.common.model.Nation;
 import net.sf.freecol.common.model.Player;
 
 /**
@@ -37,17 +39,17 @@ import net.sf.freecol.common.model.Player;
 public final class NationCellRenderer implements TableCellRenderer {
 
 
-    private static final JComboBox standardNationsComboBox = new JComboBox(Player.NATIONS);
-    private static final JComboBox indianTribesComboBox = new JComboBox(Player.TRIBES);
-    private static final JComboBox refNationsComboBox = new JComboBox(Player.REF_NATIONS);
-
-    private Vector<Player> players;
+    private final Nation[] nations;
+    private final JComboBox comboBox;
+    private List<Player> players;
     private Player thisPlayer;
 
     /**
     * The default constructor.
     */
-    public NationCellRenderer() {
+    public NationCellRenderer(Nation[] nations) {
+        this.nations = nations;
+        this.comboBox = new JComboBox(nations);
     }
 
 
@@ -57,7 +59,7 @@ public final class NationCellRenderer implements TableCellRenderer {
     * @param players The players that should be rendered in the table.
     * @param owningPlayer The player running the client that is displaying the table.
     */
-    public void setData(Vector<Player> players, Player owningPlayer) {
+    public void setData(List<Player> players, Player owningPlayer) {
         this.players = players;
         thisPlayer = owningPlayer;
     }
@@ -86,28 +88,17 @@ public final class NationCellRenderer implements TableCellRenderer {
 
         Player player = getPlayer(row);
 
-        JComboBox component;
-        if (player.isREF()) {
-            component = refNationsComboBox;
-        } else if (player.isEuropean()) {
-            component = standardNationsComboBox;
-        } else {
-            component = indianTribesComboBox;
-        }
-
-        if (player.isAI()) {
-        /*
-            int nation = player.getNation();
-            if ((nation != Player.DUTCH) && (nation != Player.ENGLISH)
-                    && (nation != Player.FRENCH) && (nation != Player.SPANISH)) {
-                // This is an indian AI player.
-
-                indianTribesComboBox.setForeground(Color.LIGHT_GRAY);
-                return indianTribesComboBox;
-            } else {
-                standardNationsComboBox.setForeground(Color.LIGHT_GRAY);
+        Component component;
+        if (player == thisPlayer) {
+            for (int index = 0; index < nations.length; index++) {
+                if (nations[index].getId().equals(player.getNationID())) {
+                    comboBox.setSelectedIndex(index);
+                    break;
+                }
             }
-            */
+            component = comboBox;
+        } else {
+            component = new JLabel(player.getNationAsString());
         }
 
         if (player.isReady()) {
@@ -117,14 +108,6 @@ public final class NationCellRenderer implements TableCellRenderer {
         }
         component.setBackground(table.getBackground());
 
-        int index = player.getNation();
-        if (player.isIndian() || player.isREF()) {
-            index -= Player.NATIONS.length;
-        }
-        if (player.isREF()) {
-            index -= Player.TRIBES.length;
-        }        
-        component.setSelectedIndex(index);
         return component;
     }
 }
