@@ -17,7 +17,6 @@
  *  along with FreeCol.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-
 package net.sf.freecol.client.gui.panel;
 
 import java.awt.Color;
@@ -31,9 +30,12 @@ import javax.swing.ButtonGroup;
 import javax.swing.JButton;
 import javax.swing.JCheckBox;
 import javax.swing.JLabel;
-import javax.swing.JPanel;
 import javax.swing.JRadioButton;
+import javax.swing.JSpinner;
 import javax.swing.JTextField;
+import javax.swing.SpinnerNumberModel;
+import javax.swing.event.ChangeEvent;
+import javax.swing.event.ChangeListener;
 
 import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.control.ConnectController;
@@ -46,8 +48,10 @@ import cz.autel.dmi.HIGLayout;
 * A panel filled with 'new game' items.
 */
 public final class NewPanel extends FreeColPanel implements ActionListener {
+
     private static final Logger logger = Logger.getLogger(NewPanel.class.getName());
 
+    private static final int CLASSIC_PLAYER_NO = 4;
 
     private static final int    OK = 0,
                                 CANCEL = 1,
@@ -64,14 +68,17 @@ public final class NewPanel extends FreeColPanel implements ActionListener {
                                 join,
                                 start,
                                 meta;
-    private final JPanel        namePanel,
-                                joinServerPanel,
-                                startServerPanel,
-                                buttonPanel;
     private final JLabel        ipLabel,
-                                port1Label,
-                                port2Label;
-    private final JCheckBox     publicServer;
+        port1Label,
+        port2Label,
+        singlePlayerNoLabel,
+        multiPlayerNoLabel;
+    private final JCheckBox     publicServer,
+        additionalNations,
+        selectAdvantages,
+        useAdvantages;
+
+    private final JSpinner singlePlayerNo, multiPlayerNo;
 
     private final Canvas        parent;
 
@@ -83,12 +90,10 @@ public final class NewPanel extends FreeColPanel implements ActionListener {
     * The constructor that will add the items to this panel.
     * 
     * @param parent The parent of this panel.
-    * @param connectController The controller responsible for
-    *       creating new connections.
     */
-    public NewPanel(Canvas parent, ConnectController connectController) {
+    public NewPanel(Canvas parent) {
         this.parent = parent;
-        this.connectController = connectController;
+        this.connectController = parent.getClient().getConnectController();
 
         JButton         cancel = new JButton( Messages.message("cancel") );
         ButtonGroup     group = new ButtonGroup();
@@ -99,73 +104,96 @@ public final class NewPanel extends FreeColPanel implements ActionListener {
         ipLabel = new JLabel( Messages.message("host") );
         port1Label = new JLabel( Messages.message("port") );
         port2Label = new JLabel( Messages.message("startServerOnPort") );
+        singlePlayerNoLabel = new JLabel(Messages.message("singlePlayerNo"));
+        multiPlayerNoLabel = new JLabel(Messages.message("multiPlayerNo"));
+
         publicServer = new JCheckBox( Messages.message("publicServer") );
+        additionalNations = new JCheckBox(Messages.message("additionalNations"));
+        additionalNations.addChangeListener(new ChangeListener() {
+                public void stateChanged(ChangeEvent e) {
+                    if (!additionalNations.isSelected()) {
+                        int players = ((Integer) singlePlayerNo.getValue()).intValue();
+                        if (players > CLASSIC_PLAYER_NO) {
+                            singlePlayerNo.setValue(CLASSIC_PLAYER_NO);
+                        }
+                    }
+                }
+            });
+
+        selectAdvantages = new JCheckBox(Messages.message("selectAdvantages"));
+        useAdvantages = new JCheckBox(Messages.message("useAdvantages"));
+
+        singlePlayerNo = new JSpinner(new SpinnerNumberModel(CLASSIC_PLAYER_NO, 1, 8, 1));
+        singlePlayerNo.addChangeListener(new ChangeListener() {
+                public void stateChanged(ChangeEvent e) {
+                    int players = ((Integer) singlePlayerNo.getValue()).intValue();
+                    if (players > 4) {
+                        additionalNations.setSelected(true);
+                    }
+                }
+            });
+        multiPlayerNo = new JSpinner(new SpinnerNumberModel(CLASSIC_PLAYER_NO, 2, 8, 1));
+
         name = new JTextField( System.getProperty("user.name", Messages.message("defaultPlayerName")) );
-        name.setColumns(30);
+        name.setColumns(20);
+
         server = new JTextField("127.0.0.1");
-        port1 = new JTextField("3541");
-        port2 = new JTextField("3541");
+        port1 = new JTextField(new Integer(FreeCol.getDefaultPort()).toString());
+        port2 = new JTextField(new Integer(FreeCol.getDefaultPort()).toString());
         single = new JRadioButton(Messages.message("singlePlayerGame"), true);
         join = new JRadioButton(Messages.message("joinMultiPlayerGame"), false);
         start = new JRadioButton(Messages.message("startMultiplayerGame"), false);
         meta = new JRadioButton( Messages.message("getServerList") + " (" + FreeCol.META_SERVER_ADDRESS + ")", false);
 
-        namePanel = new JPanel();
-        namePanel.setOpaque(false);
-        namePanel.add(nameLabel);
-        namePanel.add(name);
-
-        joinServerPanel = new JPanel();
-        joinServerPanel.setOpaque(false);
-        joinServerPanel.add(ipLabel);
-        joinServerPanel.add(server);
-        joinServerPanel.add(port1Label);
-        joinServerPanel.add(port1);
-
-        startServerPanel = new JPanel();
-        startServerPanel.setOpaque(false);
-        startServerPanel.add(port2Label);
-        startServerPanel.add(port2);
-
-        buttonPanel = new JPanel();
-        buttonPanel.setOpaque(false);
-        buttonPanel.add(ok);
-        buttonPanel.add(cancel);
-        enterPressesWhenFocused(ok);
-        enterPressesWhenFocused(cancel);
-
         group.add(single);
         group.add(join);
         group.add(start);
         group.add(meta);
 
-        int numberOfRows = 9;
-        int[] widths = {0};
+        int numberOfRows = 13;
+        int[] widths = {21, 0, margin, 0, margin, 0, margin, 0, 6*margin, 0};
         int[] heights = new int[2 * numberOfRows - 1];
         for (int index = 1; index < heights.length; index += 2) {
             heights[index] = margin;
         }
-        int column = 1;
+        heights[heights.length - 2] = 3 * margin;
+
         setLayout(new HIGLayout(widths, heights));
 
         int row = 1;
-        add(namePanel, higConst.rc(row, column));
+        add(nameLabel, higConst.rc(row, 8, "r"));
+        add(name, higConst.rc(row, 10));
         row += 2;
-        add(single, higConst.rc(row, column));
+        add(single, higConst.rcwh(row, 1, 8, 1, "l"));
         row += 2;
-        add(meta, higConst.rc(row, column));
+        add(additionalNations, higConst.rcwh(row, 2, 3, 1, "l"));
         row += 2;
-        add(join, higConst.rc(row, column));
+        add(selectAdvantages, higConst.rcwh(row, 2, 3, 1, "l"));
         row += 2;
-        add(joinServerPanel, higConst.rc(row, column, "l"));
+        add(singlePlayerNoLabel, higConst.rcwh(row, 2, 3, 1, "l"));
+        add(singlePlayerNo, higConst.rc(row, 8));
         row += 2;
-        add(start, higConst.rc(row, column));
+        add(meta, higConst.rcwh(row, 1, 8, 1, "l"));
         row += 2;
-        add(startServerPanel, higConst.rc(row, column, "l"));
+        add(join, higConst.rcwh(row, 1, 8, 1, "l"));
         row += 2;
-        add(publicServer, higConst.rc(row, column));
+        add(ipLabel, higConst.rc(row, 2));
+        add(server, higConst.rc(row, 4));
+        add(port1Label, higConst.rc(row, 6));
+        add(port1, higConst.rc(row, 8));
         row += 2;
-        add(buttonPanel, higConst.rc(row, column));
+        add(start, higConst.rcwh(row, 1, 8, 1, "l"));
+        row += 2;
+        add(port2Label, higConst.rcwh(row, 2, 3, 1, "l"));
+        add(port2, higConst.rc(row, 8));
+        add(publicServer, higConst.rc(row, 10, "l"));
+        row += 2;
+        add(multiPlayerNoLabel, higConst.rcwh(row, 2, 3, 1, "l"));
+        add(multiPlayerNo, higConst.rc(row, 8));
+        add(useAdvantages, higConst.rc(row, 10, "l"));
+        row += 2;
+        add(ok, higConst.rc(row, 8, "r"));
+        add(cancel, higConst.rc(row, 10, "l"));
 
 
         ok.setActionCommand(String.valueOf(OK));
@@ -182,13 +210,8 @@ public final class NewPanel extends FreeColPanel implements ActionListener {
         start.addActionListener(this);
         meta.addActionListener(this);
 
-        ipLabel.setEnabled(false);
-        server.setEnabled(false);
-        port1Label.setEnabled(false);
-        port1.setEnabled(false);
-        port2Label.setEnabled(false);
-        publicServer.setEnabled(false);
-        port2.setEnabled(false);
+        single.setSelected(true);
+        setEnabledComponents();
 
         setSize(getPreferredSize());
     }
@@ -211,34 +234,54 @@ public final class NewPanel extends FreeColPanel implements ActionListener {
         for (int i = 0; i < components.length; i++) {
             components[i].setEnabled(enabled);
         }
+        setEnabledComponents();
+    }
 
-        if (single.isSelected() || meta.isSelected()) {
-            ipLabel.setEnabled(false);
-            server.setEnabled(false);
-            port1Label.setEnabled(false);
-            port1.setEnabled(false);
-            port2Label.setEnabled(false);
-            port2.setEnabled(false);
-            publicServer.setEnabled(false);
+    private void setEnabledComponents() {
+        if (single.isSelected()) {
+            setSinglePlayerOptions(true);
+            setJoinGameOptions(false);
+            setServerOptions(false);
         } else if (join.isSelected()) {
-            ipLabel.setEnabled(true);
-            server.setEnabled(true);
-            port1Label.setEnabled(true);
-            port1.setEnabled(true);
-            port2Label.setEnabled(false);
-            port2.setEnabled(false);
-            publicServer.setEnabled(false);
+            setSinglePlayerOptions(false);
+            setJoinGameOptions(true);
+            setServerOptions(false);
         } else if (start.isSelected()) {
-            ipLabel.setEnabled(false);
-            server.setEnabled(false);
-            port1Label.setEnabled(false);
-            port1.setEnabled(false);
-            port2Label.setEnabled(true);
-            port2.setEnabled(true);
-            publicServer.setEnabled(true);
+            setSinglePlayerOptions(false);
+            setJoinGameOptions(false);
+            setServerOptions(true);
+        } else if (meta.isSelected()) {
+            setSinglePlayerOptions(false);
+            setJoinGameOptions(false);
+            setServerOptions(false);
         }
     }
 
+    private void setSinglePlayerOptions(boolean enabled) {
+        additionalNations.setEnabled(enabled);
+        selectAdvantages.setEnabled(enabled);
+        singlePlayerNoLabel.setEnabled(enabled);
+        singlePlayerNo.setEnabled(enabled);
+    }
+
+    private void setJoinGameOptions(boolean enabled) {
+        ipLabel.setEnabled(enabled);
+        server.setEnabled(enabled);
+        port1Label.setEnabled(enabled);
+        port1.setEnabled(enabled);
+    }
+
+    private void setServerOptions(boolean enabled) {
+        port2Label.setEnabled(enabled);
+        port2.setEnabled(enabled);
+        publicServer.setEnabled(enabled);
+        multiPlayerNoLabel.setEnabled(enabled);
+        multiPlayerNo.setEnabled(enabled);
+        useAdvantages.setEnabled(enabled);
+    }
+        
+
+
     /**
     * This function analyses an event and calls the right methods to take
     * care of the user's requests.
@@ -250,7 +293,14 @@ public final class NewPanel extends FreeColPanel implements ActionListener {
             switch (Integer.valueOf(command).intValue()) {
                 case OK:
                     if (single.isSelected()) {
-                        connectController.startSingleplayerGame(name.getText());
+                        int players = ((Integer) singlePlayerNo.getValue()).intValue();
+                        int advantages = AdvantageCellRenderer.FIXED;
+                        if (selectAdvantages.isSelected()) {
+                            advantages = AdvantageCellRenderer.SELECTABLE;
+                        }
+                        connectController.startSingleplayerGame(name.getText(), players, 
+                                                                additionalNations.isSelected(),
+                                                                advantages);
                     } else if (join.isSelected()) {
                         int port;
 
@@ -272,8 +322,13 @@ public final class NewPanel extends FreeColPanel implements ActionListener {
                             port2Label.setForeground(Color.red);
                             break;
                         }
-
-                        connectController.startMultiplayerGame(publicServer.isSelected(), name.getText(), port);
+                        int players = ((Integer) singlePlayerNo.getValue()).intValue();
+                        int advantages = AdvantageCellRenderer.NONE;
+                        if (useAdvantages.isSelected()) {
+                            advantages = AdvantageCellRenderer.SELECTABLE;
+                        }
+                        connectController.startMultiplayerGame(publicServer.isSelected(), name.getText(), port,
+                                                               players, advantages);
                     } else if (meta.isSelected()) {
                         ArrayList<ServerInfo> serverList = connectController.getServerList();
                         if (serverList != null) {
@@ -286,40 +341,10 @@ public final class NewPanel extends FreeColPanel implements ActionListener {
                     parent.showMainPanel();
                     break;
                 case SINGLE:
-                    ipLabel.setEnabled(false);
-                    server.setEnabled(false);
-                    port1Label.setEnabled(false);
-                    port1.setEnabled(false);
-                    port2Label.setEnabled(false);
-                    port2.setEnabled(false);
-                    publicServer.setEnabled(false);
-                    break;
                 case JOIN:
-                    ipLabel.setEnabled(true);
-                    server.setEnabled(true);
-                    port1Label.setEnabled(true);
-                    port1.setEnabled(true);
-                    port2Label.setEnabled(false);
-                    port2.setEnabled(false);
-                    publicServer.setEnabled(false);
-                    break;
                 case START:
-                    ipLabel.setEnabled(false);
-                    server.setEnabled(false);
-                    port1Label.setEnabled(false);
-                    port1.setEnabled(false);
-                    port2Label.setEnabled(true);
-                    port2.setEnabled(true);
-                    publicServer.setEnabled(true);
-                    break;
                 case META_SERVER:
-                    ipLabel.setEnabled(false);
-                    server.setEnabled(false);
-                    port1Label.setEnabled(false);
-                    port1.setEnabled(false);
-                    port2Label.setEnabled(false);
-                    port2.setEnabled(false);
-                    publicServer.setEnabled(false);
+                    setEnabledComponents();
                     break;
                 default:
                     logger.warning("Invalid Actioncommand: invalid number.");
