@@ -32,6 +32,7 @@ import javax.swing.JComponent;
 import net.sf.freecol.client.ClientOptions;
 import net.sf.freecol.client.gui.Canvas;
 import net.sf.freecol.common.model.Goods;
+import net.sf.freecol.common.model.GoodsType;
 
 /**
  * The ProductionLabel represents Goods that are produced in a
@@ -39,9 +40,6 @@ import net.sf.freecol.common.model.Goods;
  */
 public final class ProductionLabel extends JComponent {
 
-
-
-
     private static Logger logger = Logger.getLogger(ProductionLabel.class.getName());
 
     private final Canvas parent;
@@ -69,7 +67,7 @@ public final class ProductionLabel extends JComponent {
     /**
      * The type of goods being produced.
      */
-    private int goodsType;
+    private GoodsType goodsType;
 
     /**
      * The goodsIcon for this type of production.
@@ -114,7 +112,7 @@ public final class ProductionLabel extends JComponent {
      * @param amount an <code>int</code> value
      * @param parent a <code>Canvas</code> value
      */
-    public ProductionLabel(int goodsType, int amount, Canvas parent) {
+    public ProductionLabel(GoodsType goodsType, int amount, Canvas parent) {
         this(goodsType, amount, -1, parent);
     }
 
@@ -126,7 +124,7 @@ public final class ProductionLabel extends JComponent {
      * @param maximumProduction an <code>int</code> value
      * @param parent a <code>Canvas</code> value
      */
-    public ProductionLabel(int goodsType, int amount, int maximumProduction, Canvas parent) {
+    public ProductionLabel(GoodsType goodsType, int amount, int maximumProduction, Canvas parent) {
         super();
         this.parent = parent;
         this.production = amount;
@@ -143,11 +141,10 @@ public final class ProductionLabel extends JComponent {
         } else {
             setForeground(Color.WHITE);
         }
-        if (goodsType >= 0) {
-            goodsIcon = parent.getImageProvider().getGoodsImageIcon(goodsType);
-            compressedWidth = goodsIcon.getIconWidth()*2;
+        if (goodsType != null) {
+            setGoodsIcon(parent.getGUI().getImageLibrary().getGoodsImageIcon(goodsType));
+            setToolTipText(String.valueOf(amount) + " " + goodsType.getName());
         }
-        setToolTipText(String.valueOf(amount) + " " + Goods.getName(goodsType));
     }
     
 
@@ -194,6 +191,7 @@ public final class ProductionLabel extends JComponent {
      */
     public void setGoodsIcon(final ImageIcon newGoodsIcon) {
         this.goodsIcon = newGoodsIcon;
+        compressedWidth = goodsIcon.getIconWidth()*2;
     }
 
     /**
@@ -212,7 +210,9 @@ public final class ProductionLabel extends JComponent {
      */
     public void setProduction(final int newProduction) {
         this.production = newProduction;
-        setToolTipText(String.valueOf(production) + " " + Goods.getName(goodsType));
+        if (goodsType != null) { // goodsType is null for Schoolhouse
+            setToolTipText(String.valueOf(production) + " " + goodsType.getName());
+        }
     }
 
     /**
@@ -421,7 +421,8 @@ public final class ProductionLabel extends JComponent {
             goodsIcon.paintIcon(null, g, leftOffset + i*pixelsPerIcon, 0);
         }
 
-        if (production >= displayNumber || production < 0 || maxIcons < production || stockNumber>0) {
+        if (production >= displayNumber || production < 0 || maxIcons < production || stockNumber > 0
+            || (maximumProduction > production && production > 0)) {
             String number = "";
             if (stockNumber >= 0 ) {
                 number = Integer.toString(stockNumber);  // Show stored items in ReportColonyPanel
