@@ -23,6 +23,7 @@ import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.util.logging.Logger;
 
+import javax.swing.ImageIcon;
 import javax.swing.JButton;
 import javax.swing.JTextArea;
 
@@ -31,8 +32,10 @@ import net.sf.freecol.client.control.InGameController;
 import net.sf.freecol.client.gui.Canvas;
 import net.sf.freecol.client.gui.ImageLibrary;
 import net.sf.freecol.client.gui.i18n.Messages;
+import net.sf.freecol.common.model.Colony;
+import net.sf.freecol.common.model.Goods;
 import net.sf.freecol.common.model.Player;
-import net.sf.freecol.common.model.Unit;
+import net.sf.freecol.common.model.UnitType;
 import cz.autel.dmi.HIGLayout;
 
 /**
@@ -40,8 +43,6 @@ import cz.autel.dmi.HIGLayout;
  */
 public final class RecruitDialog extends FreeColDialog implements ActionListener {
 
-
-
     private static Logger logger = Logger.getLogger(RecruitDialog.class.getName());
 
     private static final int RECRUIT_CANCEL = -1;
@@ -100,10 +101,7 @@ public final class RecruitDialog extends FreeColDialog implements ActionListener
     public void initialize() {
 
         int[] widths = new int[] { 0 };
-        int[] heights = new int[9];
-        for (int index = 0; index < 4; index++) {
-            heights[2 * index + 1] = margin;
-        }
+        int[] heights = new int[] { 0, 3 * margin, 0, margin, 0, margin, 0, margin, 0 };
         setLayout(new HIGLayout(widths, heights));
 
         int row = 1;
@@ -115,16 +113,31 @@ public final class RecruitDialog extends FreeColDialog implements ActionListener
         int recruitPrice = 0;
         Player player = freeColClient.getMyPlayer();
         if ((freeColClient.getGame() != null) && (player != null)) {
-            ImageLibrary library = (ImageLibrary) getCanvas().getImageProvider();
+
+            int production = 0;
+            for (Colony colony : player.getColonies()) {
+                production += colony.getProductionOf(Goods.CROSSES);
+            }
+            int turns = 100;
+            if (production > 0) {
+                int crossesRequired = (player.getCrossesRequired() - player.getCrosses());
+                turns = crossesRequired / production;
+                if (crossesRequired % production > 0) {
+                    turns++;
+                }
+            }
+            ImageLibrary library = getCanvas().getGUI().getImageLibrary();
             recruitPrice = player.getRecruitPrice();
 
-            question.setText(Messages.message("recruitDialog.clickOn", "%money%", String.valueOf(recruitPrice)));
+            question.setText(Messages.message("recruitDialog.clickOn",
+                                              "%money%", String.valueOf(recruitPrice),
+                                              "%number%", String.valueOf(turns)));
 
             for (int index = 0; index < NUMBER_OF_PERSONS; index++) {
-                int unitType = player.getEurope().getRecruitable(index);
-                int graphicsType = ImageLibrary.getUnitGraphicsType(unitType, false, false, 0, false);
-                person[index].setText(Unit.getName(unitType));
-                person[index].setIcon(library.getScaledUnitImageIcon(graphicsType, 0.66f));
+                UnitType unitType = player.getEurope().getRecruitable(index);
+                ImageIcon unitIcon = library.getUnitImageIcon(unitType);
+                person[index].setText(unitType.getName());
+                person[index].setIcon(library.getScaledImageIcon(unitIcon, 0.66f));
 
                 if (recruitPrice > player.getGold()) {
                     person[index].setEnabled(false);
