@@ -21,7 +21,6 @@ package net.sf.freecol.client.gui.panel;
 
 import java.awt.Color;
 import java.awt.GridLayout;
-import java.awt.Insets;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.util.Collections;
@@ -33,11 +32,14 @@ import javax.swing.JLabel;
 import javax.swing.JPanel;
 import javax.swing.border.Border;
 
+import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.gui.Canvas;
 import net.sf.freecol.client.gui.i18n.Messages;
+import net.sf.freecol.common.model.BuildableType;
 import net.sf.freecol.common.model.Building;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.Goods;
+import net.sf.freecol.common.model.GoodsType;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Unit;
 import cz.autel.dmi.HIGLayout;
@@ -45,7 +47,7 @@ import cz.autel.dmi.HIGLayout;
 /**
  * This panel displays the Colony Report.
  */
-public final class ReportColonyPanel extends ReportPanel implements ActionListener {
+public final class ReportColonyPanel extends ReportPanel {
 
 
 
@@ -145,18 +147,19 @@ public final class ReportColonyPanel extends ReportPanel implements ActionListen
     private JPanel createProductionPanel(Colony colony) {
         JPanel goodsPanel = new JPanel(new GridLayout(0, 10));
         goodsPanel.setOpaque(false);
-        for (int goodsType = 0; goodsType < Goods.NUMBER_OF_ALL_TYPES; goodsType++) {
-            int newValue = colony.getProductionNetOf(goodsType);
-            int stockValue = -1;  // Show stored items in ReportColonyPanel
-            if (goodsType < Goods.NUMBER_OF_TYPES) {
-                stockValue = colony.getGoodsCount(goodsType);
-            } else if (goodsType == Goods.HAMMERS) {
-                stockValue = colony.getHammers();
-            } else if (goodsType == Goods.BELLS) {
-                stockValue = colony.getBells();
+        int netFood = colony.getFoodProduction() - colony.getFoodConsumption();
+        if (netFood != 0) {
+            ProductionLabel productionLabel = new ProductionLabel(Goods.FOOD, netFood, getCanvas());
+            productionLabel.setStockNumber(colony.getFoodCount());
+            goodsPanel.add(productionLabel);
+        }
+        for (GoodsType goodsType : FreeCol.getSpecification().getGoodsTypeList()) {
+            if (goodsType.isFoodType()) {
+                continue;
             }
+            int newValue = colony.getProductionNetOf(goodsType);
+            int stockValue = colony.getGoodsCount(goodsType);
             if (newValue != 0 || stockValue > 0) {
-                Goods goods = new Goods(colony.getGame(), colony, goodsType, newValue);
                 Building building = colony.getBuildingForProducing(goodsType);
                 ProductionLabel productionLabel = new ProductionLabel(goodsType, newValue, getCanvas());
                 if (building != null) {
@@ -175,42 +178,24 @@ public final class ReportColonyPanel extends ReportPanel implements ActionListen
     private JPanel createBuildingPanel(Colony colony) {
         JPanel buildingPanel = new JPanel(new GridLayout(0, 5, 12, 0));
         buildingPanel.setOpaque(false);
-        int currentType = colony.getCurrentlyBuilding();
-        for (int buildingType = 0; buildingType < Building.NUMBER_OF_TYPES; buildingType++) {
-            Building building = colony.getBuilding(buildingType);
-            if (building.getLevel() != Building.NOT_BUILT) {
-                buildingPanel.add(new JLabel(building.getName()));
-            }
-            if (buildingType == currentType) {
-                JLabel buildingLabel = new JLabel(building.getNextName());
-                buildingLabel.setForeground(Color.GRAY);
-                buildingPanel.add(buildingLabel);
-            }
+
+        for (Building building : colony.getBuildings()) {
+            buildingPanel.add(new JLabel(building.getName()));
         }
-        if (currentType >= Colony.BUILDING_UNIT_ADDITION) {
-            JLabel unitLabel = new JLabel(Unit.getName(currentType - Colony.BUILDING_UNIT_ADDITION));
-            unitLabel.setForeground(Color.GRAY);
-            buildingPanel.add(unitLabel);
-        }
-        /* If nothing is build, show it red color */
-        if (currentType == -1) {
-            JLabel nothingLabel = new JLabel(Messages.message("nothing"));
-            nothingLabel.setForeground(Color.RED);
-            buildingPanel.add(nothingLabel);
+        
+        BuildableType currentType = colony.getCurrentlyBuilding();
+        JLabel buildableLabel = new JLabel(currentType.getName());
+        if (currentType == BuildableType.NOTHING) {
+            buildableLabel.setForeground(Color.RED);
+        } else {
+            buildableLabel.setForeground(Color.GRAY);
         }
+        buildingPanel.add(buildableLabel);
         return buildingPanel;
     }
 
     private JButton createColonyButton(int index) {
-
-        JButton button = new JButton(colonies.get(index).getName());
-        button.setFont(smallHeaderFont);
-        button.setMargin(new Insets(0,0,0,0));
-        button.setOpaque(false);
-        button.setForeground(LINK_COLOR);
-        button.setAlignmentY(0.8f);
-        button.setBorder(BorderFactory.createEmptyBorder());
-        button.setActionCommand(String.valueOf(index));
+        JButton button = getLinkButton(colonies.get(index).getName(), null, String.valueOf(index));
         button.addActionListener(this);
         return button;
     }
