@@ -19,37 +19,34 @@
 
 package net.sf.freecol.client.gui.panel;
 
-import java.awt.Insets;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.util.ArrayList;
 import java.util.Collections;
-import java.util.Iterator;
 import java.util.List;
 
-import javax.swing.BorderFactory;
 import javax.swing.JButton;
 import javax.swing.JTextPane;
-import javax.swing.text.Style;
 import javax.swing.text.StyleConstants;
-import javax.swing.text.StyleContext;
 import javax.swing.text.StyledDocument;
 
+import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.gui.Canvas;
 import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.common.model.Building;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.ColonyTile;
-import net.sf.freecol.common.model.Goods;
+import net.sf.freecol.common.model.GoodsType;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Unit;
-import net.sf.freecol.common.model.WorkLocation;
+import net.sf.freecol.common.model.UnitType;
+
 import cz.autel.dmi.HIGLayout;
 
 /**
  * This panel displays the Advanced Colony Report.
  */
-public final class ReportRequirementsPanel extends ReportPanel implements ActionListener {
+public final class ReportRequirementsPanel extends ReportPanel {
 
 
 
@@ -97,26 +94,25 @@ public final class ReportRequirementsPanel extends ReportPanel implements Action
         reportPanel.setLayout(new HIGLayout(new int[] {780}, new int[] {0}));
 
         //Create a text pane.
-        JTextPane textPane = new JTextPane();
-        textPane.setOpaque(false);
-	textPane.setEditable(false);
-
+        JTextPane textPane = getDefaultTextPane();
         StyledDocument doc = textPane.getStyledDocument();
-        defineStyles(doc);
 
-        unitCount = new int[colonies.size()][Unit.UNIT_COUNT];
-        canTrain = new boolean[colonies.size()][Unit.UNIT_COUNT];
-        surplus = new int[colonies.size()][Goods.NUMBER_OF_TYPES];
+        int numberUnitTypes = FreeCol.getSpecification().numberOfUnitTypes();
+        int numberGoodsTypes = FreeCol.getSpecification().numberOfGoodsTypes();
+        unitCount = new int[colonies.size()][numberUnitTypes];
+        canTrain = new boolean[colonies.size()][numberUnitTypes];
+        surplus = new int[colonies.size()][numberGoodsTypes];
 
+        List<GoodsType> goodsTypes = FreeCol.getSpecification().getGoodsTypeList();
         // check which colonies can train which units
         for (int index = 0; index < colonies.size(); index++) {
             Colony colony = colonies.get(index);
             for (Unit unit : colony.getUnitList()) {
-                unitCount[index][unit.getType()]++;
-                canTrain[index][unit.getType()] = colony.canTrain(unit);
+                unitCount[index][unit.getType().getIndex()]++;
+                canTrain[index][unit.getType().getIndex()] = colony.canTrain(unit.getType());
             }
-            for (int goodsType = 0; goodsType < Goods.NUMBER_OF_TYPES; goodsType++) {
-                surplus[index][goodsType] = colony.getProductionNetOf(goodsType);
+            for (GoodsType goodsType : goodsTypes) {
+                surplus[index][goodsType.getIndex()] = colony.getProductionNetOf(goodsType);
             }
         }
 
@@ -133,47 +129,47 @@ public final class ReportRequirementsPanel extends ReportPanel implements Action
                 logger.warning(e.toString());
             }
 
-            boolean[] expertWarning = new boolean[Unit.UNIT_COUNT];
-            boolean[] productionWarning = new boolean[Goods.NUMBER_OF_ALL_TYPES];
+            boolean[] expertWarning = new boolean[numberUnitTypes];
+            boolean[] productionWarning = new boolean[numberGoodsTypes];
             boolean hasWarning = false;
 
             // check if all unit requirements are met
-            Iterator<WorkLocation> workLocationIterator = colony.getWorkLocationIterator();
-            while (workLocationIterator.hasNext()) {
-                // check colony tiles
-                WorkLocation workLocation = workLocationIterator.next();
-                if (workLocation instanceof ColonyTile) {
-                    Unit unit = ((ColonyTile) workLocation).getUnit();
-                    if (unit != null) {
-                        int workType = unit.getWorkType();
-                        int expert = ((ColonyTile) workLocation).getExpertForProducing(workType);
-                        if (unitCount[index][expert] == 0 && !expertWarning[expert]) {
-                            addExpertWarning(doc, index, Goods.getName(workType), expert);
-                            expertWarning[expert] = true;
-                            hasWarning = true;
-                        }
+            for (ColonyTile colonyTile : colony.getColonyTiles()) {
+                Unit unit = colonyTile.getUnit();
+                if (unit != null) {
+                    GoodsType workType = unit.getWorkType();
+                    UnitType expert = FreeCol.getSpecification().getExpertForProducing(workType);
+                    int expertIndex = expert.getIndex();
+                    if (unitCount[index][expertIndex] == 0 && !expertWarning[expertIndex]) {
+                        addExpertWarning(doc, index, workType.getName(), expert);
+                        expertWarning[expertIndex] = true;
+                        hasWarning = true;
                     }
-                } else {
-                    // check buildings
-                    Building building = (Building) workLocation;
-                    int goodsType = building.getGoodsOutputType();
-                    int expert = building.getExpertUnitType();
-                    if (goodsType != -1) {
-                        // no expert
-                        if (building.getFirstUnit() != null &&
-                            !expertWarning[expert] &&
-                            unitCount[index][expert] == 0) {
-                            addExpertWarning(doc, index, Goods.getName(goodsType), expert);
-                            expertWarning[expert] = true;
-                            hasWarning = true;
-                        }
-                        // not enough input
-                        if (building.getProductionNextTurn() < building.getMaximumProduction() &&
-                            !productionWarning[goodsType]) {
-                            addProductionWarning(doc, index, goodsType, building.getGoodsInputType());
-                            productionWarning[goodsType] = true;
-                            hasWarning = true;
-                        }
+                }
+            } 
+            for (Building building : colony.getBuildings()) {
+                GoodsType goodsType = building.getGoodsOutputType();
+                UnitType expert = building.getExpertUnitType();
+                    
+                if (goodsType != null && expert != null) {
+                    // check if this building has no expert producing goods
+                    int expertIndex = expert.getIndex();
+                    if (building.getFirstUnit() != null &&
+                        !expertWarning[expertIndex] &&
+                        unitCount[index][expertIndex] == 0) {
+                        addExpertWarning(doc, index, goodsType.getName(), expert);
+                        expertWarning[expertIndex] = true;
+                        hasWarning = true;
+                    }
+                }
+                if (goodsType != null) {
+                    // not enough input
+                    int goodsIndex = goodsType.getIndex();
+                    if (building.getProductionNextTurn() < building.getMaximumProduction() &&
+                        !productionWarning[goodsIndex]) {
+                        addProductionWarning(doc, index, goodsType, building.getGoodsInputType());
+                        productionWarning[goodsIndex] = true;
+                        hasWarning = true;
                     }
                 }
             }
@@ -195,8 +191,8 @@ public final class ReportRequirementsPanel extends ReportPanel implements Action
 
     }
 
-    private void addExpertWarning(StyledDocument doc, int colonyIndex, String goods, int workType) {
-        String expertName = Unit.getName(workType);
+    private void addExpertWarning(StyledDocument doc, int colonyIndex, String goods, UnitType workType) {
+        String expertName = workType.getName();
         String colonyName = colonies.get(colonyIndex).getName();
         String newMessage = Messages.message("report.requirements.noExpert", "%colony%", colonyName, "%goods%", goods,
                 "%unit%", expertName);
@@ -207,10 +203,10 @@ public final class ReportRequirementsPanel extends ReportPanel implements Action
             ArrayList<Colony> severalExperts = new ArrayList<Colony>();
             ArrayList<Colony> canTrainExperts = new ArrayList<Colony>();
             for (int index = 0; index < colonies.size(); index++) {
-                if (unitCount[index][workType] > 1) {
+                if (unitCount[index][workType.getIndex()] > 1) {
                     severalExperts.add(colonies.get(index));
                 }
-                if (canTrain[index][workType]) {
+                if (canTrain[index][workType.getIndex()]) {
                     canTrainExperts.add(colonies.get(index));
                 }
             }
@@ -253,12 +249,12 @@ public final class ReportRequirementsPanel extends ReportPanel implements Action
         
     }
 
-    private void addProductionWarning(StyledDocument doc, int colonyIndex, int output, int input) {
+    private void addProductionWarning(StyledDocument doc, int colonyIndex, GoodsType output, GoodsType input) {
         String colonyName = colonies.get(colonyIndex).getName();
         String newMessage = Messages.message("report.requirements.missingGoods",
                 "%colony%", colonyName,
-                "%goods%", Goods.getName(output),
-                "%input%", Goods.getName(input));
+                "%goods%", output.getName(),
+                "%input%", input.getName());
 
         try {
             doc.insertString(doc.getLength(), newMessage + "\n\n", doc.getStyle("regular"));
@@ -266,15 +262,15 @@ public final class ReportRequirementsPanel extends ReportPanel implements Action
             ArrayList<Colony> withSurplus = new ArrayList<Colony>();
             ArrayList<Integer> theSurplus = new ArrayList<Integer>();
             for (int index = 0; index < colonies.size(); index++) {
-                if (surplus[index][input] > 0) {
+                if (surplus[index][input.getIndex()] > 0) {
                     withSurplus.add(colonies.get(index));
-                    theSurplus.add(new Integer(surplus[index][input]));
+                    theSurplus.add(new Integer(surplus[index][input.getIndex()]));
                 }
             }
 
             if (!withSurplus.isEmpty()) {
                 doc.insertString(doc.getLength(),
-                        Messages.message("report.requirements.surplus", "%goods%", Goods.getName(input)) + " ",
+                        Messages.message("report.requirements.surplus", "%goods%", input.getName()) + " ",
                         doc.getStyle("regular"));
                 for (int index = 0; index < withSurplus.size() - 1; index++) {
                     Colony colony = withSurplus.get(index);
@@ -302,35 +298,14 @@ public final class ReportRequirementsPanel extends ReportPanel implements Action
     }
 
     private JButton createColonyButton(Colony colony, String info, boolean headline) {
-        JButton button = new JButton(colony.getName() + info);
+        JButton button = getLinkButton(colony.getName() + info, null, colony.getId());
         if (headline) {
             button.setFont(smallHeaderFont);
         }
-        button.setMargin(new Insets(0,0,0,0));
-        button.setOpaque(false);
-        button.setForeground(LINK_COLOR);
-        button.setAlignmentY(0.8f);
-        button.setBorder(BorderFactory.createEmptyBorder());
-        button.setActionCommand(colony.getId());
         button.addActionListener(this);
         return button;
     }
 
-    private void defineStyles(StyledDocument doc) {
-
-        //Initialize some styles.
-        Style def = StyleContext.getDefaultStyleContext().getStyle(StyleContext.DEFAULT_STYLE);
-	
-        Style regular = doc.addStyle("regular", def);
-        StyleConstants.setFontFamily(def, "Dialog");
-        StyleConstants.setBold(def, true);
-	StyleConstants.setFontSize(def, 12);
-
-        Style buttonStyle = doc.addStyle("button", regular);
-        StyleConstants.setForeground(buttonStyle, LINK_COLOR);
-    }
-
-
     /**
      * This function analyses an event and calls the right methods to take care
      * of the user's requests.
@@ -345,7 +320,7 @@ public final class ReportRequirementsPanel extends ReportPanel implements Action
         } else {
             Colony colony = (Colony) getCanvas().getClient().getGame().getFreeColGameObject(command);
             if (colony instanceof Colony) {
-                getCanvas().showColonyPanel((Colony) colony);
+                getCanvas().showColonyPanel(colony);
             }
         }
     }
