@@ -19,46 +19,40 @@
 
 package net.sf.freecol.client.gui.panel;
 
+import cz.autel.dmi.HIGLayout;
+import net.sf.freecol.FreeCol;
+import net.sf.freecol.client.gui.Canvas;
+import net.sf.freecol.client.gui.i18n.Messages;
+import net.sf.freecol.common.model.Colony;
+import net.sf.freecol.common.model.GoodsType;
+import net.sf.freecol.common.model.Market;
+import net.sf.freecol.common.model.Player;
+import net.sf.freecol.common.model.Unit;
+
+import javax.swing.JButton;
+import javax.swing.JLabel;
+import javax.swing.JPanel;
+import javax.swing.SwingConstants;
+import javax.swing.border.Border;
+import javax.swing.border.EmptyBorder;
 import java.awt.Color;
 import java.awt.Dimension;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
+import java.util.ArrayList;
 import java.util.Collections;
-import java.util.List;
 import java.util.Iterator;
-
-import javax.swing.JButton;
-import javax.swing.JLabel;
-import javax.swing.SwingConstants;
-import javax.swing.JPanel;
-import javax.swing.border.Border;
-import javax.swing.border.EmptyBorder;
-
-import net.sf.freecol.client.gui.Canvas;
-import net.sf.freecol.client.gui.i18n.Messages;
-import net.sf.freecol.common.model.Building;
-import net.sf.freecol.common.model.Colony;
-import net.sf.freecol.common.model.Goods;
-import net.sf.freecol.common.model.Market;
-import net.sf.freecol.common.model.Player;
-import net.sf.freecol.common.model.Unit;
-import cz.autel.dmi.HIGLayout;
+import java.util.List;
 
 /**
  * This panel displays the Trade Report.
  */
-public final class ReportTradePanel extends ReportPanel implements ActionListener {
-
-
-    /**
-     * How many colums are defined per label.
-     */
-    private static final int columnsPerLabel = 1;
+public final class ReportTradePanel extends ReportPanel {
 
     /**
      * How many additional rows are defined.
      */
-    private static final int extraRows = 8; // labels and separators
+    private static final int extraRows = 7; // labels and separators
     private static final int extraBottomRows = 2;
 
     /**
@@ -67,9 +61,9 @@ public final class ReportTradePanel extends ReportPanel implements ActionListene
     private static final int extraColumns = 1; // labels
 
     /**
-     * How many columns are defined all together.
+     * Storable goods types.
      */
-    private static final int columns = columnsPerLabel * Goods.NUMBER_OF_TYPES + extraColumns;
+    private List<GoodsType> storableGoods = new ArrayList<GoodsType>();
 
     /**
      * How wide the margins should be.
@@ -104,6 +98,12 @@ public final class ReportTradePanel extends ReportPanel implements ActionListene
 
         goodsHeader.setBorder(new EmptyBorder(20, 20, 0, 20));
         scrollPane.setColumnHeaderView(goodsHeader);
+
+        for (GoodsType goodsType : FreeCol.getSpecification().getGoodsTypeList()) {
+            if (goodsType.isStorable()) {
+                storableGoods.add(goodsType);
+            }
+        }
     }
 
     @Override
@@ -111,6 +111,7 @@ public final class ReportTradePanel extends ReportPanel implements ActionListene
         return new Dimension(900, 750);
     }
 
+    @Override
     protected Border createBorder() {
         return new EmptyBorder(0, 20, 20, 20);
     }
@@ -129,12 +130,13 @@ public final class ReportTradePanel extends ReportPanel implements ActionListene
         colonies = player.getColonies();
         Collections.sort(colonies, getCanvas().getClient().getClientOptions().getColonyComparator());
 
+        int columns = storableGoods.size() + extraColumns;
         int[] widths = new int[columns];
 
         //align the labels with the goodsHeader
-        widths[0] = 170; 
+        widths[0] = 170;
         for (int i = 1; i < columns; i++) {
-            widths[i] = 40;
+            widths[i] = 42;
         }
 
         int[] heights = new int[colonies.size() * 2 + extraRows + extraBottomRows];
@@ -146,7 +148,7 @@ public final class ReportTradePanel extends ReportPanel implements ActionListene
 
         reportPanel.setLayout(new HIGLayout(widths, heights));
         goodsHeader.setLayout(new HIGLayout(widths, new int[1]));
-        
+
         JLabel emptyLabel = new JLabel();
         emptyLabel.setBorder(FreeColPanel.TOPLEFTCELLBORDER);
         goodsHeader.add(emptyLabel, higConst.rc(1, labelColumn));
@@ -157,21 +159,19 @@ public final class ReportTradePanel extends ReportPanel implements ActionListene
 
         JLabel cargoUnitsLabel = new JLabel(Messages.message("report.trade.cargoUnits"), JLabel.TRAILING);
         cargoUnitsLabel.setBorder(FreeColPanel.LEFTCELLBORDER);
-        JLabel equippedUnitsLabel = new JLabel(Messages.message("report.trade.equippedUnits"), JLabel.TRAILING);
-        equippedUnitsLabel.setBorder(FreeColPanel.LEFTCELLBORDER);
         JLabel totalUnitsLabel = new JLabel(Messages.message("report.trade.totalUnits"), JLabel.TRAILING);
         totalUnitsLabel.setBorder(FreeColPanel.LEFTCELLBORDER);
         JLabel totalDeltaLabel = new JLabel(Messages.message("report.trade.totalDelta"), JLabel.TRAILING);
         totalDeltaLabel.setBorder(FreeColPanel.LEFTCELLBORDER);
 
         reportPanel.add(cargoUnitsLabel, higConst.rc(4, labelColumn));
-        reportPanel.add(equippedUnitsLabel, higConst.rc(5, labelColumn));
-        reportPanel.add(totalUnitsLabel, higConst.rc(6, labelColumn));
-        reportPanel.add(totalDeltaLabel, higConst.rc(7, labelColumn));
+        reportPanel.add(totalUnitsLabel, higConst.rc(5, labelColumn));
+        reportPanel.add(totalDeltaLabel, higConst.rc(6, labelColumn));
 
         JLabel currentLabel;
-        for (int goodsType = 0; goodsType < Goods.NUMBER_OF_TYPES; goodsType++) {
-            int column = goodsType + extraColumns + 1;
+        int column = extraColumns;
+        for (GoodsType goodsType : storableGoods) {
+            column++;
             int sales = player.getSales(goodsType);
             int beforeTaxes = player.getIncomeBeforeTaxes(goodsType);
             int afterTaxes = player.getIncomeAfterTaxes(goodsType);
@@ -204,21 +204,14 @@ public final class ReportTradePanel extends ReportPanel implements ActionListene
             reportPanel.add(currentLabel, higConst.rc(3, column));
 
             int cargoUnits = 0;
-            int equippedUnits = 0;
             for (Iterator<Unit> iterator = player.getUnitIterator(); iterator.hasNext();) {
                 Unit unit = iterator.next();
                 if (unit.isCarrier()) {
                     cargoUnits += unit.getGoodsContainer().getGoodsCount(goodsType);
-                } else if (goodsType == Goods.MUSKETS && unit.isArmed()) {
-                    equippedUnits += 50;
-                } else if (goodsType == Goods.HORSES && unit.isMounted()) {
-                    equippedUnits += 50;
-                } else if (goodsType == Goods.TOOLS) {
-                    equippedUnits += unit.getNumberOfTools();
                 }
             }
 
-            int totalUnits = cargoUnits + equippedUnits;
+            int totalUnits = cargoUnits;
             int deltaUnits = 0;
             for (Colony colony : colonies) {
                 deltaUnits += colony.getProductionNetOf(goodsType);
@@ -229,13 +222,9 @@ public final class ReportTradePanel extends ReportPanel implements ActionListene
             cargoUnitsAmount.setBorder(FreeColPanel.CELLBORDER);
             reportPanel.add(cargoUnitsAmount, higConst.rc(4, column));
 
-            JLabel equippedUnitsAmount = new JLabel(String.valueOf(equippedUnits), JLabel.TRAILING);
-            equippedUnitsAmount.setBorder(FreeColPanel.CELLBORDER);
-            reportPanel.add(equippedUnitsAmount, higConst.rc(5, column));
-
             JLabel totalUnitsAmount = new JLabel(String.valueOf(totalUnits), JLabel.TRAILING);
             totalUnitsAmount.setBorder(FreeColPanel.CELLBORDER);
-            reportPanel.add(totalUnitsAmount, higConst.rc(6, column));
+            reportPanel.add(totalUnitsAmount, higConst.rc(5, column));
 
             JLabel deltaUnitsAmount = new JLabel(String.valueOf(deltaUnits), JLabel.TRAILING);
             if (deltaUnits < 0) {
@@ -244,7 +233,7 @@ public final class ReportTradePanel extends ReportPanel implements ActionListene
                 deltaUnitsAmount.setText("+" + deltaUnits);
             }
             deltaUnitsAmount.setBorder(FreeColPanel.CELLBORDER);
-            reportPanel.add(deltaUnitsAmount, higConst.rc(7, column));
+            reportPanel.add(deltaUnitsAmount, higConst.rc(6, column));
         }
 
         int row = extraRows + 1;
@@ -253,8 +242,12 @@ public final class ReportTradePanel extends ReportPanel implements ActionListene
             Colony colony = colonies.get(colonyIndex);
             JButton colonyButton = createColonyButton(colonyIndex);
             reportPanel.add(colonyButton, higConst.rcwh(row, labelColumn, 1, 2));
-            for (int goodsType = 0; goodsType < Goods.NUMBER_OF_TYPES; goodsType++) {
-                int column = goodsType + 1 + extraColumns;
+            column = extraColumns;
+            for (GoodsType goodsType : FreeCol.getSpecification().getGoodsTypeList()) {
+                if (!goodsType.isStorable()) {
+                    continue;
+                }
+                column++;
                 int amount = colony.getGoodsCount(goodsType);
                 JLabel goodsLabel = new JLabel(String.valueOf(amount), JLabel.TRAILING);
                 if (colonyIndex == 0) {
@@ -262,7 +255,7 @@ public final class ReportTradePanel extends ReportPanel implements ActionListene
                 } else {
                     goodsLabel.setBorder(FreeColPanel.CELLBORDER);
                 }
-                if (colony.getExports(goodsType)) {
+                if (colony.getExportData(goodsType).isExported()) {
                     goodsLabel.setText("*" + String.valueOf(amount));
                 }
                 reportPanel.add(goodsLabel, higConst.rc(row, column));
@@ -300,11 +293,11 @@ public final class ReportTradePanel extends ReportPanel implements ActionListene
     private JButton createColonyButton(int index) {
 
         JButton button = new JButton();
-        if (colonies.get(index).getBuilding(Building.CUSTOM_HOUSE).isBuilt()) {
-            button.setText(colonies.get(index).getName() + "*");
-        } else {
-            button.setText(colonies.get(index).getName());
+        String name = colonies.get(index).getName();
+        if (colonies.get(index).hasAbility("model.ability.export")) {
+            name += "*";
         }
+        button.setText(name);
         button.setOpaque(false);
         button.setForeground(LINK_COLOR);
         button.setHorizontalAlignment(SwingConstants.LEADING);
