@@ -23,11 +23,17 @@ import java.awt.Color;
 import java.awt.Component;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
-import java.util.Vector;
+import java.awt.event.MouseAdapter;
+import java.awt.event.MouseEvent;
+import java.util.ArrayList;
+import java.util.List;
 import java.util.logging.Logger;
 
+import javax.swing.DefaultCellEditor;
 import javax.swing.JButton;
 import javax.swing.JCheckBox;
+import javax.swing.JComboBox;
+import javax.swing.JLabel;
 import javax.swing.JScrollPane;
 import javax.swing.JTable;
 import javax.swing.JTextArea;
@@ -35,13 +41,19 @@ import javax.swing.JTextField;
 import javax.swing.ScrollPaneConstants;
 import javax.swing.table.AbstractTableModel;
 import javax.swing.table.DefaultTableCellRenderer;
+import javax.swing.table.JTableHeader;
 import javax.swing.table.TableColumn;
+import javax.swing.table.TableCellRenderer;
 
+import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.control.PreGameController;
 import net.sf.freecol.client.gui.Canvas;
+import net.sf.freecol.client.gui.panel.ColopediaPanel.PanelType;
 import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.common.model.Game;
+import net.sf.freecol.common.model.Nation;
+import net.sf.freecol.common.model.NationType;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.server.generator.MapGeneratorOptions;
 import cz.autel.dmi.HIGLayout;
@@ -52,27 +64,13 @@ import cz.autel.dmi.HIGLayout;
  */
 public final class StartGamePanel extends FreeColPanel implements ActionListener {
 
-
-
-
     private static final Logger logger = Logger.getLogger(StartGamePanel.class.getName());
 
     private static final int START = 0, CANCEL = 1,
-    // MAPSIZE = 2,
-            READY = 3, CHAT = 4, GAME_OPTIONS = 5, MAP_GENERATOR_OPTIONS = 6;
+        READY = 3, CHAT = 4, GAME_OPTIONS = 5, MAP_GENERATOR_OPTIONS = 6;
 
-    // private final String[] colors = { Messages.message("black"),
-    // Messages.message("blue"),
-    // Messages.message("cyan"),
-    // Messages.message("gray"),
-    // Messages.message("green"),
-    // Messages.message("magenta"),
-    // Messages.message("orange"),
-    // Messages.message("pink"),
-    // Messages.message("red"),
-    // Messages.message("white"),
-    // Messages.message("yellow"),
-    // };
+
+    public static final int NAME_COLUMN = 0, NATION_COLUMN = 1, ADVANTAGE_COLUMN = 2, COLOR_COLUMN = 3;
 
     private final Canvas parent;
 
@@ -100,6 +98,60 @@ public final class StartGamePanel extends FreeColPanel implements ActionListener
 
     private JButton mapGeneratorOptions;
 
+    private int advantages;
+
+
+    private class HeaderRenderer implements TableCellRenderer {
+
+        private static final int NO_COLUMN = -1;
+        private int pressedColumn = NO_COLUMN;
+        private Component[] components;
+
+        public HeaderRenderer(Component... components) {
+            this.components = components;
+        }
+
+        public Component getTableCellRendererComponent(JTable table,
+                                                       Object value,
+                                                       boolean isSelected,
+                                                       boolean hasFocus,
+                                                       int row,
+                                                       int column) {
+            if (components[column] instanceof JButton) {
+                boolean isPressed = (column == pressedColumn);
+                ((JButton) components[column]).getModel().setPressed(isPressed);
+                ((JButton) components[column]).getModel().setArmed(isPressed);
+            }
+            return components[column];
+        }
+
+        public void setPressedColumn(int column) {
+            pressedColumn = column;
+        }
+    }
+
+    private class HeaderListener extends MouseAdapter {
+        JTableHeader header;
+
+        HeaderRenderer renderer;
+
+        HeaderListener(JTableHeader header, HeaderRenderer renderer) {
+            this.header = header;
+            this.renderer = renderer;
+        }
+
+        public void mousePressed(MouseEvent e) {
+            int col = header.columnAtPoint(e.getPoint());
+            renderer.setPressedColumn(col);
+            header.repaint();
+        }
+
+        public void mouseReleased(MouseEvent e) {
+            renderer.setPressedColumn(HeaderRenderer.NO_COLUMN);
+            header.repaint();
+        }
+    }
+
 
     /**
      * The constructor that will add the items to this panel.
@@ -129,8 +181,9 @@ public final class StartGamePanel extends FreeColPanel implements ActionListener
         tableModel = new PlayersTableModel(freeColClient, freeColClient.getPreGameController());
         table = new JTable(tableModel);
 
-        TableColumn nameColumn = table.getColumnModel().getColumn(0),
-                colorsColumn = table.getColumnModel().getColumn(2);
+        TableColumn nameColumn = table.getColumnModel().getColumn(NAME_COLUMN),
+            advantageColumn = table.getColumnModel().getColumn(ADVANTAGE_COLUMN),
+            colorsColumn = table.getColumnModel().getColumn(COLOR_COLUMN);
         DefaultTableCellRenderer dtcr = new DefaultTableCellRenderer();
 
         dtcr.setOpaque(false);
@@ -146,7 +199,7 @@ public final class StartGamePanel extends FreeColPanel implements ActionListener
         tableScroll.getViewport().setOpaque(false);
         tableScroll.getColumnHeader().setOpaque(false);
 
-        int[] widths = { 300, margin, 300 };
+        int[] widths = { 400, margin, 300 };
         int[] heights = { 200, margin, 0, margin, 0, margin, 0, margin, 0 };
         setLayout(new HIGLayout(widths, heights));
 
@@ -205,26 +258,68 @@ public final class StartGamePanel extends FreeColPanel implements ActionListener
      * 
      * @param singlePlayerGame <code>true</code> if the user wants to start a
      *            single player game, <code>false</code> otherwise.
+     * @param additionalNations whether to allow additional nations
+     * @param advantages the type of advantages used
      */
-    public void initialize(boolean singlePlayerGame) {
+    public void initialize(boolean singlePlayerGame, boolean additionalNations, int advantages) {
         this.singlePlayerGame = singlePlayerGame;
         game = freeColClient.getGame();
         thisPlayer = freeColClient.getMyPlayer();
 
-        tableModel.setData(game.getPlayers(), thisPlayer);
+        this.advantages = advantages;
 
-        TableColumn nationsColumn = table.getColumnModel().getColumn(1);
-        nationsColumn.setCellEditor(new NationCellEditor());
-        nationsColumn.setCellRenderer(new NationCellRenderer());
-        ((NationCellRenderer) table.getColumnModel().getColumn(1).getCellRenderer()).setData(game.getPlayers(),
-                thisPlayer);
+        Nation[] nations;
+        if (additionalNations) {
+            nations = FreeCol.getSpecification().getEuropeanNations().toArray(new Nation[] {});
+        } else {
+            nations = FreeCol.getSpecification().getClassicNations().toArray(new Nation[] {});
+        }
+
+        tableModel.setData(game.getPlayers(), thisPlayer, advantages);
+
+        JLabel playerLabel = new JLabel(Messages.message("player"));
+        JButton nationButton = new JButton(Messages.message("nation"));
+        JButton advantageButton = new JButton(Messages.message("advantage"));
+        JLabel colorLabel = new JLabel(Messages.message("color"));
+
+        nationButton.addActionListener(new ActionListener() {
+                public void actionPerformed(ActionEvent event) {
+                    parent.showColopediaPanel(PanelType.NATIONS);
+                }
+            });
+
+        advantageButton.addActionListener(new ActionListener() {
+                public void actionPerformed(ActionEvent event) {
+                    parent.showColopediaPanel(PanelType.NATION_TYPES);
+                }
+            });
+
+        HeaderRenderer renderer = new HeaderRenderer(playerLabel, nationButton, advantageButton, colorLabel);
+        JTableHeader header = table.getTableHeader();
+        header.addMouseListener(new HeaderListener(header, renderer));
+
+        TableColumn nationsColumn = table.getColumnModel().getColumn(NATION_COLUMN);
+        nationsColumn.setCellEditor(new DefaultCellEditor(new JComboBox(nations)));
+        nationsColumn.setCellRenderer(new NationCellRenderer(nations));
+        ((NationCellRenderer) table.getColumnModel().getColumn(NATION_COLUMN).getCellRenderer())
+        .setData(game.getPlayers(), thisPlayer);
+        nationsColumn.setHeaderRenderer(renderer);
+
+        TableColumn advantagesColumn = table.getColumnModel().getColumn(ADVANTAGE_COLUMN);
+        if (advantages == AdvantageCellRenderer.SELECTABLE) {
+            advantagesColumn.setCellEditor(new AdvantageCellEditor());
+        }
+        advantagesColumn.setCellRenderer(new AdvantageCellRenderer());
+        ((AdvantageCellRenderer) table.getColumnModel().getColumn(ADVANTAGE_COLUMN).getCellRenderer())
+        .setData(game.getPlayers(), thisPlayer, advantages);
+        advantagesColumn.setHeaderRenderer(renderer);
         
-        ((ColorCellEditor) table.getColumnModel().getColumn(2).getCellEditor()).setData(game.getPlayers(), thisPlayer);
+        ((ColorCellEditor) table.getColumnModel().getColumn(COLOR_COLUMN).getCellEditor())
+        .setData(game.getPlayers(), thisPlayer);
         
         if (singlePlayerGame) {
-            // If we set the ready flag to false then the player will be able to
-            // change the
-            // settings as he likes.
+            // If we set the ready flag to false then the player will
+            // be able to change the settings as he likes.
             thisPlayer.setReady(false);
 
             // Pretend as if the player is ready.
@@ -305,8 +400,7 @@ public final class StartGamePanel extends FreeColPanel implements ActionListener
             case CANCEL:
                 freeColClient.getConnectController().quitGame(true);
                 parent.remove(this);
-                // parent.showMainPanel();
-                parent.showNewGamePanel();
+                parent.showPanel(new NewPanel(parent));
                 break;
             case READY:
                 freeColClient.getPreGameController().setReady(readyBox.isSelected());
@@ -367,14 +461,20 @@ class PlayersTableModel extends AbstractTableModel {
     @SuppressWarnings("unused")
     private FreeColClient freeColClient;
 
-    private Vector<Player> players;
+    private List<Player> players;
 
     private Player thisPlayer;
 
     private final PreGameController preGameController;
 
-    private static final String[] columnNames = { Messages.message("player"), Messages.message("nation"),
-            Messages.message("color"), };
+    private int advantages;
+
+    private static final String[] columnNames = {
+        Messages.message("player"),
+        Messages.message("nation"),
+        Messages.message("advantage"),
+        Messages.message("color")
+    };
 
 
     /**
@@ -386,7 +486,7 @@ class PlayersTableModel extends AbstractTableModel {
      */
     public PlayersTableModel(FreeColClient freeColClient, PreGameController pgc) {
         this.freeColClient = freeColClient;
-        players = new Vector<Player>();
+        players = new ArrayList<Player>();
         thisPlayer = null;
         preGameController = pgc;
     }
@@ -400,9 +500,10 @@ class PlayersTableModel extends AbstractTableModel {
      * @param owningPlayer The player running the client that is displaying the
      *            table.
      */
-    public void setData(Vector<Player> myPlayers, Player owningPlayer) {
+    public void setData(List<Player> myPlayers, Player owningPlayer, int advantages) {
         players = myPlayers;
         thisPlayer = owningPlayer;
+        this.advantages = advantages;
     }
 
     /**
@@ -463,12 +564,14 @@ class PlayersTableModel extends AbstractTableModel {
         if ((row < getRowCount()) && (column < getColumnCount()) && (row >= 0) && (column >= 0)) {
             Player player = getPlayer(row);
             switch (column) {
-            case 0:
+            case StartGamePanel.NAME_COLUMN:
                 return player.getName();
-            case 1:
-                return new Integer(player.getNation());
-            default:
+            case StartGamePanel.NATION_COLUMN:
+                return new Integer(player.getIndex());
+            case StartGamePanel.COLOR_COLUMN:
                 return player.getColor();
+            case StartGamePanel.ADVANTAGE_COLUMN:
+                return player.getNationType();
             }
         }
         return null;
@@ -501,13 +604,20 @@ class PlayersTableModel extends AbstractTableModel {
         if ((row < getRowCount()) && (column < getColumnCount()) && (row >= 0) && (column >= 0)) {
             // Column 0 can't be updated.
 
-            if (column == 1) {
-                int nation = ((Integer) value).intValue();
+            if (column == StartGamePanel.NATION_COLUMN) {
+                //Nation nation = FreeCol.getSpecification().getNation(((Integer) value).intValue());
+                Nation nation = (Nation) value;
                 preGameController.setNation(nation);
-                preGameController.setColor(Player.getDefaultNationColor(nation));
-                fireTableCellUpdated(row, 2);
-            } else if (column == 2) {
+                preGameController.setColor(nation.getColor());
+                fireTableCellUpdated(row, StartGamePanel.COLOR_COLUMN);
+                if (advantages == AdvantageCellRenderer.FIXED) {
+                    preGameController.setNationType(nation.getType());
+                    fireTableCellUpdated(row, StartGamePanel.ADVANTAGE_COLUMN);
+                }
+            } else if (column == StartGamePanel.COLOR_COLUMN) {
                 preGameController.setColor((Color) value);
+            } else if (column == StartGamePanel.ADVANTAGE_COLUMN) {
+                preGameController.setNationType((NationType) value);
             }
 
             fireTableCellUpdated(row, column);
