@@ -25,7 +25,7 @@ import java.awt.FlowLayout;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.awt.event.KeyEvent;
-import java.util.ArrayList;
+import java.util.List;
 import java.util.logging.Logger;
 
 import javax.swing.BoxLayout;
@@ -38,12 +38,13 @@ import javax.swing.JPanel;
 import javax.swing.KeyStroke;
 import javax.swing.SwingUtilities;
 
+import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.gui.Canvas;
-import net.sf.freecol.client.gui.ImageLibrary;
 import net.sf.freecol.client.gui.i18n.Messages;
-import net.sf.freecol.common.model.Goods;
-import net.sf.freecol.common.model.Player;
+import net.sf.freecol.common.model.AbstractGoods;
+import net.sf.freecol.common.model.GoodsType;
 import net.sf.freecol.common.model.Tile;
+import net.sf.freecol.common.model.TileType;
 
 /**
  * This panel is used to show information about a tile.
@@ -56,18 +57,17 @@ public final class TilePanel extends FreeColDialog implements ActionListener {
     private static final int COLOPEDIA = 1;
     private final Canvas canvas;
 
-    private final int[] goods;
+    private final GoodsType[] goodsTypes;
     private final int number;
     
     private final JPanel goodsPanel;
     private final JLabel tileNameLabel;
     private final JLabel ownerLabel;
+    private final JLabel regionLabel;
     private final JButton okButton;
     private final JButton colopediaButton;
-    private final JLabel[] labels;
-    private final JLabel fishLabel;
 
-    private Tile tile;
+    private TileType tileType;
 
     /**
      * The constructor that will add the items to this panel.
@@ -82,31 +82,26 @@ public final class TilePanel extends FreeColDialog implements ActionListener {
         tileNameLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
         add(tileNameLabel);
 
+        regionLabel = new JLabel("", JLabel.CENTER);
+        regionLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
+        add(regionLabel);
+
         ownerLabel = new JLabel("", JLabel.CENTER);
         ownerLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
-        add(ownerLabel);        
-        
-        ArrayList<Integer> farmedGoods = new ArrayList<Integer>();
-        for (int i = 0; i < Goods.NUMBER_OF_TYPES; i++) {
-            if (Goods.isFarmedGoods(i)) {
-                farmedGoods.add(i);
-            }
+        add(ownerLabel);
 
-        }
+        List<GoodsType> farmedGoods = FreeCol.getSpecification().getFarmedGoodsTypeList();
         number = farmedGoods.size();
 
         goodsPanel = new JPanel();
         goodsPanel.setLayout(new FlowLayout());
 
-        goods = new int[number];
-        labels = new JLabel[number];
+        goodsTypes = new GoodsType[number];
+        JLabel[] labels = new JLabel[number];
         for (int k = 0; k < number; k++) {
-            int index = farmedGoods.get(k).intValue();
-            goods[k] = index;
-            labels[k] = new JLabel(canvas.getImageProvider().getGoodsImageIcon(index));
-            //goodsPanel.add(labels[k]);
+            goodsTypes[k] = farmedGoods.get(k);
+            labels[k] = new JLabel(canvas.getGUI().getImageLibrary().getGoodsImageIcon(goodsTypes[k]));
         }
-        fishLabel = new JLabel(canvas.getImageProvider().getGoodsImageIcon(Goods.FISH));
         
         goodsPanel.setSize(goodsPanel.getPreferredSize());
         add(goodsPanel);
@@ -148,14 +143,18 @@ public final class TilePanel extends FreeColDialog implements ActionListener {
      * @param tile The Tile whose information should be displayed.
      */
     public void initialize(Tile tile) {
-        this.tile = tile;
-        tileNameLabel.setText(tile.getName() + " (" + tile.getX() + ", " +
-                              tile.getY() + ")");
-        if (tile.getNationOwner() == Player.NO_NATION) {
+        this.tileType = tile.getType();
+        String name = tile.getLabel() + " (" + tile.getX() + ", " +
+            tile.getY() + ")";
+        tileNameLabel.setText(name);
+        if (tile.getRegion() != null) {
+            regionLabel.setText(tile.getRegion().getDisplayName());
+        }
+        if (tile.getOwner() == null) {
             ownerLabel.setText("");
         } else {
-            String ownerName = Player.getNationAsString(tile.getNationOwner());
-            if (ownerName.equals("INVALID")) {
+            String ownerName = tile.getOwner().getNationAsString();
+            if (ownerName == null) {
                 ownerLabel.setText("");
             } else {
                 ownerLabel.setText(ownerName);
@@ -163,20 +162,15 @@ public final class TilePanel extends FreeColDialog implements ActionListener {
         }
         
         goodsPanel.removeAll();
-        if (tile.isLand()) {
-            for (int i = 0; i < number; i++) {
-                int potential = tile.potential(goods[i]);
-                if (i == Goods.LUMBER && potential > 0) {
-                    labels[i].setText(String.valueOf(potential) + " ( "  +
-                                      String.valueOf(tile.potential(goods[i]) * 5) + ")");
-                } else {
-                    labels[i].setText(String.valueOf(potential));
-                }
-                goodsPanel.add(labels[i]);
-            }
+        if (tileType == null) {
+            colopediaButton.setEnabled(false);
         } else {
-            fishLabel.setText(String.valueOf(tile.potential(Goods.FOOD)));
-            goodsPanel.add(fishLabel);
+            List<AbstractGoods> production = tileType.getProduction();
+            for (AbstractGoods goods : production) {
+                JLabel label = new JLabel(canvas.getGUI().getImageLibrary().getGoodsImageIcon(goods.getType()));
+                label.setText(String.valueOf(tile.potential(goods.getType(), null)));
+                goodsPanel.add(label);
+            }
         }
         setSize(getPreferredSize());
 
@@ -196,18 +190,8 @@ public final class TilePanel extends FreeColDialog implements ActionListener {
                 setResponse(new Boolean(true));
                 break;
             case COLOPEDIA:
-                int type = tile.getType();
-                if (tile.getAddition() == Tile.ADD_MOUNTAINS) {
-                    type = ImageLibrary.MOUNTAINS;
-                } else if (tile.getAddition() == Tile.ADD_HILLS) {
-                    type = ImageLibrary.HILLS;
-                }
-                int action = 2 * type;
-                if (tile.isForested()) {
-                    action++;
-                }
                 setResponse(new Boolean(true));
-                canvas.showColopediaPanel(ColopediaPanel.COLOPEDIA_TERRAIN, action);
+                canvas.showColopediaPanel(ColopediaPanel.PanelType.TERRAIN, tileType);
                 break;
             default:
                 logger.warning("Invalid Actioncommand: invalid number.");
