@@ -28,11 +28,10 @@ import javax.swing.Box;
 import javax.swing.ImageIcon;
 import javax.swing.JButton;
 import javax.swing.JLabel;
-import javax.swing.UIManager;
 
-import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.gui.Canvas;
 import net.sf.freecol.client.gui.i18n.Messages;
+import net.sf.freecol.common.resources.ResourceManager;
 import cz.autel.dmi.HIGLayout;
 
 /**
@@ -40,17 +39,12 @@ import cz.autel.dmi.HIGLayout;
  */
 public final class VictoryPanel extends FreeColPanel implements ActionListener {
 
-
-
     private static final Logger logger = Logger.getLogger(VictoryPanel.class.getName());
 
     private static final int OK = 0, CONTINUE = 1;
 
-    @SuppressWarnings("unused")
     private final Canvas parent;
 
-    private final FreeColClient freeColClient;
-
     private Box buttonsBox = Box.createHorizontalBox();
             
     private JButton ok = new JButton(Messages.message("victory.yes"));
@@ -61,11 +55,9 @@ public final class VictoryPanel extends FreeColPanel implements ActionListener {
      * The constructor that will add the items to this panel.
      * 
      * @param parent The parent of this panel.
-     * @param freeColClient The main controller object for the client
      */
-    public VictoryPanel(Canvas parent, FreeColClient freeColClient) {
+    public VictoryPanel(Canvas parent) {
         this.parent = parent;
-        this.freeColClient = freeColClient;
         
         int[] widths = { 0 };
         int[] heights = { 0, margin, 0, margin, 0 };
@@ -75,7 +67,7 @@ public final class VictoryPanel extends FreeColPanel implements ActionListener {
 
         JLabel victoryLabel = getDefaultHeader(Messages.message("victory.text"));
 
-        Image tempImage = (Image) UIManager.get("VictoryImage");
+        Image tempImage = ResourceManager.getImage("VictoryImage");
         JLabel imageLabel;
 
         if (tempImage != null) {
@@ -99,16 +91,13 @@ public final class VictoryPanel extends FreeColPanel implements ActionListener {
         continueButton.addActionListener(this);
         enterPressesWhenFocused(ok);
         enterPressesWhenFocused(continueButton);
-    }
-    
-    public void initialize() {
+
         buttonsBox.removeAll();
-        if (freeColClient.isSingleplayer()) {
+        if (parent.getClient().isSingleplayer()) {
             buttonsBox.add(continueButton);
             buttonsBox.add(Box.createGlue());
         }
         buttonsBox.add(ok);
-
         setSize(getPreferredSize());
     }
 
@@ -127,10 +116,18 @@ public final class VictoryPanel extends FreeColPanel implements ActionListener {
         try {
             switch (Integer.valueOf(command).intValue()) {
             case OK:
-                freeColClient.quit();
+            	if(parent.getClient().retire()){
+            		parent.showInformationMessage("highscores.new");
+            		parent.showReportPanel(ReportHighScoresPanel.class.getName());
+            	}
+                parent.quit();
                 break;
             case CONTINUE:
-                freeColClient.continuePlaying();
+            	if(parent.getClient().retire()){
+            		parent.showInformationMessage("highscores.new");
+            		parent.showReportPanel(ReportHighScoresPanel.class.getName());
+            	}
+                parent.getClient().continuePlaying();
                 parent.remove(this);
                 break;
             default:
