@@ -33,11 +33,13 @@ import javax.swing.JPanel;
 import javax.swing.JSpinner;
 import javax.swing.SpinnerNumberModel;
 
+import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.gui.Canvas;
 import net.sf.freecol.client.gui.i18n.Messages;
-import net.sf.freecol.common.model.Building;
 import net.sf.freecol.common.model.Colony;
+import net.sf.freecol.common.model.ExportData;
 import net.sf.freecol.common.model.Goods;
+import net.sf.freecol.common.model.GoodsType;
 import cz.autel.dmi.HIGLayout;
 
 /**
@@ -110,8 +112,10 @@ public final class WarehouseDialog extends FreeColDialog implements ActionListen
     public void initialize(Colony colony) {
 
         warehouseDialog.removeAll();
-        for (int goodsType = 0; goodsType < Goods.NUMBER_OF_TYPES; goodsType++) {
-            warehouseDialog.add(new WarehouseGoodsPanel(colony, goodsType));
+        for (GoodsType goodsType : FreeCol.getSpecification().getGoodsTypeList()) {
+            if (goodsType.isStorable()) {
+                warehouseDialog.add(new WarehouseGoodsPanel(colony, goodsType));
+            }
         }
         setSize(getPreferredSize());
 
@@ -156,7 +160,7 @@ public final class WarehouseDialog extends FreeColDialog implements ActionListen
 
         private final Colony colony;
 
-        private final int goodsType;
+        private final GoodsType goodsType;
 
         private final JCheckBox export;
 
@@ -167,14 +171,15 @@ public final class WarehouseDialog extends FreeColDialog implements ActionListen
         private final JSpinner exportLevel;
 
 
-        public WarehouseGoodsPanel(Colony colony, int goodsType) {
+        public WarehouseGoodsPanel(Colony colony, GoodsType goodsType) {
 
             this.colony = colony;
             this.goodsType = goodsType;
+            int goodsIndex = goodsType.getIndex();
 
             setLayout(new HIGLayout(widths, heights));
             setOpaque(false);
-            setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder(Goods.getName(goodsType)),
+            setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder(goodsType.getName()),
                     BorderFactory.createEmptyBorder(6, 6, 6, 6)));
 
             // goods label
@@ -184,28 +189,29 @@ public final class WarehouseDialog extends FreeColDialog implements ActionListen
             goodsLabel.setHorizontalAlignment(JLabel.LEADING);
             add(goodsLabel, higConst.rcwh(1, labelColumn, 1, 3));
 
+            ExportData exportData = colony.getExportData(goodsType);
             // export checkbox
-            export = new JCheckBox(Messages.message("warehouseDialog.export"), colony.getExports(goodsType));
+            export = new JCheckBox(Messages.message("warehouseDialog.export"), exportData.isExported());
             export.setToolTipText(Messages.message("warehouseDialog.export.shortDescription"));
-            if (colony.getBuilding(Building.CUSTOM_HOUSE).getLevel() == Building.NOT_BUILT) {
+            if (!colony.hasAbility("model.ability.export")) {
                 export.setEnabled(false);
             }
             add(export, higConst.rc(5, labelColumn));
 
             // low level settings
-            SpinnerNumberModel lowLevelModel = new SpinnerNumberModel(colony.getLowLevel()[goodsType], 0, 100, 1);
+            SpinnerNumberModel lowLevelModel = new SpinnerNumberModel(exportData.getLowLevel(), 0, 100, 1);
             lowLevel = new JSpinner(lowLevelModel);
             lowLevel.setToolTipText(Messages.message("warehouseDialog.lowLevel.shortDescription"));
             add(lowLevel, higConst.rc(1, spinnerColumn));
 
             // high level settings
-            SpinnerNumberModel highLevelModel = new SpinnerNumberModel(colony.getHighLevel()[goodsType], 0, 100, 1);
+            SpinnerNumberModel highLevelModel = new SpinnerNumberModel(exportData.getHighLevel(), 0, 100, 1);
             highLevel = new JSpinner(highLevelModel);
             highLevel.setToolTipText(Messages.message("warehouseDialog.highLevel.shortDescription"));
             add(highLevel, higConst.rc(3, spinnerColumn));
 
             // export level settings
-            SpinnerNumberModel exportLevelModel = new SpinnerNumberModel(colony.getExportLevel()[goodsType], 0, colony
+            SpinnerNumberModel exportLevelModel = new SpinnerNumberModel(exportData.getExportLevel(), 0, colony
                     .getWarehouseCapacity(), 1);
             exportLevel = new JSpinner(exportLevelModel);
             exportLevel.setToolTipText(Messages.message("warehouseDialog.exportLevel.shortDescription"));
@@ -218,15 +224,16 @@ public final class WarehouseDialog extends FreeColDialog implements ActionListen
             int lowLevelValue = ((SpinnerNumberModel) lowLevel.getModel()).getNumber().intValue();
             int highLevelValue = ((SpinnerNumberModel) highLevel.getModel()).getNumber().intValue();
             int exportLevelValue = ((SpinnerNumberModel) exportLevel.getModel()).getNumber().intValue();
-            boolean changed = (export.isSelected() != colony.getExports(goodsType))
-                                || (lowLevelValue != colony.getLowLevel()[goodsType])
-                                || (highLevelValue != colony.getHighLevel()[goodsType])
-                                || (exportLevelValue != colony.getExportLevel()[goodsType]);
+            ExportData exportData = colony.getExportData(goodsType);
+            boolean changed = (export.isSelected() != exportData.isExported())
+                || (lowLevelValue != exportData.getLowLevel())
+                || (highLevelValue != exportData.getHighLevel())
+                || (exportLevelValue != exportData.getExportLevel());
             
-            colony.setExports(goodsType, export.isSelected());
-            colony.getLowLevel()[goodsType] = lowLevelValue;
-            colony.getHighLevel()[goodsType] = highLevelValue;
-            colony.getExportLevel()[goodsType] = exportLevelValue;
+            exportData.setExported(export.isSelected());
+            exportData.setLowLevel(lowLevelValue);
+            exportData.setHighLevel(highLevelValue);
+            exportData.setExportLevel(exportLevelValue);
             if (changed) {
                 parent.getClient().getInGameController().setGoodsLevels(colony, goodsType);
             }
