@@ -23,8 +23,10 @@ package net.sf.freecol.common.model;
 
 import java.util.ArrayList;
 import java.util.Collections;
+import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Set;
 import java.util.logging.Logger;
 
 import javax.xml.stream.XMLStreamConstants;
@@ -32,6 +34,7 @@ import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
+import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.gui.i18n.Messages;
 
 import org.w3c.dom.Element;
@@ -41,7 +44,6 @@ import org.w3c.dom.Element;
 */
 public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownable {
 
-
     private static final Logger logger = Logger.getLogger(ColonyTile.class.getName());
 
     private Colony colony;
@@ -126,26 +128,13 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
     }
     
     /**
-     *  Returns a description of the tile, with the name of the tile
-     *and any improvements made to it (road/plow)
+     * Returns a description of the tile, with the name of the tile
+     * and any improvements made to it (road/plow).
+     *
      * @return The description label for this tile
      */
     public String getLabel() {
-        
-        Tile tile = getWorkTile();
-        
-        String label = tile.getName();
-        
-        if(tile.hasRiver())
-            label += "/"+ Messages.message("river");
-        
-        if(tile.hasRoad())
-            label += "/"+ Messages.message("road");
-        
-        if(tile.isPlowed())
-            label += "/"+ Messages.message("plowed");
-        
-        return label;
+        return workTile.getLabel();
     }
 
 
@@ -249,11 +238,9 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
      */
     public void relocateWorkers() {
         if (getUnit() != null) {
-            Iterator<WorkLocation> wli = getColony().getWorkLocationIterator();
-            while (wli.hasNext()) {
-                WorkLocation wl = wli.next();
+            for (WorkLocation wl : getColony().getWorkLocations()) {
                 if (wl != this && wl.canAdd(getUnit())) {
-                    getUnit().setLocation(wl);
+                    getUnit().work(wl);
                     break;
                 }
             }
@@ -267,20 +254,30 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
     * @return <code>true</code> if the <code>Unit</code> may be added and <code>false</code> otherwise.
     */
     public boolean canAdd(Locatable locatable) {
-        if (getWorkTile().getOwner() != null && getWorkTile().getOwner() != getColony()) {
+        if (workTile.getSettlement() != null) {
             return false;
-        }
+        } else if (locatable instanceof Unit) {
+            Unit unit = (Unit) locatable;
+            Settlement settlement = getWorkTile().getOwningSettlement();
+            if (settlement != null) {
+                if (settlement instanceof Colony && settlement != getColony()) {
+                    return false;
+                } else if (settlement instanceof IndianSettlement &&
+                           unit.getOwner().getLandPrice(getWorkTile()) > 0) {
+                    return false;
+                }
+            }
+            if (!(workTile.isLand() || getColony().hasAbility("model.ability.produceInWater"))) {
+                return false;
+            }
 
-        if (!(workTile.isLand() || getColony().getBuilding(Building.DOCK).isBuilt())) {
+            if (!unit.getType().hasSkill()) {
+                return false;
+            }
+            return (getUnit() == null || unit == getUnit());
+        } else {
             return false;
         }
-        
-        if (!((Unit) locatable).isColonist() && ((Unit) locatable).getType() != Unit.INDIAN_CONVERT) {
-            return false;
-        }
-
-        return ( ! isColonyCenterTile()  &&  locatable instanceof Unit
-                &&  ( getUnit() == null || locatable == getUnit()) );
     }
 
 
@@ -294,7 +291,7 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
         }
 
         if (!canAdd(locatable)) {
-            if (getWorkTile().getOwner() != null && getWorkTile().getOwner() != getColony()) {
+            if (getWorkTile().getOwningSettlement() != null && getWorkTile().getOwningSettlement() != getColony()) {
                 throw new IllegalArgumentException("Cannot add locatable to this location: somebody else owns this land!");
             }
             throw new IllegalArgumentException("Cannot add locatable to this location: there is a unit here already!");
@@ -302,40 +299,16 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
 
         Unit u = (Unit) locatable;
 
-        getWorkTile().takeOwnership(u.getOwner());
+        getWorkTile().takeOwnership(u.getOwner(), getColony());
 
-        if (u != null) {
-            if (u.isArmed()) {
-                u.setArmed(false);
-            }
-
-            if (u.isMounted()) {
-                u.setMounted(false);
-            }
-
-            if (u.isMissionary()) {
-                u.setMissionary(false);
-            }
-
-            if (u.getNumberOfTools() > 0) {
-                u.setNumberOfTools(0);
-            }
-
-            Unit student = u.getStudent();
-            if (student != null) {
-                student.setTeacher(null);
-                u.setStudent(null);
-            }
-        }
+        u.removeAllEquipment(false);
 
         setUnit(u);
         
-        getColony().updatePopulation();
-
         if (unit != null) {
-            getWorkTile().setOwner(getColony());
+            getWorkTile().setOwningSettlement(getColony());
         } else {
-            getWorkTile().setOwner(null);
+            getWorkTile().setOwningSettlement(null);
         }
     }
     
@@ -354,9 +327,8 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
         }
 
         getUnit().setMovesLeft(0);
-        getWorkTile().setOwner(null);
+        getWorkTile().setOwningSettlement(null);
         setUnit(null);
-        getColony().updatePopulation();
     }
 
     public List<Unit> getUnitList() {
@@ -430,16 +402,13 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
             produceGoodsCenterTile();
         } else if (getUnit() != null && !isOccupied()) {
             produceGoods();
+            workTile.expendResource(getUnit().getWorkType(), getUnit().getType(), colony);
         }
     }
 
     private void produceGoods() {
-        int amount = getUnit().getFarmedPotential(getUnit().getWorkType(), workTile);
+        int amount = getProductionOf(getUnit().getWorkType());
 
-        if (!workTile.isLand() && !colony.getBuilding(Building.DOCK).isBuilt()) {
-            amount = 0;
-        }
-        
         if (amount > 0) {
             colony.addGoods(getUnit().getWorkType(), amount);
             unit.modifyExperience(amount);
@@ -448,14 +417,18 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
 
     private void produceGoodsCenterTile() {
         
-        colony.addGoods(Goods.FOOD, getProductionOf(Goods.FOOD));
-        int type2 = workTile.secondaryGoods();
-        colony.addGoods(type2, getProductionOf(type2));
-            
+        GoodsType goodsFood = workTile.primaryGoods();
+        colony.addGoods(goodsFood, getProductionOf(goodsFood));
+        
+        GoodsType type2 = workTile.secondaryGoods();
+        if (type2 != null)
+                colony.addGoods(type2, getProductionOf(type2));
+
+        // TODO: why is this here?
         if (unit != null) {
-            getWorkTile().setOwner(getColony());
+            getWorkTile().setOwningSettlement(getColony());
         } else {
-            getWorkTile().setOwner(null);
+            getWorkTile().setOwningSettlement(null);
         }
     }
    
@@ -465,17 +438,16 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
      * @param unit a <code>Unit</code> value
      * @return a workType
      */
-    public int getWorkType(Unit unit) {
-        int workType = unit.getWorkType();
-        int amount = unit.getFarmedPotential(workType, workTile);
+    public GoodsType getWorkType(Unit unit) {
+        GoodsType workType = unit.getWorkType();
+        int amount = getProductionOf(unit, workType);
         if (amount == 0) {
-            for (int goods = 0; goods < Goods.NUMBER_OF_TYPES; goods++) {
-                if (Goods.isFarmedGoods(goods)) {
-                    int newAmount = unit.getFarmedPotential(goods, workTile);
-                    if (newAmount > amount) {
-                        amount = newAmount;
-                        workType = goods;
-                    }
+            List<GoodsType> farmedGoodsTypes = FreeCol.getSpecification().getFarmedGoodsTypeList();
+            for(GoodsType farmedGoods : farmedGoodsTypes) {
+                int newAmount = getProductionOf(unit, farmedGoods);
+                if (newAmount > amount) {
+                    amount = newAmount;
+                    workType = farmedGoods;
                 }
             }
         }
@@ -483,83 +455,82 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
     }
     
     /**
-    * Returns the production of the given type of goods.
-    */
-    public int getProductionOf(int goodsType) {
-        if ((getUnit() == null) && !(isColonyCenterTile())) {
-            return 0; // Produce nothing if there's nobody to work the terrain.
-        }
-
-        if (!(isColonyCenterTile())) {
-            if (getUnit().getWorkType() != goodsType) {
+     * Returns the production of the given type of goods.
+     *
+     * @param goodsType a <code>GoodsType</code> value
+     * @return an <code>int</code> value
+     */
+    public int getProductionOf(GoodsType goodsType) {
+        if (goodsType == null) {
+            throw new IllegalArgumentException("GoodsType must not be 'null'.");
+        } else if (getUnit() == null) {
+            if (isColonyCenterTile() &&
+                (goodsType.isFoodType() || 
+                 goodsType.equals(workTile.secondaryGoods()))) {
+                return workTile.potential(goodsType, null);
+            } else {
                 return 0;
             }
-
-            int amount = getUnit().getFarmedPotential(getUnit().getWorkType(), workTile);
-
-            if (!workTile.isLand() && !colony.getBuilding(Building.DOCK).isBuilt()) {
-                amount = 0;
-            }
-
-            /*if (amount > 0) {
-                amount += colony.getProductionBonus();
-            }*/
-            
-            return Math.max(0, amount);
+        } else if (goodsType.equals(getUnit().getWorkType())) {
+            return getProductionOf(getUnit(), goodsType);
+        } else {
+            return 0;
         }
-
-        int production = 0;
-        if (goodsType == Goods.FOOD) {
-            production = workTile.potential(Goods.FOOD);
-        } else if (goodsType == workTile.secondaryGoods()) {
-            production = workTile.potential(workTile.secondaryGoods());
-        }
-        if (production > 0) {
-            production = Math.max(1, production + colony.getProductionBonus());
-        }
-        return production;
     }
-    
-    
+
     /**
-    * Returns the unit type producing the greatest amount of the
-    * given goods at this tile.
-    *
-    * @param goodsType The type of goods.
-    * @return The {@link Unit#getType unit type}.
-    * @see Unit#getExpertWorkType
-    * @see Building#getExpertUnitType
-    */
-    public int getExpertForProducing(int goodsType) {
-        switch (goodsType) {
-            case Goods.FOOD:
-                return ( getWorkTile().isLand() )
-                    ? Unit.EXPERT_FARMER
-                    : Unit.EXPERT_FISHERMAN;
-            case Goods.FURS:
-                return Unit.EXPERT_FUR_TRAPPER;
-            case Goods.SILVER:
-                return Unit.EXPERT_SILVER_MINER;
-            case Goods.LUMBER:
-                return Unit.EXPERT_LUMBER_JACK;
-            case Goods.ORE:
-                return Unit.EXPERT_ORE_MINER;
-            case Goods.SUGAR:
-                return Unit.MASTER_SUGAR_PLANTER;
-            case Goods.COTTON:
-                return Unit.MASTER_COTTON_PLANTER;
-            case Goods.TOBACCO:
-                return Unit.MASTER_TOBACCO_PLANTER;
-            default:
-                logger.warning("Unknown type of goods.");
-                return Unit.FREE_COLONIST;
+     * Returns the production of the given type of goods.
+     *
+     * @param goodsType a <code>GoodsType</code> value
+     * @param unitType a <code>unitType</code> value
+     * @return an <code>int</code> value
+     */
+    public Set<Modifier> getProductionModifiers(GoodsType goodsType, UnitType unitType) {
+        if (goodsType == null) {
+            throw new IllegalArgumentException("GoodsType must not be 'null'.");
+        } else {
+            Set<Modifier> result = new HashSet<Modifier>();
+            if (getUnit() == null) {
+                if (isColonyCenterTile() &&
+                    (goodsType.isFoodType() || 
+                     goodsType.equals(workTile.secondaryGoods()))) {
+                    result.addAll(workTile.getProductionBonus(goodsType, null));
+                    result.addAll(getColony().getFeatureContainer().getModifierSet(goodsType.getId()));
+                }
+            } else if (goodsType.equals(getUnit().getWorkType())) {
+                result.addAll(workTile.getProductionBonus(goodsType, unitType));
+                result.addAll(getUnit().getModifierSet(goodsType.getId()));
             }
+            return result;
+        }
+    }
+
+    /**
+     * Returns the production of the given type of goods which would
+     * be produced by the given unit
+     *
+     * @param unit an <code>Unit</code> value
+     * @param goodsType a <code>GoodsType</code> value
+     * @return an <code>int</code> value
+     */
+    public int getProductionOf(Unit unit, GoodsType goodsType) {
+        if (unit == null) {
+            throw new IllegalArgumentException("Unit must not be 'null'.");
+        } else if (workTile.isLand() || colony.hasAbility("model.ability.produceInWater")) {
+            int production = unit.getProductionOf(goodsType, workTile.potential(goodsType, unit.getType()));
+            if (production > 0) {
+                production = Math.max(1, production + colony.getProductionBonus());
+            }
+            return production;
+        } else {
+            return 0;
+        }
     }
 
 
     public void dispose() {
         if (unit != null) {
-            getWorkTile().setOwner(null);
+            getWorkTile().setOwningSettlement(null);
             unit.dispose();
         }
 
@@ -597,9 +568,7 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
         out.writeAttribute("colony", colony.getId());
         out.writeAttribute("workTile", workTile.getId());
 
-        if (unit != null) {
-            unit.toXML(out, player, showAll, toSavedGame);
-        }
+        writeFreeColGameObject(unit, out, player, showAll, toSavedGame);
 
         // End element:
         out.writeEndElement();
@@ -614,33 +583,17 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
     protected void readFromXMLImpl(XMLStreamReader in) throws XMLStreamException {
         setId(in.getAttributeValue(null, "ID"));
 
-        colony = (Colony) getGame().getFreeColGameObject(in.getAttributeValue(null, "colony"));
-        if (colony == null) {
-            colony = new Colony(getGame(), in.getAttributeValue(null, "colony"));
-        }
-        workTile = (Tile) getGame().getFreeColGameObject(in.getAttributeValue(null, "workTile"));
-        if (workTile == null) {
-            workTile = new Tile(getGame(), in.getAttributeValue(null, "workTile"));
-        }
+        colony = getFreeColGameObject(in, "colony", Colony.class);
+        workTile = getFreeColGameObject(in, "workTile", Tile.class);
+        colonyCenterTile = (colony.getTile() == workTile);
         
         unit = null;
         while (in.nextTag() != XMLStreamConstants.END_ELEMENT) {
             if (in.getLocalName().equals(Unit.getXMLElementTagName())) {
-                Unit unit = (Unit) getGame().getFreeColGameObject(in.getAttributeValue(null, "ID"));
-                if (unit != null) {
-                    unit.readFromXML(in);
-                    this.unit = unit;
-                } else {
-                    this.unit = new Unit(getGame(), in);
-                } 
+                unit = updateFreeColGameObject(in, Unit.class);
             }
         }
 
-        if (colony.getTile() == workTile) {
-            colonyCenterTile = true;
-        } else {
-            colonyCenterTile = false;
-        }
     }
 
     /**
@@ -650,7 +603,7 @@ public class ColonyTile extends FreeColGameObject implements WorkLocation, Ownab
      * @return A representation of a colony-tile that can be used for debugging.
      */
     public String toString() {
-        return getTile().getPosition().toString() + " in '" + getColony().getName() + "'" + super.toString();
+        return getWorkTile().getPosition().toString() + " in '" + getColony().getName() + "'" + super.toString();
     }
 
     /**
