@@ -30,12 +30,14 @@ import javax.xml.stream.XMLStreamWriter;
 
 import org.w3c.dom.Element;
 
+import net.sf.freecol.common.model.Player.Stance;
+
 /**
  * The class <code>DiplomaticTrade</code> represents an offer one player can
  * make another.
  * 
  */
-public class DiplomaticTrade extends PersistentObject {
+public class DiplomaticTrade extends FreeColObject {
 
 
 
@@ -44,7 +46,6 @@ public class DiplomaticTrade extends PersistentObject {
      */
     private List<TradeItem> items;
 
-    @SuppressWarnings("unused")
     private final Game game;
 
     /**
@@ -212,12 +213,12 @@ public class DiplomaticTrade extends PersistentObject {
 
 
     /**
-     * Returns the stance being offered, or Integer.MIN_VALUE if none
+     * Returns the stance being offered, or null if none
      * is being offered.
      *
      * @return an <code>int</code> value
      */
-    public int getStance() {
+    public Stance getStance() {
         Iterator<TradeItem> itemIterator = items.iterator();
         while (itemIterator.hasNext()) {
             TradeItem item = itemIterator.next();
@@ -225,10 +226,44 @@ public class DiplomaticTrade extends PersistentObject {
                 return ((StanceTradeItem) item).getStance();
             }
         }
-        return Integer.MIN_VALUE;
+        return null;
     }
-
-
+    
+    /**
+     * Returns a list of goods given by <code>Player</code>
+     *
+     * @return a list of <code>Goods</code> offered by the player, empty if none given
+     */
+    public List<Goods> getGoodsGivenBy(Player player){
+    	List<Goods> goodsList = new ArrayList<Goods>();
+    	Iterator<TradeItem> itemIterator = items.iterator();
+        while (itemIterator.hasNext()) {
+            TradeItem item = itemIterator.next();
+            if (item instanceof GoodsTradeItem && player == item.getSource()) {
+            	goodsList.add(((GoodsTradeItem) item).getGoods());
+            }
+        }
+        return goodsList;
+    }
+    
+    /**
+     * Returns a list of colonies given by <code>Player</code>
+     *
+     * @return a list of <code>Colony</code> offered by the player, empty if none given
+     */
+    public List<Colony> getColoniesGivenBy(Player player){
+    	List<Colony> colonyList = new ArrayList<Colony>();
+    	Iterator<TradeItem> itemIterator = items.iterator();
+        while (itemIterator.hasNext()) {
+            TradeItem item = itemIterator.next();
+            if (item instanceof ColonyTradeItem && player == item.getSource()) {
+            	colonyList.add(((ColonyTradeItem) item).getColony());
+            }
+        }
+        return colonyList;
+    }
+    
+    
     /**
      * Calls the <code>makeTrade</code> method of all TradeItems.
      *
@@ -306,29 +341,18 @@ public class DiplomaticTrade extends PersistentObject {
      * This method writes an XML-representation of this object to the given
      * stream.
      * 
-     * <br>
-     * <br>
-     * 
-     * Only attributes visible to the given <code>Player</code> will be added
-     * to that representation if <code>showAll</code> is set to
-     * <code>false</code>.
-     * 
-     * @param out
+      * @param out
      *            The target stream.
-     * @param player
-     *            The <code>Player</code> this XML-representation should be
-     *            made for, or <code>null</code> if
-     *            <code>showAll == true</code>.
-     * @throws XMLStreamException
+      * @throws XMLStreamException
      *             if there are any problems writing to the stream.
      */
-    public void toXML(XMLStreamWriter out, Player player) throws XMLStreamException {
+    public void toXMLImpl(XMLStreamWriter out) throws XMLStreamException {
         out.writeStartElement(getXMLElementTagName());
         out.writeAttribute("accept", accept ? "accept" : "");
         out.writeAttribute("sender", sender.getId());
         out.writeAttribute("recipient", recipient.getId());
         for (TradeItem item : items) {
-            item.toXML(out, player);
+            item.toXML(out);
         }
         out.writeEndElement();
     }
