@@ -25,12 +25,8 @@ import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 
 import net.sf.freecol.FreeCol;
-import net.sf.freecol.common.option.BooleanOption;
-import net.sf.freecol.common.option.IntegerOption;
-import net.sf.freecol.common.option.OptionGroup;
+import net.sf.freecol.common.Specification;
 import net.sf.freecol.common.option.OptionMap;
-import net.sf.freecol.common.option.RangeOption;
-
 import org.w3c.dom.Element;
 
 
@@ -44,48 +40,52 @@ public class GameOptions extends OptionMap {
 
 
     /** The amount of money each player will receive before the game starts. */
-    public static final String STARTING_MONEY = "startingMoney";
+    public static final String STARTING_MONEY = "model.option.startingMoney";
 
     /** The cost of a single hammer when buying a building in a colony. */
-    public static final String HAMMER_PRICE = "hammerPrice";
+    //Unused at the moment
+    // public static final String HAMMER_PRICE = "hammerPrice";
 
     /** Does the Custom House sell boycotted goods **/
-    public static final String CUSTOM_IGNORE_BOYCOTT = "customIgnoreBoycott";
+    public static final String CUSTOM_IGNORE_BOYCOTT = "model.option.customIgnoreBoycott";
 
     /** Whether experts have connections, producing without raw materials in factories */
-    public static final String EXPERTS_HAVE_CONNECTIONS = "expertsHaveConnections";
+    public static final String EXPERTS_HAVE_CONNECTIONS = "model.option.expertsHaveConnections";
 
-    public static final String SAVE_PRODUCTION_OVERFLOW = "saveProductionOverflow";
+    public static final String SAVE_PRODUCTION_OVERFLOW = "model.option.saveProductionOverflow";
+
+    /** Whether to award exploration points or not. */
+    public static final String EXPLORATION_POINTS = "model.option.explorationPoints";
 
     /** Enables/disables fog of war. */
-    public static final String FOG_OF_WAR = "fogOfWar";
+    public static final String FOG_OF_WAR = "model.option.fogOfWar";
 
     /** No units are hidden on carriers or settlements if this option is set to <code>false</code>. */
-    public static final String UNIT_HIDING = "unitHiding";
+    public static final String UNIT_HIDING = "model.option.unitHiding";
     
     /** 
      * Victory condition: Should the <code>Player</code> who first defeats the
      * Royal Expeditionary Force win the game?
      */
-    public static final String VICTORY_DEFEAT_REF = "victoryDefeatREF";
+    public static final String VICTORY_DEFEAT_REF = "model.option.victoryDefeatREF";
     
     /** 
      * Victory condition: Should a <code>Player</code> who first defeats all
      * other european players win the game?
      */
-    public static final String VICTORY_DEFEAT_EUROPEANS = "victoryDefeatEuropeans";    
+    public static final String VICTORY_DEFEAT_EUROPEANS = "model.option.victoryDefeatEuropeans";    
 
     /** 
      * Victory condition: Should a <code>Player</code> who first defeats all
      * other human players win the game?
      */
-    public static final String VICTORY_DEFEAT_HUMANS = "victoryDefeatHumans";
+    public static final String VICTORY_DEFEAT_HUMANS = "model.option.victoryDefeatHumans";
 
 
     /**
      * The difficulty of the game.
      */
-    public static final String DIFFICULTY = "difficulty";    
+    public static final String DIFFICULTY = "model.option.difficulty";
     
     /**
     * Creates a new <code>GameOptions</code>.
@@ -126,49 +126,22 @@ public class GameOptions extends OptionMap {
     * Adds the options to this <code>GameOptions</code>.
     */
     protected void addDefaultOptions() {
-        /* Add options here: */
-
-        /* Initial values: */
-        OptionGroup starting = new OptionGroup("gameOptions.starting");
+        Specification spec = Specification.getSpecification();
+        
         if (FreeCol.isInDebugMode()) {
-            new IntegerOption(STARTING_MONEY, starting, 0, 50000, 10000);
-        } else {
-            new IntegerOption(STARTING_MONEY, starting, 0, 50000, 0);
+            spec.getIntegerOption(STARTING_MONEY).setValue(10000);
         }
-        add(starting);
-
+        
+        /* Starting options: */
+        add(spec.getOptionGroup("gameOptions.starting"));
         /* Map options: */
-        OptionGroup map = new OptionGroup("gameOptions.map");
-        new BooleanOption(FOG_OF_WAR, map, true);
-        new BooleanOption(UNIT_HIDING, map, true);
-        add(map);        
-
+        add(spec.getOptionGroup("gameOptions.map"));
         /* Colony options: */
-        OptionGroup colony = new OptionGroup("gameOptions.colony");
-        new IntegerOption(HAMMER_PRICE, colony, 0, 50, 20);
-        new BooleanOption(CUSTOM_IGNORE_BOYCOTT, colony, false);
-        new BooleanOption(EXPERTS_HAVE_CONNECTIONS, colony, false);
-        new BooleanOption(SAVE_PRODUCTION_OVERFLOW, colony, false);
-
-        add(colony);
-
+        add(spec.getOptionGroup("gameOptions.colony"));
         /* Victory Conditions */
-        OptionGroup victoryConditions = new OptionGroup("gameOptions.victoryConditions");
-        new BooleanOption(VICTORY_DEFEAT_REF, victoryConditions, true);
-        new BooleanOption(VICTORY_DEFEAT_EUROPEANS, victoryConditions, true);
-        new BooleanOption(VICTORY_DEFEAT_HUMANS, victoryConditions, false);
-        add(victoryConditions);
-
+        add(spec.getOptionGroup("gameOptions.victoryConditions"));
         /* Difficulty settings */
-        OptionGroup difficultySettings = new OptionGroup("gameOptions.difficultySettings");
-        new RangeOption(DIFFICULTY, difficultySettings,
-                         new String[] {"veryEasy", 
-                                       "easy", 
-                                       "normal", 
-                                       "hard", 
-                                       "veryHard"}, 
-                         2);
-        add(difficultySettings);
+        add(spec.getOptionGroup("gameOptions.difficultySettings"));
     }
 
     protected boolean isCorrectTagName(String tagName) {
