@@ -17,82 +17,50 @@
  *  along with FreeCol.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-
 package net.sf.freecol.common.model;
 
-
-import java.io.IOException;
-import java.io.PrintWriter;
-import java.io.StringReader;
-import java.io.StringWriter;
+import java.util.List;
 import java.util.logging.Logger;
 
-import javax.xml.parsers.DocumentBuilder;
-import javax.xml.parsers.DocumentBuilderFactory;
-import javax.xml.parsers.ParserConfigurationException;
-import javax.xml.stream.XMLInputFactory;
-import javax.xml.stream.XMLOutputFactory;
 import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
-import javax.xml.transform.Transformer;
-import javax.xml.transform.TransformerException;
-import javax.xml.transform.TransformerFactory;
-import javax.xml.transform.dom.DOMSource;
-import javax.xml.transform.stream.StreamResult;
 
 import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.gui.i18n.Messages;
 
-import org.w3c.dom.Document;
 import org.w3c.dom.Element;
-import org.xml.sax.InputSource;
-import org.xml.sax.SAXException;
-
 
 /**
-* Represents a locatable goods of a specified type and amount.
-*/
-public class Goods implements Locatable, Ownable, Nameable {
-
+ * Represents locatable goods of a specified type and amount. Use
+ * AbstractGoods to represent abstract or potential goods that need
+ * not be present in any particular location.
+ *
+ * @see AbstractGoods
+ */
+public class Goods extends AbstractGoods implements Locatable, Ownable, Named {
 
     private static Logger logger = Logger.getLogger(Goods.class.getName());
 
-    public static final int FOOD = 0,
-                            SUGAR = 1,
-                            TOBACCO = 2,
-                            COTTON = 3,
-                            FURS = 4,
-                            LUMBER = 5,
-                            ORE = 6,
-                            SILVER = 7,
-                            HORSES = 8,
-                            RUM = 9,
-                            CIGARS = 10,
-                            CLOTH = 11,
-                            COATS = 12,
-                            TRADE_GOODS = 13,
-                            TOOLS = 14,
-                            MUSKETS = 15;
-
-    public static final int NUMBER_OF_TYPES = 16;
-
-    // Unstorable goods:
-    public static final int FISH = 16, // Stored as food.
-                            BELLS = 17,
-                            CROSSES = 18,
-                            HAMMERS = 19;
-
-    public static final int NUMBER_OF_ALL_TYPES = 20;
+    /**
+     * Need to change various classes to use the specification in
+     * order to remove reliance on these static quick links.  Only the
+     * essential should have a quick link.
+     */
+    public static GoodsType FOOD, LUMBER, ORE, SILVER, HORSES,
+        RUM, CIGARS, CLOTH, COATS, TRADEGOODS, TOOLS, MUSKETS, 
+        FISH, BELLS, CROSSES, HAMMERS,
+    // these goods are no longer used; define them only to avoid warnings
+        SUGAR, TOBACCO, FURS, COTTON;
 
     private Game game;
     private Location location;
-    private int type;
-    private int amount;
+
+    // ------------------------------------------------------------ constructor
 
     /**
      * Creates a standard <code>Goods</code>-instance given the place where
-     * the goods .
+     * the goods is.
      * 
      * This constructor only asserts that the game and 
      * that the location (if given) can store goods. The goods will not
@@ -105,45 +73,72 @@ public class Goods implements Locatable, Ownable, Nameable {
      * 
      * @throws IllegalArgumentException if the location cannot store any goods.
      */
-    public Goods(Game game, Location location, int type, int amount) {
+    public Goods(Game game, Location location, GoodsType type, int amount) {
         if (game == null) {
-            throw new NullPointerException();
+            throw new IllegalArgumentException("Parameter 'game' must not be 'null'.");
         }
         
+        if (type == null) {
+            throw new IllegalArgumentException("Parameter 'type' must not be 'null'.");
+        }
+
         if (location != null && location.getGoodsContainer() == null){
             throw new IllegalArgumentException("This location cannot store goods");
         }
 
         this.game = game;
         this.location = location;
-        this.type = type;
-        this.amount = amount;
+        setType(type);
+        setAmount(amount);
     }
 
-
+    /**
+     * Creates a new <code>Goods</code> instance.
+     *
+     * @param game a <code>Game</code> value
+     * @param in a <code>XMLStreamReader</code> value
+     * @exception XMLStreamException if an error occurs
+     */
     public Goods(Game game, XMLStreamReader in) throws XMLStreamException {
         this.game = game;
         readFromXML(in);
+
     }
     
+    /**
+     * Creates a new <code>Goods</code> instance.
+     *
+     * @param game a <code>Game</code> value
+     * @param e an <code>Element</code> value
+     */
     public Goods(Game game, Element e) {
         this.game = game;
         readFromXMLElement(e);
     }
 
+
+    // ------------------------------------------------------------ static methods
+
     /**
-     * This constructor should be used to get an object that represents a goods
-     * type (and not an actual amoutn of goods of that type).
-     * 
-     * @param type
+     * Initializes the important Types for quick reference - performed by Specification.java
+     * Should be softcoded as much as possible, and this should be amended later
+     * @param numberOfTypes Initializer for NUMBER_OF_TYPES
      */
-    public Goods(int type) {
-        this.game = null;
-        this.location = null;
-        this.type = type;
-        this.amount = 0;
+    public static void initialize(List<GoodsType> goodsList, int numberOfTypes) {
+        for (GoodsType g : goodsList) {
+            try {
+                String fieldName = g.getId().substring(g.getId().lastIndexOf('.') + 1).toUpperCase();
+                Goods.class.getDeclaredField(fieldName).set(null, g);
+            } catch (Exception e) {
+                logger.warning("Error assigning a GoodsType to Goods." +
+                        g.getId().toUpperCase() + "\n" + e.toString());
+            }
+        }
+        //NUMBER_OF_TYPES = numberOfTypes;
     }
 
+    // ------------------------------------------------------------ retrieval methods
+
     /**
     * Gets the owner of this <code>Ownable</code>.
     *
@@ -174,7 +169,9 @@ public class Goods implements Locatable, Ownable, Nameable {
     *         <br>15 Cotton
     */
     public String toString() {
-        return Integer.toString(amount) + " " + getName();
+        return Messages.message("model.goods.goodsAmount",
+                                "%goods%", getName(),
+                                "%amount%", String.valueOf(getAmount()));
     }
 
     /**
@@ -183,16 +180,7 @@ public class Goods implements Locatable, Ownable, Nameable {
      * @return The name of this type of goods.
      */
     public String getName() {
-        return getName(type);
-    }
-
-    /**
-     * This method does not do anything.
-     *
-     * @param newName The new Name value.
-     */
-    public void setName(String newName) {
-        // this.name = newName;
+        return getType().getName();
     }
 
     /**
@@ -202,9 +190,27 @@ public class Goods implements Locatable, Ownable, Nameable {
      * @return The name of this type of goods.
      */
     public String getName(boolean sellable) {
-        return getName(type, sellable);
+        return getType().getName(sellable);
     }
 
+    /**
+     * Returns a textual representation of the Good of type <code>type</code>.
+     * @param type  The type of good to return
+     * @return
+     *
+     *//*   COMEBACKHERE
+    public static String getName(GoodsType type) {
+        return type.getName();
+    }
+
+    public static String getName(GoodsType type, boolean sellable) {
+        if (sellable) {
+            return type.getName();
+        } else {
+            return type.getName() + " (" + Messages.message("model.goods.boycotted") + ")";
+        }
+    }
+*/
     /**
     * Returns the <code>Tile</code> where this <code>Goods</code> is located,
     * or <code>null</code> if it's location is <code>Europe</code>.
@@ -217,71 +223,6 @@ public class Goods implements Locatable, Ownable, Nameable {
     }
 
 
-    /**
-    * Gets the type of goods that is needed to produce the given type
-    * of goods.
-    *
-    * @param goodsType The type of manufactured goods.
-    * @return The type of raw material or <code>-1</code> if the given type
-    *         of goods does not have a raw material.
-    */
-    public static int getRawMaterial(int goodsType) {
-        GoodsType  good = FreeCol.getSpecification().goodsType( goodsType );
-        return good.isRefined() ? good.madeFrom.index : -1;        
-    }
-
-
-    /**
-    * Gets the type of goods which can be produced by the given
-    * raw material.
-    *
-    * @param rawMaterialGoodsType The type of raw material.
-    * @return The type of manufactured goods or <code>-1</code> if the given type
-    *         of goods does not have a manufactured goods.
-    */
-    public static int getManufactoredGoods(int rawMaterialGoodsType) {
-        GoodsType  good = FreeCol.getSpecification().goodsType( rawMaterialGoodsType );
-        return good.isRawMaterial() ? good.makes.index : -1;
-    }
-
-
-    /**
-    * Checks if the given type of goods can be produced on a {@link ColonyTile}.
-    * @param goodsType The type of goods to test.
-    * @return The result.
-    */
-    public static boolean isFarmedGoods(int goodsType) {
-
-        return FreeCol.getSpecification().goodsType(goodsType).isFarmed;
-    }
-
-
-    /**
-     * Returns a textual representation of the Good of type <code>type</code>.
-     * @param type  The type of good to return
-     * @return
-     *
-     * TODO - needs to be completed
-     */
-    public static String getName(int type) {
-
-        if ( 0 <= type  &&  type < FreeCol.getSpecification().numberOfGoodsTypes() ) {
-            return Messages.message(FreeCol.getSpecification().goodsType(type).name);
-        }
-        return Messages.message("model.goods.Unknown");
-    }
-
-
-    public static String getName(int type, boolean sellable) {
-
-        if (sellable) {
-            return getName(type);
-        } else {
-            return getName(type) + " (" + Messages.message("model.goods.Boycotted") + ")";
-        }
-    }
-
-
     /**
     * Sets the location of the goods.
     * @param location The new location of the goods,
@@ -304,7 +245,7 @@ public class Goods implements Locatable, Ownable, Nameable {
             this.location = location;
         } catch (IllegalStateException e) {
             throw new IllegalStateException("Could not move the goods of type: "
-                    + getName(getType()) + " (" + type + ") with amount: " + getAmount() + " from "
+                    + getType().getName() + " with amount: " + getAmount() + " from "
                     + this.location + " to " + location, e);
         }
     }
@@ -323,38 +264,10 @@ public class Goods implements Locatable, Ownable, Nameable {
     * Gets the amount of space this <code>Goods</code> take.
     * @return The amount.
     */
-    public int getTakeSpace() {
+    public int getSpaceTaken() {
         return 1;
     }
 
-    /**
-    * Gets the value <code>amount</code>.
-    * @return The current value of amount.
-    */
-    public int getAmount() {
-        return amount;
-    }
-
-    /**
-    * Sets the value <code>amount</code>.
-    * 
-    * This method does not check this new amount in any way (thus the amount might be negative)
-    * 
-    * @param newAmount The new value for amount.
-    */
-    public void setAmount(int newAmount) {
-        amount = newAmount;
-    }
-
-    /**
-    * Gets the value <code>type</code>. Note that type of goods should NEVER change.
-    * @return The current value of type.
-    */
-    public int getType() {
-        return type;
-    }
-
-
     /**
     * If the amount of goods is greater than the container can hold,
     * then this method adjusts the amount to the maximum amount possible.
@@ -430,40 +343,20 @@ public class Goods implements Locatable, Ownable, Nameable {
         return game;
     }
 
-    /**
-     * Initializes this object from an XML-representation of this object.
-     * @param in The input stream with the XML.
-     * @throws XMLStreamException if there are any problems writing
-     *      to the stream.
-     */
-    public void readFromXML(XMLStreamReader in) throws XMLStreamException {
-        readFromXMLImpl(in);
-    }
-
-    
     /**
      * This method writes an XML-representation of this object to
      * the given stream.
      * 
-     * <br><br>
-     * 
-     * Only attributes visible to the given <code>Player</code> will 
-     * be added to that representation if <code>showAll</code> is
-     * set to <code>false</code>.
-     *  
      * @param out The target stream.
-     * @param player The <code>Player</code> this XML-representation 
-     *      should be made for, or <code>null</code> if
-     *      <code>showAll == true</code>.
      * @throws XMLStreamException if there are any problems writing
      *      to the stream.
      */
-    public void toXML(XMLStreamWriter out, Player player) throws XMLStreamException {
+    public void toXMLImpl(XMLStreamWriter out) throws XMLStreamException {
         // Start element:
         out.writeStartElement(getXMLElementTagName());
 
-        out.writeAttribute("type", Integer.toString(type));
-        out.writeAttribute("amount", Integer.toString(amount));
+        out.writeAttribute("type", getType().getId());
+        out.writeAttribute("amount", Integer.toString(getAmount()));
 
         if (location != null) {
             out.writeAttribute("location", location.getId());
@@ -474,89 +367,6 @@ public class Goods implements Locatable, Ownable, Nameable {
         out.writeEndElement();
     }
     
-    /**
-     * This method writes an XML-representation of this object to
-     * the given stream.
-     * 
-     * <br><br>
-     * 
-     * Only attributes visible to the given <code>Player</code> will 
-     * be added to that representation if <code>showAll</code> is
-     * set to <code>false</code>.
-     *  
-     * @param player The <code>Player</code> this XML-representation 
-     *      should be made for, or <code>null</code> if
-     *      <code>showAll == true</code>.
-     * @param document The <code>Document</code> the <code>Element</code>
-     *      should be created within.
-     * @return An XML-representation of this object.
-     */    
-    public Element toXMLElement(Player player, Document document) {
-        try {
-            StringWriter sw = new StringWriter();
-            XMLOutputFactory xif = XMLOutputFactory.newInstance();
-            XMLStreamWriter xsw = xif.createXMLStreamWriter(sw);
-            toXML(xsw, player);
-            xsw.close();
-            
-            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
-            Document tempDocument = null;
-            try {
-                DocumentBuilder builder = factory.newDocumentBuilder();
-                tempDocument = builder.parse(new InputSource(new StringReader(sw.toString())));
-                return (Element) document.importNode(tempDocument.getDocumentElement(), true);
-            } catch (ParserConfigurationException pce) {
-                // Parser with specified options can't be built
-                StringWriter swe = new StringWriter();
-                pce.printStackTrace(new PrintWriter(swe));
-                logger.warning(swe.toString());
-                throw new IllegalStateException("ParserConfigurationException");
-            } catch (SAXException se) {
-                StringWriter swe = new StringWriter();
-                se.printStackTrace(new PrintWriter(swe));
-                logger.warning(swe.toString());
-                throw new IllegalStateException("SAXException");
-            } catch (IOException ie) {
-                StringWriter swe = new StringWriter();
-                ie.printStackTrace(new PrintWriter(swe));
-                logger.warning(swe.toString());
-                throw new IllegalStateException("IOException");
-            }                                    
-        } catch (XMLStreamException e) {
-            logger.warning(e.toString());
-            throw new IllegalStateException("XMLStreamException");
-        }
-    }    
-
-    /**
-     * Initialize this object from an XML-representation of this object.
-     * @param element An XML-element that will be used to initialize
-     *      this object.
-     */
-    public void readFromXMLElement(Element element) {
-        XMLInputFactory xif = XMLInputFactory.newInstance();        
-        try {
-            try {
-                TransformerFactory factory = TransformerFactory.newInstance();
-                Transformer xmlTransformer = factory.newTransformer();
-                StringWriter stringWriter = new StringWriter();
-                xmlTransformer.transform(new DOMSource(element), new StreamResult(stringWriter));
-                String xml = stringWriter.toString();
-                XMLStreamReader xsr = xif.createXMLStreamReader(new StringReader(xml));
-                xsr.nextTag();
-                readFromXML(xsr);
-            } catch (TransformerException e) {
-                StringWriter sw = new StringWriter();
-                e.printStackTrace(new PrintWriter(sw));
-                logger.warning(sw.toString());
-                throw new IllegalStateException("TransformerException");
-            }
-        } catch (XMLStreamException e) {
-            logger.warning(e.toString());
-            throw new IllegalStateException("XMLStreamException");
-        }
-    }
-    
     /**
      * Initialize this object from an XML-representation of this object.
      * @param in The input stream with the XML.
@@ -564,8 +374,8 @@ public class Goods implements Locatable, Ownable, Nameable {
      *      during parsing.
      */
     protected void readFromXMLImpl(XMLStreamReader in) throws XMLStreamException {        
-        type = Integer.parseInt(in.getAttributeValue(null, "type"));
-        amount = Integer.parseInt(in.getAttributeValue(null, "amount"));
+        setType(FreeCol.getSpecification().getGoodsType(in.getAttributeValue(null, "type")));
+        setAmount(Integer.parseInt(in.getAttributeValue(null, "amount")));
 
         final String locationStr = in.getAttributeValue(null, "location");
         if (locationStr != null) {
@@ -583,5 +393,4 @@ public class Goods implements Locatable, Ownable, Nameable {
     public static String getXMLElementTagName() {
         return "goods";
     }
-
 }
