@@ -156,31 +156,21 @@ public class GoodsTradeItem extends TradeItem {
                 this.goods = new Goods(getGame(), in);
             }
         }
-        in.nextTag();
     }
 
     /**
      * This method writes an XML-representation of this object to
      * the given stream.
-     * 
-     * <br><br>
-     * 
-     * Only attributes visible to the given <code>Player</code> will 
-     * be added to that representation if <code>showAll</code> is
-     * set to <code>false</code>.
      *  
      * @param out The target stream.
-     * @param player The <code>Player</code> this XML-representation 
-     *      should be made for, or <code>null</code> if
-     *      <code>showAll == true</code>.
      * @throws XMLStreamException if there are any problems writing
      *      to the stream.
      */
-    public void toXML(XMLStreamWriter out, Player player) throws XMLStreamException {
+    public void toXMLImpl(XMLStreamWriter out) throws XMLStreamException {
         out.writeStartElement(getXMLElementTagName());
-        super.toXML(out, player);
+        super.toXMLImpl(out);
         out.writeAttribute("settlement", settlement.getId());
-        this.goods.toXML(out, player);
+        this.goods.toXML(out);
         out.writeEndElement();
     }
     
