@@ -1366,9 +1366,16 @@ public class IndianSettlement extends Settlement {
                 Unit u = (Unit) getGame().getFreeColGameObject(token);
                 if (u == null) {
                     u = new Unit(getGame(), token);
-                    owner.setUnit(u);
+                    if(u.getOwner() == null || u.getOwner() == owner){
+                    	owner.setUnit(u);
+                    }
+                }
+                if(u.getOwner() != null && u.getOwner() != owner){
+                	logger.warning("Error in savegame: unit " + u.getId() + " does not belong to settlement " + getId());
+                }
+                else{
+                	ownedUnits.add(u);
                 }
-                ownedUnits.add(u);
             }
         }
         if (getAttribute(in, "hasBeenVisited", false)) {
@@ -1424,8 +1431,13 @@ public class IndianSettlement extends Settlement {
                 }
             } else if (OWNED_UNITS_TAG_NAME.equals(in.getLocalName())) {
                 Unit unit = getFreeColGameObject(in, ID_ATTRIBUTE, Unit.class);
-                ownedUnits.add(unit);
-                owner.setUnit(unit);
+                if(unit.getOwner() != null && unit.getOwner() != owner){
+                	logger.warning("Error in savegame: unit " + unit.getId() + " does not belong to settlement " + getId());
+                }
+                else{
+                	ownedUnits.add(unit);
+                	owner.setUnit(unit);
+                }
                 in.nextTag();
             } else if (in.getLocalName().equals(GoodsContainer.getXMLElementTagName())) {
                 goodsContainer = (GoodsContainer) getGame().getFreeColGameObject(in.getAttributeValue(null, ID_ATTRIBUTE));
