@@ -23,6 +23,7 @@ package net.sf.freecol.common.model;
 import java.util.ArrayList;
 
 import net.sf.freecol.common.PseudoRandom;
+import net.sf.freecol.common.model.Player.Stance;
 
 /**
 * The <code>ModelController</code> is used by the model to perform
@@ -58,7 +59,25 @@ public interface ModelController {
     * @param type   The type of unit (Unit.FREE_COLONIST...).
     * @return The created <code>Unit</code>.
     */
-    public Unit createUnit(String taskID, Location location, Player owner, int type);
+    public Unit createUnit(String taskID, Location location, Player owner, UnitType type);
+
+    /**
+    * Creates a new building.
+    *
+    * @param taskID The <code>taskID</code> should be a unique identifier.
+    *               One method to make a unique <code>taskID</code>:
+    *               <br><br>
+    *               getId() + "methodName:taskDescription"
+    *               <br><br>
+    *               As long as the "taskDescription" is unique
+    *               within the method ("methodName"), you get a unique
+    *               identifier.
+    * @param colony The <code>Colony</code> where the <code>Building</code>
+    *               will be created.
+    * @param type   The type of building.
+    * @return The created <code>Building</code>.
+    */
+    public Building createBuilding(String taskID, Colony colony, BuildingType type);
 
     /**
     * Puts the specified <code>Unit</code> in America.
@@ -88,7 +107,7 @@ public interface ModelController {
      * @param second The second <code>Player</code>.
      * @param stance The new stance.
      */
-    public void setStance(Player first, Player second, int stance);
+    public void setStance(Player first, Player second, Stance stance);
     
     
     /**
