@@ -175,7 +175,8 @@ public class SimpleCombatModel implements CombatModel {
         if (result.compareTo(CombatResultType.WIN) >= 0 &&
             defender.getTile().getSettlement() != null) {
             if (defender.getTile().getSettlement() instanceof Colony) {
-                if (!defender.isDefensiveUnit()) {
+                if (!defender.isDefensiveUnit() &&
+                    defender.getAutomaticEquipment() == null) {
                     result = CombatResultType.DONE_SETTLEMENT;
                 }
             } else if (defender.getTile().getSettlement() instanceof IndianSettlement) {
@@ -249,10 +250,10 @@ public class SimpleCombatModel implements CombatModel {
     
     /**
      * Return the offensive power of the attacker versus the defender.
-     *  
+     * 
      * Null can be passed as the defender when only the attacker unit stats
      *  are required
-     *   
+     * 
      * @param attacker an <code>Unit</code> value
      * @param defender an <code>Unit</code> value
      * @return a <code>float</code> value
@@ -265,7 +266,7 @@ public class SimpleCombatModel implements CombatModel {
     /**
      * Return a list of all offensive modifiers that apply to the attacker
      * versus the defender.
-     *
+     * 
      * Null can be passed as the defender when only the attacker unit stats
      *  are required
      * 
@@ -283,11 +284,11 @@ public class SimpleCombatModel implements CombatModel {
 
     /**
      * Return a list of all offensive modifiers that apply to the attacker
-     *  versus the defender.
+     * versus the defender.
      * 
      * Null can be passed as the defender when only the attacker unit stats
      *  are required
-     * 
+     *  
      * @param attacker an <code>Unit</code> value
      * @param defender an <code>Unit</code> value
      * @return a <code>List</code> of Modifiers
@@ -441,27 +442,11 @@ public class SimpleCombatModel implements CombatModel {
             // a stock-piled musket if attacked, so the bonus should be applied
             // for unarmed colonists inside colonies where there are muskets
             // available. Indians can also pick up equipment.
-            if (!defender.isArmed() &&
-                (defender.getLocation() instanceof WorkLocation ||
-                 defender.getLocation() instanceof IndianSettlement) &&
-                defender.getOwner().hasAbility("model.ability.automaticEquipment")) {
-                Set<Ability> autoDefence = defender.getOwner().getFeatureContainer()
-                    .getAbilitySet("model.ability.automaticEquipment");
-                Settlement settlement;
-                if (defender.getLocation() instanceof WorkLocation) {
-                    settlement = defender.getColony();
-                } else {
-                    settlement = (Settlement) defender.getLocation();
-                }
-                for (EquipmentType equipment : Specification.getSpecification().getEquipmentTypeList()) {
-                    if (defender.canBeEquippedWith(equipment)) {
-                        for (Ability ability : autoDefence) {
-                            if (ability.appliesTo(equipment) && 
-                                settlement.canBuildEquipment(equipment)) {
-                                result.addAll(equipment.getModifierSet("model.modifier.defence"));
-                            }
-                        }
-                    }
+            List<EquipmentType> autoEquipList = defender.getAutomaticEquipment();
+            if (autoEquipList != null) {
+                // add modifiers given by equipment of defense
+                for(EquipmentType equipment : autoEquipList){
+                    result.addAll(equipment.getModifierSet("model.modifier.defence"));
                 }
             }
 
@@ -537,31 +522,14 @@ public class SimpleCombatModel implements CombatModel {
         //attacker.adjustTension(defender);
         Settlement settlement = newTile.getSettlement();
 
-        /*
-         * TODO: This is a bit clumsy. Should we make information like
-         * this part of the CombatResult?
-         */
-        if (!defender.isArmed() &&
-            (defender.getLocation() instanceof WorkLocation ||
-             defender.getLocation() instanceof IndianSettlement) &&
-            defender.getOwner().hasAbility("model.ability.automaticEquipment")) {
-            Set<Ability> autoDefence = defender.getOwner().getFeatureContainer()
-                .getAbilitySet("model.ability.automaticEquipment");
-            outer: for (EquipmentType equipment : Specification.getSpecification().getEquipmentTypeList()) {
-                if (defender.canBeEquippedWith(equipment)) {
-                    for (Ability ability : autoDefence) {
-                        if (ability.appliesTo(equipment) && 
-                            settlement.canBuildEquipment(equipment)) {
-                            defender.addModelMessage(defender, ModelMessage.MessageType.COMBAT_RESULT,
-                                                     defender,
-                                                     "model.unit.automaticDefence",
-                                                     "%unit%", defender.getName(),
-                                                     "%colony%", defender.getColony().getName());
-                            break outer;
-                        }
-                    }
-                }
-            }
+        // Warn of automatic arming
+        List<EquipmentType> autoEquipList = defender.getAutomaticEquipment();
+        if (autoEquipList != null) {
+            defender.addModelMessage(defender, ModelMessage.MessageType.COMBAT_RESULT,
+                    defender,
+                    "model.unit.automaticDefence",
+                    "%unit%", defender.getName(),
+                    "%colony%", settlement.getLocationName());
         }
 
         switch (result.type) {
@@ -608,7 +576,8 @@ public class SimpleCombatModel implements CombatModel {
             } else if (attacker.hasAbility("model.ability.pillageUnprotectedColony") && 
                        !defender.isDefensiveUnit() &&
                        defender.getColony() != null &&
-                       !defender.getColony().hasStockade()) {
+                       !defender.getColony().hasStockade() &&
+                       defender.getAutomaticEquipment() == null) {
                 pillageColony(attacker, defender.getColony(), repairLocation);
             } else {
                 if (!defender.isNaval()) {
@@ -1092,8 +1061,8 @@ public class SimpleCombatModel implements CombatModel {
         damagedShip.setHitpoints(1);
         damagedShip.disposeAllUnits();
         damagedShip.getGoodsContainer().removeAll();
+        damagedShip.setDestination(null);
         damagedShip.sendToRepairLocation(repairLocation);
-
     }
 
     /**
@@ -1149,8 +1118,16 @@ public class SimpleCombatModel implements CombatModel {
     private EquipmentType findEquipmentTypeToLose(Unit victim) {
         EquipmentType toLose = null;
         int combatLossPriority = 0;
+        
+        List<EquipmentType> equipmentList = victim.getEquipment();
+        if(equipmentList.isEmpty()){
+            List<EquipmentType> autoEquipment = victim.getAutomaticEquipment(); 
+            if(autoEquipment != null){
+                equipmentList = autoEquipment;
+            }
+        }
 
-        for (EquipmentType equipmentType : victim.getEquipment()) {
+        for (EquipmentType equipmentType : equipmentList) {
             if (equipmentType.getCombatLossPriority() > combatLossPriority) {
                 toLose = equipmentType;
                 combatLossPriority = equipmentType.getCombatLossPriority();
@@ -1167,22 +1144,50 @@ public class SimpleCombatModel implements CombatModel {
      * @param enemyUnit a <code>Unit</code> that has won a combat
      */
     private void loseCombat(Unit unit, Unit enemyUnit) {
-        EquipmentType typeToLose;
-        UnitType downgrade;
-
         if (unit.hasAbility("model.ability.disposeOnCombatLoss")) {
             slaughterUnit(unit, enemyUnit);
-        } else if ((typeToLose = findEquipmentTypeToLose(unit)) != null) {
-            disarmUnit(unit, typeToLose, enemyUnit);
-        } else if ((downgrade = unit.getType().getDowngrade(DowngradeType.DEMOTION))
-                   != null) {
-            demoteUnit(unit, downgrade, enemyUnit);
-        } else if (unit.hasAbility("model.ability.canBeCaptured")
-                   && enemyUnit.hasAbility("model.ability.captureUnits")) {
-            captureUnit(unit, enemyUnit);
-        } else {
-            slaughterUnit(unit, enemyUnit);
+            return;
         }
+        
+        EquipmentType typeToLose = findEquipmentTypeToLose(unit);
+        boolean hasEquipToLose = typeToLose != null;
+        if(hasEquipToLose){
+            if(losingEquipDiscardsUnit(unit,typeToLose)){
+                slaughterUnit(unit, enemyUnit);
+            }
+            else{
+                disarmUnit(unit, typeToLose, enemyUnit);
+            }
+            return;
+        }
+        
+        UnitType downgrade = unit.getType().getDowngrade(DowngradeType.DEMOTION); 
+        if (downgrade != null) {
+            demoteUnit(unit, downgrade, enemyUnit);
+            return;
+        }
+        
+        boolean unitCanBeCaptured = unit.hasAbility("model.ability.canBeCaptured")
+                                    && enemyUnit.hasAbility("model.ability.captureUnits");
+        if(unitCanBeCaptured){
+            captureUnit(unit, enemyUnit);
+        }
+        else{
+            slaughterUnit(unit, enemyUnit);
+        }        
+    }
+
+	private boolean losingEquipDiscardsUnit(Unit unit, EquipmentType typeToLose) {
+        if(!unit.hasAbility("model.ability.disposeOnAllEquipLost")){
+            return false;
+        }
+        // verifies if unit has other equipment
+        for(EquipmentType equip : unit.getEquipment()){
+            if(equip != typeToLose){
+                return false;
+            }
+        }
+        return true;
     }
 
     /**
@@ -1279,9 +1284,28 @@ public class SimpleCombatModel implements CombatModel {
         String messageID = Messages.getKey(unit.getType().getId() + ".demoted",
                                            "model.unit.unitDemoted");
 
-        unit.removeEquipment(typeToLose, true);
-        if (unit.getEquipment().isEmpty()) {
-            messageID = "model.unit.unitDemotedToUnarmed";
+        boolean hasAutoEquipment = unit.getEquipment().isEmpty() && unit.getAutomaticEquipment() != null;
+
+        if(hasAutoEquipment){
+            // auto equipment isnt actually equiped by the unit (cannot be)
+            // its kept stored in the settlement of the unit
+            // we need to remove it from there if its lost
+            Settlement settlement = null;
+            if(unit.getLocation() instanceof IndianSettlement){
+                settlement = unit.getIndianSettlement();
+            }
+            else{
+                settlement = unit.getColony();
+            }
+            for(AbstractGoods goods : typeToLose.getGoodsRequired()){
+                settlement.removeGoods(goods);
+            }
+        }
+        else{
+            unit.removeEquipment(typeToLose, true);
+            if (unit.getEquipment().isEmpty()) {
+                messageID = "model.unit.unitDemotedToUnarmed";
+            }
         }
         enemyUnit.addModelMessage(enemyUnit,
                                   ModelMessage.MessageType.COMBAT_RESULT,
@@ -1301,6 +1325,14 @@ public class SimpleCombatModel implements CombatModel {
                              "%enemyNation%", enemyNation,
                              "%enemyUnit%", enemyUnit.getName(),
                              "%location%", locationName);
+               
+        enemyCapturesEquipment(unit,enemyUnit,typeToLose);
+    }
+
+    private void enemyCapturesEquipment(Unit unit, Unit enemyUnit, EquipmentType typeToLose) {
+        if (!enemyUnit.hasAbility("model.ability.captureEquipment")){
+            return;
+        }
         
         // we may need to change the equipment type if:
         //    - the attacker is an indian and the defender is not, or vice-versa
@@ -1325,21 +1357,22 @@ public class SimpleCombatModel implements CombatModel {
         	}
         }
         
-        if (enemyUnit.hasAbility("model.ability.captureEquipment")
-            && enemyUnit.canBeEquippedWith(newEquipType)) {
-            IndianSettlement settlement = enemyUnit.getIndianSettlement();
+        if (!enemyUnit.canBeEquippedWith(newEquipType)) {
+        	return;
+        }
+        
+        IndianSettlement settlement = enemyUnit.getIndianSettlement();
 
-            enemyUnit.equipWith(newEquipType, true);
-            unit.addModelMessage(unit,
-                                 ModelMessage.MessageType.COMBAT_RESULT,
-                                 "model.unit.equipmentCaptured",
-                                 "%nation%", enemyNation,
-                                 "%equipment%", typeToLose.getName());
-            if (settlement != null) {
-                for (AbstractGoods goods : typeToLose.getGoodsRequired()) {
-                    settlement.addGoods(goods);
-                }
-            }
+        enemyUnit.equipWith(newEquipType, true);
+        unit.addModelMessage(unit,
+        		ModelMessage.MessageType.COMBAT_RESULT,
+        		"model.unit.equipmentCaptured",
+        		"%nation%", enemyUnit.getOwner().getNationAsString(),
+        		"%equipment%", typeToLose.getName());
+        if (settlement != null) {
+        	for (AbstractGoods goods : typeToLose.getGoodsRequired()) {
+        		settlement.addGoods(goods);
+        	}
         }
     }
 
@@ -1373,6 +1406,11 @@ public class SimpleCombatModel implements CombatModel {
                              "%enemyNation%", enemyNation,
                              "%enemyUnit%", enemyUnit.getName(),
                              "%location%", locationName);
+        
+        for(EquipmentType equip : unit.getEquipment()){
+        	enemyCapturesEquipment(unit, enemyUnit, equip);
+        }
+        
         loser.divertModelMessages(unit, unit.getTile());
         unit.dispose();
     }
