@@ -523,7 +523,7 @@ public final class Tile extends FreeColGameObject implements Location, Named, Ow
                 // on land tiles, ships are docked in port and cannot defend
                 // on ocean tiles, land units behave as ship cargo and cannot defend
                 float tmpPower = getGame().getCombatModel().getDefencePower(attacker,nextUnit);
-                if (tmpPower > defencePower) {
+                if (tmpPower > defencePower || tileDefender == null) {
                     tileDefender = nextUnit;
                     defencePower = tmpPower;
                 }
