@@ -29,6 +29,7 @@ import javax.xml.stream.XMLStreamWriter;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.Europe;
 import net.sf.freecol.common.model.Goods;
+import net.sf.freecol.common.model.GoodsType;
 import net.sf.freecol.common.model.Locatable;
 import net.sf.freecol.common.model.Location;
 import net.sf.freecol.common.model.Tile;
@@ -91,7 +92,7 @@ public class AIGoods extends AIObject implements Transportable {
      * @param destination The destination of the goods. This is the
      *      <code>Location</code> to which the goods should be transported.
      */
-    public AIGoods(AIMain aiMain, Location location, int type, int amount, Location destination) {
+    public AIGoods(AIMain aiMain, Location location, GoodsType type, int amount, Location destination) {
         super(aiMain, getXMLElementTagName() + ":" + aiMain.getNextID());
 
         goods = new Goods(aiMain.getGame(), location, type, amount);
@@ -299,14 +300,6 @@ public class AIGoods extends AIObject implements Transportable {
     }
     
     
-    /**
-     * Returns the ID of this {@link AIObject}.
-     * @return The ID.
-     */
-    public String getId() {
-        return id;
-    }
-
     /**
      * Writes this object to an XML stream.
      *
@@ -317,7 +310,7 @@ public class AIGoods extends AIObject implements Transportable {
     protected void toXMLImpl(XMLStreamWriter out) throws XMLStreamException {
         out.writeStartElement(getXMLElementTagName());
 
-        out.writeAttribute("ID", id);
+        out.writeAttribute("ID", getId());
         if (destination != null) {
             out.writeAttribute("destination", destination.getId());
         }
@@ -345,7 +338,7 @@ public class AIGoods extends AIObject implements Transportable {
      *      from the stream.
      */
     protected void readFromXMLImpl(XMLStreamReader in) throws XMLStreamException {
-        id = in.getAttributeValue(null, "ID");
+        setId(in.getAttributeValue(null, "ID"));
         final String destinationStr = in.getAttributeValue(null, "destination");
         if (destinationStr != null) {
             destination = (Location) getAIMain().getFreeColGameObject(destinationStr);
