@@ -20,27 +20,26 @@
 package net.sf.freecol.server.ai;
 
 import java.io.IOException;
+import java.util.ArrayList;
 import java.util.Iterator;
+import java.util.List;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
+import net.sf.freecol.FreeCol;
 import net.sf.freecol.common.model.Colony;
-import net.sf.freecol.common.model.ColonyTradeItem;
 import net.sf.freecol.common.model.DiplomaticTrade;
 import net.sf.freecol.common.model.FoundingFather;
+import net.sf.freecol.common.model.FoundingFather.FoundingFatherType;
 import net.sf.freecol.common.model.Game;
-import net.sf.freecol.common.model.GoldTradeItem;
 import net.sf.freecol.common.model.Goods;
-import net.sf.freecol.common.model.GoodsTradeItem;
-import net.sf.freecol.common.model.Monarch;
+import net.sf.freecol.common.model.Monarch.MonarchAction;
 import net.sf.freecol.common.model.Player;
-import net.sf.freecol.common.model.StanceTradeItem;
-import net.sf.freecol.common.model.TradeItem;
+import net.sf.freecol.common.model.Player.Stance;
 import net.sf.freecol.common.model.Unit;
-import net.sf.freecol.common.model.UnitTradeItem;
 import net.sf.freecol.common.networking.Connection;
 import net.sf.freecol.common.networking.Message;
 import net.sf.freecol.common.networking.MessageHandler;
@@ -56,11 +55,9 @@ import org.w3c.dom.NodeList;
  * Handles the network messages that arrives while in the game.
  */
 public final class AIInGameInputHandler implements MessageHandler, StreamedMessageHandler {
+
     private static final Logger logger = Logger.getLogger(AIInGameInputHandler.class.getName());
 
-
-
-
     /** The player for whom I work. */
     private final ServerPlayer serverPlayer;
 
@@ -146,6 +143,7 @@ public final class AIInGameInputHandler implements MessageHandler, StreamedMessa
                 } else if (type.equals("giveIndependence")) {
                 } else if (type.equals("lostCityRumour")) {
                 } else if (type.equals("updateMarket")) {
+                } else if (type.equals("marketElement")) {
                 } else if (type.equals("diplomaticTrade")) {
                     reply = diplomaticTrade((DummyConnection) connection, element);
                 } else {
@@ -195,7 +193,7 @@ public final class AIInGameInputHandler implements MessageHandler, StreamedMessa
 
         if (serverPlayer.getId() == currentPlayer.getId()) {
             logger.finest("Starting new Thread for " + serverPlayer.getName());
-            Thread t = new Thread("AIPlayer (" + serverPlayer.getName() + ")") {
+            Thread t = new Thread(FreeCol.SERVER_THREAD+"AIPlayer (" + serverPlayer.getName() + ")") {
                 public void run() {
                     try {
                         getAIPlayer().startWorking();
@@ -229,14 +227,17 @@ public final class AIInGameInputHandler implements MessageHandler, StreamedMessa
      * 
      */
     private Element chooseFoundingFather(DummyConnection connection, Element element) {
-        int[] possibleFoundingFathers = new int[FoundingFather.TYPE_COUNT];
-        for (int i = 0; i < FoundingFather.TYPE_COUNT; i++) {
-            possibleFoundingFathers[i] = Integer.parseInt(element.getAttribute("foundingFather" + Integer.toString(i)));
+        final List<FoundingFather> possibleFoundingFathers = new ArrayList<FoundingFather>();
+        for (FoundingFatherType type : FoundingFatherType.values()) {
+            String id = element.getAttribute(type.toString());
+            if (id != null && id != "") {
+                possibleFoundingFathers.add(FreeCol.getSpecification().getFoundingFather(id));
+            }
         }
 
-        int foundingFather = getAIPlayer().selectFoundingFather(possibleFoundingFathers);
+        FoundingFather foundingFather = getAIPlayer().selectFoundingFather(possibleFoundingFathers);
         Element reply = Message.createNewRootElement("chosenFoundingFather");
-        reply.setAttribute("foundingFather", Integer.toString(foundingFather));
+        reply.setAttribute("foundingFather", foundingFather.getId());
         serverPlayer.setCurrentFather(foundingFather);
 
         return reply;
@@ -251,28 +252,26 @@ public final class AIInGameInputHandler implements MessageHandler, StreamedMessa
      * 
      */
     private Element monarchAction(DummyConnection connection, Element element) {
-        int action = Integer.parseInt(element.getAttribute("action"));
+        MonarchAction action = Enum.valueOf(MonarchAction.class, element.getAttribute("action"));
         Element reply = null;
+        boolean accept = false;
         switch (action) {
-        case Monarch.RAISE_TAX:
+        case RAISE_TAX:
             int tax = Integer.parseInt(element.getAttribute("amount"));
-            boolean accept = getAIPlayer().acceptTax(tax);
+            accept = getAIPlayer().acceptTax(tax);
             reply = Message.createNewRootElement("acceptTax");
             reply.setAttribute("accepted", String.valueOf(accept));
             break;
-
-        case Monarch.LOWER_TAX:
-            int newTax = Integer.parseInt(element.getAttribute("amount")); 
+            
+        case LOWER_TAX:
+            int newTax = Integer.parseInt(element.getAttribute("amount"));
             getAIPlayer().getPlayer().setTax(newTax);
             break;
-            
-        case Monarch.OFFER_MERCENARIES:
+
+        case OFFER_MERCENARIES:
             reply = Message.createNewRootElement("hireMercenaries");
-            if (getAIPlayer().getStrategy() == AIPlayer.STRATEGY_CONQUEST || getAIPlayer().getPlayer().isAtWar()) {
-                reply.setAttribute("accepted", String.valueOf(true));
-            } else {
-                reply.setAttribute("accepted", String.valueOf(false));
-            }
+            accept = getAIPlayer().acceptMercenaryOffer();
+            reply.setAttribute("accepted", String.valueOf(accept));
             break;
 
         default:
@@ -318,63 +317,8 @@ public final class AIInGameInputHandler implements MessageHandler, StreamedMessa
         NodeList childElements = element.getChildNodes();
         Element childElement = (Element) childElements.item(0);
         DiplomaticTrade agreement = new DiplomaticTrade(freeColServer.getGame(), childElement);
-        int stance = Integer.MIN_VALUE;
-        int value = 0;
-        Iterator<TradeItem> itemIterator = agreement.iterator();
-        while (itemIterator.hasNext()) {
-            TradeItem item = itemIterator.next();
-            if (item instanceof GoldTradeItem) {
-                int gold = ((GoldTradeItem) item).getGold();
-                if (item.getSource() == serverPlayer) {
-                    value -= gold;
-                } else {
-                    value += gold;
-                }
-            } else if (item instanceof StanceTradeItem) {
-                stance = ((StanceTradeItem) item).getStance();
-            } else if (item instanceof ColonyTradeItem) {
-                // TODO: evaluate whether we might wish to give up a colony
-                if (item.getSource() == serverPlayer) {
-                    value = Integer.MIN_VALUE;
-                    break;
-                } else {
-                    value += 1000;
-                }
-            } else if (item instanceof UnitTradeItem) {
-                // TODO: evaluate whether we might wish to give up a unit
-                if (item.getSource() == serverPlayer) {
-                    value = Integer.MIN_VALUE;
-                    break;
-                } else {
-                    value += 100;
-                }
-            } else if (item instanceof GoodsTradeItem) {
-                Goods goods = ((GoodsTradeItem) item).getGoods();
-                if (item.getSource() == serverPlayer) {
-                    value -= serverPlayer.getMarket().getBidPrice(goods.getType(), goods.getAmount());
-                } else {
-                    value += serverPlayer.getMarket().getSalePrice(goods.getType(), goods.getAmount());
-                }
-            }
-        }
+        boolean accept = getAIPlayer().acceptDiplomaticTrade(agreement);
 
-        boolean accept = false;
-        if (stance == Player.PEACE) {
-            if (agreement.getSender().hasFather(FoundingFather.BENJAMIN_FRANKLIN) &&
-                value >= 0) {
-                // TODO: introduce some kind of counter in order to avoid
-                // Benjamin Franklin exploit
-                accept = true;
-            } else if (value >= 1000) {
-                accept = true;
-            }
-        } else if (serverPlayer.getStance(agreement.getSender()) >= Player.PEACE) {
-            if (value > 100) {
-                accept = true;
-            }
-        }
-
-        System.out.println("value is " + value + ", accept is " + accept);
         if (accept) {
             Element reply = Message.createNewRootElement("diplomaticTrade");
             reply.setAttribute("accept", "accept");
