@@ -20,49 +20,34 @@
 
 package net.sf.freecol.server.ai;
 
-import java.io.IOException;
 import java.io.PrintWriter;
-import java.io.StringReader;
 import java.io.StringWriter;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.logging.Logger;
 
-import javax.xml.parsers.DocumentBuilder;
-import javax.xml.parsers.DocumentBuilderFactory;
-import javax.xml.parsers.ParserConfigurationException;
-import javax.xml.stream.XMLInputFactory;
-import javax.xml.stream.XMLOutputFactory;
 import javax.xml.stream.XMLStreamConstants;
 import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
-import javax.xml.transform.Transformer;
-import javax.xml.transform.TransformerException;
-import javax.xml.transform.TransformerFactory;
-import javax.xml.transform.dom.DOMSource;
-import javax.xml.transform.stream.StreamResult;
 
 import net.sf.freecol.common.PseudoRandom;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.FreeColGameObject;
 import net.sf.freecol.common.model.FreeColGameObjectListener;
+import net.sf.freecol.common.model.FreeColObject;
 import net.sf.freecol.common.model.Game;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Unit;
 import net.sf.freecol.server.FreeColServer;
 import net.sf.freecol.server.model.ServerPlayer;
 
-import org.w3c.dom.Document;
 import org.w3c.dom.Element;
-import org.xml.sax.InputSource;
-import org.xml.sax.SAXException;
-
 
 /**
 * The main AI-class. Keeps references to all other AI-classes.
 */
-public class AIMain implements FreeColGameObjectListener {
+public class AIMain extends FreeColObject implements FreeColGameObjectListener {
     private static final Logger logger = Logger.getLogger(AIMain.class.getName());
 
 
@@ -160,7 +145,8 @@ public class AIMain implements FreeColGameObjectListener {
         Iterator<FreeColGameObject> fit = getGame().getFreeColGameObjectIterator();
         while (fit.hasNext()) {
             FreeColGameObject f = fit.next();
-            if ((f instanceof Unit || f instanceof Player || f instanceof Colony)
+            if ((f instanceof Unit || f instanceof Colony ||
+                 (f instanceof Player && !((Player)f).getName().equals(Player.UNKNOWN_ENEMY)) )
                     && !aiObjects.containsKey(f.getId())) {
                 logger.warning("Missing AIObject for: " + f.getId());
                 ok = false;
@@ -321,7 +307,7 @@ public class AIMain implements FreeColGameObjectListener {
         if (freeColGameObject instanceof Unit) {
             new AIUnit(this, (Unit) freeColGameObject);
         } else if (freeColGameObject instanceof ServerPlayer) {
-            new AIPlayer(this, (ServerPlayer) freeColGameObject);
+            new StandardAIPlayer(this, (ServerPlayer) freeColGameObject);
         } else if (freeColGameObject instanceof Colony) {
             new AIColony(this, (Colony) freeColGameObject);
         }
@@ -337,89 +323,7 @@ public class AIMain implements FreeColGameObjectListener {
         if (o != null) {
             o.dispose();
         }
-        //removeAIObject(id);
-    }
-
-    /**
-     * This method writes an XML-representation of this object to
-     * the given stream.
-     * 
-     * <br><br>
-     *  
-     * @param document The <code>Document</code>.
-     * @return An XML-representation of this object.
-     */    
-    public Element toXMLElement(Document document) {
-        try {
-            StringWriter sw = new StringWriter();
-            XMLOutputFactory xif = XMLOutputFactory.newInstance();
-            XMLStreamWriter xsw = xif.createXMLStreamWriter(sw);
-            toXML(xsw);      
-            
-            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
-            Document tempDocument = null;
-            try {
-                DocumentBuilder builder = factory.newDocumentBuilder();
-                tempDocument = builder.parse(new InputSource(new StringReader(sw.toString())));;
-                return (Element) document.importNode(tempDocument.getDocumentElement(), true);
-            } catch (ParserConfigurationException pce) {
-                // Parser with specified options can't be built
-                StringWriter swe = new StringWriter();
-                pce.printStackTrace(new PrintWriter(swe));
-                logger.warning(swe.toString());
-                IllegalStateException ex = new IllegalStateException("ParserConfigurationException");
-                ex.initCause(pce);
-                throw ex;
-            } catch (SAXException se) {
-                StringWriter swe = new StringWriter();
-                se.printStackTrace(new PrintWriter(swe));
-                logger.warning(swe.toString());
-                IllegalStateException ex = new IllegalStateException("SAXException");
-                ex.initCause(se);
-                throw ex;
-            } catch (IOException ie) {
-                StringWriter swe = new StringWriter();
-                ie.printStackTrace(new PrintWriter(swe));
-                logger.warning(swe.toString());
-                IllegalStateException ex = new IllegalStateException("IOException");
-                ex.initCause(ie);
-                throw ex;
-            }                                    
-        } catch (XMLStreamException e) {
-            logger.warning(e.toString());
-            IllegalStateException ex = new IllegalStateException("XMLStreamException");
-            ex.initCause(e);
-            throw ex;
-        }
-    }
-    
-    /**
-     * Initialize this object from an XML-representation of this object.
-     * @param element An XML-element that will be used to initialize
-     *      this object.
-     */
-    public void readFromXMLElement(Element element) {
-        XMLInputFactory xif = XMLInputFactory.newInstance();        
-        try {
-            try {
-                TransformerFactory factory = TransformerFactory.newInstance();
-                Transformer xmlTransformer = factory.newTransformer();
-                StringWriter stringWriter = new StringWriter();
-                xmlTransformer.transform(new DOMSource(element), new StreamResult(stringWriter));
-                String xml = stringWriter.toString();
-                XMLStreamReader xsr = xif.createXMLStreamReader(new StringReader(xml));
-                xsr.nextTag();
-                readFromXML(xsr);
-            } catch (TransformerException e) {
-                StringWriter sw = new StringWriter();
-                e.printStackTrace(new PrintWriter(sw));
-                logger.warning(sw.toString());
-                throw new IllegalStateException("TransformerException");
-            }
-        } catch (XMLStreamException e) {
-            logger.warning(e.toString());
-            throw new IllegalStateException("XMLStreamException");
-        }
+        removeAIObject(id);
     }
 
     /**
@@ -438,14 +342,16 @@ public class AIMain implements FreeColGameObjectListener {
         Iterator<AIObject> i = aiObjects.values().iterator();
         while (i.hasNext()) {
             AIObject aio = i.next();
+            
+            if ((aio instanceof Wish) && !((Wish) aio).shouldBeStored()) {
+                continue;
+            }
 
             try {
-                if (!(aio instanceof Wish) || ((Wish) aio).shouldBeStored()) {
-                    if (aio.getId() != null) {
-                        aio.toXML(out);
-                    } else {
-                        logger.warning("aio.getId() == null, for: " + aio.getClass().getName());
-                    }
+                if (aio.getId() != null) {
+                    aio.toXML(out);
+                } else {
+                    logger.warning("aio.getId() == null, for: " + aio.getClass().getName());
                 }
             } catch (Exception e) {
                 StringWriter sw = new StringWriter();
@@ -457,28 +363,6 @@ public class AIMain implements FreeColGameObjectListener {
         out.writeEndElement();
     }
 
-    /**
-     * Writes all of the <code>AIObject</code>s and other AI-related 
-     * information to an XML-stream.
-     *
-     * @param out The target stream.
-     * @throws XMLStreamException if there are any problems writing
-     *      to the stream.
-     */
-    public void toXML(XMLStreamWriter out) throws XMLStreamException {
-        toXMLImpl(out);
-    }
-    
-    /**
-     * Reads all the <code>AIObject</code>s and other AI-related information
-     * from XML data.
-     * @param in The input stream with the XML.
-     * @throws XMLStreamException if an error occured during parsing.
-     */
-    public void readFromXML(XMLStreamReader in) throws XMLStreamException {
-        readFromXMLImpl(in);
-    }
-
     /**
      * Reads all the <code>AIObject</code>s and other AI-related information
      * from XML data.
@@ -506,7 +390,7 @@ public class AIMain implements FreeColGameObjectListener {
                 } else if (tagName.equals(AIUnit.getXMLElementTagName())) {
                     new AIUnit(this, in);
                 } else if (tagName.equals(AIPlayer.getXMLElementTagName())) {
-                    new AIPlayer(this, in);
+                    new StandardAIPlayer(this, in);
                 } else if (tagName.equals(AIColony.getXMLElementTagName())) {
                     new AIColony(this, in);
                 } else if (tagName.equals(AIGoods.getXMLElementTagName())) {
@@ -515,8 +399,8 @@ public class AIMain implements FreeColGameObjectListener {
                     new WorkerWish(this, in);
                 } else if (tagName.equals(GoodsWish.getXMLElementTagName())) {
                     new GoodsWish(this, in);
-                } else if (tagName.equals(TileImprovement.getXMLElementTagName())) {
-                    new TileImprovement(this, in);                
+                } else if (tagName.equals(TileImprovementPlan.getXMLElementTagName())) {
+                    new TileImprovementPlan(this, in);                
                 } else {
                     logger.warning("Unkown AI-object read: " + tagName + "(" + lastTag + ")");
                 }
@@ -550,4 +434,28 @@ public class AIMain implements FreeColGameObjectListener {
     public static String getXMLElementTagName() {
         return "aiMain";
     }
+    
+    /**
+     * Computes how many objects of each class have been created, 
+     * to track memory leaks over time
+     */
+    public HashMap<String, Long> getAIStatistics() {
+        
+        HashMap<String, Long> map = new HashMap<String, Long>();
+        Iterator<AIObject> iter = aiObjects.values().iterator();
+        while (iter.hasNext()) {
+            AIObject obj = iter.next();
+            String className = obj.getClass().getSimpleName();
+            if (map.containsKey(className)) {
+                Long count = map.get(className);
+                count++;
+                map.put(className, count);
+            } else {
+                Long count = new Long(1);
+                map.put(className, count);
+            }
+        }
+        
+        return map;
+    }
 }
