@@ -43,6 +43,7 @@ import net.sf.freecol.server.ai.mission.BuildColonyMission;
 import net.sf.freecol.server.ai.mission.CashInTreasureTrainMission;
 import net.sf.freecol.server.ai.mission.Mission;
 import net.sf.freecol.server.ai.mission.PioneeringMission;
+import net.sf.freecol.server.ai.mission.PrivateerMission;
 import net.sf.freecol.server.ai.mission.ScoutingMission;
 import net.sf.freecol.server.ai.mission.TransportMission;
 import net.sf.freecol.server.ai.mission.UnitWanderHostileMission;
@@ -273,6 +274,32 @@ public abstract class ColonialAIPlayer extends EuropeanAIPlayer {
             }
         }
     }
+    
+    /**
+     * Gives missions to all the naval units this player owns.
+     */
+    public void giveNavalMissions() {
+        logger.finest("Entering method giveNavalMissions");
+        if (!getPlayer().isEuropean()) {
+            return;
+        }
+        Iterator<AIUnit> aiUnitsIterator = getAIUnitIterator();
+        while (aiUnitsIterator.hasNext()) {
+            AIUnit aiUnit = aiUnitsIterator.next();
+            Unit unit = aiUnit.getUnit();
+            
+            if (aiUnit.hasMission() || !unit.isNaval()){
+                continue;
+            }
+
+            if (PrivateerMission.isValid(aiUnit)) {
+                aiUnit.setMission(new PrivateerMission(getAIMain(), aiUnit));
+            }
+            else{
+                aiUnit.setMission(new TransportMission(getAIMain(), aiUnit));
+            }
+        }
+    }
 
     /**
      * Cheats for the AI :-)
