@@ -169,6 +169,16 @@ public class ColonyPlan {
             }
         }
 
+        Building building = colony.getStockade();
+        if (building == null) {
+            if (colony.canBuild(stockade)) {
+                buildList.add(stockade);
+            }
+        }
+        else if (building.canBuildNext()) {
+        	buildList.add(building.getType().getUpgradesTo());            
+        }
+        
         Iterator<WorkLocationPlan> wlpIt = getSortedWorkLocationPlans().iterator();
         while (wlpIt.hasNext()) {
             WorkLocationPlan wlp = wlpIt.next();
@@ -180,10 +190,10 @@ public class ColonyPlan {
 
                 GoodsType outputType = b.getGoodsOutputType();
                 if (outputType != null) {
-                    for (Building building : colony.getBuildings()) {
-                        if (!building.getType().getModifierSet(outputType.getId()).isEmpty()
-                                && building.canBuildNext()) {
-                            buildList.add(building.getType().getUpgradesTo());
+                    for (Building manufacture : colony.getBuildings()) {
+                        if (!manufacture.getType().getModifierSet(outputType.getId()).isEmpty()
+                                && manufacture.canBuildNext()) {
+                            buildList.add(manufacture.getType().getUpgradesTo());
                         }
                     }
                 }
@@ -200,7 +210,7 @@ public class ColonyPlan {
         }
 
         // Check if we should improve the warehouse:
-        Building building = colony.getWarehouse();
+        building = colony.getWarehouse();
         if (building.canBuildNext()) {
             if (colony.getGoodsContainer().hasReachedCapacity(colony.getWarehouseCapacity())) {
                 buildList.add(0, building.getType().getUpgradesTo());
@@ -231,15 +241,6 @@ public class ColonyPlan {
             }
         }
 
-        building = colony.getStockade();
-        if (building == null) {
-            if (colony.canBuild(stockade)) {
-                buildList.add(stockade);
-            }
-        } else if (building.canBuildNext()) {
-            buildList.add(building.getType().getUpgradesTo());
-        }
-
         building = colony.getBuildingForProducing(Goods.MUSKETS);
         if (building == null) {
             if (colony.canBuild(armory)) {
