@@ -26,6 +26,8 @@ import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
+import net.sf.freecol.FreeCol;
+import net.sf.freecol.common.model.GoodsType;
 import net.sf.freecol.common.model.Location;
 import net.sf.freecol.common.model.Unit;
 
@@ -42,7 +44,7 @@ public class GoodsWish extends Wish {
     private static final Logger logger = Logger.getLogger(GoodsWish.class.getName());
 
     
-    private int goodsType;
+    private GoodsType goodsType;
 
     /**
     * Creates a new <code>GoodsWish</code>.
@@ -56,7 +58,7 @@ public class GoodsWish extends Wish {
     * @param goodsType The type of goods needed for releasing this wish
     *       completly.
     */
-    public GoodsWish(AIMain aiMain, Location destination, int value, int goodsType) {
+    public GoodsWish(AIMain aiMain, Location destination, int value, GoodsType goodsType) {
         super(aiMain, getXMLElementTagName() + ":" + aiMain.getNextID());
 
         if (destination == null) {
@@ -108,7 +110,7 @@ public class GoodsWish extends Wish {
       * Returns the type of unit needed for releasing this wish.
       * @return The {@link Unit#getType type of unit}.
       */
-     public int getGoodsType() {
+     public GoodsType getGoodsType() {
          return goodsType;
      }
      
@@ -130,7 +132,7 @@ public class GoodsWish extends Wish {
          }
          out.writeAttribute("value", Integer.toString(value));
          
-         out.writeAttribute("goodsType", Integer.toString(goodsType));
+         out.writeAttribute("goodsType", goodsType.getId());
          
          out.writeEndElement();
      }
@@ -142,7 +144,7 @@ public class GoodsWish extends Wish {
       *      from the stream.
       */
      protected void readFromXMLImpl(XMLStreamReader in) throws XMLStreamException {        
-         id = in.getAttributeValue(null, "ID");
+         setId(in.getAttributeValue(null, "ID"));
          destination = (Location) getAIMain().getFreeColGameObject(in.getAttributeValue(null, "destination"));
          
          final String transportableStr = in.getAttributeValue(null, "transportable");
@@ -156,7 +158,7 @@ public class GoodsWish extends Wish {
          }
          value = Integer.parseInt(in.getAttributeValue(null, "value"));
          
-         goodsType = Integer.parseInt(in.getAttributeValue(null, "goodsType"));
+         goodsType = FreeCol.getSpecification().getGoodsType(in.getAttributeValue(null, "goodsType"));
          
          in.nextTag();
      }
