@@ -146,24 +146,45 @@ public abstract class NewAIPlayer extends AIObject {
     protected void determineStances() {
         logger.finest("Entering method determineStances");
         for (Player p : getGame().getPlayers()) {
-            if (p != player) {
-                Stance stance = getPlayer().getStance(p);
-                Tension tension = getPlayer().getTension(p);
-                if (stance != null && tension != null) {
-                    if (p.getREFPlayer() == getPlayer() && p.getPlayerType() == PlayerType.REBEL) {
-                        tension.modify(1000);
-                    }
-                    if (stance != Stance.WAR &&
-                        tension.getLevel() == Tension.Level.HATEFUL) {
-                        getPlayer().changeRelationWithPlayer(p, Stance.WAR);
-                    } else if (stance == Stance.WAR
-                               && tension.getLevel().compareTo(Tension.Level.CONTENT) <= 0) {
-                        getPlayer().changeRelationWithPlayer(p, Stance.CEASE_FIRE);
-                    } else if (stance == Stance.CEASE_FIRE
-                               && tension.getLevel().compareTo(Tension.Level.HAPPY) <= 0) {
-                        getPlayer().changeRelationWithPlayer(p, Stance.PEACE);
-                    }
+        	if(p == player){
+                continue;
+            }
+            
+            Stance stance = player.getStance(p);
+            Tension tension = player.getTension(p);
+            // not contacted yet
+            if (stance == null || tension == null) {
+                continue;
+            }
+
+            // REF is always at war with rebel player
+            if (p.getREFPlayer() == player && p.getPlayerType() == PlayerType.REBEL) {
+                tension.modify(1000);
+                if(stance != Stance.WAR){
+                    player.changeRelationWithPlayer(p, Stance.WAR);
                 }
+                continue;
+            }
+            
+            // Ready to go to war
+            if (stance != Stance.WAR 
+                    && tension.getLevel() == Tension.Level.HATEFUL) {
+                player.changeRelationWithPlayer(p, Stance.WAR);
+                continue;
+            }
+            
+            // Ready to back down
+            if (stance == Stance.WAR
+                    && tension.getLevel().compareTo(Tension.Level.CONTENT) <= 0) {
+                player.changeRelationWithPlayer(p, Stance.CEASE_FIRE);
+                continue;
+            }
+            
+            // all friends now
+            if (stance == Stance.CEASE_FIRE
+                    && tension.getLevel().compareTo(Tension.Level.HAPPY) <= 0) {
+                player.changeRelationWithPlayer(p, Stance.PEACE);
+                continue;
             }
         }
     }
