@@ -21,15 +21,13 @@ package net.sf.freecol.server.ai;
 
 import java.io.PrintWriter;
 import java.io.StringWriter;
+import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.HashMap;
 import java.util.Iterator;
-import java.util.ArrayList;
-import java.util.LinkedList;
 import java.util.List;
 import java.util.Vector;
-import java.util.logging.Level;
 import java.util.logging.Logger;
 
 import javax.xml.stream.XMLStreamException;
@@ -58,8 +56,6 @@ import net.sf.freecol.common.model.Map;
 import net.sf.freecol.common.model.Ownable;
 import net.sf.freecol.common.model.PathNode;
 import net.sf.freecol.common.model.Player;
-import net.sf.freecol.common.model.Player.PlayerType;
-import net.sf.freecol.common.model.Player.Stance;
 import net.sf.freecol.common.model.Settlement;
 import net.sf.freecol.common.model.StanceTradeItem;
 import net.sf.freecol.common.model.Tension;
@@ -67,11 +63,12 @@ import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.TileImprovement;
 import net.sf.freecol.common.model.TradeItem;
 import net.sf.freecol.common.model.Unit;
-import net.sf.freecol.common.model.Unit.UnitState;
 import net.sf.freecol.common.model.UnitTradeItem;
 import net.sf.freecol.common.model.UnitType;
 import net.sf.freecol.common.model.Map.Position;
-import net.sf.freecol.common.networking.Connection;
+import net.sf.freecol.common.model.Player.PlayerType;
+import net.sf.freecol.common.model.Player.Stance;
+import net.sf.freecol.common.model.Unit.UnitState;
 import net.sf.freecol.common.networking.Message;
 import net.sf.freecol.common.networking.NetworkConstants;
 import net.sf.freecol.server.ai.mission.BuildColonyMission;
@@ -82,6 +79,7 @@ import net.sf.freecol.server.ai.mission.IndianBringGiftMission;
 import net.sf.freecol.server.ai.mission.IndianDemandMission;
 import net.sf.freecol.server.ai.mission.Mission;
 import net.sf.freecol.server.ai.mission.PioneeringMission;
+import net.sf.freecol.server.ai.mission.PrivateerMission;
 import net.sf.freecol.server.ai.mission.ScoutingMission;
 import net.sf.freecol.server.ai.mission.TransportMission;
 import net.sf.freecol.server.ai.mission.UnitSeekAndDestroyMission;
@@ -354,25 +352,47 @@ public class StandardAIPlayer extends AIPlayer {
      */
     private void determineStances() {
         logger.finest("Entering method determineStances");
+        Player player = getPlayer();
         for (Player p : getGame().getPlayers()) {
-            if (p != getPlayer()) {
-                Stance stance = getPlayer().getStance(p);
-                Tension tension = getPlayer().getTension(p);
-                if (stance != null && tension != null) {
-                    if (p.getREFPlayer() == getPlayer() && p.getPlayerType() == PlayerType.REBEL) {
-                        tension.modify(1000);
-                    }
-                    if (stance != Stance.WAR &&
-                        tension.getLevel() == Tension.Level.HATEFUL) {
-                        getPlayer().changeRelationWithPlayer(p, Stance.WAR);
-                    } else if (stance == Stance.WAR
-                               && tension.getLevel().compareTo(Tension.Level.CONTENT) <= 0) {
-                        getPlayer().changeRelationWithPlayer(p, Stance.CEASE_FIRE);
-                    } else if (stance == Stance.CEASE_FIRE
-                               && tension.getLevel().compareTo(Tension.Level.HAPPY) <= 0) {
-                        getPlayer().changeRelationWithPlayer(p, Stance.PEACE);
-                    }
+            if(p == player){
+                continue;
+            }
+            
+            Stance stance = player.getStance(p);
+            Tension tension = player.getTension(p);
+            // not contacted yet
+            if (stance == null || tension == null) {
+                continue;
+            }
+
+            // REF is always at war with rebel player
+            if (p.getREFPlayer() == player && p.getPlayerType() == PlayerType.REBEL) {
+                tension.modify(1000);
+                if(stance != Stance.WAR){
+                    player.changeRelationWithPlayer(p, Stance.WAR);
                 }
+                continue;
+            }
+            
+            // Ready to go to war
+            if (stance != Stance.WAR 
+                    && tension.getLevel() == Tension.Level.HATEFUL) {
+                player.changeRelationWithPlayer(p, Stance.WAR);
+                continue;
+            }
+            
+            // Ready to back down
+            if (stance == Stance.WAR
+                    && tension.getLevel().compareTo(Tension.Level.CONTENT) <= 0) {
+                player.changeRelationWithPlayer(p, Stance.CEASE_FIRE);
+                continue;
+            }
+            
+            // all friends now
+            if (stance == Stance.CEASE_FIRE
+                    && tension.getLevel().compareTo(Tension.Level.HAPPY) <= 0) {
+                player.changeRelationWithPlayer(p, Stance.PEACE);
+                continue;
             }
         }
     }
@@ -427,8 +447,16 @@ public class StandardAIPlayer extends AIPlayer {
         Iterator<AIUnit> aiUnitsIterator = getAIUnitIterator();
         while (aiUnitsIterator.hasNext()) {
             AIUnit aiUnit = aiUnitsIterator.next();
-            if (aiUnit.getUnit().isNaval() && !aiUnit.hasMission()) {
-                // TODO: privateers should have military naval mission?
+            Unit unit = aiUnit.getUnit();
+            
+            if (aiUnit.hasMission() || !unit.isNaval()){
+                continue;
+            }
+
+            if (PrivateerMission.isValid(aiUnit)) {
+                aiUnit.setMission(new PrivateerMission(getAIMain(), aiUnit));
+            }
+            else{
                 aiUnit.setMission(new TransportMission(getAIMain(), aiUnit));
             }
         }
@@ -858,17 +886,28 @@ public class StandardAIPlayer extends AIPlayer {
             
             if (unit.canCarryTreasure()) {
                 aiUnit.setMission(new CashInTreasureTrainMission(getAIMain(), aiUnit));
-            } else if (unit.hasAbility("model.ability.scoutIndianSettlement") &&
+                continue;
+            }
+            
+            if (unit.hasAbility("model.ability.scoutIndianSettlement") &&
                        ScoutingMission.isValid(aiUnit)) {
                 aiUnit.setMission(new ScoutingMission(getAIMain(), aiUnit));
-            } else if ((unit.isOffensiveUnit() || unit.isDefensiveUnit())
-                       && (!unit.isColonist() || unit.hasAbility("model.ability.expertSoldier") || 
-                        getGame().getTurn().getNumber() > 5)) {
+                continue;
+            }
+            
+            if ((unit.isOffensiveUnit() || unit.isDefensiveUnit())
+            		&& (!unit.isColonist() || unit.hasAbility("model.ability.expertSoldier") || 
+            				getGame().getTurn().getNumber() > 5)) {
                 giveMilitaryMission(aiUnit);
-            } else if (unit.hasAbility("model.ability.improveTerrain")
+                continue;
+            }
+            if (unit.hasAbility("model.ability.improveTerrain")
                        && PioneeringMission.isValid(aiUnit)) {
                 aiUnit.setMission(new PioneeringMission(getAIMain(), aiUnit));
-            } else if (unit.isColonist()) {                
+                continue;
+            }
+            
+            if (unit.isColonist()) {                
                 /*
                  * Motivated by (speed) performance: This map stores the
                  * distance between the unit and the destination of a Wish:
@@ -915,13 +954,14 @@ public class StandardAIPlayer extends AIPlayer {
                     aiUnit.setMission(new WishRealizationMission(getAIMain(), aiUnit, bestWish));
                     continue;
                 }
+                
                 // Find a site for a new colony:
                 Tile colonyTile = null;
                 if (getPlayer().canBuildColonies()) {
-                    colonyTile = BuildColonyMission.findColonyLocation(aiUnit.getUnit());
-                }
-                if (colonyTile != null) {
-                    bestTurns = unit.getTurnsToReach(colonyTile);
+                    colonyTile = BuildColonyMission.findColonyLocation(aiUnit.getUnit());            
+                    if (colonyTile != null) {
+                    	bestTurns = unit.getTurnsToReach(colonyTile);
+                    }
                 }
                 
                 // Check if we can find a better site to work than a new colony:
@@ -986,6 +1026,7 @@ public class StandardAIPlayer extends AIPlayer {
                     continue;
                 }
             }
+            
             if (!aiUnit.hasMission()) {
                 if (aiUnit.getUnit().isOffensiveUnit()) {
                     aiUnit.setMission(new UnitWanderHostileMission(getAIMain(), aiUnit));
@@ -1550,6 +1591,10 @@ public class StandardAIPlayer extends AIPlayer {
         Iterator<AIUnit> aui = getAIUnitIterator();
         while (aui.hasNext()) {
             AIUnit au = aui.next();
+            // naval units do not need transports
+            if(au.getUnit().isNaval()){
+            	continue;
+            }
             if (au.getTransportDestination() != null && au.getTransport() == null) {
                 transportables.add(au);
             }
