@@ -97,15 +97,6 @@ public abstract class Wish extends AIObject {
         return (transportable != null);
     }
     
-    /**
-    * Returns the ID for this <code>Wish</code>.
-    * @return The ID of this <code>Wish</code>.
-    */
-    public String getId() {
-        return id;
-    }
-
-
     /**
     * Returns the value for this <code>Wish</code>.
     * @return The value identifying the importance of
