@@ -22,12 +22,16 @@ package net.sf.freecol.server.ai;
 
 import java.util.logging.Logger;
 
+import net.sf.freecol.FreeCol;
 import net.sf.freecol.common.model.Building;
 import net.sf.freecol.common.model.ColonyTile;
 import net.sf.freecol.common.model.Game;
 import net.sf.freecol.common.model.Goods;
+import net.sf.freecol.common.model.GoodsType;
 import net.sf.freecol.common.model.Tile;
-import net.sf.freecol.common.model.Unit;
+import net.sf.freecol.common.model.TileImprovement;
+import net.sf.freecol.common.model.TileImprovementType;
+import net.sf.freecol.common.model.UnitType;
 import net.sf.freecol.common.model.WorkLocation;
 
 import org.w3c.dom.Document;
@@ -38,19 +42,18 @@ import org.w3c.dom.Element;
 * Objects of this class contains AI-information for a single {@link WorkLocation}.
 */
 public class WorkLocationPlan {
+
     @SuppressWarnings("unused")
     private static final Logger logger = Logger.getLogger(WorkLocationPlan.class.getName());
 
-
     private AIMain aiMain;
-    
 
     /**
     * The FreeColGameObject this AIObject contains AI-information for.
     */
     private WorkLocation workLocation;
     private int priority;
-    private int goodsType;
+    private GoodsType goodsType;
 
 
     /**
@@ -62,7 +65,7 @@ public class WorkLocationPlan {
      * @param goodsType The goodsType to be produced on the 
      *      <code>workLocation</code> using this plan.
      */
-    public WorkLocationPlan(AIMain aiMain, WorkLocation workLocation, int goodsType) {
+    public WorkLocationPlan(AIMain aiMain, WorkLocation workLocation, GoodsType goodsType) {
         this.aiMain = aiMain;
         this.workLocation = workLocation;
         this.goodsType = goodsType;
@@ -100,65 +103,50 @@ public class WorkLocationPlan {
     }
     
     /**
-     * Gets a <code>TileImprovement</code> which will improve
+     * Gets a <code>TileImprovementPlan</code> which will improve
      * the production of the goods type specified by this
      * <code>WorkLocationPlan</code>.
      * 
-     * @return The <code>TileImprovement</code> if there is an
+     * @return The <code>TileImprovementPlan</code> if there is an
      *      improvement, plow or build road, which will increase
      *      the production of the goods type specified by this 
      *      plan. <code>null</code> gets returned if this plan is 
      *      for a <code>Building</code> or that the <code>Tile</code> 
      *      does not have an improvement.
      */
-    public TileImprovement createTileImprovement() {   
-        return updateTileImprovement(null);
+    public TileImprovementPlan createTileImprovementPlan() {   
+        return updateTileImprovementPlan(null);
     }
         
     /**
-     * Updates the given <code>TileImprovement</code>.
+     * Updates the given <code>TileImprovementPlan</code>.
      * 
-     * @param ti The <code>TileImprovement</code> to update.
-     * @return The same <code>TileImprovement</code>-object
+     * @param tip The <code>TileImprovementPlan</code> to update.
+     * @return The same <code>TileImprovementPlan</code>-object
      *      as provided to the method or <code>null</code> if 
      *      there is no more need for the improvement. 
      */
-    public TileImprovement updateTileImprovement(TileImprovement ti) {
+    public TileImprovementPlan updateTileImprovementPlan(TileImprovementPlan tip) {
         if (workLocation instanceof ColonyTile) {
             Tile tile = ((ColonyTile) workLocation).getWorkTile();
             
-            if (ti != null && ti.getTarget() != tile) {
-                throw new IllegalArgumentException("The given TileImprovement was not created for this Tile.");
+            if (tip != null && tip.getTarget() != tile) {
+                throw new IllegalArgumentException("The given TileImprovementPlan was not created for this Tile.");
             }
             
-            int gain = tile.getMaximumPotential(goodsType) - tile.potential(goodsType);
-            if (gain > 0) {
-                int value = gain;
-                if (tile.hasBonus()) {
-                    value += 5;
-                }
-                if (!tile.isForested()) {
-                    value += 5;
-                }
-                boolean roadIsImprovement = (goodsType == Goods.FURS) 
-                        || goodsType == Goods.LUMBER 
-                        || goodsType == Goods.ORE 
-                        || goodsType == Goods.SILVER;
-                int type = (roadIsImprovement) ? TileImprovement.BUILD_ROAD : TileImprovement.PLOW;
-                
-                if (type == TileImprovement.BUILD_ROAD && tile.canGetRoad()
-                        || type == TileImprovement.PLOW && tile.canBePlowed()) {
-                    if (ti == null) {
-                        return new TileImprovement(getAIMain(), tile, type, value);
-                    } else {
-                        ti.setType(type);
-                        ti.setValue(value);
-                        return ti;
-                    }
+            // Update to find the best thing to do now
+            TileImprovementType impType = TileImprovement.findBestTileImprovementType(tile, goodsType);
+            if (impType != null) {
+                int value = impType.getValue(tile.getType(), goodsType);
+                if (tip == null) {
+                    return new TileImprovementPlan(getAIMain(), tile, impType, value);
+                } else {
+                    tip.setType(impType);
+                    tip.setValue(value);
+                    return tip;
                 }
             }
         }
-        
         return null;
     }
 
@@ -174,39 +162,45 @@ public class WorkLocationPlan {
 
     
     /**
-    * Gets the production of the given type of goods according to this
-    * <code>WorkLocationPlan</code>. The plan has been created for either
-    * a {@link ColonyTile} or a {@link Building}. If this is a plan for a
-    * <code>ColonyTile</code> then the maximum possible production of the
-    * tile gets returned, while the <code>Building</code>-plans only returns
-    * a number used for identifying the value of the goods produced.
-    *
-    * @param goodsType The type of goods to get the production for.
-    * @return The production.
-    */
-    public int getProductionOf(int goodsType) {
+     * Gets the production of the given type of goods according to this
+     * <code>WorkLocationPlan</code>. The plan has been created for either
+     * a {@link ColonyTile} or a {@link Building}. If this is a plan for a
+     * <code>ColonyTile</code> then the maximum possible production of the
+     * tile gets returned, while the <code>Building</code>-plans only returns
+     * a number used for identifying the value of the goods produced.
+     *
+     * @param goodsType The type of goods to get the production for.
+     * @return The production.
+     */
+    public int getProductionOf(GoodsType goodsType) {
         if (goodsType != this.goodsType) {
             return 0;
         }
         
         if (workLocation instanceof ColonyTile) {
-            if (!Goods.isFarmedGoods(goodsType)) {
+            if (!goodsType.isFarmed()) {
                 return 0;
             }
 
             ColonyTile ct = (ColonyTile) workLocation;
             Tile t = ct.getWorkTile();
-            int expertUnitType = ct.getExpertForProducing(goodsType);
+            UnitType expertUnitType = FreeCol.getSpecification().getExpertForProducing(goodsType);
 
-            int base = t.getMaximumPotential(goodsType);
+            int base = t.getMaximumPotential(goodsType, expertUnitType);
 
             if (t.isLand() && base != 0) {
                 base++;
             }
-
-            return Unit.getProductionUsing(expertUnitType, goodsType, base, t) * ((goodsType == Goods.FURS) ? 2 : 1);
+            /**
+             * What's this supposed to be? Are we checking for the
+             * possible production bonus granted by Henry Hudson? If
+             * so, we should check all possible production bonuses instead.
+             *
+             * return expertUnitType.getProductionFor(goodsType, base) * ((goodsType == Goods.FURS) ? 2 : 1);
+             */
+            return expertUnitType.getProductionFor(goodsType, base);
         } else {
-            if (Goods.isFarmedGoods(goodsType)) {
+            if (goodsType.isFarmed()) {
                 return 0;
             } else {
                 /* These values are not really the production, but are
@@ -226,7 +220,6 @@ public class WorkLocationPlan {
         }
     }
 
-
     /**
     * Gets the type of goods which should be produced at the <code>WorkLocation</code>.
     *
@@ -234,7 +227,7 @@ public class WorkLocationPlan {
     * @see Goods
     * @see WorkLocation
     */
-    public int getGoodsType() {
+    public GoodsType getGoodsType() {
         return goodsType;
     }
     
@@ -246,7 +239,7 @@ public class WorkLocationPlan {
     * @see Goods
     * @see WorkLocation
     */
-    public void setGoodsType(int goodsType) {
+    public void setGoodsType(GoodsType goodsType) {
         this.goodsType = goodsType;
     }
 
@@ -262,7 +255,7 @@ public class WorkLocationPlan {
 
         element.setAttribute("ID", workLocation.getId());
         element.setAttribute("priority", Integer.toString(priority));
-        element.setAttribute("goodsType", Integer.toString(goodsType));
+        element.setAttribute("goodsType", goodsType.getId());
 
         return element;
     }
@@ -277,7 +270,7 @@ public class WorkLocationPlan {
     public void readFromXMLElement(Element element) {
         workLocation = (WorkLocation) getAIMain().getFreeColGameObject(element.getAttribute("ID"));
         priority = Integer.parseInt(element.getAttribute("priority"));
-        goodsType = Integer.parseInt(element.getAttribute("goodsType"));
+        goodsType = FreeCol.getSpecification().getGoodsType(element.getAttribute("goodsType"));
     }
 
 
