@@ -26,8 +26,10 @@ import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
+import net.sf.freecol.FreeCol;
 import net.sf.freecol.common.model.Location;
 import net.sf.freecol.common.model.Unit;
+import net.sf.freecol.common.model.UnitType;
 
 import org.w3c.dom.Element;
 
@@ -40,7 +42,7 @@ public class WorkerWish extends Wish {
     private static final Logger logger = Logger.getLogger(WorkerWish.class.getName());
 
 
-    private int unitType;
+    private UnitType unitType;
     private boolean expertNeeded;
 
 
@@ -58,7 +60,7 @@ public class WorkerWish extends Wish {
     * @param expertNeeded Determines wether the <code>unitType</code> is
     *       required or not.
     */
-    public WorkerWish(AIMain aiMain, Location destination, int value, int unitType, boolean expertNeeded) {
+    public WorkerWish(AIMain aiMain, Location destination, int value, UnitType unitType, boolean expertNeeded) {
         super(aiMain, getXMLElementTagName() + ":" + aiMain.getNextID());
 
         if (destination == null) {
@@ -121,7 +123,7 @@ public class WorkerWish extends Wish {
      * @param expertNeeded Determines wether the <code>unitType</code> is
      *       required or not.
      */
-    public void update(int value, int unitType, boolean expertNeeded) {
+    public void update(int value, UnitType unitType, boolean expertNeeded) {
         this.value = value;
         this.unitType = unitType;
         this.expertNeeded = expertNeeded;
@@ -131,7 +133,7 @@ public class WorkerWish extends Wish {
     * Returns the type of unit needed for releasing this wish.
     * @return The {@link Unit#getType type of unit}.
     */
-    public int getUnitType() {
+    public UnitType getUnitType() {
         return unitType;
     }
 
@@ -152,7 +154,7 @@ public class WorkerWish extends Wish {
         }
         out.writeAttribute("value", Integer.toString(value));
 
-        out.writeAttribute("unitType", Integer.toString(unitType));
+        out.writeAttribute("unitType", unitType.getId());
         out.writeAttribute("expertNeeded", Boolean.toString(expertNeeded));
 
         out.writeEndElement();
@@ -165,7 +167,7 @@ public class WorkerWish extends Wish {
      *      from the stream.
      */
     protected void readFromXMLImpl(XMLStreamReader in) throws XMLStreamException {        
-        id = in.getAttributeValue(null, "ID");
+        setId(in.getAttributeValue(null, "ID"));
         destination = (Location) getAIMain().getFreeColGameObject(in.getAttributeValue(null, "destination"));
         
         final String transportableStr = in.getAttributeValue(null, "transportable"); 
@@ -179,7 +181,7 @@ public class WorkerWish extends Wish {
         }
         value = Integer.parseInt(in.getAttributeValue(null, "value"));
 
-        unitType = Integer.parseInt(in.getAttributeValue(null, "unitType"));
+        unitType = FreeCol.getSpecification().getUnitType(in.getAttributeValue(null, "unitType"));
         expertNeeded = Boolean.valueOf(in.getAttributeValue(null, "expertNeeded")).booleanValue();
         in.nextTag();
     }
@@ -194,6 +196,6 @@ public class WorkerWish extends Wish {
     }
     
     public String toString() {
-        return "WorkerWish: " + Unit.getName(unitType) + " (" + expertNeeded + ")";
+        return "WorkerWish: " + unitType.getName() + " (" + expertNeeded + ")";
     }
 }
