@@ -29,6 +29,7 @@ import javax.xml.stream.XMLStreamWriter;
 
 import net.sf.freecol.common.model.GoalDecider;
 import net.sf.freecol.common.model.Map;
+import net.sf.freecol.common.model.Map.Direction;
 import net.sf.freecol.common.model.PathNode;
 import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.Unit;
@@ -132,13 +133,8 @@ public class CashInTreasureTrainMission extends Mission {
             PathNode bestPath = map.search(getUnit(), destinationDecider, Integer.MAX_VALUE);        
             
             if (bestPath != null) {
-                int direction = moveTowards(connection, bestPath);
-                if (direction >= 0) {
-                    final int mt = getUnit().getMoveType(direction);
-                    if (mt != Unit.ILLEGAL_MOVE && mt != Unit.ATTACK) {
-                        move(connection, direction);                    
-                    } 
-                }
+                Direction direction = moveTowards(connection, bestPath);
+                moveButDontAttack(connection, direction);
             }
         }
         
