@@ -28,14 +28,16 @@ import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
+import net.sf.freecol.FreeCol;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.Goods;
+import net.sf.freecol.common.model.GoodsType;
 import net.sf.freecol.common.model.GoodsContainer;
 import net.sf.freecol.common.model.IndianSettlement;
+import net.sf.freecol.common.model.Map.Direction;
 import net.sf.freecol.common.model.Player;
+import net.sf.freecol.common.model.Player.Stance;
 import net.sf.freecol.common.model.Tension;
-import net.sf.freecol.common.model.Tile;
-import net.sf.freecol.common.model.Unit;
 import net.sf.freecol.common.networking.Connection;
 import net.sf.freecol.common.networking.Message;
 import net.sf.freecol.server.ai.AIMain;
@@ -67,7 +69,7 @@ public class IndianBringGiftMission extends Mission {
     /** The <code>Colony</code> receiving the gift. */
     private Colony target;
 
-    /** Desides wether this mission has been completed or not. */
+    /** Decides whether this mission has been completed or not. */
     private boolean giftDelivered;
 
 
@@ -82,10 +84,11 @@ public class IndianBringGiftMission extends Mission {
         super(aiMain, aiUnit);
 
         this.target = target;
+        this.giftDelivered = false;
 
-        if (getUnit().getType() != Unit.BRAVE) {
-            logger.warning("Only an indian brave can be given the mission: IndianBringGiftMission");
-            throw new IllegalArgumentException("Only an indian brave can be given the mission: IndianBringGiftMission");
+        if (!getUnit().getOwner().isIndian() || !getUnit().canCarryGoods()) {
+            logger.warning("Only an indian which can carry goods can be given the mission: IndianBringGiftMission");
+            throw new IllegalArgumentException("Only an indian which can carry goods can be given the mission: IndianBringGiftMission");
         }
     }
 
@@ -121,24 +124,26 @@ public class IndianBringGiftMission extends Mission {
      * @param connection The <code>Connection</code> to the server.
      */
     public void doMission(Connection connection) {
+        
+        if (!isValid()) {
+            // the destination colony may have been destroyed
+            return;
+        }
+        
         if (!hasGift()) {
             if (getUnit().getTile() != getUnit().getIndianSettlement().getTile()) {
                 // Move to the owning settlement:
-                int r = moveTowards(connection, getUnit().getIndianSettlement().getTile());
-                if (r >= 0) {
-                    final int mt = getUnit().getMoveType(r);
-                    if (mt != Unit.ILLEGAL_MOVE && mt != Unit.ATTACK) {
-                        move(connection, r);
-                    }
-                }
+                Direction r = moveTowards(connection, getUnit().getIndianSettlement().getTile());
+                moveButDontAttack(connection, r);
             } else {
                 // Load the goods:
                 List<Goods> goodsList = new ArrayList<Goods>();
                 GoodsContainer gc = getUnit().getIndianSettlement().getGoodsContainer();
-                for (int i = 1; i <= 4; i++) {
-                    if (gc.getGoodsCount(i) >= IndianSettlement.KEEP_RAW_MATERIAL + 25) {
-                        goodsList.add(new Goods(getGame(), getUnit().getIndianSettlement(), i,
-                                getRandom().nextInt(15) + 10));
+                for (GoodsType goodsType : FreeCol.getSpecification().getNewWorldGoodsTypeList()) {
+                    if (gc.getGoodsCount(goodsType) >= IndianSettlement.KEEP_RAW_MATERIAL + 25) {
+                        goodsList.add(new Goods(getGame(), getUnit().getIndianSettlement(),
+                                                goodsType,
+                                                getRandom().nextInt(15) + 10));
                     }
                 }
 
@@ -149,8 +154,9 @@ public class IndianBringGiftMission extends Mission {
             }
         } else {
             // Move to the target's colony and deliver
-            int r = moveTowards(connection, target.getTile());
-            if (r >= 0 && getGame().getMap().getNeighbourOrNull(r, getUnit().getTile()) == target.getTile()) {
+            Direction r = moveTowards(connection, target.getTile());
+            if (r != null && 
+                getGame().getMap().getNeighbourOrNull(r, getUnit().getTile()) == target.getTile()) {
                 // We have arrived.
                 Element deliverGiftElement = Message.createNewRootElement("deliverGift");
                 deliverGiftElement.setAttribute("unit", getUnit().getId());
@@ -170,32 +176,7 @@ public class IndianBringGiftMission extends Mission {
         }
 
         // Walk in a random direction if we have any moves left:
-        Tile thisTile = getUnit().getTile();
-        Unit unit = getUnit();
-        while (unit.getMovesLeft() > 0) {
-            int direction = (int) (Math.random() * 8);
-            int j;
-            for (j = 8; j > 0
-                    && ((unit.getGame().getMap().getNeighbourOrNull(direction, thisTile) == null)
-                    || (unit.getMoveType(direction) == Unit.ILLEGAL_MOVE)
-                    || (unit.getMoveType(direction) == Unit.ATTACK)
-                    ); j--) {
-                direction = (int) (Math.random() * 8);
-            }
-            if (j == 0)
-                break;
-            thisTile = unit.getGame().getMap().getNeighbourOrNull(direction, thisTile);
-
-            Element moveElement = Message.createNewRootElement("move");
-            moveElement.setAttribute("unit", unit.getId());
-            moveElement.setAttribute("direction", Integer.toString(direction));
-
-            try {
-                connection.sendAndWait(moveElement);
-            } catch (IOException e) {
-                logger.warning("Could not send \"move\"-message!");
-            }
-        }
+        moveRandomly(connection);
     }
 
     /**
@@ -215,8 +196,8 @@ public class IndianBringGiftMission extends Mission {
      * <BR>
      * 
      * This mission will be invalidated when the gift has been delivered. In
-     * case of the stances {@link Player#WAR WAR} or
-     * {@link Player#CEASE_FIRE CEASE_FIRE} towards the target player, the
+     * case of the stances {@link Player#Stance#WAR WAR} or
+     * {@link Player#Stance#CEASE_FIRE CEASE_FIRE} towards the target player, the
      * mission would be invalidated as well.
      * 
      * @return <code>true</code> if this mission is still valid.
@@ -236,9 +217,14 @@ public class IndianBringGiftMission extends Mission {
      *         with regard to the tension towards the target player.
      */
     public static boolean isValidMission(Player owner, Player targetPlayer) {
-        int stance = owner.getStance(targetPlayer);
-        return (stance != Player.WAR && stance != Player.CEASE_FIRE)
-                && owner.getTension(targetPlayer).getLevel() <= Tension.HAPPY;
+        Stance stance = owner.getStance(targetPlayer);
+        if (stance == null) {
+            return false;
+        } else {
+            return (stance != Stance.WAR && stance != Stance.CEASE_FIRE) &&
+                owner.getTension(targetPlayer).getLevel() != null && 
+                owner.getTension(targetPlayer).getLevel().compareTo(Tension.Level.HAPPY) <= 0;
+        }
     }
 
     /**
@@ -253,7 +239,10 @@ public class IndianBringGiftMission extends Mission {
         out.writeStartElement(getXMLElementTagName());
 
         out.writeAttribute("unit", getUnit().getId());
-        out.writeAttribute("target", target.getId());
+        if (target!=null) {
+            // the destination colony is still alive
+            out.writeAttribute("target", target.getId());
+        }
         out.writeAttribute("giftDelivered", Boolean.toString(giftDelivered));
 
         out.writeEndElement();
