@@ -26,7 +26,9 @@ import java.util.logging.Logger;
 import net.sf.freecol.common.model.CombatModel;
 import net.sf.freecol.common.model.GoalDecider;
 import net.sf.freecol.common.model.Map.Direction;
+import net.sf.freecol.common.model.Goods;
 import net.sf.freecol.common.model.PathNode;
+import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Tension;
 import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.Unit;
@@ -157,8 +159,10 @@ public abstract class Mission extends AIObject {
                     break;
                 }
             }
-            if (j == randomDirections.length)
+            if (j == randomDirections.length){
+            	unit.setMovesLeft(0);
                 break;
+            }
             thisTile = unit.getGame().getMap().getNeighbourOrNull(direction, thisTile);
 
             move(connection, direction);
@@ -184,6 +188,27 @@ public abstract class Mission extends AIObject {
         }
     }
     
+    protected void moveUnitToAmerica(Connection connection, Unit unit) {
+        Element moveToAmericaElement = Message.createNewRootElement("moveToAmerica");
+        moveToAmericaElement.setAttribute("unit", unit.getId());
+        try {
+            connection.sendAndWait(moveToAmericaElement);
+        } catch (IOException e) {
+            logger.warning("Could not send \"moveToAmericaElement\"-message!");
+        }
+    }
+    
+    protected void moveUnitToEurope(Connection connection, Unit unit) {
+        Element moveToAmericaElement = Message.createNewRootElement("moveToEurope");
+        moveToAmericaElement.setAttribute("unit", unit.getId());
+        try {
+            connection.sendAndWait(moveToAmericaElement);
+        } catch (IOException e) {
+            logger.warning("Could not send \"moveToAmericaElement\"-message!");
+        }
+    }
+    
+    
     protected void moveButDontAttack(Connection connection, Direction direction) {
         if (direction != null) {
             switch (getUnit().getMoveType(direction)) {
@@ -230,6 +255,7 @@ public abstract class Mission extends AIObject {
         
         GoalDecider gd = new GoalDecider() {
             private PathNode bestTarget = null;
+            private int higherTension = 0;
             
             public PathNode getGoal() {
                 return bestTarget;              
@@ -243,46 +269,61 @@ public abstract class Mission extends AIObject {
                 CombatModel combatModel = getGame().getCombatModel();
                 Tile newTile = pathNode.getTile();
                 Unit defender = newTile.getDefendingUnit(unit);
-                if ((newTile.isLand() && !unit.isNaval() || !newTile.isLand() && unit.isNaval()) &&
-                        defender != null && defender.getOwner() != unit.getOwner()) {
-
-                    int tension = 0;
-                    Tension alarm = unit.getOwner().getTension(defender.getOwner());
-                    if (alarm != null) {
-                        tension = alarm.getValue();
-                    }
-                    if (unit.getIndianSettlement() != null &&
-                        unit.getIndianSettlement().getAlarm(defender.getOwner()) != null) {
-                        tension += unit.getIndianSettlement().getAlarm(defender.getOwner()).getValue();
-                    }
-                    if (defender.canCarryTreasure()) {
-                        tension += Math.min(defender.getTreasureAmount() / 10, 600);
-                    }
-                    if (defender.getType().getDefence() > 0 &&
-                        newTile.getSettlement() == null) {
-                        tension += 100 - combatModel.getDefencePower(unit, defender) * 2;
-                    }
-                    if (defender.hasAbility("model.ability.expertSoldier") &&
-                        !defender.isArmed()) {
-                        tension += 50 - combatModel.getDefencePower(unit, defender) * 2;
-                    }
-                    // TODO-AI-CHEATING: REMOVE WHEN THE AI KNOWNS HOW TO HANDLE PEACE WITH THE INDIANS:
-                    if (unit.getOwner().isIndian() 
-                            && defender.getOwner().isAI()) {
-                        tension -= 200;
-                    }
-                    // END: TODO-AI-CHEATING
-                    if (tension > Tension.Level.CONTENT.getLimit()) {
-                        if (bestTarget == null) {
-                            bestTarget = pathNode;                           
-                        } else if (bestTarget.getTurns() == pathNode.getTurns()) {
-                            // TODO: Check if the new target is better than the previous:
-                        }
-                    }
-                    return true;
-                } else {
+                
+                if( defender == null){
                     return false;
                 }
+                
+                if( defender.getOwner() == unit.getOwner()){
+                    return false;
+                }
+                
+                if (newTile.isLand() && unit.isNaval() || !newTile.isLand() && !unit.isNaval()) {
+                    return false;
+                }
+
+                int tension = 0;
+                Tension alarm = unit.getOwner().getTension(defender.getOwner());
+                if (alarm != null) {
+                    tension = alarm.getValue();
+                }
+                if (unit.getIndianSettlement() != null &&
+                        unit.getIndianSettlement().getAlarm(defender.getOwner()) != null) {
+                    tension += unit.getIndianSettlement().getAlarm(defender.getOwner()).getValue();
+                }
+                if (defender.canCarryTreasure()) {
+                    tension += Math.min(defender.getTreasureAmount() / 10, 600);
+                }
+                if (defender.getType().getDefence() > 0 &&
+                        newTile.getSettlement() == null) {
+                    tension += 100 - combatModel.getDefencePower(unit, defender) * 2;
+                }
+                if (defender.hasAbility("model.ability.expertSoldier") &&
+                        !defender.isArmed()) {
+                    tension += 50 - combatModel.getDefencePower(unit, defender) * 2;
+                }
+                if (unit.hasAbility("model.ability.piracy")){
+                	tension += PrivateerMission.getModifierValueForTarget(combatModel, unit, defender);
+                }
+                // TODO-AI-CHEATING: REMOVE WHEN THE AI KNOWNS HOW TO HANDLE PEACE WITH THE INDIANS:
+                if (unit.getOwner().isIndian() 
+                        && defender.getOwner().isAI()) {
+                    tension -= 200;
+                }
+                // END: TODO-AI-CHEATING
+                if (tension > Tension.Level.CONTENT.getLimit()) {
+                    if (bestTarget == null) {
+                        bestTarget = pathNode;
+                        higherTension = tension;
+                        return true;
+                    } else if (bestTarget.getTurns() == pathNode.getTurns()
+                                 && tension > higherTension) {
+                        bestTarget = pathNode;
+                        higherTension = tension;
+                        return true;
+                    }
+                }
+                return false;
             }
         };
         return getGame().getMap().search(getUnit(), gd, maxTurns);
@@ -355,6 +396,39 @@ public abstract class Mission extends AIObject {
         }
     }
     
+    protected boolean unloadCargoInColony(Connection connection, Unit carrier, Goods goods) {
+        Element unloadCargoElement = Message.createNewRootElement("unloadCargo");
+        unloadCargoElement.appendChild(goods.toXMLElement(carrier.getOwner(),
+                unloadCargoElement.getOwnerDocument()));
+        try {
+            connection.sendAndWait(unloadCargoElement);
+        } catch (IOException e) {
+            logger.warning("Could not send \"unloadCargoElement\"-message!");
+            return false;
+        }
+        return true;
+    }
+
+    protected boolean sellCargoInEurope(Connection connection, Unit carrier, Goods goods) {
+        // TODO-AI-CHEATING: REMOVE WHEN THE AI IS GOOD ENOUGH:
+        Player p = carrier.getOwner();
+        if (p.isAI() && getAIMain().getFreeColServer().isSingleplayer()) {
+            // Double the income by adding this bonus:
+            p.modifyGold(p.getMarket().getSalePrice(goods));
+        }
+        // END: TODO-AI-CHEATING.
+        Element sellGoodsElement = Message.createNewRootElement("sellGoods");
+        sellGoodsElement.appendChild(goods.toXMLElement(carrier.getOwner(),
+                sellGoodsElement.getOwnerDocument()));
+        try {
+            connection.sendAndWait(sellGoodsElement);
+        } catch (IOException e) {
+            logger.warning("Could not send \"sellGoodsElement\"-message!");
+            return false;
+        }
+        return true;
+    }
+    
     /**
      * Disposes this mission by removing any references to it.
      */
@@ -428,4 +502,29 @@ public abstract class Mission extends AIObject {
     public String getDebuggingInfo() {
         return "";
     }
+
+
+	public void attack(Connection connection, Unit unit, Direction direction) {
+		assert direction != null;
+		Element element = Message.createNewRootElement("attack");
+		element.setAttribute("unit", unit.getId());
+		element.setAttribute("direction", direction.toString());
+	
+		try {
+		    connection.ask(element);
+		} catch (IOException e) {
+		    logger.warning("Could not send message!");
+		}
+	}
+
+
+	protected void unitLeavesShip(Connection connection, Unit u) {
+		Element leaveShipElement = Message.createNewRootElement("leaveShip");
+		leaveShipElement.setAttribute("unit", u.getId());
+		try {
+		    connection.sendAndWait(leaveShipElement);
+		} catch (IOException e) {
+		    logger.warning("Could not send \"leaveShipElement\"-message!");
+		}
+	}
 }
