@@ -28,11 +28,19 @@ import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
+import net.sf.freecol.FreeCol;
+import net.sf.freecol.common.model.AbstractGoods;
+import net.sf.freecol.common.model.EquipmentType;
 import net.sf.freecol.common.model.GoalDecider;
-import net.sf.freecol.common.model.Goods;
 import net.sf.freecol.common.model.PathNode;
 import net.sf.freecol.common.model.Tile;
+import net.sf.freecol.common.model.TileImprovement;
+import net.sf.freecol.common.model.TileItemContainer;
 import net.sf.freecol.common.model.Unit;
+import net.sf.freecol.common.model.Map.Direction;
+import net.sf.freecol.common.model.Unit.MoveType;
+import net.sf.freecol.common.model.Unit.Role;
+import net.sf.freecol.common.model.Unit.UnitState;
 import net.sf.freecol.common.networking.Connection;
 import net.sf.freecol.common.networking.Message;
 import net.sf.freecol.server.ai.AIColony;
@@ -40,7 +48,7 @@ import net.sf.freecol.server.ai.AIMain;
 import net.sf.freecol.server.ai.AIObject;
 import net.sf.freecol.server.ai.AIPlayer;
 import net.sf.freecol.server.ai.AIUnit;
-import net.sf.freecol.server.ai.TileImprovement;
+import net.sf.freecol.server.ai.TileImprovementPlan;
 
 import org.w3c.dom.Element;
 
@@ -48,18 +56,19 @@ import org.w3c.dom.Element;
 /**
  * Mission for controlling a pioneer.
  * 
- * @see Unit#isPioneer
+ * @see Unit#Role#PIONNER
  */
 public class PioneeringMission extends Mission {
     /* 
-     * TODO-LATER: "updateTileImprovement" should be called
+     * TODO-LATER: "updateTileImprovementPlan" should be called
      *             only once (in the beginning of the turn).
      */
     
     private static final Logger logger = Logger.getLogger(PioneeringMission.class.getName());
 
+    private static final EquipmentType toolsType = FreeCol.getSpecification().getEquipmentType("model.equipment.tools");
 
-    private TileImprovement tileImprovement = null;
+    private TileImprovementPlan tileImprovementPlan = null;
     
     /**
      * Temporary variable for skipping the mission.
@@ -110,33 +119,52 @@ public class PioneeringMission extends Mission {
      * Disposes this <code>Mission</code>.
      */
     public void dispose() {
-        if (tileImprovement != null) {
-            tileImprovement.setPioneer(null);
-            tileImprovement = null;
+        if (tileImprovementPlan != null) {
+            tileImprovementPlan.setPioneer(null);
+            tileImprovementPlan = null;
         }
         super.dispose();
     }
 
     /**
-     * Sets the <code>TileImprovement</code> which should
+     * Sets the <code>TileImprovementPlan</code> which should
      * be the next target.
      * 
-     * @param tileImprovement The <code>TileImprovement</code>.
+     * @param tileImprovementPlan The <code>TileImprovementPlan</code>.
      */
-    public void setTileImprovement(TileImprovement tileImprovement) {
-        this.tileImprovement = tileImprovement;
+    public void setTileImprovementPlan(TileImprovementPlan tileImprovementPlan) {
+        this.tileImprovementPlan = tileImprovementPlan;
     }
 
-    private void updateTileImprovement() {
-        if (tileImprovement != null) {
+    private void updateTileImprovementPlan() {
+        final AIPlayer aiPlayer = (AIPlayer) getAIMain().getAIObject(getUnit().getOwner().getId());
+        final Unit carrier = (getUnit().isOnCarrier()) ? (Unit) getUnit().getLocation() : null;
+        
+        Tile improvementTarget = (tileImprovementPlan != null)? tileImprovementPlan.getTarget():null;
+        // invalid tileImprovementPlan, remove and get a new valid one
+        if (tileImprovementPlan != null && improvementTarget == null) {
+        	logger.warning("Found invalid TileImprovementPlan, removing it and assigning a new one");
+        	aiPlayer.removeTileImprovementPlan(tileImprovementPlan);
+        	tileImprovementPlan.dispose();
+        }
+        
+        // Verify if the improvement has been applied already
+        // If it has, remove this improvement
+        if( tileImprovementPlan != null &&
+            improvementTarget != null &&
+            improvementTarget.hasImprovement(tileImprovementPlan.getType())){
+                aiPlayer.removeTileImprovementPlan(tileImprovementPlan);
+                tileImprovementPlan.dispose();
+        }
+        
+        // mission still valid, no update needed
+        if (tileImprovementPlan != null && improvementTarget != null) {
             return;
         }
-        final AIPlayer aiPlayer = (AIPlayer) getAIMain().getAIObject(getUnit().getOwner().getId());
-        final Unit carrier = (getUnit().getLocation() instanceof Unit) ? (Unit) getUnit().getLocation() : null;
         
         final Tile startTile;
         if (getUnit().getTile() == null) {
-            if (getUnit().getLocation() instanceof Unit) {
+            if (getUnit().isOnCarrier()) {
                 startTile = (Tile) ((Unit) getUnit().getLocation()).getEntryLocation();
             } else {
                 startTile = (Tile) getUnit().getOwner().getEntryLocation();
@@ -145,12 +173,20 @@ public class PioneeringMission extends Mission {
             startTile = getUnit().getTile();
         }
                 
-        TileImprovement bestChoice = null;
+        TileImprovementPlan bestChoice = null;
         int bestValue = 0;
-        Iterator<TileImprovement> tiIterator = aiPlayer.getTileImprovementIterator();            
+        Iterator<TileImprovementPlan> tiIterator = aiPlayer.getTileImprovementPlanIterator();            
         while (tiIterator.hasNext()) {
-            TileImprovement ti = tiIterator.next();
+            TileImprovementPlan ti = tiIterator.next();
             if (ti.getPioneer() == null) {
+            	// invalid tileImprovementPlan, remove and get a new valid one
+                if (ti.getTarget() == null) {
+                	logger.warning("Found invalid TileImprovementPlan, removing it and finding a new one");
+                	aiPlayer.removeTileImprovementPlan(ti);
+                	ti.dispose();
+                	continue;
+                }
+            	
                 PathNode path = null;
                 int value;
                 if (startTile != ti.getTarget()) {
@@ -159,7 +195,7 @@ public class PioneeringMission extends Mission {
                         value = ti.getValue() + 10000 - (path.getTotalTurns()*5);
                         
                         /*
-                         * Avoid picking a TileImprovement with a path being blocked 
+                         * Avoid picking a TileImprovementPlan with a path being blocked 
                          * by an enemy unit (apply a penalty to the value):
                          */
                         PathNode pn = path;
@@ -184,7 +220,7 @@ public class PioneeringMission extends Mission {
         }
         
         if (bestChoice != null) {
-            tileImprovement = bestChoice;
+            tileImprovementPlan = bestChoice;
             bestChoice.setPioneer(getAIUnit());
         }    
     }
@@ -204,13 +240,11 @@ public class PioneeringMission extends Mission {
             public boolean check(Unit u, PathNode pathNode) {
                 Tile t = pathNode.getTile();
                 boolean target = false;
-                if (t.getColony() != null 
-                        && t.getColony().getOwner() == u.getOwner()
-                        && t.getColony().getGoodsContainer().getGoodsCount(Goods.TOOLS) >= 20) {
+                if (t.getColony() != null && 
+                    t.getColony().getOwner() == u.getOwner() &&
+                    t.getColony().canBuildEquipment(toolsType)) {
                     AIColony ac = (AIColony) getAIMain().getAIObject(t.getColony());
-                    if (ac.getAvailableTools() >= 20) {
-                        target = true;
-                    }
+                    target = ac.canBuildEquipment(toolsType);
                 }
                 if (target) {
                     best = pathNode;
@@ -231,71 +265,109 @@ public class PioneeringMission extends Mission {
         }
         
         if (getUnit().getTile() != null) {
-            if (getUnit().getNumberOfTools() == 0) {
+            if (!getUnit().hasAbility("model.ability.improveTerrain")) {
                 // Get tools from a Colony.
                 if (getUnit().getColony() == null) {
                     PathNode bestPath = findColonyWithTools();        
 
                     if (bestPath != null) {
-                        int direction = moveTowards(connection, bestPath);
-                        if (direction >= 0) {
-                            final int mt = getUnit().getMoveType(direction);
-                            if (mt != Unit.ILLEGAL_MOVE && mt != Unit.ATTACK) {
-                                move(connection, direction);                    
-                            }
-                        }
+                        Direction direction = moveTowards(connection, bestPath);
+                        moveButDontAttack(connection, direction);
                     } else {
                         skipMission = true;
                     }
                 }
                 if (getUnit().getColony() != null) {
                     AIColony ac = (AIColony) getAIMain().getAIObject(getUnit().getColony());
-                    final int tools = ac.getAvailableTools();
-                    if (tools >= 20) {                    
-                        Element equipUnitElement = Message.createNewRootElement("equipunit");
+                    int amount = toolsType.getMaximumCount();
+                    for (AbstractGoods materials : toolsType.getGoodsRequired()) {
+                        int availableAmount = ac.getAvailableGoods(materials.getType());
+                        int requiredAmount = materials.getAmount();
+                        if (availableAmount < requiredAmount) {
+                            skipMission = true;
+                            break;
+                        } else {
+                            amount = Math.min(amount, availableAmount / requiredAmount);
+                        }
+                    }
+                    if (!skipMission) {
+                        Element equipUnitElement = Message.createNewRootElement("equipUnit");
                         equipUnitElement.setAttribute("unit", getUnit().getId());
-                        equipUnitElement.setAttribute("type", Integer.toString(Goods.TOOLS));
-                        equipUnitElement.setAttribute("amount", Integer.toString(Math.min(tools - tools % 20, 100)));
+                        equipUnitElement.setAttribute("type", toolsType.getId());
+                        equipUnitElement.setAttribute("amount", Integer.toString(amount));
                         try {
                             connection.sendAndWait(equipUnitElement);
                         } catch (Exception e) {
                             logger.warning("Could not send equip message.");
                         }
-                    } else {
-                        skipMission = true;
                     }
                 }
                 return;
             }
         }
         
-        if (tileImprovement == null) {
-            updateTileImprovement();
+        if (tileImprovementPlan == null) {
+            updateTileImprovementPlan();
         }
         
-        if (tileImprovement != null) {
+        if (tileImprovementPlan != null) {
             if (getUnit().getTile() != null) {
-                if (getUnit().getTile() != tileImprovement.getTarget()) {
-                    PathNode pathToTarget = getUnit().findPath(tileImprovement.getTarget());
+                // move toward the target tile
+                if (getUnit().getTile() != tileImprovementPlan.getTarget()) {
+                    PathNode pathToTarget = getUnit().findPath(tileImprovementPlan.getTarget());
                     if (pathToTarget != null) {
-                        int direction = moveTowards(connection, pathToTarget);
-                        if (direction >= 0 && (getUnit().getMoveType(direction) == Unit.MOVE
-                                || getUnit().getMoveType(direction) == Unit.EXPLORE_LOST_CITY_RUMOUR)) {
+                        Direction direction = moveTowards(connection, pathToTarget);
+                        if (direction != null &&
+                            (getUnit().getMoveType(direction) == MoveType.MOVE
+                             || getUnit().getMoveType(direction) == MoveType.EXPLORE_LOST_CITY_RUMOUR)) {
                             move(connection, direction);
                         }
                     }
                 }
-                if (getUnit().getTile() == tileImprovement.getTarget()
-                        && getUnit().getState() != tileImprovement.getType()
-                        && getUnit().checkSetState(tileImprovement.getType())) {
-                    Element changeStateElement = Message.createNewRootElement("changeState");
-                    changeStateElement.setAttribute("unit", getUnit().getId());
-                    changeStateElement.setAttribute("state", Integer.toString(tileImprovement.getType()));
+                // are we there yet ?
+                if (getUnit().getTile() == tileImprovementPlan.getTarget()
+                    && getUnit().getState() != UnitState.IMPROVING
+                    && getUnit().checkSetState(UnitState.IMPROVING)) {
+                    // start improving now
+                    int price = getUnit().getOwner().getLandPrice(getUnit().getTile());
+                    // Buy the land from the Indians first?
+                    if (price > 0) {
+                        // TODO: the AI should buy the land, to avoid indian wars
+                    }
+                    // ask to create the TileImprovement
+                    Element changeWorkTypeElement = Message.createNewRootElement("workImprovement");
+                    changeWorkTypeElement.setAttribute("unit", getUnit().getId());
+                    changeWorkTypeElement.setAttribute("improvementType", tileImprovementPlan.getType().getId());
+                    Element reply = null;
                     try {
-                        connection.sendAndWait(changeStateElement);
+                        reply = connection.ask(changeWorkTypeElement);
                     } catch (IOException e) {
                         logger.warning("Could not send message!");
                     }
+                    if (reply!=null && reply.getTagName().equals("workImprovementConfirmed")) {
+                        // get the TileItemContainer
+                        Element containerElement = (Element)reply.getElementsByTagName(TileItemContainer.getXMLElementTagName()).item(0);
+                        if (containerElement != null) {
+                            TileItemContainer container = (TileItemContainer) getGame().getFreeColGameObject(containerElement.getAttribute("ID"));
+                            if (container == null) {
+                                container = new TileItemContainer(getGame(), getUnit().getTile(), containerElement);
+                                getUnit().getTile().setTileItemContainer(container);
+                            } else {
+                                container.readFromXMLElement(containerElement);
+                            }
+                        }
+                        // get the TileImprovement
+                        Element improvementElement = (Element)reply.getElementsByTagName(TileImprovement.getXMLElementTagName()).item(0);
+                        if (improvementElement!=null) {
+                            TileImprovement improvement = (TileImprovement) getGame().getFreeColGameObject(improvementElement.getAttribute("ID"));
+                            if (improvement == null) {
+                                improvement = new TileImprovement(getGame(), improvementElement);
+                                getUnit().getTile().add(improvement);
+                            } else {
+                                improvement.readFromXMLElement(improvementElement);
+                            }
+                        }
+                    }
                 }
             }
         }        
@@ -312,16 +384,16 @@ public class PioneeringMission extends Mission {
      * @return The destination for this <code>Transportable</code>.
      */    
     public Tile getTransportDestination() {
-        updateTileImprovement();
-        if (tileImprovement == null) {
+        updateTileImprovementPlan();
+        if (tileImprovementPlan == null) {
             return null;
         }
-        if (getUnit().getLocation() instanceof Unit) {
-            return tileImprovement.getTarget();
-        } else if (getUnit().getTile() == tileImprovement.getTarget()) {
+        if (getUnit().isOnCarrier()) {
+            return tileImprovementPlan.getTarget();
+        } else if (getUnit().getTile() == tileImprovementPlan.getTarget()) {
             return null;
-        } else if (getUnit().getTile() == null || getUnit().findPath(tileImprovement.getTarget()) == null) {
-            return tileImprovement.getTarget();
+        } else if (getUnit().getTile() == null || getUnit().findPath(tileImprovementPlan.getTarget()) == null) {
+            return tileImprovementPlan.getTarget();
         } else {
             return null;
         }
@@ -348,8 +420,9 @@ public class PioneeringMission extends Mission {
      *         and <code>false</code> otherwise.
      */
     public boolean isValid() {  
-        updateTileImprovement();
-        return !skipMission && (tileImprovement != null) && (getUnit().isPioneer() || getUnit().getType() == Unit.HARDY_PIONEER);
+        updateTileImprovementPlan();
+        return !skipMission && tileImprovementPlan != null &&
+                (getUnit().getRole() == Role.PIONEER || getUnit().hasAbility("model.ability.expertPioneer"));
     }
 
     /**
@@ -361,9 +434,9 @@ public class PioneeringMission extends Mission {
      */    
     public static boolean isValid(AIUnit aiUnit) {
         AIPlayer aiPlayer = (AIPlayer) aiUnit.getAIMain().getAIObject(aiUnit.getUnit().getOwner().getId());
-        Iterator<TileImprovement> tiIterator = aiPlayer.getTileImprovementIterator();            
+        Iterator<TileImprovementPlan> tiIterator = aiPlayer.getTileImprovementPlanIterator();            
         while (tiIterator.hasNext()) {
-            TileImprovement ti = tiIterator.next();
+            TileImprovementPlan ti = tiIterator.next();
             if (ti.getPioneer() == null) {
                 return true;
             }
@@ -383,8 +456,8 @@ public class PioneeringMission extends Mission {
         out.writeStartElement(getXMLElementTagName());
         
         out.writeAttribute("unit", getUnit().getId());
-        if (tileImprovement != null) {
-            out.writeAttribute("tileImprovement", tileImprovement.getId());
+        if (tileImprovementPlan != null) {
+            out.writeAttribute("tileImprovementPlan", tileImprovementPlan.getId());
         }
 
         out.writeEndElement();
@@ -398,14 +471,14 @@ public class PioneeringMission extends Mission {
     protected void readFromXMLImpl(XMLStreamReader in) throws XMLStreamException {
         setAIUnit((AIUnit) getAIMain().getAIObject(in.getAttributeValue(null, "unit")));
         
-        final String tileImprovementStr = in.getAttributeValue(null, "tileImprovement");
-        if (tileImprovementStr != null) {
-            tileImprovement = (TileImprovement) getAIMain().getAIObject(tileImprovementStr);
-            if (tileImprovement == null) {
-                tileImprovement = new TileImprovement(getAIMain(), tileImprovementStr);
+        final String tileImprovementPlanStr = in.getAttributeValue(null, "tileImprovementPlan");
+        if (tileImprovementPlanStr != null) {
+            tileImprovementPlan = (TileImprovementPlan) getAIMain().getAIObject(tileImprovementPlanStr);
+            if (tileImprovementPlan == null) {
+                tileImprovementPlan = new TileImprovementPlan(getAIMain(), tileImprovementPlanStr);
             }
         } else {
-            tileImprovement = null;
+            tileImprovementPlan = null;
         }
         
         in.nextTag();
@@ -416,7 +489,7 @@ public class PioneeringMission extends Mission {
      * @return The <code>String</code> "wishRealizationMission".
      */
     public static String getXMLElementTagName() {
-        return "tileImprovementMission";
+        return "tileImprovementPlanMission";
     }
     
     /**
@@ -432,9 +505,9 @@ public class PioneeringMission extends Mission {
      *      </ul>
      */
     public String getDebuggingInfo() {
-        if (tileImprovement != null) {
-            final String action = (tileImprovement.getType() == Unit.PLOW) ? "P" : "R";
-            return tileImprovement.getTarget().getPosition().toString() + " " + action;
+        if (tileImprovementPlan != null) {
+            final String action = tileImprovementPlan.getType().getName();
+            return tileImprovementPlan.getTarget().getPosition().toString() + " " + action;
         } else {
             PathNode bestPath = findColonyWithTools();
             if (bestPath != null) {
