@@ -27,16 +27,21 @@ import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
+import net.sf.freecol.FreeCol;
+import net.sf.freecol.common.model.AbstractGoods;
+import net.sf.freecol.common.model.EquipmentType;
 import net.sf.freecol.common.model.GoalDecider;
-import net.sf.freecol.common.model.Goods;
 import net.sf.freecol.common.model.IndianSettlement;
 import net.sf.freecol.common.model.Map;
 import net.sf.freecol.common.model.PathNode;
 import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.Unit;
+import net.sf.freecol.common.model.Map.Direction;
 import net.sf.freecol.common.model.Map.Position;
+import net.sf.freecol.common.model.Unit.MoveType;
 import net.sf.freecol.common.networking.Connection;
 import net.sf.freecol.common.networking.Message;
+import net.sf.freecol.server.ai.AIColony;
 import net.sf.freecol.server.ai.AIMain;
 import net.sf.freecol.server.ai.AIObject;
 import net.sf.freecol.server.ai.AIUnit;
@@ -46,16 +51,16 @@ import org.w3c.dom.Element;
 /**
  * Mission for controlling a scout.
  * 
- * @see Unit#isScout
+ * @see Unit#Role#SCOUT
  */
 public class ScoutingMission extends Mission {
+
     private static final Logger logger = Logger.getLogger(ScoutingMission.class.getName());
 
-
-
-
     private boolean valid = true;
 
+    private EquipmentType scoutEquipment;
+
     private Tile transportDestination = null;
 
     // Debug variable:
@@ -120,33 +125,33 @@ public class ScoutingMission extends Mission {
             return;
         }
 
-        if (!getUnit().isMounted()) {
-            if (getUnit().getColony() != null
-                    && getUnit().getColony().getGoodsContainer().getGoodsCount(Goods.HORSES) >= 50) {
-                if (getUnit().getColony().getFoodProduction()
-                        - getUnit().getColony().getFoodConsumption() < 2
-                        || getUnit().getColony().getGoodsContainer().getGoodsCount(Goods.HORSES) >= 52) {
-                    Element equipUnitElement = Message.createNewRootElement("equipunit");
-                    equipUnitElement.setAttribute("unit", getUnit().getId());
-                    equipUnitElement.setAttribute("type", Integer.toString(Goods.HORSES));
-                    equipUnitElement.setAttribute("amount", Integer.toString(50));
-                    try {
-                        connection.ask(equipUnitElement);
-                    } catch (IOException e) {
-                        logger.warning("Could not send \"equipUnit (50)\"-message!");
+        if (getUnit().getRole() != Unit.Role.SCOUT) {
+            if (getUnit().getColony() != null) {
+                AIColony colony = (AIColony) getAIMain().getAIObject(getUnit().getColony());
+                for (EquipmentType equipment : FreeCol.getSpecification().getEquipmentTypeList()) {
+                    if (equipment.getRole() == Unit.Role.SCOUT &&
+                        getUnit().canBeEquippedWith(equipment) && 
+                        colony.canBuildEquipment(equipment)) {
+                        Element equipUnitElement = Message.createNewRootElement("equipUnit");
+                        equipUnitElement.setAttribute("unit", getUnit().getId());
+                        equipUnitElement.setAttribute("type", equipment.getId());
+                        equipUnitElement.setAttribute("amount", "1");
+                        try {
+                            connection.ask(equipUnitElement);
+                            scoutEquipment = equipment;
+                        } catch (IOException e) {
+                            logger.warning("Could not send \"equipUnit\"-message!");
+                        }
+                        return;
                     }
-                    return;
-                } else {
-                    debugAction = "Awaiting 52 horses";
-                    return;
                 }
-            } else {
                 valid = false;
                 return;
             }
+
         }
 
-        if (!isTarget(getUnit().getTile(), getUnit())) {
+        if (!isTarget(getUnit().getTile(), getUnit(), scoutEquipment)) {
             GoalDecider destinationDecider = new GoalDecider() {
                 private PathNode best = null;
 
@@ -161,7 +166,7 @@ public class ScoutingMission extends Mission {
 
                 public boolean check(Unit u, PathNode pathNode) {
                     Tile t = pathNode.getTile();
-                    boolean target = isTarget(t, getUnit());
+                    boolean target = isTarget(t, getUnit(), scoutEquipment);
                     if (target) {
                         best = pathNode;
                         debugAction = "Target: " + t.getPosition();
@@ -173,13 +178,13 @@ public class ScoutingMission extends Mission {
 
             if (bestPath != null) {
                 transportDestination = null;
-                int direction = moveTowards(connection, bestPath);
-                if (direction >= 0) {
-                    final int mt = getUnit().getMoveType(direction);                                            
-                    if (getUnit().getMoveType(direction) == Unit.ENTER_INDIAN_VILLAGE_WITH_SCOUT) {
+                Direction direction = moveTowards(connection, bestPath);
+                if (direction != null) {
+                    final MoveType mt = getUnit().getMoveType(direction);             
+                    if (getUnit().getMoveType(direction) == MoveType.ENTER_INDIAN_VILLAGE_WITH_SCOUT) {
                         Element scoutMessage = Message.createNewRootElement("scoutIndianSettlement");
                         scoutMessage.setAttribute("unit", getUnit().getId());
-                        scoutMessage.setAttribute("direction", Integer.toString(direction));
+                        scoutMessage.setAttribute("direction", direction.toString());
                         scoutMessage.setAttribute("action", "basic");
                         try {
                             connection.ask(scoutMessage);
@@ -197,12 +202,12 @@ public class ScoutingMission extends Mission {
                         if (getUnit().isDisposed()) {
                             return;
                         }
-                    } else if (mt != Unit.ILLEGAL_MOVE && mt != Unit.ATTACK) {
+                    } else if (mt != MoveType.ILLEGAL_MOVE && mt != MoveType.ATTACK) {
                         move(connection, direction);
                     }
                 }
             } else {
-                if (transportDestination != null && !isTarget(transportDestination, getUnit())) {
+                if (transportDestination != null && !isTarget(transportDestination, getUnit(), scoutEquipment)) {
                     transportDestination = null;
                 }
                 if (transportDestination == null) {
@@ -216,25 +221,29 @@ public class ScoutingMission extends Mission {
             return;
         }
 
-        if (isTarget(getUnit().getTile(), getUnit()) && getUnit().getColony() != null) {
-            Element equipUnitElement = Message.createNewRootElement("equipunit");
-            equipUnitElement.setAttribute("unit", getUnit().getId());
-            equipUnitElement.setAttribute("type", Integer.toString(Goods.HORSES));
-            equipUnitElement.setAttribute("amount", Integer.toString(0));
-            try {
-                connection.ask(equipUnitElement);
-            } catch (IOException e) {
-                logger.warning("Could not send \"equipUnit (0)\"-message!");
-                return;
+        if (isTarget(getUnit().getTile(), getUnit(), scoutEquipment) &&
+            getUnit().getColony() != null) {
+            if (scoutEquipment != null) {
+                Element equipUnitElement = Message.createNewRootElement("equipUnit");
+                equipUnitElement.setAttribute("unit", getUnit().getId());
+                equipUnitElement.setAttribute("type", scoutEquipment.getId());
+                equipUnitElement.setAttribute("amount", "0");
+                try {
+                    connection.ask(equipUnitElement);
+                    scoutEquipment = null;
+                } catch (IOException e) {
+                    logger.warning("Could not send \"equipUnit (0)\"-message!");
+                    return;
+                }
+                debugAction = "Awaiting 52 horses";
             }
-            debugAction = "Awaiting 52 horses";
         }
     }
         
     private void updateTransportDestination() {
         if (getUnit().getTile() == null) {
             transportDestination = (Tile) getUnit().getOwner().getEntryLocation();
-        } else if (getUnit().getLocation() instanceof Unit) {
+        } else if (getUnit().isOnCarrier()) {
             GoalDecider destinationDecider = new GoalDecider() {
                 private PathNode best = null;
 
@@ -249,7 +258,7 @@ public class ScoutingMission extends Mission {
 
                 public boolean check(Unit u, PathNode pathNode) {
                     Tile t = pathNode.getTile();
-                    boolean target = isTarget(t, getUnit());
+                    boolean target = isTarget(t, getUnit(), scoutEquipment);
                     if (target) {
                         best = pathNode;
                         debugAction = "Target: " + t.getPosition();
@@ -269,7 +278,7 @@ public class ScoutingMission extends Mission {
             Iterator<Position> it = getGame().getMap().getFloodFillIterator(getUnit().getTile().getPosition());
             while (it.hasNext()) {
                 Tile t = getGame().getMap().getTile(it.next());
-                if (isTarget(t, getUnit())) {
+                if (isTarget(t, getUnit(), scoutEquipment)) {
                     transportDestination = t;
                     debugAction = "Transport to: " + transportDestination.getPosition();
                     return;
@@ -280,13 +289,20 @@ public class ScoutingMission extends Mission {
         }
     }
 
-    private static boolean isTarget(Tile t, Unit u) {
+    private static boolean isTarget(Tile t, Unit u, EquipmentType scoutEquipment) {
         if (t.hasLostCityRumour()) {
             return true;
-        } else if (t.getColony() != null && t.getColony().getOwner() == u.getOwner()
-                && t.getColony().getGoodsContainer().getGoodsCount(Goods.HORSES) <= 1
-                && t.getColony().getFoodProduction() - t.getColony().getFoodConsumption() >= 2) {
-            return true;
+        } else if (scoutEquipment != null && t.getColony() != null &&
+                   t.getColony().getOwner() == u.getOwner()) {
+            for (AbstractGoods goods : scoutEquipment.getGoodsRequired()) {
+                if (goods.getType().isBreedable() &&
+                    !t.getColony().canBreed(goods.getType()) &&
+                    // TODO: remove assumptions about auto-production implementation
+                    t.getColony().getProductionNetOf(goods.getType()) > 1) {
+                    return true;
+                }
+            }
+            return false;
         } else if (t.getSettlement() != null && t.getSettlement() instanceof IndianSettlement
                 && !((IndianSettlement) t.getSettlement()).hasBeenVisited()) {
             return true;
@@ -304,7 +320,7 @@ public class ScoutingMission extends Mission {
      * @return The destination for this <code>Transportable</code>.
      */
     public Tile getTransportDestination() {
-        if (getUnit().getLocation() instanceof Unit
+        if (getUnit().isOnCarrier()
                 || getUnit().getTile() == null) {
             if (transportDestination == null
                     || !transportDestination.isLand()) {
@@ -339,7 +355,13 @@ public class ScoutingMission extends Mission {
      *         <code>false</code> otherwise.
      */
     public boolean isValid() {
-        return valid;
+        Unit unit = getUnit();
+        // unit no longer has horses and not in a colony where it may get some
+        // cannot fulfill role of scout anymore
+        if(!unit.isMounted() && unit.getTile().getColony() == null){
+            return false;
+        }
+        return valid && super.isValid();
     }
 
     /**
@@ -356,7 +378,7 @@ public class ScoutingMission extends Mission {
         Iterator<Position> it = au.getGame().getMap().getFloodFillIterator(au.getUnit().getTile().getPosition());
         while (it.hasNext()) {
             Tile t = au.getGame().getMap().getTile(it.next());
-            if (isTarget(t, au.getUnit())) {
+            if (isTarget(t, au.getUnit(), null)) {
                 return true;
             }
         }
@@ -408,4 +430,5 @@ public class ScoutingMission extends Mission {
     public String getDebuggingInfo() {
         return debugAction;
     }
+
 }
