@@ -72,9 +72,6 @@ import org.w3c.dom.Element;
 public class TransportMission extends Mission {
     private static final Logger logger = Logger.getLogger(TransportMission.class.getName());
 
-
-
-
     private static final String ELEMENT_TRANSPORTABLE = "transportable";
 
     private static final int MINIMUM_GOLD_TO_STAY_IN_EUROPE = 600;
@@ -473,13 +470,7 @@ public class TransportMission extends Mission {
                     // Tile target = getGame().getMap().getNeighbourOrNull(r,
                     // carrier.getTile());
                     if (carrier.getMoveType(r) == MoveType.MOVE_HIGH_SEAS && moveToEurope) {
-                        Element moveToEuropeElement = Message.createNewRootElement("moveToEurope");
-                        moveToEuropeElement.setAttribute("unit", carrier.getId());
-                        try {
-                            connection.sendAndWait(moveToEuropeElement);
-                        } catch (IOException e) {
-                            logger.warning("Could not send \"moveToEuropeElement\"-message!");
-                        }
+                        moveUnitToEurope(connection, carrier);
                     } else {
                         move(connection, r);
                     }
@@ -491,13 +482,7 @@ public class TransportMission extends Mission {
 
                 transportListChanged = restockCargoAtDestination(connection);
             } else if (moveToEurope && carrier.canMoveToEurope()) {
-                Element moveToEuropeElement = Message.createNewRootElement("moveToEurope");
-                moveToEuropeElement.setAttribute("unit", carrier.getId());
-                try {
-                    connection.sendAndWait(moveToEuropeElement);
-                } catch (IOException e) {
-                    logger.warning("Could not send \"moveToEuropeElement\"-message (2)!");
-                }
+                moveUnitToEurope(connection, carrier);
             }
         }
     }
@@ -940,13 +925,7 @@ public class TransportMission extends Mission {
                             && au.getTransportDestination().getTile() == carrier.getTile()
                             && carrier.getState() != UnitState.TO_EUROPE && carrier.getState() != UnitState.TO_AMERICA) {
                         if (u.getLocation() instanceof Europe || u.getColony() != null) {
-                            Element leaveShipElement = Message.createNewRootElement("leaveShip");
-                            leaveShipElement.setAttribute("unit", u.getId());
-                            try {
-                                connection.sendAndWait(leaveShipElement);
-                            } catch (IOException e) {
-                                logger.warning("Could not send \"leaveShipElement\"-message!");
-                            }
+                            unitLeavesShip(connection, u);
                         }
                         mission.doMission(connection);
                         if (u.getLocation() != getUnit()) {
@@ -1004,35 +983,18 @@ public class TransportMission extends Mission {
                         && ag.getTransportDestination().getTile() == carrier.getLocation().getTile()
                         && carrier.getState() != UnitState.TO_EUROPE && carrier.getState() != UnitState.TO_AMERICA) {
                     if (carrier.getLocation() instanceof Europe) {
-                        // TODO-AI-CHEATING: REMOVE WHEN THE AI IS GOOD ENOUGH:
-                        Player p = carrier.getOwner();
-                        if (p.isAI() && getAIMain().getFreeColServer().isSingleplayer()) {
-                            // Double the income by adding this bonus:
-                            p.modifyGold(p.getMarket().getSalePrice(ag.getGoods()));
-                        }
-                        // END: TODO-AI-CHEATING.
-                        Element sellGoodsElement = Message.createNewRootElement("sellGoods");
-                        sellGoodsElement.appendChild(ag.getGoods().toXMLElement(carrier.getOwner(),
-                                sellGoodsElement.getOwnerDocument()));
-                        try {
-                            connection.sendAndWait(sellGoodsElement);
+                        boolean success = sellCargoInEurope(connection, carrier, ag.getGoods());
+                        if(success){
                             removeFromTransportList(ag);
                             ag.dispose();
                             transportListChanged = true;
-                        } catch (IOException e) {
-                            logger.warning("Could not send \"sellGoodsElement\"-message!");
                         }
                     } else {
-                        Element unloadCargoElement = Message.createNewRootElement("unloadCargo");
-                        unloadCargoElement.appendChild(ag.getGoods().toXMLElement(carrier.getOwner(),
-                                unloadCargoElement.getOwnerDocument()));
-                        try {
-                            connection.sendAndWait(unloadCargoElement);
+                        boolean success = unloadCargoInColony(connection, carrier, ag.getGoods());
+                        if(success){
                             removeFromTransportList(ag);
                             ag.dispose();
                             transportListChanged = true;
-                        } catch (IOException e) {
-                            logger.warning("Could not send \"unloadCargoElement\"-message!");
                         }
                     }
                 }
@@ -1044,7 +1006,7 @@ public class TransportMission extends Mission {
         return transportListChanged;
     }
 
-    /**
+	/**
      * Loads any <code>Transportable</code>s being in range of the carrier.
      * 
      * @param connection The <code>Connection</code> to the server.
@@ -1121,17 +1083,60 @@ public class TransportMission extends Mission {
         return transportListChanged;
     }
 
+    /**
+     * Checks if this mission is valid for the given unit.
+     *
+     * @param aiUnit The unit.
+     * @return <code>true</code> if this mission is valid to perform
+     *         and <code>false</code> otherwise.
+     */    
+    public static boolean isValid(AIUnit aiUnit) {
+        Unit unit = aiUnit.getUnit();
+        
+        if(!unit.isNaval()){
+        	return true;
+        }
+        
+        if(unit.isUnderRepair()){
+        	return false;
+        }
+
+        boolean hasCargo = unit.getGoodsCount() > 0 || unit.getUnitCount() > 0;
+        if(hasCargo){
+        	return true;
+        }
+    
+        // Verify if empty unit is a privateer and able to be assigned a PrivateerMisison 
+        if(unit.hasAbility("model.ability.piracy")){
+            AIPlayer aiPlayer = (AIPlayer) aiUnit.getAIMain().getAIObject(unit.getOwner().getId());
+        	// Do not consider this unit mission
+        	int transportMissions = getPlayerNavalTransportMissionCount(aiPlayer,unit);
+        	if(transportMissions > 0){
+        		// there are other naval units doing Transport missions
+        		// this one can do some pirating
+        		logger.finest("Privateer (" + unit.getId() + ") at " + unit.getTile() + " does no longer have TransportMission");
+        		return false;
+        	}
+        }
+        
+        return true;
+    }
+    
     /**
      * Checks if this mission is still valid to perform.
      * 
      * @return <code>true</code>
      */
     public boolean isValid() {
-        Unit unit = this.getAIUnit().getUnit();
-        if (unit.isNaval() && unit.isUnderRepair())
+        if(!super.isValid()){
             return false;
-        else
-            return true;
+        }
+        
+        AIUnit aiUnit = getAIUnit(); 
+        if(!isValid(aiUnit)){
+            return false;
+        }
+        return true;
     }
 
     /**
@@ -1164,13 +1169,7 @@ public class TransportMission extends Mission {
         // Move back to America:
         Unit carrier = getUnit();
         if (carrier.getOwner().getGold() < MINIMUM_GOLD_TO_STAY_IN_EUROPE || transportList.size() > 0) {
-            Element moveToAmericaElement = Message.createNewRootElement("moveToAmerica");
-            moveToAmericaElement.setAttribute("unit", carrier.getId());
-            try {
-                connection.sendAndWait(moveToAmericaElement);
-            } catch (IOException e) {
-                logger.warning("Could not send \"moveToAmericaElement\"-message!");
-            }
+            moveUnitToAmerica(connection, carrier);
         }        
     }
     
@@ -1185,6 +1184,28 @@ public class TransportMission extends Mission {
     protected PathNode findPathToEurope(Tile start) {
         return getGame().getMap().findPathToEurope(getUnit(), start);
     }
+    
+    /**
+     * Gives the number of naval units assigned with a Transport Mission
+     */
+    public static int getPlayerNavalTransportMissionCount(AIPlayer aiPlayer, Unit unitExcluded){
+        Player player = aiPlayer.getPlayer();
+        int units = 0;
+        
+        for(Unit unit : player.getUnits()){
+        	if(unit == unitExcluded){
+        		continue;
+        	}
+            if(!unit.isNaval()){
+                continue;
+            }
+            AIUnit aiUnit = (AIUnit) aiPlayer.getAIMain().getAIObject(unit);
+            if(aiUnit.getMission() instanceof TransportMission){
+                units++;
+            }
+        }
+        return units;
+    }
 
     /**
      * Writes all of the <code>AIObject</code>s and other AI-related
