@@ -696,10 +696,22 @@ public final class InGameController extends Controller {
     }
     
     public boolean createMission(IndianSettlement settlement,Unit missionary){
-        settlement.setMissionary(missionary);
-        
+    	boolean success = false;
+    	switch(settlement.getAlarm(missionary.getOwner()).getLevel()){
+    		case HAPPY:
+    		case CONTENT:
+    		case DISPLEASED:
+    			success = true;
+    			settlement.setMissionary(missionary);
+    			break;
+    		case ANGRY:
+    		case HATEFUL:
+    			success = false;
+    			missionary.dispose();
+    			break;
+    	}   
         //TODO: make possibility of indians refusing the mission 
-        return true;
+        return success;
     }
 
     private void bombardEnemyShips(ServerPlayer currentPlayer) {
