@@ -24,18 +24,23 @@ import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
-import java.util.Vector;
 import java.util.logging.Logger;
 
 import net.sf.freecol.common.PseudoRandom;
+import net.sf.freecol.common.model.Building;
+import net.sf.freecol.common.model.BuildingType;
+import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.FreeColGameObject;
 import net.sf.freecol.common.model.Game;
 import net.sf.freecol.common.model.Location;
 import net.sf.freecol.common.model.ModelController;
 import net.sf.freecol.common.model.Player;
+import net.sf.freecol.common.model.Player.Stance;
 import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.TradeRoute;
 import net.sf.freecol.common.model.Unit;
+import net.sf.freecol.common.model.Unit.UnitState;
+import net.sf.freecol.common.model.UnitType;
 import net.sf.freecol.common.networking.Connection;
 import net.sf.freecol.common.networking.Message;
 import net.sf.freecol.server.FreeColServer;
@@ -47,11 +52,9 @@ import org.w3c.dom.Element;
  * A server-side implementation of the <code>ModelController</code> interface.
  */
 public class ServerModelController implements ModelController {
+
     private static final Logger logger = Logger.getLogger(ServerModelController.class.getName());
 
-
-
-
     private final FreeColServer freeColServer;
 
     private final HashMap<String, TaskEntry> taskRegister = new HashMap<String, TaskEntry>();
@@ -83,7 +86,7 @@ public class ServerModelController implements ModelController {
     public synchronized int getRandom(String taskID, int n) {
         int turnNumber = freeColServer.getGame().getTurn().getNumber();
         String extendedTaskID = taskID + Integer.toString(turnNumber);
-        logger.info("Entering getRandom with taskID " + taskID);
+        //logger.info("Entering getRandom with taskID " + taskID);
         if (taskRegister.containsKey(extendedTaskID)) {
             return ((Integer) taskRegister.get(extendedTaskID).entry).intValue();
         } else {
@@ -130,7 +133,7 @@ public class ServerModelController implements ModelController {
 
     /**
      * Creates a new unit. This method is the same as running
-     * {@link #createUnit(String, Location, Player, int, boolean, Connection)}
+     * {@link #createUnit(String, Location, Player, UnitType, boolean, Connection)}
      * with <code>secure = true</code> and <code>connection = null</code>.
      * 
      * @param taskID The <code>taskID</code> should be a unique identifier.
@@ -145,7 +148,7 @@ public class ServerModelController implements ModelController {
      * @param type The type of unit (Unit.FREE_COLONIST...).
      * @return A reference to the <code>Unit</code> which has been created.
      */
-    public synchronized Unit createUnit(String taskID, Location location, Player owner, int type) {
+    public synchronized Unit createUnit(String taskID, Location location, Player owner, UnitType type) {
         return createUnit(taskID, location, owner, type, true, null);
     }
 
@@ -170,7 +173,7 @@ public class ServerModelController implements ModelController {
      *            or null if this request is internal to the server.
      * @return A reference to the <code>Unit</code> which has been created.
      */
-    public synchronized Unit createUnit(String taskID, Location location, Player owner, int type, boolean secure,
+    public synchronized Unit createUnit(String taskID, Location location, Player owner, UnitType type, boolean secure,
             Connection connection) {
         String extendedTaskID = taskID + owner.getId()
                 + Integer.toString(freeColServer.getGame().getTurn().getNumber());
@@ -201,7 +204,7 @@ public class ServerModelController implements ModelController {
                 taskEntry.secure = true;
             }
         } else {
-            unit = new Unit(freeColServer.getGame(), location, owner, type, Unit.ACTIVE);
+            unit = new Unit(freeColServer.getGame(), location, owner, type, UnitState.ACTIVE);
             taskEntry = new TaskEntry(extendedTaskID, freeColServer.getGame().getTurn().getNumber(), secure, unit);
             taskRegister.put(extendedTaskID, taskEntry);
         }
@@ -214,6 +217,91 @@ public class ServerModelController implements ModelController {
         return unit;
     }
 
+    /**
+     * Creates a new building. This method is the same as running
+     * {@link #createBuilding(String, Colony, BuildingType, boolean, Connection)}
+     * with <code>secure = true</code> and <code>connection = null</code>.
+     * 
+     * @param taskID The <code>taskID</code> should be a unique identifier.
+     *            One method to make a unique <code>taskID</code>: <br>
+     *            <br>
+     *            getId() + "methodName:taskDescription" <br>
+     *            <br> As long as the "taskDescription" is unique within the
+     *            method ("methodName"), you get a unique identifier.
+     * @param colony The <code>Colony</code> where the <code>Building</code>
+     *            will be created.
+     * @param type The type of building.
+     * @return A reference to the <code>Building</code> which has been created.
+     */
+    public synchronized Building createBuilding(String taskID, Colony colony, BuildingType type) {
+        return createBuilding(taskID, colony, type, true, null);
+    }
+
+    /**
+     * Creates a new building.
+     * 
+     * @param taskID The <code>taskID</code> should be a unique identifier.
+     *            One method to make a unique <code>taskID</code>: <br>
+     *            <br>
+     *            getId() + "methodName:taskDescription" <br>
+     *            <br> As long as the "taskDescription" is unique within the
+     *            method ("methodName"), you get a unique identifier.
+     * @param colony The <code>Colony</code> where the <code>Building</code>
+     *            will be created.
+     * @param type The type of building.
+     * @param secure This variable should be set to <code>false</code> in case
+     *            this method is called when serving a client. Setting this
+     *            variable to <code>false</code> signals that the request
+     *            might be illegal.
+     * @param connection The connection that has requested to create the building,
+     *            or null if this request is internal to the server.
+     * @return A reference to the <code>Building</code> which has been created.
+     */
+    public synchronized Building createBuilding(String taskID, Colony colony, BuildingType type, boolean secure,
+                                                Connection connection) {
+        String extendedTaskID = taskID + colony.getOwner().getId()
+                + Integer.toString(freeColServer.getGame().getTurn().getNumber());
+        Building building;
+        TaskEntry taskEntry;
+        Player owner = colony.getOwner();
+
+        logger.info("Entering createBuilding.");
+
+        if (taskRegister.containsKey(extendedTaskID)) {
+            taskEntry = taskRegister.get(extendedTaskID);
+            building = (Building) taskEntry.entry;
+
+            if (building.getColony().getTile() != colony.getTile() ||
+                building.getOwner() != colony.getOwner() ||
+                building.getType() != type) {
+                logger.warning("Unsynchronization between the client and the server. Maybe a cheating attempt! Differences: "
+                               + ((building.getColony().getTile() != colony.getTile()) ? "colony: "
+                                  + building.getColony().getTile() + "!=" + colony.getTile() : "")
+                               + ((building.getOwner() != owner) ? "owner: " + building.getOwner() + "!=" + owner : "")
+                               + ((building.getType() != type) ? "type: " + building.getType() + "!=" + type : ""));
+                
+                taskRegister.remove(extendedTaskID);
+                building.dispose();
+                return null;
+            }
+
+            if (secure) {
+                taskEntry.secure = true;
+            }
+        } else {
+            building = new Building(freeColServer.getGame(), colony,type);
+            taskEntry = new TaskEntry(extendedTaskID, freeColServer.getGame().getTurn().getNumber(), secure, building);
+            taskRegister.put(extendedTaskID, taskEntry);
+        }
+
+        /*
+         * if (connection != null) { update(building,
+         * freeColServer.getPlayer(connection)); }
+         */
+
+        return building;
+    }
+
     /**
      * Puts the specified <code>Unit</code> in America.
      * 
@@ -238,11 +326,11 @@ public class ServerModelController implements ModelController {
         }
 
         unit.setLocation(entryLocation);
-        unit.setState(Unit.ACTIVE);
+        unit.setState(UnitState.ACTIVE);
 
         // Display the tiles surrounding the Unit:
         Element updateElement = Message.createNewRootElement("update");
-        Vector<Tile> surroundingTiles = game.getMap().getSurroundingTiles(unit.getTile(), unit.getLineOfSight());
+        List<Tile> surroundingTiles = game.getMap().getSurroundingTiles(unit.getTile(), unit.getLineOfSight());
 
         for (int i = 0; i < surroundingTiles.size(); i++) {
             Tile t = surroundingTiles.get(i);
@@ -300,9 +388,9 @@ public class ServerModelController implements ModelController {
      * @param second The second <code>Player</code>.
      * @param stance The new stance.
      */
-    public void setStance(Player first, Player second, int stance) {
+    public void setStance(Player first, Player second, Stance stance) {
         Element element = Message.createNewRootElement("setStance");
-        element.setAttribute("stance", Integer.toString(stance));
+        element.setAttribute("stance", stance.toString());
         element.setAttribute("first", first.getId());
         element.setAttribute("second", second.getId());
 
