@@ -30,6 +30,7 @@ import javax.xml.stream.XMLStreamWriter;
 import net.sf.freecol.common.model.Game;
 import net.sf.freecol.common.model.GameOptions;
 import net.sf.freecol.common.model.Map;
+import net.sf.freecol.common.model.Nation;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.Unit;
@@ -82,7 +83,6 @@ public class ServerPlayer extends Player implements ServerModelObject {
         connected = (connection != null);
     }
 
-
     /**
     * Creates a new <code>ServerPlayer</code>.
     *
@@ -94,7 +94,8 @@ public class ServerPlayer extends Player implements ServerModelObject {
     * @param connection The <code>Connection</code> for the above mentioned socket.
     * @param nation The nation of the <code>Player</code>.
     */
-    public ServerPlayer(Game game, String name, boolean admin, boolean ai, Socket socket, Connection connection, int nation) {
+    public ServerPlayer(Game game, String name, boolean admin, boolean ai, Socket socket, Connection connection,
+                        Nation nation) {
         super(game, name, admin, ai, nation);
 
         this.socket = socket;
@@ -311,6 +312,6 @@ public class ServerPlayer extends Player implements ServerModelObject {
     
     @Override
     public String toString() {
-        return "ServerPlayer[id=" + serverID + ",conn=" + connection + "]";
+        return "ServerPlayer[name="+getName()+",serverID=" + serverID + ",conn=" + connection + "]";
     }
 }
