@@ -19,13 +19,20 @@
 
 package net.sf.freecol.common.model;
 
+import java.util.ArrayList;
+import java.util.HashMap;
 import java.util.Iterator;
+import java.util.List;
 import java.util.Set;
 
+import net.sf.freecol.common.PseudoRandom;
 import net.sf.freecol.common.model.CombatModel.CombatResult;
+import net.sf.freecol.common.model.CombatModel.CombatResultType;
 import net.sf.freecol.common.model.Player.Stance;
+import net.sf.freecol.common.model.Unit.MoveType;
 import net.sf.freecol.common.model.Unit.UnitState;
 import net.sf.freecol.util.test.FreeColTestCase;
+import net.sf.freecol.util.test.MockPseudoRandom;
 
 public class CombatTest extends FreeColTestCase {
 
@@ -212,6 +219,8 @@ public class CombatTest extends FreeColTestCase {
         assertEquals(SimpleCombatModel.CARGO_PENALTY_SOURCE, goodsPenalty1.getSource());
         assertEquals(-12.5f, goodsPenalty1.getValue());
 
+        // Verify that the move is correctly interpreted
+        assertEquals("Wrong move type",MoveType.ATTACK, privateer.getMoveType(tile1));
     }
 
     public void testAtackedNavalUnitIsDamaged(){
@@ -233,6 +242,9 @@ public class CombatTest extends FreeColTestCase {
         Tile tile2 = map.getTile(4, 8);
         tile2.setExploredBy(dutch, true);
         tile2.setExploredBy(french, true);
+        Tile tile3 = map.getTile(6, 8);
+        tile3.setExploredBy(dutch, true);
+        tile3.setExploredBy(french, true);
 
         Unit galleon = new Unit(game, tile1, dutch, galleonType, UnitState.ACTIVE);
         Unit privateer = new Unit(game, tile2, french, privateerType, UnitState.ACTIVE);
@@ -249,6 +261,10 @@ public class CombatTest extends FreeColTestCase {
         errMsg = "Galeon should not be repairing";
         assertFalse(errMsg,galleon.isUnderRepair());
         
+        galleon.setDestination(tile3);
+        errMsg = "Wrong destination for Galeon";
+        assertEquals(errMsg, tile3, galleon.getDestination());
+        
         int damage = galleon.getHitpoints() -1;
         CombatModel.CombatResultType combatResolution = CombatModel.CombatResultType.WIN;
         
@@ -261,6 +277,9 @@ public class CombatTest extends FreeColTestCase {
         
         errMsg = "Galleon should be empty";
         assertEquals(errMsg,galleon.getGoodsCount(),0);
+        
+        errMsg = "Galeon should no longer have a destination";
+        assertTrue(errMsg, galleon.getDestination() == null);
     }
 
     public void testUnarmedAttack() {
@@ -436,7 +455,7 @@ public class CombatTest extends FreeColTestCase {
 
     }
 
-    public void testDefendColonyWithScout() {
+    public void testDefendColonyWithUnarmedColonist() {
     	Game game = getGame();
     	Map map = getTestMap(plainsType,true);
     	game.setMap(map);
@@ -445,35 +464,24 @@ public class CombatTest extends FreeColTestCase {
 
         SimpleCombatModel combatModel = new SimpleCombatModel(game.getModelController().getPseudoRandom());
         Player dutch = game.getPlayer("model.nation.dutch");
-        Player french = game.getPlayer("model.nation.frenchREF");
-
-        assertTrue(french.isREF());
+        Player inca = game.getPlayer("model.nation.inca");
 
         Tile tile2 = map.getTile(4, 8);
         tile2.setExploredBy(dutch, true);
-        tile2.setExploredBy(french, true);
+        tile2.setExploredBy(inca, true);
 
         Unit colonist = colony.getUnitIterator().next();
-        Unit defender = new Unit(getGame(), colony.getTile(), dutch, colonistType, UnitState.ACTIVE, horses);
-        Unit attacker = new Unit(getGame(), tile2, french, veteranType, UnitState.ACTIVE, horses, muskets);
+        Unit attacker = new Unit(getGame(), tile2, inca, braveType, UnitState.ACTIVE, indianHorses, indianMuskets);
 
-        assertTrue(defender.isDefensiveUnit());
+        assertEquals(colonist, colony.getDefendingUnit(attacker));
+        assertEquals(colonist, colony.getTile().getDefendingUnit(attacker));
+
+        Unit defender = new Unit(getGame(), colony.getTile(), dutch, colonistType, UnitState.ACTIVE);
+        assertFalse("Colonist should not be defensive unit",defender.isDefensiveUnit());
         assertEquals(defender, colony.getTile().getDefendingUnit(attacker));
 
-        // defender should lose horses
-        combatModel.attack(attacker, defender, victory, 0, null);
-        assertTrue(attacker.isMounted());
-        assertTrue(attacker.isArmed());
-        assertEquals(veteranType, attacker.getType());
-        assertFalse(defender.isMounted());
-        assertFalse(defender.isArmed());
-        assertEquals(colonistType, defender.getType());
-        // this should force DONE_SETTLEMENT
-        assertFalse(defender.isDefensiveUnit());
-
     }
 
-
     public void testDefendColonyWithRevere() {
     	Game game = getGame();
     	Map map = getTestMap(plainsType,true);
@@ -509,6 +517,57 @@ public class CombatTest extends FreeColTestCase {
             assertFalse(defenceModifiers.contains(defenceModifier));
         }
     }
+    
+    public void testLoseColonyDefenceWithRevere() {
+        Game game = getGame();
+        Map map = getTestMap(plainsType,true);
+        game.setMap(map);
+        
+        Colony colony = getStandardColony();
+     
+        Player dutch = game.getPlayer("model.nation.dutch");
+        Player inca = game.getPlayer("model.nation.inca");
+        dutch.changeRelationWithPlayer(inca, Stance.WAR);
+        
+        Tile tile2 = map.getTile(4, 8);
+        tile2.setExploredBy(dutch, true);
+        tile2.setExploredBy(inca, true);
+
+        Unit colonist = colony.getUnitIterator().next();
+        Unit attacker = new Unit(getGame(), tile2, inca, braveType, UnitState.ACTIVE, indianHorses, indianMuskets);
+
+        assertEquals(colonist, colony.getDefendingUnit(attacker));
+
+        dutch.addFather(spec().getFoundingFather("model.foundingFather.paulRevere"));
+        
+        java.util.Map<GoodsType,Integer> goodsAdded = new HashMap<GoodsType,Integer>();
+        for (AbstractGoods goods : muskets.getGoodsRequired()) {
+            colony.addGoods(goods);
+            goodsAdded.put(goods.getType(), goods.getAmount());
+        }
+
+        // We need a deterministic random
+        List<Integer> setValues = new ArrayList<Integer>();
+        setValues.add(1);
+        PseudoRandom mockRandom = new MockPseudoRandom(setValues,false);
+        SimpleCombatModel combatModel = new SimpleCombatModel(mockRandom);
+
+        CombatResult result = combatModel.generateAttackResult(attacker, colonist);
+        
+        String errMsg = "Wrong combat result, cannot be DONE_SETTLEMENT";
+        assertFalse(errMsg,result.type == CombatResultType.DONE_SETTLEMENT);
+        
+        combatModel.attack(attacker, colonist, victory, 0, null);
+        
+        assertFalse("Colonist should not be disposed",colonist.isDisposed());
+        assertFalse("Colonist should not be captured",colonist.getOwner() == attacker.getOwner());
+        
+        for (AbstractGoods goods : muskets.getGoodsRequired()) {
+            boolean goodsLost = colony.getGoodsCount(goods.getType()) < goodsAdded.get(goods.getType());
+            errMsg = "Colony should have lose " + goods.getType().getName();
+            assertTrue(errMsg,goodsLost);
+        }
+    }
 
     public void testDefendSettlement() {
 
@@ -550,7 +609,7 @@ public class CombatTest extends FreeColTestCase {
         }
     }
     
-    public void testPionnerDiesNotLosesEquipment() {
+    public void testPioneerDiesNotLosesEquipment() {
     	Game game = getStandardGame();
         CombatModel combatModel = game.getCombatModel();
         Player dutch = game.getPlayer("model.nation.dutch");
@@ -565,16 +624,42 @@ public class CombatTest extends FreeColTestCase {
         tile2.setExploredBy(dutch, true);
         tile2.setExploredBy(french, true);
 
-        Unit pionner = new Unit(game, tile1, dutch, colonistType, UnitState.ACTIVE);
+        Unit pioneer = new Unit(game, tile1, dutch, colonistType, UnitState.ACTIVE);
         Unit soldier = new Unit(game, tile2, french, veteranType, UnitState.ACTIVE);
 
         soldier.equipWith(muskets, true);
         soldier.equipWith(horses, true);
         soldier.setMovesLeft(1);
-        pionner.equipWith(tools, true);
+        pioneer.equipWith(tools, true);
 
-        combatModel.attack(soldier, pionner, victory, 0, null);
-        assertTrue("Pionner should be dead",pionner.isDisposed());
+        combatModel.attack(soldier, pioneer, victory, 0, null);
+        assertTrue("Pioneer should be dead", pioneer.isDisposed());
     }
 
+    public void testScoutDiesNotLosesEquipment() {
+        Game game = getStandardGame();
+        CombatModel combatModel = game.getCombatModel();
+        Player dutch = game.getPlayer("model.nation.dutch");
+        Player french = game.getPlayer("model.nation.french");
+        Map map = getTestMap();
+        game.setMap(map);
+        
+        Tile tile1 = map.getTile(5, 8);       
+        tile1.setExploredBy(dutch, true);
+        tile1.setExploredBy(french, true);
+        Tile tile2 = map.getTile(4, 8);
+        tile2.setExploredBy(dutch, true);
+        tile2.setExploredBy(french, true);
+
+        Unit scout = new Unit(game, tile1, dutch, colonistType, UnitState.ACTIVE);
+        Unit soldier = new Unit(game, tile2, french, veteranType, UnitState.ACTIVE);
+
+        soldier.equipWith(muskets, true);
+        soldier.equipWith(horses, true);
+        scout.setMovesLeft(1);
+        scout.equipWith(horses, true);
+
+        combatModel.attack(soldier, scout, victory, 0, null);
+        assertTrue("Scout should be dead", scout.isDisposed());
+    }
 }
\ No newline at end of file
