@@ -19,25 +19,26 @@
 
 package net.sf.freecol.common.model;
 
-import java.util.Iterator;
-
 import net.sf.freecol.util.test.FreeColTestCase;
 
 public class SoLTest extends FreeColTestCase {
 
 
     public void testSoL() {
+    	Game game = getGame();
+    	game.setMap(getTestMap(plainsType,true));
         
         Colony colony = getStandardColony(7);
-	for (int i = 1; i <= 100; i++) {
-	    colony.addSoL(1);
-	    colony.updatePopulation();
-	    assertEquals(colony.getSoL(), i);
-	    assertEquals(colony.getTory(), 100 - i);
-	    assertEquals(colony.getMembers(), Math.round(i * 7 / 100f));
+        for (int i = 1; i <= 100; i++) {
+            int requiredBells = ((colony.getSoL() + 1) * Colony.BELLS_PER_REBEL *
+                                 colony.getUnitCount()) / 100;
+            colony.addGoods(Goods.BELLS, requiredBells - colony.getGoodsCount(Goods.BELLS));
+            colony.updatePopulation(0);
+            assertEquals(colony.getSoL(), i);
+            assertEquals(colony.getTory(), 100 - i);
+            assertEquals(colony.getMembers(), Math.round(i * 7 / 100f));
 
-	}
+        }
     }
 
-
 }
\ No newline at end of file
