@@ -23,9 +23,13 @@ import java.util.ArrayList;
 import java.util.List;
 
 import net.sf.freecol.FreeCol;
+import net.sf.freecol.common.PseudoRandom;
 import net.sf.freecol.common.Specification;
+import net.sf.freecol.common.model.Map.Direction;
 import net.sf.freecol.common.model.Unit.UnitState;
 import net.sf.freecol.util.test.FreeColTestCase;
+import net.sf.freecol.util.test.MockModelController;
+import net.sf.freecol.util.test.MockPseudoRandom;
 
 public class UnitTest extends FreeColTestCase {
 
@@ -59,6 +63,8 @@ public class UnitTest extends FreeColTestCase {
     EquipmentType musketsType = spec().getEquipmentType("model.equipment.muskets");
 
     UnitType colonistType = spec().getUnitType("model.unit.freeColonist");
+    UnitType hardyPioneerType = spec().getUnitType("model.unit.hardyPioneer");
+    UnitType expertFarmerType = spec().getUnitType("model.unit.expertFarmer");
     UnitType galleonType = spec().getUnitType("model.unit.galleon");
     UnitType caravelType = spec().getUnitType("model.unit.caravel");
     UnitType wagonType = spec().getUnitType("model.unit.wagonTrain");
@@ -403,8 +409,7 @@ public class UnitTest extends FreeColTestCase {
         Game game = getStandardGame();
         Player dutch = game.getPlayer("model.nation.dutch");
 
-        Unit galleon = new Unit(game, null, dutch, spec().getUnitType("model.unit.galleon"),
-                                UnitState.ACTIVE);
+        Unit galleon = new Unit(game, null, dutch, galleonType, UnitState.ACTIVE);
         Unit caravel = new Unit(game, null, dutch, spec().getUnitType("model.unit.caravel"),
                                 UnitState.ACTIVE);
         Unit colonist = new Unit(game, null, dutch, spec().getUnitType("model.unit.freeColonist"),
@@ -442,6 +447,34 @@ public class UnitTest extends FreeColTestCase {
 
     }
     
+    public void testFailedAddGoods(){
+        Game game = getStandardGame();
+        Map map = getTestMap();
+        game.setMap(map);
+        
+        Colony colony = this.getStandardColony();
+        int foodInColony = 300;
+        colony.addGoods(foodType, foodInColony);
+        assertEquals("Setup error, colony does not have expected goods quantities",foodInColony,colony.getGoodsCount(foodType));
+        
+        Player dutch = game.getPlayer("model.nation.dutch");
+        Unit wagonTrain = new Unit(game, colony.getTile(), dutch, spec().getUnitType("model.unit.wagonTrain"),
+                UnitState.ACTIVE);
+        int initialMoves = wagonTrain.getInitialMovesLeft();
+        assertEquals("Setup error, unit has wrong initial moves", initialMoves, wagonTrain.getMovesLeft());
+        assertTrue("Setup error, unit should not carry anything", wagonTrain.getGoodsCount() == 0);
+        
+        Goods tooManyGoods = colony.goodsContainer.getGoods(foodType);
+        try{
+        	wagonTrain.add(tooManyGoods);
+        	fail("Should have thrown an IllegalStateException");
+        }
+        catch(IllegalStateException e){
+        	assertTrue("Unit should not carry anything", wagonTrain.getGoodsCount() == 0);
+        	assertEquals("Unit moves should not have been modified", initialMoves, wagonTrain.getMovesLeft());
+        }
+    }
+        
     public void testMissionary() {
         Game game = getStandardGame();
         Map map = getTestMap(plains, true);
@@ -763,6 +796,31 @@ public class UnitTest extends FreeColTestCase {
 
     }
     
+    public void testEquipmentChange(){
+        Game game = getStandardGame();
+        Player dutch = game.getPlayer("model.nation.dutch");
+        Map map = getTestMap();
+        game.setMap(map);
+
+        Tile unitTile = map.getTile(6, 8);
+        
+        Unit unit = new Unit(game, unitTile, dutch, colonistType, UnitState.ACTIVE);
+        
+        assertFalse("Soldier should not have muskets",unit.getEquipmentCount(musketsType) > 0);
+        assertFalse("Soldier should not have horses",unit.getEquipmentCount(horsesType) > 0);
+        assertFalse("Soldier should not have tools",unit.getEquipmentCount(toolsType) > 0);
+        unit.equipWith(musketsType, true);
+        unit.equipWith(horsesType, true);
+        assertTrue("Soldier should be equiped with muskets",unit.getEquipmentCount(musketsType) > 0);
+        assertTrue("Soldier should be equiped with horses",unit.getEquipmentCount(horsesType) > 0);
+        assertFalse("Soldier should not have tools",unit.getEquipmentCount(toolsType) > 0);
+        
+        unit.equipWith(toolsType, true);
+        assertFalse("Soldier should no longer have muskets",unit.getEquipmentCount(musketsType) > 0);
+        assertFalse("Soldier should no longer have horses",unit.getEquipmentCount(horsesType) > 0);
+        assertTrue("Soldier should be equiped with tools",unit.getEquipmentCount(toolsType) > 0);
+    }
+    
     public void testUnitLocationAfterBuildingColony() {
         Game game = getStandardGame();
         Player dutch = game.getPlayer("model.nation.dutch");
@@ -898,5 +956,51 @@ public class UnitTest extends FreeColTestCase {
         assertEquals(UnitState.SENTRY, colonist.getState());
 
     }
+    
+    public void testUnitPromotionWorkingInWorkTile(){
+        Game game = getStandardGame();
+        Map map = getTestMap(plains);
+        game.setMap(map);
+        
+        Colony colony = getStandardColony();
+        
+        assertTrue("Colony should only have 1 colonist for test setup",colony.getUnitCount() == 1);
+        
+        Unit colonist = colony.getRandomUnit();
+        String errMsg = "Error setting test, colonist should not be an expert";
+        assertTrue(errMsg, colonist.getType() == colonistType);
+        
+        ColonyTile workTile = colony.getColonyTile(map.getNeighbourOrNull(Direction.N, colony.getTile()));
+        
+        // set colonist as farmer
+        colonist.setLocation(workTile);
+        colonist.setWorkType(foodType);
+        assertEquals("Wrong work allocation",foodType,colonist.getWorkType());
+        
+        // set some experience
+        int expectXP = 10;
+        colonist.modifyExperience(expectXP);
+        assertEquals("Wrong colonist experience",expectXP,colonist.getExperience());
+        
+        // We need a deterministic random
+        List<Integer> setValues = new ArrayList<Integer>();
+        setValues.add(1);
+        PseudoRandom mockRandom = new MockPseudoRandom(setValues,true);
+        MockModelController controller = (MockModelController) game.getModelController();
+        controller.setPseudoRandom(mockRandom);
+        
+        // Verify initial state
+        boolean isExpert = colonist.getType() == expertFarmerType;
+        assertFalse("Unit should not be an expert", isExpert);
+        
+        // Make upgrade
+        colonist.newTurn();
 
+        // verify upgrade
+        isExpert = colonist.getType() == expertFarmerType;
+        assertTrue("Unit should now be an expert", isExpert);
+        
+        // necessary to undo the deterministic random
+        controller.setPseudoRandom(null);
+    }
 }
