@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2007  The FreeCol Team
+ *  Copyright (C) 2002-2008  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -25,7 +25,7 @@ import junit.framework.TestSuite;
 public class AllTests {
 
 	public static Test suite() {
-		TestSuite suite = new TestSuite("Test for net.sf.freecol.common");
+		TestSuite suite = new TestSuite("Test for net.sf.freecol.server.generator");
 		//$JUnit-BEGIN$
 		suite.addTestSuite(MapGeneratorTest.class);
 		//$JUnit-END$
